' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports System.Text
Imports System.Security.Cryptography

Friend Module General
    ' ----- Public constants.
    Public Const ProgramTitle As String = "The Library Project"
    Public Const NotAuthorizedMessage As String = _
        "You are not authorized to perform this task."
    Public Const UseDBVersion As Integer = 1

    ' ----- Constants for the MatchingImages image list.
    Public Const MatchPresent As Integer = 0
    Public Const MatchNone As Integer = 1

    Public Enum LookupMethods As Integer
        ByTitle = 1
        ByAuthor = 2
        BySubject = 3
        ByKeywordAny = 4
        ByKeywordAll = 5
        ByPublisher = 6
        BySeries = 7
        BySeriesID = 8
        ByBarcode = 9
        ByDatabaseID = 10
        ByAuthorID = 11
        BySubjectID = 12
        ByPublisherID = 13
    End Enum

    ' ----- Security values.
    Public Enum LibrarySecurity As Integer
        ManageAuthors = 1
        ManageAuthorTypes = 2
        ManageCopyStatus = 3
        ManageMediaTypes = 4
        ManageSeries = 5
        ManageGroups = 6
        ManageItems = 7
        ManagePatrons = 8
        ManagePublishers = 9
        ManageValues = 10
        ManageUsers = 11
        ProcessFees = 12
        ManageLocations = 13
        CheckOutItems = 14
        CheckInItems = 15
        AdminFeatures = 16
        DailyProcessing = 17
        RunReports = 18
        PatronOverride = 19
        ManageBarcodeTemplates = 20
        ManageHolidays = 21
        ManagePatronGroups = 22
        ViewAdminPatronMessages = 23
    End Enum
    Public Const MaxLibrarySecurity As LibrarySecurity = LibrarySecurity.ViewAdminPatronMessages

    ' ----- Global variables.
    Public LibraryDB As System.Data.SqlClient.SqlConnection
    Public HoldTransaction As System.Data.SqlClient.SqlTransaction
    Public LoggedInUserID As Integer
    Public LoggedInUserName As String
    Public LoggedInGroupID As Integer
    Public SecurityProfile(MaxLibrarySecurity) As Boolean

    Public Function CenterText(ByVal origText As String, ByVal textWidth As Integer) As String
        ' ----- Center a piece of text in a field width. If the text is too
        '       wide, truncate it.
        Dim resultText As String

        resultText = Trim(origText)
        If (Len(resultText) >= textWidth) Then
            ' ----- Truncate as needed.
            Return Trim(Left(origText, textWidth))
        Else
            ' ----- Start with extra spaces.
            Return Space((textWidth - Len(origText)) \ 2) & resultText
        End If
    End Function

    Public Sub CleanUpProgram()
        ' ----- Prepare to exit the program.
        On Error Resume Next
        LibraryDB.Close()
    End Sub

    Public Function ConnectDatabase() As Boolean
        ' ----- Connect to the database. Return True on success.
        Dim connectionString As String

        ' ----- Initialize.
        HoldTransaction = Nothing

        ' ----- Build the connection string.
        ' !!! WARNING: Hardcoded for now.
        connectionString = "Data Source=MYSYSTEM\SQLEXPRESS;" & _
           "Initial Catalog=Library;Integrated Security=true"

        ' ----- Attempt to open the database.
        Try
            LibraryDB = New SqlClient.SqlConnection(connectionString)
            LibraryDB.Open()
        Catch ex As Exception
            GeneralError("ConnectDatabase", ex)
            Return False
        End Try

        ' ----- Success.
        Return True
    End Function

    Public Function CountSubStr(ByVal mainText As String, ByVal subText As String) As Integer
        ' ----- Return a count of the number of times that a subText occurs in
        '       a string (mainText).
        Dim totalTimes As Integer
        Dim startPos As Integer
        Dim foundPos As Integer

        totalTimes = 0
        startPos = 1

        ' ----- Keep searching until we don't find it no more!
        Do
            ' ----- Search for the subText.
            foundPos = InStr(startPos, mainText, subText)
            If (foundPos = 0) Then Exit Do
            totalTimes = totalTimes + 1

            ' ----- Move to just after the occurrence.
            startPos = foundPos + Len(subText)
        Loop

        ' ----- Return the count.
        Return totalTimes
    End Function

    Public Function CreateDataTable(ByVal sqlText As String) As Data.DataTable
        ' ----- Given a SQL statement, return a data table.
        Dim dbCommand As SqlClient.SqlCommand
        Dim dbAdapter As SqlClient.SqlDataAdapter
        Dim dbTable As Data.DataTable

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        dbCommand = New SqlClient.SqlCommand(sqlText, LibraryDB)
        If Not (HoldTransaction Is Nothing) Then dbCommand.Transaction = HoldTransaction
        dbAdapter = New SqlClient.SqlDataAdapter(dbCommand)
        dbTable = New Data.DataTable
        dbAdapter.Fill(dbTable)
        dbAdapter = Nothing
        dbCommand = Nothing
        Return dbTable
    End Function

    Public Function CreateReader(ByVal sqlText As String) As SqlClient.SqlDataReader
        ' ----- Given a SQL statement, return a data reader.
        Dim dbCommand As SqlClient.SqlCommand
        Dim dbScan As SqlClient.SqlDataReader

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        dbCommand = New SqlClient.SqlCommand(sqlText, LibraryDB)
        If Not (HoldTransaction Is Nothing) Then dbCommand.Transaction = HoldTransaction
        dbScan = dbCommand.ExecuteReader()
        dbCommand = Nothing
        Return dbScan
    End Function

    Public Function DBCombo(ByRef whichField As ComboBox) As String
        ' ----- Prepare the value of a combobox for use in the database.
        Dim listItem As Library.ListItemData

        ' ----- Get the stored data for this item.
        If (whichField.SelectedItem Is Nothing) Then Return "NULL"
        listItem = CType(whichField.SelectedItem, Library.ListItemData)

        ' ----- An item value of "-1" has special meaning: unselected.
        If (listItem.ItemData = -1) Then
            Return "NULL"
        Else
            Return CStr(listItem.ItemData)
        End If
    End Function

    Public Function DBDate(ByVal origText As String) As String
        ' ----- Prepare a date for insertion in a SQL statement.
        If (Trim(origText) = "") Then
            Return "NULL"
        ElseIf (IsDate(origText)) Then
            Return "'" & Format(CDate(origText), "d-MMM-yyyy") & "'"
        Else
            Return "NULL"
        End If
    End Function

    Public Function DBDate(ByVal origDate As Date) As String
        ' ----- Prepare a date for insertion in a SQL statement.
        Return "'" & Format(origDate, "d-MMM-yyyy") & "'"
    End Function

    Public Function DBGetDecimal(ByRef dataField As Object) As Decimal
        ' ----- Return the decimal equivalent of an optional database field.
        If (IsDbNull(dataField) = True) Then
            Return 0@
        Else
            Return CDec(dataField)
        End If
    End Function

    Public Function DBGetInteger(ByRef dataField As Object) As Integer
        ' ----- Return the integer equivalent of an optional database field.
        If (IsDbNull(dataField) = True) Then
            Return 0
        Else
            Return CInt(dataField)
        End If
    End Function

    Public Function DBGetText(ByRef dataField As Object) As String
        ' ----- Return the text equivalent of an optional database field.
        If (IsDbNull(dataField) = True) Then
            Return ""
        Else
            Return CStr(dataField)
        End If
    End Function

    Public Function DBNum(ByVal origText As String) As String
        ' ----- Prepare a number for insertion in a SQL statement.
        If (Trim(origText) = "") Then
            Return "NULL"
        Else
            Return Trim(origText)
        End If
    End Function

    Public Function DBText(ByVal origText As String) As String
        ' ----- Prepare a string for insertion in a SQL statement.
        If (Trim(origText) = "") Then
            Return "NULL"
        Else
            Return "'" & Replace(origText, "'", "''") & "'"
        End If
    End Function

    Public Function DigitsOnly(ByVal origText As String) As String
        ' ----- Return only the digits found in a string.
        Dim destText As String
        Dim counter As Integer

        ' ----- Examine each character.
        destText = ""
        For counter = 1 To Len(origText)
            If (IsNumeric(Mid(origText, counter, 1))) Then _
                destText &= Mid(origText, counter, 1)
        Next counter
        Return destText
    End Function

    Public Function EncryptPassword(ByVal loginID As String, _
            ByVal passwordText As String) As String
        ' ----- Given a user name and a password, encrypt the password so that it
        '       is not easy to decrypt. There is no limit on password length since
        '       it is going to be hashed anyway.
        Dim hashingFunction As HMACSHA1
        Dim secretKey() As Byte
        Dim hashValue() As Byte
        Dim counter As Integer
        Dim result As String = ""

        ' ----- Prepare the secret key. Force it to uppercase for consistency,
        '       and then stuff it in a byte array.
        secretKey = (New UnicodeEncoding).GetBytes(UCase(loginID))

        ' ----- Create the hashing component using Managed SHA-1.
        hashingFunction = New HMACSHA1(secretKey, True)

        ' ----- Calculate the hash value. One simple line of code.
        hashValue = hashingFunction.ComputeHash( _
            (New UnicodeEncoding).GetBytes(passwordText))

        ' ----- The hash value is ready, but I like things in plaintext when
        '       possible. Let's convert it to a long hex string.
        For counter = 0 To hashValue.Length - 1
            result &= Hex(hashValue(counter))
        Next counter

        ' ----- Stored passwords are limited to 20 characters.
        Return Left(result, 20)
    End Function

    Public Sub ExecuteSQL(ByVal sqlText As String)
        ' ----- Given a SQL statement, run it.
        Dim DBCommand As SqlClient.SqlCommand

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        DBCommand = New SqlClient.SqlCommand(sqlText, LibraryDB)
        If Not (HoldTransaction Is Nothing) Then DBCommand.Transaction = HoldTransaction
        DBCommand.ExecuteNonQuery()
    End Sub

    Public Function ExecuteSQLReturn(ByVal sqlText As String) As Object
        ' ----- Given a SQL statement, run it, and return a related result.
        Dim DBCommand As SqlClient.SqlCommand

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        DBCommand = New SqlClient.SqlCommand(sqlText, LibraryDB)
        If Not (HoldTransaction Is Nothing) Then DBCommand.Transaction = HoldTransaction
        Return DBCommand.ExecuteScalar()
    End Function

    Public Sub GeneralError(ByVal routineName As String, _
            ByVal theError As System.Exception)
        ' ----- Report an error to the user.
        On Error Resume Next

        MsgBox("The following error occurred at location '" & routineName & _
            "':" & vbCrLf & vbCrLf & theError.Message, _
            MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
        My.Application.Log.WriteException(theError)
    End Sub

    Public Function GetSubStr(ByVal origString As String, ByVal delim As String, _
            ByVal whichField As Integer) As String
        ' ----- Extracts a delimited string from another larger string.
        Dim stringParts() As String

        ' ----- Handle some errors.
        If (whichField < 0) Then Return ""
        If (Len(origString) < 1) Then Return ""
        If (Len(delim) = 0) Then Return ""

        ' ----- Break the string up into delimited parts.
        stringParts = Split(origString, delim)

        ' ----- See if the part we want exists and return it.
        If (whichField > UBound(stringParts) + 1) Then Return "" _
            Else Return stringParts(whichField - 1)
    End Function

    Public Function GetSystemValue(ByVal valueName As String) As String
        ' ----- Return the data portion of a system value name-data pair.
        Dim sqlText As String
        Dim returnValue As String

        ' ----- Retrieve the value.
        returnValue = ""
        sqlText = "SELECT ValueData FROM SystemValue WHERE UPPER(ValueName) = " & _
            DBText(UCase(valueName))
        Try
            returnValue = DBGetText(ExecuteSQLReturn(sqlText))
        Catch ex As Exception
            GeneralError("GetSystemValue", ex)
        End Try
        Return returnValue
    End Function

    Public Sub InitializeSystem()
        ' ----- Initialize global variables here.
        Dim counter As Integer

        ' ----- Clear security-related values.
        LoggedInUserID = -1
        LoggedInUserName = ""
        LoggedInGroupID = -1
        For counter = 1 To MaxLibrarySecurity
            SecurityProfile(counter) = False
        Next counter
    End Sub

    Public Function LeftAndRightText(ByVal origLeft As String, ByVal origRight As String, _
            ByVal textWidth As Integer) As String
        ' ----- Build a string of the indicated width that has both left-justified
        '       and right-justified text.
        Dim leftText As String
        Dim rightText As String

        leftText = Trim(origLeft)
        rightText = Trim(origRight)
        If (Len(leftText & rightText) >= textWidth) Then
            Return Trim(Left(leftText & rightText, textWidth))
        Else
            Return leftText & Space(textWidth - Len(leftText & rightText)) & rightText
        End If
    End Function

    Public Sub ReprocessSecuritySet()
        ' ----- Reload in the security set for the current user. If no user
        '       is logged in, clear all settings.
        Dim counter As Integer
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader = Nothing

        ' ----- Clear out the existing items.
        For counter = 1 To MaxLibrarySecurity
            SecurityProfile(counter) = False
        Next counter

        ' ----- Exit if there is no user logged in.
        If (LoggedInUserID = -1) Or (LoggedInGroupID = -1) Then Return

        Try
            ' ----- Load in the security elements for this user.
            sqlText = "SELECT ActivityID FROM GroupActivity WHERE GroupID = " & _
                LoggedInGroupID
            dbInfo = CreateReader(sqlText)
            Do While (dbInfo.Read)
                SecurityProfile(CInt(dbInfo!ActivityID)) = True
            Loop
            dbInfo.Close()
        Catch ex As Exception
            ' ----- Some database-related error.
            GeneralError("ReprocessSecuritySet", ex)
            If (dbInfo IsNot Nothing) Then dbInfo.Close()

            ' ----- Un-login the administrator through recursion.
            LoggedInUserID = -1
            LoggedInGroupID = -1
            ReprocessSecuritySet()
        Finally
            dbInfo = Nothing
        End Try
    End Sub

    Public Sub SetSystemValue(ByVal valueName As String, ByVal valueData As String)
        ' ----- Update a record in the SystemValue table.
        Dim sqlText As String

        Try
            ' ----- See if the entry already exists.
            sqlText = "SELECT COUNT(*) FROM SystemValue WHERE UPPER(ValueName) = " & _
                DBText(UCase(valueName))
            If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
                ' ----- Value already exists.
                sqlText = "UPDATE SystemValue SET ValueData = " & DBText(valueData) & _
                    " WHERE UPPER(ValueName) = " & DBText(UCase(valueName))
            Else
                ' ----- Need to create value.
                sqlText = "INSERT INTO SystemValue (ValueName, ValueData) VALUES (" & _
                    DBText(valueName) & ", " & DBText(valueData) & ")"
            End If

            ' ----- Update the value.
            ExecuteSQL(sqlText)
        Catch ex As System.Exception
            GeneralError("SetSystemValue", ex)
        End Try
    End Sub

    Public Sub TransactionBegin()
        ' ----- Create a new database transaction.
        On Error Resume Next
        HoldTransaction = LibraryDB.BeginTransaction()
    End Sub

    Public Sub TransactionCommit()
        ' ----- Ignore if there is no transaction.
        On Error Resume Next
        If (HoldTransaction Is Nothing) Then Return

        ' ----- Commit the transaction.
        HoldTransaction.Commit()
        HoldTransaction = Nothing
    End Sub

    Public Sub TransactionRollback()
        ' ----- Ignore if there is no transaction.
        On Error Resume Next
        If (HoldTransaction Is Nothing) Then Return

        ' ----- Rollback the transaction.
        HoldTransaction.Rollback()
        HoldTransaction = Nothing
    End Sub
End Module
