' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class CodeCopyStatus
    Private ActiveID As Integer
    Private StartingOver As Boolean

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageCopyStatus)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageCopyStatus)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageCopyStatus)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the copy status code?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE Status = " & recordID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("You cannot delete this record because it is in use.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Delete the record.
        sqlText = "DELETE FROM CodeCopyStatus WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        Return True

ErrorHandler:
        GeneralError("CodeCopyStatus.DeleteRecord", Err.GetException())
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT * FROM CodeCopyStatus ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("CodeCopyStatus.FillListWithRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM CodeCopyStatus WHERE ID = " & recordID
        Return CStr(ExecuteSQLReturn(sqlText))

ErrorHandler:
        GeneralError("CodeCopyStatus.FormatRecordName", Err.GetException())
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use these codes to establish the different status conditions in " & _
            "which library materials can be found."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Copy Status Codes"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT FullName FROM CodeCopyStatus WHERE ID = " & ActiveID
        RecordFullName.Text = CStr(ExecuteSQLReturn(sqlText))
        Return

ErrorHandler:
        GeneralError("CodeCopyStatus.PrepareFormFields", Err.GetException())
        Return
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the record.
        Dim sqlText As String
        Dim newID As Integer

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO CodeCopyStatus (FullName) OUTPUT INSERTED.ID " & _
                "VALUES (" & DBText(Trim(RecordFullName.Text)) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE CodeCopyStatus SET FullName = " & _
                DBText(Trim(RecordFullName.Text)) & " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("CodeCopyStatus.SaveFormData", Err.GetException())
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Name is a required field.
        If (Trim(RecordFullName.Text) = "") Then
            MsgBox("Please supply the copy status name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM CodeCopyStatus WHERE UPPER(FullName) = " & _
            DBText(UCase(Trim(RecordFullName.Text)))
        If (ActiveID <> -1) Then sqlText &= " AND ID <> " & ActiveID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            ' ----- Duplicate entry.
            MsgBox("A record already exists with the name '" & Trim(RecordFullName.Text) & _
                "'.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("CodeCopyStatus.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub RecordFullName_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordFullName.GotFocus
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub CodeCopyStatus_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub
End Class
