' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class CodeMediaType
    Private ActiveID As Integer
    Private StartingOver As Boolean

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageMediaTypes)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageMediaTypes)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageMediaTypes)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the media type?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM NamedItem WHERE MediaType = " & recordID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("You cannot delete this record because it is in use.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Delete the record.
        sqlText = "DELETE FROM CodeMediaType WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        Return True

ErrorHandler:
        GeneralError("CodeMediaType.DeleteRecord", Err.GetException())
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, FullName FROM CodeMediaType ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("CodeMediaType.FillListWithRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM CodeMediaType WHERE ID = " & recordID
        Return CStr(ExecuteSQLReturn(sqlText))

ErrorHandler:
        GeneralError("CodeMediaType.FormatRecordName", Err.GetException())
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use these codes to distinguish between the different types of " & _
            "materials available in the library, such as books, CDs, etc."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Media Types"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        RecordCheckoutDays.Text = ""
        RecordRenewDays.Text = ""
        RecordRenewTimes.Text = ""
        RecordDailyFine.Text = ""
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM CodeMediaType WHERE ID = " & ActiveID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            RecordFullName.Text = CStr(dbInfo!FullName)
            RecordCheckoutDays.Text = CStr(dbInfo!CheckoutDays)
            RecordRenewDays.Text = CStr(dbInfo!RenewDays)
            RecordRenewTimes.Text = CStr(dbInfo!RenewTimes)
            RecordDailyFine.Text = Format(dbInfo!DailyFine, "0.00")
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("CodeMediaType.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the database.
        Dim sqlText As String
        Dim newID As Integer

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO CodeMediaType (FullName, CheckoutDays, " & _
                "RenewDays, RenewTimes, DailyFine) OUTPUT INSERTED.ID VALUES (" & _
                DBText(Trim(RecordFullName.Text)) & ", " & _
                RecordCheckoutDays.Text & ", " & RecordRenewDays.Text & ", " & _
                RecordRenewTimes.Text & ", " & RecordDailyFine.Text & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE CodeMediaType SET FullName = " & _
                DBText(Trim(RecordFullName.Text)) & _
                ", CheckoutDays = " & RecordCheckoutDays.Text & _
                ", RenewDays = " & RecordRenewDays.Text & _
                ", RenewTimes = " & RecordRenewTimes.Text & _
                ", DailyFine = " & RecordDailyFine.Text & _
                " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("CodeMediaType.SaveFormData", Err.GetException())
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Name is a required field.
        If (Trim(RecordFullName.Text) = "") Then
            MsgBox("Please supply the media type name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Checkout days must be an integer.
        If (IsNumeric(RecordCheckoutDays.Text) = False) Or _
                (InStr(RecordCheckoutDays.Text, ".") > 0) Or _
                (Val(RecordCheckoutDays.Text) < 0) Or _
                (Val(RecordCheckoutDays.Text) > 99) Then
            MsgBox("Checkout Days must be an integer between 0 and 99.")
            RecordCheckoutDays.Focus()
            Return False
        End If

        ' ----- Renewal Days must be an integer.
        If (IsNumeric(RecordRenewDays.Text) = False) Or _
                (InStr(RecordRenewDays.Text, ".") > 0) Or _
                (Val(RecordRenewDays.Text) < 0) Or _
                (Val(RecordRenewDays.Text) > 99) Then
            MsgBox("Renewal Days must be an integer between 0 and 99.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordRenewDays.Focus()
            Return False
        End If

        ' ----- Renewal Limit must be an integer.
        If (IsNumeric(RecordRenewTimes.Text) = False) Or _
                (InStr(RecordRenewTimes.Text, ".") > 0) Or _
                (Val(RecordRenewTimes.Text) < 0) Or _
                (Val(RecordRenewTimes.Text) > 99) Then
            MsgBox("Renewal Limit must be an integer between 0 and 99.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordRenewTimes.Focus()
            Return False
        End If

        ' ----- Daily Fine must be a number.
        If (IsNumeric(RecordDailyFine.Text) = False) Then
            MsgBox("Daily Fine must be a valid number.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordDailyFine.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM CodeMediaType WHERE UPPER(FullName) = " & _
            DBText(UCase(Trim(RecordFullName.Text)))
        If (ActiveID <> -1) Then sqlText &= " AND ID <> " & ActiveID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            ' ----- Duplicate entry.
            MsgBox("A record already exists with the name '" & Trim(RecordFullName.Text) & _
                "'.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("CodeMediaType.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub CodeMediaType_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordFullName_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordFullName.GotFocus
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub RecordCheckoutDays_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordCheckoutDays.GotFocus
        ' ----- Highlight the entire text.
        RecordCheckoutDays.SelectAll()
    End Sub

    Private Sub RecordRenewDays_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordRenewDays.GotFocus
        ' ----- Highlight the entire text.
        RecordRenewDays.SelectAll()
    End Sub

    Private Sub RecordRenewTimes_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordRenewTimes.GotFocus
        ' ----- Highlight the entire text.
        RecordRenewTimes.SelectAll()
    End Sub

    Private Sub RecordDailyFine_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordDailyFine.GotFocus
        ' ----- Highlight the entire text.
        RecordDailyFine.SelectAll()
    End Sub

    Private Sub RecordCheckoutDays_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles RecordCheckoutDays.KeyPress
        ' ----- Only allow digits and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Then Return
        e.Handled = True
    End Sub

    Private Sub RecordRenewDays_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles RecordRenewDays.KeyPress
        ' ----- Only allow digits and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Then Return
        e.Handled = True
    End Sub

    Private Sub RecordRenewTimes_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles RecordRenewTimes.KeyPress
        ' ----- Only allow digits and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Then Return
        e.Handled = True
    End Sub

    Private Sub RecordDailyFine_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles RecordDailyFine.KeyPress
        ' ----- Only allow digits, decimals, and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Or _
            (e.KeyChar = ".") Then Return
        e.Handled = True
    End Sub
End Class
