' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports MVB = Microsoft.VisualBasic

Public Class Patron
    Private ActiveID As Integer
    Private StartingOver As Boolean
    Private AllowPatronRecord As Boolean

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        AllowPatronRecord = False
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManagePatrons)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManagePatrons)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManagePatrons)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the patron?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM PatronCopy WHERE Patron = " & recordID & _
            " AND Returned = 0 AND Missing = 0"
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("You cannot delete this record because the patron still has " & _
                "items checked out.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, _
                ProgramTitle)
            Return False
        End If

        ' ----- Delete the records.
        TransactionBegin()
        sqlText = "DELETE FROM PatronCopy WHERE Patron = " & recordID
        ExecuteSQL(sqlText)
        sqlText = "DELETE FROM Patron WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        TransactionCommit()
        Return True

ErrorHandler:
        GeneralError("Patron.DeleteRecord", Err.GetException())
        TransactionRollback()
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record. NOTE: If you modify this routine, 
        '       you must also modify the EditRecordLimited routine.
        ActiveID = recordID
        AllowPatronRecord = True
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Function EditRecordLimited(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record. NOTE: If you modify this routine, 
        '       you must also modify the EditRecord routine. The Limited
        '       version does not allow access to the PatronRecord form.
        ActiveID = recordID
        AllowPatronRecord = False
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items. Prompt the
        '       user first.
        destList.Items.Clear()
        SearchForRecord(destList, exceededMatches)
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim nameText As String

        On Error GoTo ErrorHandler
        nameText = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT LastName, FirstName, Active FROM Patron WHERE ID = " & recordID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            nameText = CStr(dbInfo!LastName) & ", " & CStr(dbInfo!FirstName)
            If (CBool(dbInfo!Active) = False) Then nameText &= " (Inactive)"
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return nameText

ErrorHandler:
        GeneralError("Patron.FormatRecordName", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use this form to manage the patrons who have check-out " & _
            "privileges at the library."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Patrons"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Fill in the list of patron groups, if needed.
        If (RecordGroup.Items.Count = 0) Then
            RecordGroup.Items.Add(New ListItemData("<Not Selected>", -1))
            sqlText = "SELECT * FROM CodePatronGroup ORDER BY FullName"
            dbInfo = CreateReader(sqlText)
            Do While dbInfo.Read
                RecordGroup.Items.Add(New ListItemData(CStr(dbInfo!FullName), _
                    CInt(dbInfo!ID)))
            Loop
            dbInfo.Close()
        End If

        ' ----- Clear the existing data.
        StartingOver = True
        RecordLastName.Text = ""
        RecordFirstName.Text = ""
        RecordActive.Checked = True
        RecordBarcode.Text = ""
        RecordPassword.Text = ""
        RecordConfirm.Text = ""
        RecordPhone.Text = ""
        RecordEmail.Text = ""
        RecordAddress.Text = ""
        RecordCity.Text = ""
        RecordState.Text = ""
        RecordPostal.Text = ""
        RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(-1)
        RecordComments.Text = ""
        RecordMessage.Text = ""
        RecordLastActivity.Text = "Not Available"
        TabPatron.SelectedTab = TabGeneral
        ActItems.Visible = False
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM Patron WHERE ID = " & ActiveID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            RecordLastName.Text = CStr(dbInfo!LastName)
            RecordFirstName.Text = CStr(dbInfo!FirstName)
            RecordActive.Checked = CBool(dbInfo!Active)
            RecordBarcode.Text = DBGetText(dbInfo!Barcode)
            RecordPhone.Text = DBGetText(dbInfo!Phone)
            RecordEmail.Text = DBGetText(dbInfo!Email)
            RecordAddress.Text = DBGetText(dbInfo!Address)
            RecordCity.Text = DBGetText(dbInfo!City)
            RecordState.Text = DBGetText(dbInfo!State)
            RecordPostal.Text = DBGetText(dbInfo!Postal)
            If (IsDBNull(dbInfo!PatronGroup) = False) Then
                RecordGroup.SelectedIndex = _
                    RecordGroup.Items.IndexOf(CInt(dbInfo!PatronGroup))
                If (RecordGroup.SelectedIndex = -1) Then _
                    RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(-1)
            End If
            RecordComments.Text = DBGetText(dbInfo!Comments)
            RecordMessage.Text = DBGetText(dbInfo!AdminMessage)
            If (IsDBNull(dbInfo!LastActivity) = False) Then _
                RecordLastActivity.Text = Format(CDate(dbInfo!LastActivity), _
                "MMMM d, yyyy")
            ActItems.Visible = AllowPatronRecord
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("Patron.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Sub SearchForRecord(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim limitText As String
        Dim nameText As String
        Dim resultCount As Integer
        Dim limitLast As String
        Dim limitFirst As String

        On Error GoTo ErrorHandler

        ' ----- Prompt the user for the limited item title.
        exceededMatches = False
        limitText = (New PatronLimit).PromptUser()

        ' ----- Use the limit to prepare the selection criteria.
        limitLast = Trim(GetSubStr(limitText, ",", 1))
        limitFirst = Trim(GetSubStr(limitText, ",", 2))
        If (limitLast = "") And (limitFirst = "") Then Return
        If (InStr(limitLast, "*") = 0) Then limitLast &= "*"
        If (InStr(limitFirst, "*") = 0) Then limitFirst &= "*"
        limitLast = Replace(limitLast, "*", "%")
        limitFirst = Replace(limitFirst, "*", "%")

        ' ----- Clear any existing values.
        destList.Items.Clear()
        resultCount = 0

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, LastName, FirstName, Active FROM Patron " & _
            "WHERE LastName LIKE " & DBText(limitLast) & _
            " AND FirstName LIKE " & DBText(limitFirst) & _
            " ORDER BY LastName, FirstName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            resultCount += 1
            If (resultCount > SearchMatchLimit) Then
                exceededMatches = True
                Exit Do
            End If
            nameText = CStr(dbInfo!LastName) & ", " & CStr(dbInfo!FirstName)
            If (CBool(dbInfo!Active) = False) Then nameText &= " (Inactive)"
            destList.Items.Add(New ListItemData(nameText, CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("Patron.SearchForRecord", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function UsesSearch() As Boolean
        ' ----- Patron records are searchable.
        Return True
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save the record to the database.
        Dim sqlText As String
        Dim newID As Integer

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO Patron (LastName, FirstName, Active, Comments, " & _
                "AdminMessage, Barcode, [Password], Phone, Email, Address, City, " & _
                "State, Postal, PatronGroup) OUTPUT INSERTED.ID VALUES (" & _
                DBText(Trim(RecordLastName.Text)) & _
                ", " & DBText(Trim(RecordFirstName.Text)) & _
                ", " & CInt(RecordActive.Checked) & _
                ", " & DBText(Trim(RecordComments.Text)) & _
                ", " & DBText(Trim(RecordMessage.Text)) & _
                ", " & DBText(Trim(RecordBarcode.Text)) & _
                ", " & DBText(EncryptPassword("patron", Trim(RecordPassword.Text))) & _
                ", " & DBText(Trim(RecordPhone.Text)) & _
                ", " & DBText(Trim(RecordEmail.Text)) & _
                ", " & DBText(Trim(RecordAddress.Text)) & _
                ", " & DBText(Trim(RecordCity.Text)) & _
                ", " & DBText(Trim(RecordState.Text)) & _
                ", " & DBText(Trim(RecordPostal.Text)) & _
                ", " & DBCombo(RecordGroup) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE Patron SET" & _
                " LastName = " & DBText(Trim(RecordLastName.Text)) & _
                ", FirstName = " & DBText(Trim(RecordFirstName.Text)) & _
                ", Active = " & CInt(RecordActive.Checked) & _
                ", Comments = " & DBText(Trim(RecordComments.Text)) & _
                ", AdminMessage = " & DBText(Trim(RecordMessage.Text)) & _
                ", Barcode = " & DBText(Trim(RecordBarcode.Text)) & _
                ", Phone = " & DBText(Trim(RecordPhone.Text)) & _
                ", Email = " & DBText(Trim(RecordEmail.Text)) & _
                ", Address = " & DBText(Trim(RecordAddress.Text)) & _
                ", City = " & DBText(Trim(RecordCity.Text)) & _
                ", State = " & DBText(Trim(RecordState.Text)) & _
                ", Postal = " & DBText(Trim(RecordPostal.Text)) & _
                ", PatronGroup = " & DBCombo(RecordGroup)
            If (Trim(RecordPassword.Text) <> "") Then sqlText &= ", [Password] = " & _
                DBText(EncryptPassword("patron", Trim(RecordPassword.Text)))
            sqlText &= " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("Patron.SaveFormData", Err.GetException())
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Last name is a required field.
        If (Trim(RecordLastName.Text) = "") Then
            MsgBox("Please supply the last name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordLastName.Focus()
            Return False
        End If

        ' ----- First name is a required field.
        If (Trim(RecordFirstName.Text) = "") Then
            MsgBox("Please supply the first name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFirstName.Focus()
            Return False
        End If

        ' ----- Barcode, if supplied, must be numeric.
        If (Trim(RecordBarcode.Text) <> "") Then
            If (DigitsOnly(RecordBarcode.Text) <> Trim(RecordBarcode.Text)) Then
                MsgBox("Barcode, if supplied, must be numeric.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                TabPatron.SelectedTab = TabGeneral
                RecordBarcode.Focus()
                Return False
            End If
        End If

        ' ----- Make sure the bar code is not already in use.
        If (Trim(RecordBarcode.Text) <> "") Then
            sqlText = "SELECT COUNT(*) FROM Patron WHERE UPPER(Barcode) = " & _
                DBText(UCase(Trim(RecordBarcode.Text)))
            If (ActiveID <> -1) Then sqlText &= " AND ID <> " & ActiveID
            If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
                ' ----- Duplicate entry.
                MsgBox("A record already exists with the barcode'" & _
                    Trim(RecordBarcode.Text) & "'.", MsgBoxStyle.OkOnly Or _
                    MsgBoxStyle.Exclamation, ProgramTitle)
                TabPatron.SelectedTab = TabGeneral
                RecordBarcode.Focus()
                Return False
            End If
        End If

        ' ----- Password and confirm must match.
        If (RecordPassword.Text <> RecordConfirm.Text) Then
            MsgBox("The Password and Confirm fields must match.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            TabPatron.SelectedTab = TabGeneral
            RecordPassword.Focus()
            Return False
        End If

        ' ----- For new patrons, a password is required.
        If (ActiveID = -1) And (Trim(RecordPassword.Text) = "") Then
            MsgBox("You must supply a password for new patron records. For an " & _
                "initial password, consider using the last four digits of the " & _
                "patron's phone number.", MsgBoxStyle.OkOnly Or _
                MsgBoxStyle.Exclamation, ProgramTitle)
            TabPatron.SelectedTab = TabGeneral
            RecordPassword.Focus()
            Return False
        End If

        ' ----- Check the matching patrons, just in case.
        If (ActiveID = -1) Then
            RefreshMatchingPatrons()
            If (ShowMatches.ImageIndex = MatchPresent) Then
                If (MsgBox("The name you supplied is similar to one or more names " & _
                    "already in the database. Do you still want to save this record?", _
                    MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
                    <> MsgBoxResult.Yes) Then Return False
            End If
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("Patron.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub Patron_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordLastName.Focus()
        StartingOver = False
    End Sub

    Private Sub RefreshMatchingPatrons()
        ' ----- See if there are name matches for this patron.
        Dim sqlText As String
        Dim holdText As String
        Dim matches As Integer

        ' TODO: Provide an option to run in the current thread or in another thread.
        On Error GoTo ErrorHandler

        ' ----- Only performed for new records.
        If (ActiveID <> -1) Then Return

        ' ----- Reset the display.
        ShowMatches.Text = "      No name matches"
        ShowMatches.ImageIndex = MatchNone

        ' ----- Ignore if we have insufficient data.
        If (Trim(RecordLastName.Text) = "") Then Return
        If (Trim(RecordFirstName.Text) = "") Then Return

        ' ----- Retrieve a count of matching records.
        sqlText = "SELECT COUNT(*) FROM Patron WHERE LastName LIKE " & _
            DBText(Trim(RecordLastName.Text)) & " AND FirstName LIKE " & _
            DBText(MVB.Left(Trim(RecordFirstName.Text), 1) & "%")
        If (Trim(RecordFirstName.Text) <> "") Then
            sqlText &= " AND FirstName LIKE " & _
                DBText(MVB.Left(Trim(RecordFirstName.Text), 1) & "%")
        End If
        matches = CInt(ExecuteSQLReturn(sqlText))
        If (matches > 0) Then
            ' ----- There are matches.
            If (matches = 1) Then
                holdText = "1 match"
            Else
                holdText = matches & " matches"
            End If
            holdText &= ". Click for details."
            ShowMatches.Text = "      " & holdText
            ShowMatches.ImageIndex = MatchPresent
        End If
        Return

ErrorHandler:
        GeneralError("Patron.RefreshMatchingPatrons", Err.GetException())
        Return
    End Sub

    Private Sub RecordLastName_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordLastName.GotFocus
        ' ----- Highlight the entire text.
        RecordLastName.SelectAll()
    End Sub

    Private Sub RecordFirstName_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordFirstName.GotFocus
        ' ----- Highlight the entire text.
        RecordFirstName.SelectAll()
    End Sub

    Private Sub RecordBarcode_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordBarcode.GotFocus
        ' ----- Highlight the entire text.
        RecordBarcode.SelectAll()

        ' ----- Don't allow an enter to close the form.
        Me.AcceptButton = Nothing
    End Sub

    Private Sub RecordPassword_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordPassword.GotFocus
        ' ----- Highlight the entire text.
        RecordPassword.SelectAll()
    End Sub

    Private Sub RecordConfirm_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordConfirm.GotFocus
        ' ----- Highlight the entire text.
        RecordConfirm.SelectAll()
    End Sub

    Private Sub RecordPhone_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordPhone.GotFocus
        ' ----- Highlight the entire text.
        RecordPhone.SelectAll()
    End Sub

    Private Sub RecordEmail_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordEmail.GotFocus
        ' ----- Highlight the entire text.
        RecordEmail.SelectAll()
    End Sub

    Private Sub RecordAddress_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordAddress.GotFocus
        ' ----- Highlight the entire text.
        RecordAddress.SelectAll()
    End Sub

    Private Sub RecordCity_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordCity.GotFocus
        ' ----- Highlight the entire text.
        RecordCity.SelectAll()
    End Sub

    Private Sub RecordState_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordState.GotFocus
        ' ----- Highlight the entire text.
        RecordState.SelectAll()
    End Sub

    Private Sub RecordPostal_GotFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordPostal.GotFocus
        ' ----- Highlight the entire text.
        RecordPostal.SelectAll()
    End Sub

    Private Sub RecordLastName_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordLastName.LostFocus
        ' ----- Update the matching display.
        RefreshMatchingPatrons()
    End Sub

    Private Sub RecordFirstName_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordFirstName.LostFocus
        ' ----- Update the matching display.
        RefreshMatchingPatrons()
    End Sub

    Private Sub ShowMatches_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ShowMatches.Click
        ' ----- The user wants more information on the matches.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim holdText As String
        Dim nameText As String
        Dim message As String
        Dim matches As Integer
        Dim counter As Integer

        On Error GoTo ErrorHandler

        ' ----- Only performed for new records.
        If (ActiveID <> -1) Then Return

        ' ----- Ignore if there are no matches.
        If (ShowMatches.ImageIndex = MatchNone) Then
            MsgBox("No matches.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
            Return
        End If

        ' ----- Retrieve a count of matching records.
        matches = CInt(Val(ShowMatches.Text))
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Retrieve the matching records.
        sqlText = "SELECT * FROM Patron WHERE LastName LIKE " & _
            DBText(Trim(RecordLastName.Text))
        If (Trim(RecordFirstName.Text) <> "") Then
            sqlText &= " AND FirstName LIKE " & _
                DBText(MVB.Left(Trim(RecordFirstName.Text), 1) & "%")
        End If
        sqlText &= " ORDER BY LastName, FirstName"
        dbInfo = CreateReader(sqlText)
        counter = 0
        holdText = ""
        Do While (dbInfo.Read)
            ' ----- Add this author.
            nameText = CStr(dbInfo!LastName) & ", " & CStr(dbInfo!FirstName)
            If (CBool(dbInfo!Active) = False) Then nameText &= " (Inactive)"
            holdText &= vbCrLf & vbTab & nameText

            ' ----- Limit to ten matches.
            counter += 1
            If (counter >= 10) Then Exit Do
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Me.Cursor = Windows.Forms.Cursors.Default

        ' ----- Format and display the message.
        message = "The database contains " & CStr(matches) & " match"
        If (matches > 1) Then message &= "es"
        message &= " for the name:" & vbCrLf & vbCrLf & vbTab & _
            Trim(RecordLastName.Text)
        If (Trim(RecordFirstName.Text) <> "") Then message &= ", " & _
            Trim(RecordFirstName.Text)
        If (matches > 10) Then
            message &= vbCrLf & vbCrLf & "The first ten matches are:"
        Else
            message &= vbCrLf & vbCrLf & "The matches are:"
        End If
        message &= vbCrLf & holdText
        MsgBox(message, MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("Author.ShowMatches_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub ActItems_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActItems.Click
        ' ----- Provide access to patron's circulation records and fees.
        ' TODO: Add this code.
        MsgBox("Not yet implemented.")
    End Sub

    Private Sub RecordBarcode_LostFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordBarcode.LostFocus
        ' ----- Allow an enter to close the form again.
        Me.AcceptButton = ActOK
    End Sub

    Private Sub RecordBarcode_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles RecordBarcode.KeyPress
        ' ----- Ignore the Enter key.
        If (e.KeyChar = ChrW(Keys.Return)) Then e.Handled = True
    End Sub
End Class
