' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class AuthorAddLocate
    Private ActivePanel As Integer

    Private Const PanelType As Integer = 1
    Private Const PanelCriteria As Integer = 2
    Private Const PanelResults As Integer = 3

    Private Function FindUserRecords(ByVal lastText As String, _
            ByVal firstText As String) As Integer
        ' ----- Search for author names using user-specified criteria.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim matches As Integer

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        SearchResults.Items.Clear()
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Retrieve the values from the database.
        matches = 0
        sqlText = "SELECT * FROM Author WHERE LastName LIKE " & DBText(lastText)
        If (firstText <> "") Then sqlText &= " AND FirstName LIKE " & DBText(firstText)
        sqlText &= " ORDER BY LastName, FirstName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            SearchResults.Items.Add(New ListItemData(FormatAuthorName(dbInfo), _
                CInt(dbInfo!ID)))
            matches += 1
            If (matches >= SearchMatchLimit) Then Exit Do
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Me.Cursor = Windows.Forms.Cursors.Default
        Return matches

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("AuthorAddLocate.FindUserRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return -1
    End Function

    Public Function PromptUser(ByRef nameTypeID As Integer) As Integer
        ' ----- Prompt the user for a author. Return the matching ID. Return
        '       -1 if no change made.
        ActivePanel = PanelType
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then
            nameTypeID = CInt(CType(NameType.SelectedItem, ListItemData))
            If (CInt(CType(SearchResults.SelectedItem, ListItemData)) = -1) Then
                ' ----- Prompt for a new author using the original
                '       typed name.
                Return (New Author).AddRecord(Trim(LastName.Text), _
                    Trim(FirstName.Text))
            Else
                ' ----- The user selected a match.
                Return CInt(CType(SearchResults.SelectedItem, ListItemData))
            End If
        Else
            ' ----- Cancelled by user.
            nameTypeID = -1
            Return -1
        End If
    End Function

    Private Sub ActBack_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ActBack.Click
        ' ----- Go back to the previous panel.
        Select Case ActivePanel
            Case PanelCriteria
                ' ----- Go back to the Name Type panel.
                ActivePanel = PanelType
                SecondPanel.Visible = False
                FirstPanel.Visible = True
                ActBack.Enabled = False
                NameType.Focus()
            Case PanelResults
                ' ----- Go back to the Last/First Name panel.
                ActivePanel = PanelCriteria
                ThirdPanel.Visible = False
                SecondPanel.Visible = True
                ActNext.Text = "&Next >"
                LastName.Focus()
        End Select
    End Sub

    Private Sub ActNext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActNext.Click
        ' ----- Take action on the user input.
        Dim lastText As String
        Dim firstText As String
        Dim matches As Integer

        ' ----- Different code for each of the panels.
        If (ActivePanel = PanelType) Then
            ' ----- Make sure a name type is selected.
            If (CInt(CType(NameType.SelectedItem, ListItemData)) = -1) Then
                MsgBox("Please select a name type from the list.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                NameType.Focus()
                Return
            End If

            ' ----- Move to the search panel.
            ActivePanel = PanelCriteria
            SecondPanel.Visible = True
            FirstPanel.Visible = False
            ActBack.Enabled = True
            LastName.Focus()
        ElseIf (ActivePanel = PanelCriteria) Then
            ' ----- Determine the search string.
            lastText = Trim(LastName.Text)
            If (lastText = "") Then
                MsgBox("Please supply a last name.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                LastName.Focus()
                Return
            End If
            If (InStr(lastText, "*") = 0) Then lastText &= "*"
            lastText = Replace(lastText, "*", "%")
            firstText = Trim(FirstName.Text)
            If (InStr(firstText, "*") = 0) Then firstText &= "*"
            firstText = Replace(firstText, "*", "%")

            ' ----- Search for matches. 
            matches = FindUserRecords(lastText, firstText)
            If (matches = -1) Then Return

            ' ----- If there are no matches, then either error out, or
            '       add a new record, depending on the user's security.
            '       If there are matches, show them.
            If (matches = 0) Then
                If (SecurityProfile(LibrarySecurity.ManageAuthors)) Then
                    ' ----- Prompt to add a new record.
                    SearchResults.ClearSelected()
                    SearchResults.Items.Add(New ListItemData( _
                        "<No matches. Add new author or name.>", -1))
                    SearchResults.SelectedIndex = 0

                    ' ----- Return to the PromptUser() routine.
                    Me.DialogResult = Windows.Forms.DialogResult.OK
                Else
                    ' ----- Give an error.
                    MsgBox("No matches.", _
                        MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                    Return
                End If
            Else
                ' ----- If the user can add new author, add that option.
                If (SecurityProfile(LibrarySecurity.ManageAuthors)) Then _
                    SearchResults.Items.Insert(0, New ListItemData( _
                        "<None of these matches. Add new author or name.>", -1))
                ActivePanel = PanelResults
                ThirdPanel.Visible = True
                SecondPanel.Visible = False
                ActNext.Text = "Finish"
                SearchResults.Focus()
            End If
        Else
            ' ----- Check for a selected result.
            If (SearchResults.SelectedIndex = -1) Then
                MsgBox("Please select a match from the list.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                SearchResults.Focus()
                Return
            End If

            ' ----- Return to the PromptUser() routine.
            Me.DialogResult = Windows.Forms.DialogResult.OK
        End If
    End Sub

    Private Sub SearchResults_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles SearchResults.DoubleClick
        ' ----- Same as the finish button.
        If (SearchResults.SelectedIndex = -1) Then Return
        ActNext.PerformClick()
    End Sub

    Private Sub AuthorAddLocate_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Adjust the wizard panels.
        SecondPanel.Location = New System.Drawing.Point(FirstPanel.Location.X, _
            FirstPanel.Location.Y)
        ThirdPanel.Location = New System.Drawing.Point(FirstPanel.Location.X, _
            FirstPanel.Location.Y)

        ' ----- Fill in the list of named items.
        NameType.Items.Add(New ListItemData("<Select a Name Type>", -1))
        NameType.SelectedIndex = 0
        sqlText = "SELECT * FROM CodeAuthorType ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            NameType.Items.Add(New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("AuthorAddLocate.AuthorAddLocate_Load", Err.GetException())
        Return
    End Sub

    Private Sub LastName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles LastName.Enter
        ' ----- Highlight the entire text.
        LastName.SelectAll()
    End Sub

    Private Sub FirstName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles FirstName.Enter
        ' ----- Highlight the entire text.
        FirstName.SelectAll()
    End Sub
End Class
