' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports MVB = Microsoft.VisualBasic

Public Class Holiday
    Private ActiveID As Integer
    Private StartingOver As Boolean

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageHolidays)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageHolidays)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageHolidays)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the selected holiday?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Delete the record.
        sqlText = "DELETE FROM Holiday WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        Return True

ErrorHandler:
        GeneralError("Holiday.DeleteRecord", Err.GetException())
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT * FROM Holiday ORDER BY EntryType, EntryDetail"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            destList.Items.Add(New ListItemData(FormatOneEntry(dbInfo), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("Holiday.FillListWithRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function FormatOneEntry(ByVal dbInfo As SqlClient.SqlDataReader) As String
        ' ----- Given an active record, format its content for display.
        Dim itemText As String

        On Error GoTo ErrorHandler

        ' ----- Different formatting for different entry types.
        itemText = ""
        Select Case UCase(CStr(dbInfo!EntryType))
            Case "A"
                ' ----- Annually.
                itemText = CStr(dbInfo!EntryDetail) & "/2004"
                If (IsDate(itemText) = True) Then
                    itemText = "Annually - " & Format(CDate(itemText), "MMMM d")
                Else
                    itemText = "Annually - Invalid Entry"
                End If
            Case "E"
                ' ----- Weekly.
                If (Val(dbInfo!EntryDetail) < FirstDayOfWeek.Sunday) Or _
                        (Val(dbInfo!EntryDetail) > FirstDayOfWeek.Saturday) Then
                    itemText = "Weekly - Invalid Entry"
                Else
                    itemText = "Weekly - Every " & WeekdayName(CInt(dbInfo!EntryDetail), _
                        False, FirstDayOfWeek.Sunday)
                End If
            Case "O"
                ' ----- One-time.
                itemText = CStr(dbInfo!EntryDetail)
                If (IsDate(itemText) = True) Then
                    itemText = Format(CDate(itemText), "yyyy") & " - " & _
                        Format(CDate(itemText), "MMMM d, yyyy")
                Else
                    itemText = "???? - Invalid Entry"
                End If
        End Select
        itemText &= " [" & CStr(dbInfo!FullName) & "]"
        Return itemText

ErrorHandler:
        GeneralError("Holiday.FormatOneEntry", Err.GetException())
        Return "Error"
    End Function

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim result As String

        On Error GoTo ErrorHandler
        result = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT * FROM Holiday WHERE ID = " & recordID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()

        ' ----- Format the holiday for display.
        result = FormatOneEntry(dbInfo)

        ' ----- Finished.
        dbInfo.Close()
        dbInfo = Nothing
        Return result

ErrorHandler:
        GeneralError("Holiday.FormatRecordName", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return a description of this editor.
        Return "Use these codes to establish the different holidays, which can " & _
            "not be used for item due dates."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the title-bar text for this editor.
        Return "Holidays"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim counter As Integer
        Dim holdText As String

        On Error GoTo ErrorHandler

        ' ----- Load in the list values.
        If (RecordType.Items.Count = 0) Then
            ' ----- Fill in the list of item types.
            RecordType.Items.Add(New ListItemData("Annual Holiday", Asc("A")))
            RecordType.Items.Add(New ListItemData("Weekly Holiday", Asc("E")))
            RecordType.Items.Add(New ListItemData("One-time Holiday", Asc("O")))

            ' ----- Fill in the list of months.
            RecordMonth.Items.Clear()
            For counter = 1 To 12
                RecordMonth.Items.Add(New ListItemData(MonthName(counter), counter))
            Next counter

            ' ----- Fill in the list of days.
            RecordDay.Items.Clear()
            For counter = 1 To 31
                RecordDay.Items.Add(New ListItemData(CStr(counter), counter))
            Next counter

            ' ----- Fill in the list of weekdays.
            RecordWeekday.Items.Clear()
            For counter = 1 To 7
                RecordWeekday.Items.Add(New ListItemData(WeekdayName(counter, False, _
                    FirstDayOfWeek.Sunday), counter))
            Next counter
        End If

        ' ----- Clear the existing data.
        StartingOver = True
        RecordName.Text = ""
        recordtype.SelectedIndex = recordtype.Items.IndexOf(Asc("O"))
        RecordMonth.SelectedIndex = RecordMonth.Items.IndexOf(1)
        RecordDay.SelectedIndex = RecordDay.Items.IndexOf(1)
        RecordWeekday.SelectedIndex = RecordWeekday.Items.IndexOf(FirstDayOfWeek.Sunday)
        RecordOneTime.Value = Today
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM Holiday WHERE ID = " & ActiveID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            RecordName.Text = CStr(dbInfo!FullName)
            RecordType.SelectedIndex = _
                RecordType.Items.IndexOf(Asc(UCase(CStr(dbInfo!EntryType))))
            If (RecordType.SelectedIndex = -1) Then
                ' ----- Something wrong with this record.
                RecordType.SelectedIndex = RecordType.Items.IndexOf(Asc("O"))
            Else
                Select Case UCase(CStr(dbInfo!EntryType))
                    Case "A"
                        ' ----- Annually.
                        holdText = CStr(dbInfo!EntryDetail) & "/2004"
                        If (IsDate(holdText) = True) Then
                            RecordMonth.SelectedIndex = _
                                RecordMonth.Items.IndexOf(Month(CDate(holdText)))
                            RecordDay.SelectedIndex = _
                                RecordDay.Items.IndexOf(MVB.Day(CDate(holdText)))
                        End If
                    Case "E"
                        ' ----- Weekly.
                        holdText = CStr(dbInfo!EntryDetail)
                        If (Val(holdText) >= FirstDayOfWeek.Sunday) And _
                                (Val(holdText) <= FirstDayOfWeek.Saturday) Then
                            RecordWeekday.SelectedIndex = _
                                RecordWeekday.Items.IndexOf(CInt(Val(holdText)))
                        End If
                    Case "O"
                        ' ----- One-time.
                        If (IsDate(CStr(dbInfo!EntryDetail)) = True) Then _
                            RecordOneTime.Value = CDate(dbInfo!EntryDetail)
                End Select
            End If
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("Holiday.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the record.
        Dim sqlText As String
        Dim entryType As String
        Dim entryDetail As String
        Dim newID As Integer

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Get the entry detail content.
        entryType = CStr(Chr(CInt(CType(RecordType.SelectedItem, ListItemData))))
        Select Case entryType
            Case "A"
                ' ----- A: Annually.
                entryDetail = CInt(CType(RecordMonth.SelectedItem, ListItemData)) & "/" & _
                    CInt(CType(RecordDay.SelectedItem, ListItemData))
            Case "E"
                ' ----- W: Weekly.
                entryDetail = CStr(CInt(CType(RecordWeekday.SelectedItem, ListItemData)))
            Case Else
                ' ----- O: One-time.
                entryDetail = Format(RecordOneTime.Value, "yyyy/MM/dd")
        End Select

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO Holiday(FullName, EntryType, EntryDetail) " & _
                "OUTPUT INSERTED.ID VALUES (" & DBText(Trim(RecordName.Text)) & ", " & _
                DBText(entryType) & ", " & DBText(entryDetail) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE Holiday SET FullName = " & _
                DBText(Trim(RecordName.Text)) & _
                ", EntryType = " & DBText(entryType) & _
                ", EntryDetail = " & DBText(entryDetail) & _
                " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("Holiday.SaveFormData", Err.GetException())
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String
        Dim holdText As String

        On Error GoTo ErrorHandler

        ' ----- Name is a required field.
        If (Trim(RecordName.Text) = "") Then
            MsgBox("Please supply the holiday name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordName.Focus()
            Return False
        End If

        ' ----- If an annual holiday is used, check for a valid date. I
        '       used 2004 since it has a leap day in it, but the year used
        '       isn't that important.
        If (Chr(CInt(CType(RecordType.SelectedItem, ListItemData))) = "A") Then
            holdText = CInt(CType(RecordMonth.SelectedItem, ListItemData)) & "/" & _
                    CInt(CType(RecordDay.SelectedItem, ListItemData)) & "/2004"
            If (IsDate(holdText) = False) Then
                MsgBox("Please select a valid day for the selected month.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                RecordDay.Focus()
                Return False
            End If
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("Holiday.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub Holiday_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Adjust the elements on the form.
        PanelAnnually.Location = New System.Drawing.Point(PanelWeekly.Location.X, _
            PanelWeekly.Location.Y)
        PanelOneTime.Location = New System.Drawing.Point(PanelWeekly.Location.X, _
            PanelWeekly.Location.Y)
    End Sub

    Private Sub ActOK_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub Holiday_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordName.Enter
        ' ----- Highlight the entire text.
        RecordName.SelectAll()
    End Sub

    Private Sub RecordType_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordType.SelectedIndexChanged
        ' ----- Display the correct detail fields.
        If (RecordType.SelectedIndex = -1) Then Return
        Select Case CStr(Chr(CInt(CType(RecordType.SelectedItem, ListItemData))))
            Case "A"
                ' ----- Annually.
                PanelAnnually.Visible = True
                PanelWeekly.Visible = False
                PanelOneTime.Visible = False
            Case "E"
                ' ----- Weekly.
                PanelAnnually.Visible = False
                PanelWeekly.Visible = True
                PanelOneTime.Visible = False
            Case "O"
                ' ----- One-time.
                PanelAnnually.Visible = False
                PanelWeekly.Visible = False
                PanelOneTime.Visible = True
        End Select
    End Sub
End Class
