' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports MVB = Microsoft.VisualBasic

Public Class ItemLookup
    ' ----- Class to track the pseudo-links on the single item view.
    Private Class SingleItemLink
        Public LinkType As Library.LookupMethods
        Public LinkID As Integer
    End Class

    ' ----- Class to track the history of the views in this form instance.
    Private Class ItemLookupHistory
        Public HistoryDisplay As String
        Public LookupType As Library.LookupMethods
        Public LookupData As String
    End Class

    ' ----- Class to track items in the MatchingItems list.
    Private Class MatchingItemData
        Public ItemID As Integer  ' NamedItem.ID
        Public Title As String
        Public Subtitle As String
        Public Author As String
        Public MediaType As String
        Public CallNumber As String

        Public Overrides Function ToString() As String
            ' ----- Build a simple display string.
            If (Subtitle = "") Then
                Return Title & ", by " & Author
            Else
                Return Title & ": " & Subtitle & ", by " & Author
            End If
        End Function
    End Class

    ' ----- Class-level member variables.
    Private UseLC As Boolean
    Private ItemLinkSet As Collections.Generic.Dictionary(Of Integer, SingleItemLink)
    Private LookupHistorySet As Collections.Generic.Stack(Of ItemLookupHistory)
    Private LimitByMedia As Integer
    Private LimitByLocation As Integer
    Private SecondItemRow As Integer

    Friend Sub InitiateSearch(ByVal searchType As Library.LookupMethods, _
            ByVal searchText As String, ByVal limitMedia As Integer, _
            ByVal limitLocation As Integer)
        ' ----- Start a new item search.
        LimitByMedia = limitMedia
        LimitByLocation = limitLocation

        ' ----- Determine if Dewey or LC is used, and other initialization.
        UseLC = CBool(Val(GetSystemValue("UseLC")))
        ItemLinkSet = New Collections.Generic.Dictionary(Of Integer, SingleItemLink)
        LookupHistorySet = New Collections.Generic.Stack(Of ItemLookupHistory)

        ' ----- Perform the first search.
        If (PerformLookup(searchType, searchText, True) = False) Then
            MsgBox("Your search resulted in no matches.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
            ItemLinkSet = Nothing
            LookupHistorySet = Nothing
            Return
        End If

        ' ----- Add a history item.
        AddLookupHistory(searchType, searchText)

        ' ----- Show the results to the user.
        Me.ShowDialog()
        ItemLinkSet = Nothing
        LookupHistorySet = Nothing
    End Sub

    Private Function HTMLEncode(ByVal origText As String) As String
        ' ----- Prepare some text for use in a web page.
        Dim result As String

        result = Replace(origText, "&", "&amp;")
        result = Replace(result, "<", "&lt;")
        result = Replace(result, ">", "&gt;")
        Return result
    End Function

    Private Sub AddLookupHistory(ByVal searchType As Library.LookupMethods, _
            ByVal searchText As String)
        ' ----- Add an item to the lookup history.
        Dim newHistory As ItemLookupHistory
        Dim displayText As String

        ' ----- Build the text for display in the new item.
        displayText = BuildDisplayText(searchType, searchText)

        ' ----- Build the new history item.
        newHistory = New ItemLookupHistory
        newHistory.LookupType = searchType
        newHistory.LookupData = searchText
        newHistory.HistoryDisplay = displayText
        LookupHistorySet.Push(newHistory)

        ' ----- Update the back button.
        RefreshBackButtons()
    End Sub
    Private Function BuildDisplayText(ByVal searchType As Library.LookupMethods, _
            ByVal searchText As String) As String
        ' ----- Add an item to the lookup history.
        Dim newHistory As ItemLookupHistory
        Dim displayText As String
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Build the text for display in the new item.
        Select Case searchType
            Case LookupMethods.ByTitle
                ' ----- Search by title text.
                displayText = "Titles matching '" & searchText & "'"
            Case LookupMethods.ByAuthor
                ' ----- Search by author name text.
                displayText = "Authors matching '" & searchText & "'"
            Case LookupMethods.BySubject
                ' ----- Search by subject name text.
                displayText = "Subjects matching '" & searchText & "'"
            Case LookupMethods.ByKeywordAny
                ' ----- Search by keyword (any).
                displayText = "Items with any keyword(s): " & searchText
            Case LookupMethods.ByKeywordAll
                ' ----- Search by keyword (all).
                displayText = "Items with all keyword(s): " & searchText
            Case LookupMethods.ByPublisher
                ' ----- Search by publisher name text.
                displayText = "Publishers matching '" & searchText & "'"
            Case LookupMethods.BySeries
                ' ----- Search by series name text.
                displayText = "Series matching '" & searchText & "'"
            Case LookupMethods.BySeriesID
                ' ----- Items for a specific series.
                sqlText = "SELECT FullName FROM CodeSeries WHERE ID = " & _
                    searchText
                displayText = "Items from series '" & _
                    CStr(ExecuteSQLReturn(sqlText)) & "'"
            Case LookupMethods.ByBarcode
                ' ----- Item with a copy of a specific barcode.
                displayText = "Item with copy barcode '" & searchText & "'"
            Case LookupMethods.ByDatabaseID
                ' ----- Item with a specific ID.
                sqlText = "SELECT Title FROM NamedItem WHERE ID = " & searchText
                displayText = "Item with title '" & _
                    CStr(ExecuteSQLReturn(sqlText)) & "'"
            Case LookupMethods.ByAuthorID
                ' ----- Items for a specific author.
                sqlText = "SELECT * FROM Author WHERE ID = " & searchText
                dbInfo = CreateReader(sqlText)
                dbInfo.Read()
                displayText = "Items by author/name '" & _
                    FormatAuthorName(dbInfo) & "'"
                dbInfo.Close()
            Case LookupMethods.BySubjectID
                ' ----- Items for a specific subject.
                sqlText = "SELECT FullName FROM Subject WHERE ID = " & searchText
                displayText = "Items for subject '" & _
                    CStr(ExecuteSQLReturn(sqlText)) & "'"
            Case LookupMethods.ByPublisherID
                ' ----- Items for a specific publisher.
                sqlText = "SELECT FullName FROM Publisher WHERE ID = " & searchText
                displayText = "Items for publisher '" & _
                    CStr(ExecuteSQLReturn(sqlText)) & "'"
            Case Else
                displayText = "Error"
        End Select

        Return displayText

ErrorHandler:
        GeneralError("ItemLookup.BuildDisplayText", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return "Error"
    End Function

    Private Function PerformLookup(ByVal searchType As Library.LookupMethods, _
            ByVal searchText As String, ByVal breakOnFailure As Boolean) As Boolean
        ' ----- Perform a search based on the user's request. resultType is
        '       "S" for single, "I" for list of items, "M" for matches, "A"
        '       for author matches.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim resultType As String
        Dim counter As Integer
        Dim matches As Integer
        Dim firstName As String
        Dim lastName As String
        Dim itemID As Integer
        Dim holdText As String
        Dim matchEntry As MatchingItemData

        Const ItemSearchSQL As String = "SELECT NI.ID, NI.Title, NI.Subtitle, " & _
            "NI.Dewey, NI.LC, CMT.FullName AS MediaName, AU.LastName, " & _
            "AU.FirstName, AU.MiddleName, AU.Suffix, AU.BirthYear, AU.DeathYear " & _
            "FROM NamedItem AS NI INNER JOIN CodeMediaType AS CMT " & _
            "ON NI.MediaType = CMT.ID LEFT JOIN ItemAuthor AS IA " & _
            "ON NI.ID = IA.ItemID LEFT JOIN Author AS AU ON IA.AuthorID = AU.ID"
        Dim ItemSearchSQL2 As String = _
            " ORDER BY NI.Title, NI.Subtitle, IA.Sequence"

        ' ----- Add in the media and location limits if needed.
        Me.Cursor = Windows.Forms.Cursors.WaitCursor
        If (LimitByMedia <> -1) Then ItemSearchSQL2 = " AND NI.MediaType = " & _
            LimitByMedia & ItemSearchSQL2
        If (LimitByLocation <> -1) Then ItemSearchSQL2 = " AND NI.ID IN (" & _
            "SELECT ZZ.ItemID FROM ItemCopy AS ZZ WHERE ZZ.ItemID = NI.ID " & _
            "AND ZZ.Location = " & LimitByLocation & ")" & ItemSearchSQL2
        If (LimitByMedia = -1) And (LimitByLocation = -1) Then
            LabelSpecialLimit.Visible = False
        ElseIf (LimitByMedia = -1) Then
            LabelSpecialLimit.Text = "Limited by location."
            LabelSpecialLimit.Visible = True
        ElseIf (LimitByLocation = -1) Then
            LabelSpecialLimit.Text = "Limited by media type."
            LabelSpecialLimit.Visible = True
        Else
            LabelSpecialLimit.Text = "Limited by location and media type."
            LabelSpecialLimit.Visible = True
        End If

        ' ----- Different code for each lookup method.
        Select Case searchType
            Case LookupMethods.ByTitle
                ' ----- Perform a title search.
                sqlText = Trim(searchText)
                If (InStr(sqlText, "*") = 0) Then sqlText &= "*"
                sqlText = Replace(sqlText, "*", "%")
                sqlText = ItemSearchSQL & " WHERE NI.Title LIKE " & _
                    DBText(sqlText) & ItemSearchSQL2
                resultType = "I"
            Case LookupMethods.ByAuthor
                ' ----- Lookup matching author.
                lastName = Trim(GetSubStr(searchText, ",", 1))
                firstName = Trim(GetSubStr(searchText, ",", 2))
                If (InStr(lastName, "*") = 0) And (lastName <> "") Then _
                    lastName &= "*"
                If (InStr(firstName, "*") = 0) And (firstName <> "") Then _
                    firstName &= "*"
                lastName = Replace(lastName, "*", "%")
                firstName = Replace(firstName, "*", "%")
                sqlText = "SELECT * FROM Author WHERE "
                If (lastName <> "") Then _
                    sqlText &= "LastName LIKE " & DBText(lastName)
                If (firstName <> "") Then
                    If (lastName <> "") Then sqlText &= " AND "
                    sqlText &= "FirstName LIKE " & DBText(firstName)
                End If
                sqlText &= " ORDER BY LastName, FirstName"
                resultType = "A"
            Case LookupMethods.BySubject
                ' ----- Lookup matching subject.
                sqlText = Trim(searchText)
                If (InStr(sqlText, "*") = 0) Then sqlText &= "*"
                sqlText = Replace(sqlText, "*", "%")
                sqlText = "SELECT ID, FullName FROM Subject WHERE FullName LIKE " & _
                    DBText(sqlText) & " ORDER BY FullName"
                resultType = "M"
            Case LookupMethods.ByKeywordAny
                ' ----- Perform a keyword (any) search.
                sqlText = ItemSearchSQL & " WHERE NI.ID IN (SELECT IK.ItemID " & _
                    "FROM ItemKeyword AS IK INNER JOIN Keyword AS KW " & _
                    "ON IK.KeywordID = KW.ID WHERE "
                For counter = 1 To CountSubStr(searchText, ",") + 1
                    If (counter > 1) Then sqlText &= " OR "
                    sqlText &= "KW.FullName LIKE " & _
                    DBText(Trim(GetSubStr(searchText, ",", counter)))
                Next counter
                sqlText &= ")" & ItemSearchSQL2
                resultType = "I"
            Case LookupMethods.ByKeywordAll
                ' ----- Perform a keyword (all) search.
                sqlText = ItemSearchSQL & " WHERE "
                For counter = 1 To CountSubStr(searchText, ",") + 1
                    If (counter > 1) Then sqlText &= " AND "
                    sqlText &= "NI.ID IN (SELECT ItemKeyword.ItemID " & _
                        "FROM ItemKeyword INNER JOIN Keyword ON ItemKeyword.KeywordID " & _
                        "= Keyword.ID WHERE Keyword.FullName LIKE " & _
                        DBText(Trim(GetSubStr(searchText, ",", counter))) & ")"
                Next counter
                sqlText &= ItemSearchSQL2
                resultType = "I"
            Case LookupMethods.ByPublisher
                ' ----- Lookup matching publishers.
                sqlText = Trim(searchText)
                If (InStr(sqlText, "*") = 0) Then sqlText &= "*"
                sqlText = Replace(sqlText, "*", "%")
                sqlText = "SELECT ID, FullName FROM Publisher WHERE FullName LIKE " & _
                    DBText(sqlText) & " ORDER BY FullName"
                resultType = "M"
            Case LookupMethods.BySeries
                ' ----- Lookup matching series names.
                sqlText = Trim(searchText)
                If (InStr(sqlText, "*") = 0) Then sqlText &= "*"
                sqlText = Replace(sqlText, "*", "%")
                sqlText = "SELECT ID, FullName FROM CodeSeries WHERE FullName LIKE " & _
                    DBText(sqlText) & " ORDER BY FullName"
                resultType = "M"
            Case LookupMethods.BySeriesID
                ' ----- Return all items within a series.
                sqlText = ItemSearchSQL & " WHERE NI.Series = " & searchText & _
                    ItemSearchSQL2
                resultType = "I"
            Case LookupMethods.ByBarcode
                ' ----- Lookup by the barcode of a copy.
                sqlText = "SELECT ItemID AS ID FROM ItemCopy WHERE Barcode = " & _
                    DBText(Trim(searchText))
                resultType = "S"
            Case LookupMethods.ByDatabaseID
                ' ----- Lookup by database ID of the Named Item. Just make sure
                '       it exists.
                sqlText = "SELECT ID FROM NamedItem WHERE ID = " & searchText
                resultType = "S"
            Case LookupMethods.ByAuthorID
                ' ----- Return all items by an author.
                sqlText = ItemSearchSQL & " WHERE AU.ID = " & searchText & ItemSearchSQL2
                resultType = "I"
            Case LookupMethods.BySubjectID
                ' ----- Return all items on a certain subject.
                sqlText = ItemSearchSQL & " WHERE NI.ID IN (SELECT ItemID " & _
                    "FROM ItemSubject WHERE SubjectID = " & searchText & ")" & _
                    ItemSearchSQL2
                resultType = "I"
            Case LookupMethods.ByPublisherID
                ' ----- Return all items by a publisher.
                sqlText = ItemSearchSQL & " WHERE NI.Publisher = " & searchText & _
                    ItemSearchSQL2
                resultType = "I"
            Case Else
                Return False
        End Select

        ' ----- Perform the search.
        dbInfo = CreateReader(sqlText)
        If (dbInfo.HasRows = False) And (breakOnFailure = True) Then
            ' ----- This was the initial search, and it failed.
            Me.Cursor = Windows.Forms.Cursors.Default
            dbInfo.Close()
            dbInfo = Nothing
            Return False
        End If

        ' ----- Prepare the correct display.
        PanelMatches.Visible = False
        PanelItems.Visible = False
        PanelOneItem.Visible = False
        Select Case resultType
            Case "S"
                ' ----- Prepare for a single-item display.
                If (dbInfo.Read()) Then
                    itemID = CInt(dbInfo!ID)
                    dbInfo.Close()
                    SingleItemDetail.DocumentText = BuildHTMLAndLinks(itemID)
                Else
                    SingleItemDetail.Text = "Error"
                    ItemLinks.Items.Clear()
                End If
                LabelSingleItem.Text = _
                    BuildDisplayText(searchType, searchText)
                PanelOneItem.Visible = True
                SingleItemDetail.Focus()
            Case "I"
                ' ----- Prepare for item matches.
                MatchingItems.Items.Clear()
                matches = 0
                itemID = -1
                Do While dbInfo.Read
                    If (itemID <> CInt(dbInfo!ID)) Then
                        ' ----- Store a new display item.
                        matchEntry = New MatchingItemData
                        matchEntry.ItemID = CInt(dbInfo!ID)
                        matchEntry.Title = CStr(dbInfo!Title)
                        matchEntry.Subtitle = DBGetText(dbInfo!Subtitle)
                        matchEntry.Author = FormatAuthorName(dbInfo)
                        matchEntry.MediaType = CStr(dbInfo!MediaName)
                        holdText = ""
                        If (UseLC = True) Then
                            If (IsDBNull(dbInfo!LC) = False) Then holdText = _
                                GetSubStr(CStr(dbInfo!LC), "/", 1)
                        Else
                            If (IsDBNull(dbInfo!Dewey) = False) Then holdText = _
                                GetSubStr(CStr(dbInfo!Dewey), "/", 1)
                        End If
                        matchEntry.CallNumber = holdText

                        ' ----- Store the match in the list.
                        itemID = CInt(dbInfo!ID)
                        MatchingItems.Items.Add(matchEntry)

                        ' ----- Limit the search to SearchMatchLimit matches.
                        matches += 1
                        If (matches >= SearchMatchLimit) Then
                            If (dbInfo.Read) Then matches += 1
                            Exit Do
                        End If
                    Else
                        ' ----- We found another match for the previous item, 
                        '       caused() by the presence of another author record.
                        '       Modify the last item by adding "et. al." to the
                        '       display.
                        matchEntry = CType(MatchingItems.SelectedItem, _
                            MatchingItemData)
                        If (MVB.Right(matchEntry.Author, 9) <> ", et. al.") Then
                            matchEntry.Author &= ", et. al."
                            MatchingItems.Items(MatchingItems.Items.Count - 1) = _
                                matchEntry
                        End If
                    End If
                Loop
                dbInfo.Close()

                ' ----- Display the results.
                LabelMatchingItems.Text = _
                    BuildDisplayText(searchType, searchText)
                If (matches = 1) Then
                    LabelItemsCount.Text = "1 matching item found."
                ElseIf (matches > SearchMatchLimit) Then
                    LabelItemsCount.Text = "Limited to first " & _
                    SearchMatchLimit & " matching items."
                Else
                    LabelItemsCount.Text = matches & " matching items found."
                End If
                ActItemLookup.Enabled = False
                PanelItems.Visible = True
            Case "M", "A"
                ' ----- General matches.
                matches = 0
                MatchingGeneral.Items.Clear()
                Do While dbInfo.Read
                    If (resultType = "M") Then
                        ' ----- General list item.
                        MatchingGeneral.Items.Add(New ListItemData( _
                            CStr(dbInfo!FullName), CInt(dbInfo!ID)))
                    Else
                        ' ----- Author list item.
                        MatchingGeneral.Items.Add(New ListItemData( _
                            FormatAuthorName(dbInfo), CInt(dbInfo!ID)))
                    End If

                    ' ----- Limit the search to SearchMatchLimit matches.
                    matches += 1
                    If (matches >= SearchMatchLimit) Then
                        If (dbInfo.Read) Then matches += 1
                        Exit Do
                    End If
                Loop
                dbInfo.Close()

                ' ----- Display the results.
                LabelMatchingGeneral.Text = _
                    BuildDisplayText(searchType, searchText)
                If (matches = 1) Then
                    LabelMatchCount.Text = "1 match found."
                ElseIf (matches > SearchMatchLimit) Then
                    LabelMatchCount.Text = "Limited to first " & _
                    SearchMatchLimit & " matches."
                Else
                    LabelMatchCount.Text = matches & " matches found."
                End If
                ActGeneralLookup.Enabled = False
                PanelMatches.Visible = True
        End Select
        dbInfo = Nothing
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("ItemLookup.PerformLookup", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return False
    End Function

    Private Sub RefreshBackButtons()
        ' ----- Refresh the display of the back button items.
        Dim counter As Integer
        Dim scanSet As System.Collections.IEnumerator
        Dim scanHistory As ItemLookupHistory
        Dim whichMenu As System.Windows.Forms.ToolStripMenuItem

        ' ----- See if there is any history.
        If (LookupHistorySet.Count <= 1) Then
            ' ----- No extra history available.
            ActBack.Enabled = False
            ActBackMenu.Enabled = False
        Else
            ' ----- Build the history. Skip the first item as it is the
            '       one that is displayed.
            counter = 1
            scanSet = LookupHistorySet.GetEnumerator()
            scanSet.MoveNext() ' Skip first entry.
            Do While scanSet.MoveNext()
                scanHistory = CType(scanSet.Current, ItemLookupHistory)
                Select Case counter
                    Case 1 : whichMenu = BackMenu1
                    Case 2 : whichMenu = BackMenu2
                    Case 3 : whichMenu = BackMenu3
                    Case 4 : whichMenu = BackMenu4
                    Case 5 : whichMenu = BackMenu5
                    Case 6 : whichMenu = BackMenu6
                    Case 7 : whichMenu = BackMenu7
                    Case 8 : whichMenu = BackMenu8
                    Case 9 : whichMenu = BackMenu9
                    Case Else : whichMenu = BackMenu10
                End Select
                whichMenu.Text = scanHistory.HistoryDisplay
                whichMenu.Visible = True
                counter += 1
                If (counter > 10) Then Exit Do
            Loop

            ' ----- Hide unused menu items.
            Do While (counter <= 10)
                Select Case counter
                    Case 1 : whichMenu = BackMenu1
                    Case 2 : whichMenu = BackMenu2
                    Case 3 : whichMenu = BackMenu3
                    Case 4 : whichMenu = BackMenu4
                    Case 5 : whichMenu = BackMenu5
                    Case 6 : whichMenu = BackMenu6
                    Case 7 : whichMenu = BackMenu7
                    Case 8 : whichMenu = BackMenu8
                    Case 9 : whichMenu = BackMenu9
                    Case Else : whichMenu = BackMenu10
                End Select
                whichMenu.Visible = False
                counter += 1
            Loop
            ActBack.Enabled = True
            ActBackMenu.Enabled = True
        End If
    End Sub

    Private Function BuildHTMLAndLinks(ByVal itemID As Integer) As String
        ' ----- Given the ID of a library item, build the HTML text display
        '       that shows all of the relevant detail.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim dbTable As DataTable
        Dim detailBody As New System.Text.StringBuilder
        Dim counter As Integer
        Dim rowScan As DataRow
        Dim holdText As String
        Dim entryID As Integer
        Dim newLink As SingleItemLink

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Initialize.
        entryID = 0
        ItemLinkSet.Clear()
        ItemLinks.Items.Clear()

        ' ----- The main HTML headers are stored in a text file resource.
        '       Add it to the body content now.
        detailBody.Append(My.Resources.ItemLookupBody)

        ' ----- Add in the title.
        sqlText = "SELECT Title, Subtitle FROM NamedItem WHERE ID = " & itemID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        detailBody.AppendLine("<h1>" & HTMLEncode(CStr(dbInfo!Title)) & "</h1>")

        ' ----- Add in the subtitle if available.
        If (IsDBNull(dbInfo!Subtitle) = False) Then
            detailBody.AppendLine("<h3>" & HTMLEncode(CStr(dbInfo!Subtitle)) & "</h3>")
        End If
        dbInfo.Close()

        ' ----- Add in the authors and names.
        sqlText = "SELECT AU.*, IA.Sequence, CA.FullName FROM ItemAuthor AS IA " & _
            "INNER JOIN Author AS AU ON IA.AuthorID = AU.ID INNER JOIN " & _
            "CodeAuthorType AS CA ON IA.AuthorType = CA.ID WHERE IA.ItemID = " & _
            itemID & " ORDER BY IA.Sequence"
        dbInfo = CreateReader(sqlText)
        If (dbInfo.HasRows) Then
            ' ----- There are authors or names. Display a heading.
            detailBody.AppendLine("<h2>Authors and Names</h2>")
            Do While dbInfo.Read
                ' ----- Add in this one author name.
                holdText = FormatAuthorName(dbInfo)
                entryID += 1
                detailBody.AppendLine("<p><a href=""library://" & entryID & """>" & _
                    HTMLEncode(holdText & " [" & CStr(dbInfo!FullName) & "]") & _
                    "</a></p>")

                ' ----- Add in an author link.
                newLink = New SingleItemLink
                newLink.LinkType = General.LookupMethods.ByAuthorID
                newLink.LinkID = CInt(dbInfo!ID)
                ItemLinkSet.Add(entryID, newLink)
                ItemLinks.Items.Add(New ListItemData("Author: " & _
                    holdText & " [" & CStr(dbInfo!FullName) & "]", entryID))
            Loop
        End If
        dbInfo.Close()

        ' ----- Retrieve most of the remaining fields.
        sqlText = "SELECT NI.Dewey, NI.Dewey, NI.LC, NI.Description, NI.Publisher, " & _
            "PB.FullName AS PubName, NI.Edition, NI.Copyright, NI.ISxN, NI.Series, " & _
            "CS.FullName AS SeriesName, CMT.FullName AS MediaName " & _
            "FROM NamedItem AS NI LEFT JOIN Publisher AS PB ON NI.Publisher = PB.ID " & _
            "LEFT JOIN CodeSeries AS CS ON NI.Series = CS.ID " & _
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " & _
            "WHERE NI.ID = " & itemID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()

        ' ----- Add in the call number information.
        detailBody.AppendLine("<h2>Call Number</h2>")
        holdText = "No call number available."
        If (UseLC) Then
            ' ----- Show the Library of Congress number.
            If (IsDBNull(dbInfo!LC) = False) Then holdText = CStr(dbInfo!LC)
        Else
            ' ----- Show the Dewey Decimal number.
            If (IsDBNull(dbInfo!Dewey) = False) Then holdText = CStr(dbInfo!Dewey)
        End If
        For counter = 1 To CountSubStr(holdText, "/") + 1
            ' ----- Add one line of the number.
            detailBody.AppendLine("<p>" & HTMLEncode(Trim( _
                GetSubStr(holdText, "/", counter))) & "</p>")
        Next counter

        ' ----- Display the media type name.
        detailBody.AppendLine("<p><i>" & HTMLEncode(CStr(dbInfo!MediaName)) & _
            "</i></p>")

        ' ----- Add in the item description.
        If (IsDBNull(dbInfo!Description) = False) Then
            detailBody.AppendLine("<h2>Description</h2>")
            detailBody.AppendLine("<p>" & HTMLEncode(CStr(dbInfo!Description)) & "</p>")
        End If

        ' ----- Add in the publishing information.
        counter = 0
        If (IsDBNull(dbInfo!PubName) = False) Then counter += 1
        If (IsDBNull(dbInfo!Edition) = False) Then counter += 1
        If (IsDBNull(dbInfo!Copyright) = False) Then counter += 1
        If (IsDBNull(dbInfo!ISxN) = False) Then counter += 1
        If (counter > 0) Then
            ' ----- There is some publishing information to display.
            detailBody.AppendLine("<h2>Publishing Information</h2>")

            ' ----- Add the publisher name, if available.
            If (IsDBNull(dbInfo!PubName) = False) Then
                entryID += 1
                holdText = CStr(dbInfo!PubName)
                detailBody.AppendLine("<p><a href=""library://" & entryID & """>" & _
                    HTMLEncode(holdText) & "</a></p>")

                ' ----- Add in a publisher link.
                newLink = New SingleItemLink
                newLink.LinkType = General.LookupMethods.ByPublisherID
                newLink.LinkID = CInt(dbInfo!Publisher)
                ItemLinkSet.Add(entryID, newLink)
                ItemLinks.Items.Add(New ListItemData("Publisher: " & _
                    holdText, entryID))
            End If

            ' ----- Add the edition number, if available.
            If (IsDBNull(dbInfo!Edition) = False) Then
                detailBody.AppendLine("<p>Edition: " & _
                    HTMLEncode(CStr(dbInfo!Edition)) & "</p>")
            End If

            ' ----- Add the copyright year, if available.
            If (IsDBNull(dbInfo!Copyright) = False) Then
                detailBody.AppendLine("<p>Copyright: " & _
                    HTMLEncode(CStr(dbInfo!Copyright)) & "</p>")
            End If

            ' ----- Add the ISBN/ISSN number, if available.
            If (IsDBNull(dbInfo!ISxN) = False) Then
                detailBody.AppendLine("<p>ISBN / ISSN: " & _
                    HTMLEncode(CStr(dbInfo!ISxN)) & "</p>")
            End If
        End If

        ' ----- Add in the series information.
        If (IsDBNull(dbInfo!SeriesName) = False) Then
            detailBody.AppendLine("<h2>Related Series</h2>")

            entryID += 1
            holdText = CStr(dbInfo!SeriesName)
            detailBody.AppendLine("<p><a href=""library://" & entryID & """>" & _
                HTMLEncode(holdText) & "</a></p>")

            ' ----- Add in a series link.
            newLink = New SingleItemLink
            newLink.LinkType = General.LookupMethods.BySeriesID
            newLink.LinkID = CInt(dbInfo!Series)
            ItemLinkSet.Add(entryID, newLink)
            ItemLinks.Items.Add(New ListItemData("Series: " & holdText, entryID))
        End If
        dbInfo.Close()

        ' ----- Add in the related subjects.
        sqlText = "SELECT SB.* FROM ItemSubject AS IB INNER JOIN Subject AS SB " & _
            "ON IB.SubjectID = SB.ID WHERE IB.ItemID = " & _
            itemID & " ORDER BY SB.FullName"
        dbInfo = CreateReader(sqlText)
        If (dbInfo.HasRows) Then
            ' ----- There are subjects. Display a heading.
            detailBody.AppendLine("<h2>Related Subjects</h2>")

            Do While dbInfo.Read
                ' ----- Add in this one subject name.
                entryID += 1
                holdText = CStr(dbInfo!FullName)
                detailBody.AppendLine("<p><a href=""library://" & entryID & """>" & _
                    HTMLEncode(holdText) & "</a></p>")

                ' ----- Add in a subject link.
                newLink = New SingleItemLink
                newLink.LinkType = General.LookupMethods.BySubjectID
                newLink.LinkID = CInt(dbInfo!ID)
                ItemLinkSet.Add(entryID, newLink)
                ItemLinks.Items.Add(New ListItemData("Subject: " & holdText, entryID))
            Loop
        End If
        dbInfo.Close()

        ' ----- See how many copies there are.
        sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE ItemID = " & itemID
        counter = CInt(ExecuteSQLReturn(sqlText))

        ' ----- Add in the copies information if needed.
        If (counter > 0) Then
            ' ----- Introduce the copies.
            If (counter = 1) Then holdText = "1 copy" Else holdText = counter & " copies"
            detailBody.AppendLine("<h2>Copies</h2>")
            detailBody.AppendLine("<p>" & holdText & "</p>")

            ' ----- Build a table that lists the copies. Use a data table
            '       since the call to GetCopyDisposition() will open
            '       a separate data reader.
            detailBody.AppendLine("<table cellspacing=""0"" cellpadding=""3"">")
            detailBody.AppendLine("<tr><th>Copy</th><th>Location</th>" & _
                "<th>Status</th></tr>")
            sqlText = "SELECT IC.ID, IC.CopyNumber, CL.FullName FROM ItemCopy AS IC " & _
                "LEFT JOIN CodeLocation AS CL ON IC.Location = CL.ID " & _
                "WHERE IC.ItemID = " & itemID & " ORDER BY IC.CopyNumber"
            dbTable = CreateDataTable(sqlText)
            For Each rowScan In dbTable.Rows
                ' ----- Add the copy number.
                detailBody.Append("<tr><td>" & CStr(CInt(rowScan!CopyNumber)) & "</td>")

                ' ----- Add the location.
                If (IsDBNull(rowScan!FullName) = True) Then holdText = "Not listed" _
                    Else holdText = HTMLEncode(CStr(rowScan!FullName))
                detailBody.Append("<td>" & holdText & "</td>")

                ' ----- Add the disposition.
                detailBody.AppendLine("<td>" & HTMLEncode(GetCopyDisposition( _
                    CInt(rowScan!ID))) & "</td></tr>")
            Next rowScan
            dbTable.Dispose()
            dbTable = Nothing
            detailBody.AppendLine("</table>")
        End If

        ' ----- Finished.
        detailBody.AppendLine("</body>")
        detailBody.AppendLine("</html>")
        Return detailBody.ToString()

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("ItemLookup.BuildHTMLAndLinks", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return "Error"
    End Function

    Private Sub MatchingGeneral_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles MatchingGeneral.DoubleClick
        ' ----- Same as a click on the Lookup button.
        ActGeneralLookup.PerformClick()
    End Sub

    Private Sub ActGeneralLookup_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActGeneralLookup.Click
        ' ----- Lookup the items associated with the selected match.
        Dim baseType As Library.LookupMethods
        Dim searchType As Library.LookupMethods
        Dim itemID As Integer

        ' ----- Ignore if no match is selected.
        If (MatchingGeneral.SelectedIndex = -1) Then Return
        itemID = CType(MatchingGeneral.SelectedItem, ListItemData).ItemData

        ' ----- Determine the type of search.
        baseType = LookupHistorySet.Peek.LookupType
        Select Case baseType
            Case LookupMethods.ByAuthor
                searchType = LookupMethods.ByAuthorID
            Case LookupMethods.BySubject
                searchType = LookupMethods.BySubjectID
            Case LookupMethods.ByPublisher
                searchType = LookupMethods.ByPublisherID
            Case LookupMethods.BySeries
                searchType = LookupMethods.BySeriesID
        End Select

        ' ----- Perform the lookup.
        If (PerformLookup(searchType, CStr(itemID), False) = False) Then Return

        ' ----- Store the history.
        AddLookupHistory(searchType, CStr(itemID))
    End Sub

    Private Sub MatchingItems_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles MatchingItems.DoubleClick
        ' ----- Same as the lookup button.
        ActItemLookup.PerformClick()
    End Sub

    Private Sub ActItemLookup_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActItemLookup.Click
        ' ----- Lookup the item with the selected ID.
        Dim itemID As Integer

        ' ----- Ignore if no match is selected.
        If (MatchingItems.SelectedIndex = -1) Then Return
        itemID = CType(MatchingItems.SelectedItem, MatchingItemData).ItemID

        ' ----- Perform the lookup.
        If (PerformLookup(LookupMethods.ByDatabaseID, CStr(itemID), False) = False) _
            Then Return

        ' ----- Store the history.
        AddLookupHistory(LookupMethods.ByDatabaseID, CStr(itemID))
    End Sub

    Private Sub SingleItemDetail_Navigating(ByVal sender As Object, ByVal e As System.Windows.Forms.WebBrowserNavigatingEventArgs) Handles SingleItemDetail.Navigating
        ' ----- Follow the clicked link.
        If (e.Url.Scheme = "library") Then _
            FollowItemLink(CInt(e.Url.Host()))
    End Sub

    Private Sub BackMenuItems_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
            Handles BackMenu1.Click, BackMenu2.Click, BackMenu3.Click, BackMenu4.Click, _
            BackMenu5.Click, BackMenu6.Click, BackMenu7.Click, BackMenu8.Click, _
            BackMenu9.Click, BackMenu10.Click
        ' ----- One of the back menu items was clicked.
        Dim whichItem As Integer
        Dim counter As Integer
        Dim scanHistory As ItemLookupHistory

        ' ----- Determine the clicked item.
        whichItem = CInt(DigitsOnly(CType(sender, _
            System.Windows.Forms.ToolStripMenuItem).Name))
        If (whichItem >= LookupHistorySet.Count) Then Return

        ' ----- Get rid of the in-between items.
        For counter = 1 To whichItem
            LookupHistorySet.Pop()
        Next counter

        ' ----- Perform a lookup as requested.
        scanHistory = LookupHistorySet.Peek
        If (PerformLookup(scanHistory.LookupType, scanHistory.LookupData, _
            False) = False) Then Return
        RefreshBackButtons()
    End Sub

    Private Sub ActBackMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActBackMenu.Click
        ' ----- Show the popup menu for back items.
        BackMenu.Show(ActBackMenu, New System.Drawing.Point(-ActBack.Width, _
            ActBackMenu.Height))
    End Sub

    Private Sub ActBack_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActBack.Click
        ' ----- Same as a click on the first menu item.
        If (LookupHistorySet.Count > 1) Then BackMenu1.PerformClick()
    End Sub

    Private Sub MatchingGeneral_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MatchingGeneral.SelectedIndexChanged
        ' ----- Activate the lookup button if needed.
        ActGeneralLookup.Enabled = CBool(MatchingGeneral.SelectedIndex <> -1)
    End Sub

    Private Sub MatchingGeneral_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MatchingGeneral.Click
        ' ----- Activate the lookup button if needed.
        ActGeneralLookup.Enabled = CBool(MatchingGeneral.SelectedIndex <> -1)
    End Sub

    Private Sub MatchingItems_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles MatchingItems.SelectedIndexChanged
        ' ----- Activate the lookup button if needed.
        ActItemLookup.Enabled = CBool(MatchingItems.SelectedIndex <> -1)
    End Sub

    Private Sub MatchingItems_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MatchingItems.Click
        ' ----- Activate the lookup button if needed.
        ActItemLookup.Enabled = CBool(MatchingItems.SelectedIndex <> -1)
    End Sub

    Private Sub ActItemLinks_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActItemLinks.Click
        ' ----- Jump to the link indicated in the drop-down list.
        Dim entryLink As Integer

        ' ----- Ignore if no link is selected.
        If (ItemLinks.SelectedIndex = -1) Then Return
        entryLink = CType(ItemLinks.SelectedItem, ListItemData).ItemData
        FollowItemLink(entryLink)
    End Sub

    Private Sub FollowItemLink(ByVal entryID As Integer)
        ' ----- Given a character position in the single item text panel,
        '       follow the link indicated by that item.
        Dim scanLink As SingleItemLink

        ' ----- Access the link.
        scanLink = ItemLinkSet.Item(entryID)
        If (scanLink Is Nothing) Then Return

        ' ----- Perform a lookup as requested.
        If (PerformLookup(scanLink.LinkType, CStr(scanLink.LinkID), False) = False) _
            Then Return

        ' ----- Store the history.
        AddLookupHistory(scanLink.LinkType, CStr(scanLink.LinkID))
    End Sub

    Private Sub ItemLookup_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles MyBase.KeyPress
        ' ----- Treat the backspace key like a click on the Back button.
        If (e.KeyChar = Chr(Keys.Back)) Then
            If (ActBack.Enabled = True) Then ActBack.PerformClick()
        End If
    End Sub
End Class
