Namespace My

    'The following events are available for MyApplication
    '
    'Startup: Raised when the application starts, before the startup form is created.
    'Shutdown: Raised after all application forms are closed.  This event is not raised if the application is terminating abnormally.
    'UnhandledException: Raised if the application encounters an unhandled exception.
    'StartupNextInstance: Raised when launching a single-instance application and the application is already active. 
    'NetworkAvailabilityChanged: Raised when the network connection is connected or disconnected.

    Class MyApplication

        Private Sub MyApplication_Shutdown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shutdown
            ' ----- Close the database.
            CleanUpProgram()
        End Sub

        Private Sub MyApplication_Startup(ByVal sender As Object, ByVal e As Microsoft.VisualBasic.ApplicationServices.StartupEventArgs) Handles Me.Startup
            ' ----- The application starts here.
            Dim productionDBVersion As String

            ' ----- Display the splash form for at least 3 seconds.
            My.Application.MinimumSplashScreenDisplayTime = 3000

            ' ----- Perform general initialization.
            InitializeSystem()

            ' ----- Connect to the database.
            If (ConnectDatabase() = False) Then
                Me.HideSplashScreen()
                e.Cancel = True
                Return
            End If

            ' ----- Check the database version.
            productionDBVersion = Trim(GetSystemValue("DatabaseVersion"))
            If (CInt(Val(productionDBVersion)) <> UseDBVersion) Then
                MsgBox("The program cannot continue due to an incompatible database. " & _
                    "The current database version is '" & productionDBVersion & _
                    "'. The application version is '" & UseDBVersion & "'.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Critical, ProgramTitle)
                CleanUpProgram()
                Me.HideSplashScreen()
                e.Cancel = True
                Return
            End If

            ' ----- Load some settings that reside in the database.
            LoadDatabaseSettings()

            ' ***** Insert Code Snippet #8 here.

        End Sub

        Private Sub MyApplication_StartupNextInstance(ByVal sender As Object, _
              ByVal e As Microsoft.VisualBasic.ApplicationServices.StartupNextInstanceEventArgs) _
              Handles Me.StartupNextInstance
            ' ----- Force the main form to the front.
            My.Application.MainForm.Activate()
        End Sub

        Private Sub MyApplication_UnhandledException(ByVal sender As Object, ByVal e As Microsoft.VisualBasic.ApplicationServices.UnhandledExceptionEventArgs) Handles Me.UnhandledException
            ' ----- Record the error, and keep running.
            e.ExitApplication = False
            GeneralError("Unhandled Exception", e.Exception)
        End Sub
    End Class
End Namespace
