' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports System.ComponentModel

Public MustInherit Class BarcodeItemGeneric
    <Browsable(False)> Public MustOverride ReadOnly Property ItemType() As String
    Public MustOverride Overrides Function ToString() As String
End Class

Public Class BarcodeItemText
    ' ----- Includes a static text element in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of attributes.
    Private StoredTextLeft As Single
    Private StoredTextTop As Single
    Private StoredTextWidth As Single
    Private StoredTextHeight As Single
    Private StoredTextColor As Drawing.Color
    Private StoredTextAngle As Short
    Private StoredTextFont As Drawing.Font
    Private StoredDisplayText As String
    Private StoredTextAlignment As Drawing.ContentAlignment

    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        ' ----- The ItemType constant helps when saving the record
        '       to the database.
        Get
            Return "T"
        End Get
    End Property

    Public Overrides Function ToString() As String
        If (StoredDisplayText = "") Then
            Return "Text: [Undefined]"
        Else
            Return "Text: " & StoredDisplayText
        End If
    End Function

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the left-edge position of the text box.")> _
    Public Property Left() As Single
        ' ----- Upper-left corner of the text box.
        Get
            Return StoredTextLeft
        End Get
        Set(ByVal Value As Single)
            StoredTextLeft = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the top-edge position of the text box.")> _
    Public Property Top() As Single
        ' ----- Upper-left corner of the text box.
        Get
            Return StoredTextTop
        End Get
        Set(ByVal Value As Single)
            StoredTextTop = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the width of the text box.")> _
    Public Property Width() As Single
        ' ----- The width and height of the text box.
        Get
            Return StoredTextWidth
        End Get
        Set(ByVal Value As Single)
            StoredTextWidth = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the height of the text box.")> _
    Public Property Height() As Single
        ' ----- The width and height of the text box.
        Get
            Return StoredTextHeight
        End Get
        Set(ByVal Value As Single)
            StoredTextHeight = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the color of the displayed text.")> _
    Public Property TextColor() As Drawing.Color
        ' ----- The text color.
        Get
            Return StoredTextColor
        End Get
        Set(ByVal Value As Drawing.Color)
            StoredTextColor = Value
        End Set
    End Property

    <Browsable(True), _
    DescriptionAttribute("Sets the clockwise rotation angle, from 0 to 359 degrees.")> _
    Public Property RotationAngle() As Short
        ' ----- Rotation angle of the text.
        Get
            Return StoredTextAngle
        End Get
        Set(ByVal Value As Short)
            ' ----- Make sure the angle is in the valid range.
            If (Value < 0) Or (Value > 359) Then
                Throw New System.ArgumentException( _
                    "Invalid rotation angle: " & Value & _
                    ". Rotation angle must range from 0 to 359 degrees.")
                Return
            End If
            StoredTextAngle = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the font of the text.")> _
    Public Property Font() As Drawing.Font
        ' ----- The font of the text.
        Get
            Return StoredTextFont
        End Get
        Set(ByVal Value As Drawing.Font)
            StoredTextFont = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the text to display.")> _
    Public Property DisplayText() As String
        ' ----- The actual text.
        Get
            Return StoredDisplayText
        End Get
        Set(ByVal Value As String)
            ' ----- Make sure the angle is in the valid range.
            If (Len(Value) > 100) Then
                Throw New System.ArgumentException( _
                    "Invalid description. Text is limited to 100 characters maximum.")
                Return
            End If
            StoredDisplayText = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the alignment of the text.")> _
    Public Property Alignment() As Drawing.ContentAlignment
        ' ----- The alignment of the text.
        Get
            Return StoredTextAlignment
        End Get
        Set(ByVal Value As Drawing.ContentAlignment)
            StoredTextAlignment = Value
        End Set
    End Property
End Class

Public Class BarcodeItemLine
    ' ----- Includes a basic line element in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of attributes.
    Private StoredLineX1 As Single
    Private StoredLineX2 As Single
    Private StoredLineY1 As Single
    Private StoredLineY2 As Single
    Private StoredLineColor As Drawing.Color

    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        ' ----- The ItemType constant helps when saving the record
        '       to the database.
        Get
            Return "L"
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return "Line: (" & StoredLineX1 & "," & StoredLineY1 & ") to (" & StoredLineX2 & _
            "," & StoredLineY2 & ")"
    End Function

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the starting horizontal position of the line.")> _
    Public Property X1() As Single
        ' ----- Starting point of the line.
        Get
            Return StoredLineX1
        End Get
        Set(ByVal Value As Single)
            StoredLineX1 = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the starting vertical position of the line.")> _
    Public Property Y1() As Single
        ' ----- Starting point of the line.
        Get
            Return StoredLineY1
        End Get
        Set(ByVal Value As Single)
            StoredLineY1 = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the ending horizontal position of the line.")> _
    Public Property X2() As Single
        ' ----- Ending point of the line.
        Get
            Return StoredLineX2
        End Get
        Set(ByVal Value As Single)
            StoredLineX2 = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the ending vertical position of the line.")> _
    Public Property Y2() As Single
        ' ----- Ending point of the line.
        Get
            Return StoredLineY2
        End Get
        Set(ByVal Value As Single)
            StoredLineY2 = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the color of the line.")> _
    Public Property LineColor() As Drawing.Color
        ' ----- The line color.
        Get
            Return StoredLineColor
        End Get
        Set(ByVal Value As Drawing.Color)
            StoredLineColor = Value
        End Set
    End Property
End Class

Public Class BarcodeItemBarcodeImage
    ' ----- Includes an actual barcode element in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of attributes.
    Private StoredBarLeft As Single
    Private StoredBarTop As Single
    Private StoredBarWidth As Single
    Private StoredBarHeight As Single
    Private StoredBarColor As Drawing.Color
    Private StoredBarFontSize As Single
    Private StoredBarAngle As Short
    Private StoredBarAlignment As Drawing.ContentAlignment
    Private StoredBarPadDigits As Byte

    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        ' ----- The ItemType constant helps when saving the record
        '       to the database.
        Get
            Return "B"
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return "Barcode: At (" & StoredBarLeft & "," & StoredBarTop & ")"
    End Function

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the left-edge position of the barcode box.")> _
    Public Property Left() As Single
        ' ----- Upper-left corner of the text box.
        Get
            Return StoredBarLeft
        End Get
        Set(ByVal Value As Single)
            StoredBarLeft = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the top-edge position of the barcode box.")> _
    Public Property Top() As Single
        ' ----- Upper-left corner of the text box.
        Get
            Return StoredBarTop
        End Get
        Set(ByVal Value As Single)
            StoredBarTop = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the width of the barcode box.")> _
    Public Property Width() As Single
        ' ----- The width of the text box.
        Get
            Return StoredBarWidth
        End Get
        Set(ByVal Value As Single)
            StoredBarWidth = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the height of the barcode box.")> _
    Public Property Height() As Single
        ' ----- The height of the text box.
        Get
            Return StoredBarHeight
        End Get
        Set(ByVal Value As Single)
            StoredBarHeight = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the size of the barcode, in points.")> _
    Public Property BarcodeSize() As Single
        ' ----- The barcode font size.
        Get
            Return StoredBarFontSize
        End Get
        Set(ByVal Value As Single)
            StoredBarFontSize = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the color of the displayed barcode.")> _
    Public Property BarcodeColor() As Drawing.Color
        ' ----- The barcode color.
        Get
            Return StoredBarColor
        End Get
        Set(ByVal Value As Drawing.Color)
            StoredBarColor = Value
        End Set
    End Property

    <Browsable(True), _
    DescriptionAttribute("Sets the rotation angle, from 0 to 359 degrees.")> _
    Public Property RotationAngle() As Short
        ' ----- Rotation angle of the text.
        Get
            Return StoredBarAngle
        End Get
        Set(ByVal Value As Short)
            ' ----- Make sure the angle is in the valid range.
            If (Value < 0) Or (Value > 359) Then
                Throw New System.ArgumentException( _
                    "Invalid rotation angle: " & Value & _
                    ". Rotation angle must range from 0 to 359 degrees.")
                Return
            End If
            StoredBarAngle = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the alignment of the barcode.")> _
    Public Property Alignment() As Drawing.ContentAlignment
        ' ----- The alignment of the text.
        Get
            Return StoredBarAlignment
        End Get
        Set(ByVal Value As Drawing.ContentAlignment)
            StoredBarAlignment = Value
        End Set
    End Property

    <Browsable(True), _
    DescriptionAttribute("The number of digits in which to pad the barcode number, or 0 to ignore.")> _
    Public Property PadDigits() As Byte
        ' ----- Number of pad digits.
        Get
            Return StoredBarPadDigits
        End Get
        Set(ByVal Value As Byte)
            ' ----- Make sure the count is in the valid range.
            If (Value < 0) Or (Value > 20) Then
                Throw New System.ArgumentException( _
                    "Invalid pad digits value: " & Value & _
                    ". Pad digits must range from 0 to 20.")
                Return
            End If
            StoredBarPadDigits = Value
        End Set
    End Property
End Class

Public Class BarcodeItemBarcodeNumber
    ' ----- Includes the numeric version of a barcode in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of attributes.
    Private StoredTextLeft As Single
    Private StoredTextTop As Single
    Private StoredTextWidth As Single
    Private StoredTextHeight As Single
    Private StoredTextColor As Drawing.Color
    Private StoredTextAngle As Short
    Private StoredTextFont As Drawing.Font
    Private StoredTextAlignment As Drawing.ContentAlignment
    Private StoredTextPadDigits As Byte

    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        ' ----- The ItemType constant helps when saving the record
        '       to the database.
        Get
            Return "N"
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return "Barcode Text: At (" & StoredTextLeft & "," & StoredTextTop & ")"
    End Function

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the left-edge position of the text box.")> _
    Public Property Left() As Single
        ' ----- Upper-left corner of the text box.
        Get
            Return StoredTextLeft
        End Get
        Set(ByVal Value As Single)
            StoredTextLeft = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the top-edge position of the text box.")> _
    Public Property Top() As Single
        ' ----- Upper-left corner of the text box.
        Get
            Return StoredTextTop
        End Get
        Set(ByVal Value As Single)
            StoredTextTop = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the width of the text box.")> _
    Public Property Width() As Single
        ' ----- The width of the text box.
        Get
            Return StoredTextWidth
        End Get
        Set(ByVal Value As Single)
            StoredTextWidth = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the height of the text box.")> _
    Public Property Height() As Single
        ' ----- The height of the text box.
        Get
            Return StoredTextHeight
        End Get
        Set(ByVal Value As Single)
            StoredTextHeight = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the color of the displayed text.")> _
    Public Property TextColor() As Drawing.Color
        ' ----- The text color.
        Get
            Return StoredTextColor
        End Get
        Set(ByVal Value As Drawing.Color)
            StoredTextColor = Value
        End Set
    End Property

    <Browsable(True), _
    DescriptionAttribute("Sets the rotation angle, from 0 to 359 degrees.")> _
    Public Property RotationAngle() As Short
        ' ----- Rotation angle of the text.
        Get
            Return StoredTextAngle
        End Get
        Set(ByVal Value As Short)
            ' ----- Make sure the angle is in the valid range.
            If (Value < 0) Or (Value > 359) Then
                Throw New System.ArgumentException( _
                    "Invalid rotation angle: " & Value & _
                    ". Rotation angle must range from 0 to 359 degrees.")
                Return
            End If
            StoredTextAngle = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the font of the text.")> _
    Public Property Font() As Drawing.Font
        ' ----- The font of the text.
        Get
            Return StoredTextFont
        End Get
        Set(ByVal Value As Drawing.Font)
            StoredTextFont = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the alignment of the text.")> _
    Public Property Alignment() As Drawing.ContentAlignment
        ' ----- The alignment of the text.
        Get
            Return StoredTextAlignment
        End Get
        Set(ByVal Value As Drawing.ContentAlignment)
            StoredTextAlignment = Value
        End Set
    End Property

    <Browsable(True), _
    DescriptionAttribute("The number of digits in which to pad the barcode number, or 0 to ignore.")> _
    Public Property PadDigits() As Byte
        ' ----- Number of pad digits.
        Get
            Return StoredTextPadDigits
        End Get
        Set(ByVal Value As Byte)
            ' ----- Make sure the count is in the valid range.
            If (Value < 0) Or (Value > 20) Then
                Throw New System.ArgumentException( _
                    "Invalid pad digits value: " & Value & _
                    ". Pad digits must range from 0 to 20.")
                Return
            End If
            StoredTextPadDigits = Value
        End Set
    End Property
End Class

Public Class BarcodeItemRect
    ' ----- Includes a basic rectangle element in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of attributes.
    Private StoredRectLeft As Single
    Private StoredRectTop As Single
    Private StoredRectWidth As Single
    Private StoredRectHeight As Single
    Private StoredRectColor As Drawing.Color
    Private StoredFillColor As Drawing.Color
    Private StoredRectAngle As Short

    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        ' ----- The ItemType constant helps when saving the record
        '       to the database.
        Get
            Return "R"
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return "Rectangle: At (" & StoredRectLeft & "," & StoredRectTop & ")"
    End Function

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the left-edge position of the rectangle.")> _
    Public Property Left() As Single
        ' ----- Upper-left corner of the text box.
        Get
            Return StoredRectLeft
        End Get
        Set(ByVal Value As Single)
            StoredRectLeft = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the top-edge position of the rectangle.")> _
    Public Property Top() As Single
        ' ----- Upper-left corner of the text box.
        Get
            Return StoredRectTop
        End Get
        Set(ByVal Value As Single)
            StoredRectTop = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the width of the rectangle.")> _
    Public Property Width() As Single
        ' ----- The width and height of the text box.
        Get
            Return StoredRectWidth
        End Get
        Set(ByVal Value As Single)
            StoredRectWidth = Value
        End Set
    End Property

    <Browsable(True), DefaultValueAttribute("0"), _
    DescriptionAttribute("Sets the height of the rectangle.")> _
    Public Property Height() As Single
        ' ----- The width and height of the text box.
        Get
            Return StoredRectHeight
        End Get
        Set(ByVal Value As Single)
            StoredRectHeight = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the border color of the rectangle.")> _
    Public Property BorderColor() As Drawing.Color
        ' ----- The line color.
        Get
            Return StoredRectColor
        End Get
        Set(ByVal Value As Drawing.Color)
            StoredRectColor = Value
        End Set
    End Property

    <Browsable(True), DescriptionAttribute("Sets the fill color of the rectangle.")> _
    Public Property FillColor() As Drawing.Color
        ' ----- The fill color.
        Get
            Return StoredFillColor
        End Get
        Set(ByVal Value As Drawing.Color)
            StoredFillColor = Value
        End Set
    End Property

    <Browsable(True), _
    DescriptionAttribute("Sets the rotation angle, from 0 to 359 degrees.")> _
    Public Property RotationAngle() As Short
        ' ----- Rotation angle of the rectangle.
        Get
            Return StoredRectAngle
        End Get
        Set(ByVal Value As Short)
            ' ----- Make sure the angle is in the valid range.
            If (Value < 0) Or (Value > 359) Then
                Throw New System.ArgumentException( _
                    "Invalid rotation angle: " & Value & _
                    ". Rotation angle must range from 0 to 359 degrees.")
                Return
            End If
            StoredRectAngle = Value
        End Set
    End Property
End Class
