' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class CheckLookup
    Private UseLC As Boolean
    Private SecondItemRow As Integer
    Private CheckInMode As Boolean

    Private Class CheckItemDetail
        ' ----- Used instead of ListItemData, with extra fields.
        Public TitleText As String = ""
        Public AuthorName As String = ""
        Public MultiAuthor As Boolean = False
        Public MediaType As String = ""
        Public CallNumber As String = "-----"
        Public Barcode As String = "-----"
        Public CopyNumber As Integer = -1
        Public ItemCopyID As Integer = -1
    End Class

    Public Function CheckItemByTitle(ByVal CheckIn As Boolean, _
            ByVal searchText As String) As Integer
        ' ----- Prompt the user for a title item. Return the item copy
        '       ID on success, or -1 if no item is selected.
        Dim itemDetail As CheckItemDetail

        ' ----- Determine if Dewey or LC is used, and other initialization.
        UseLC = CBool(Val(GetSystemValue("UseLC")))

        ' ----- Perform the first search.
        If (PerformLookup(CheckIn, searchText) = False) Then
            MsgBox("Your search resulted in no matches.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
            Return -1
        End If
        CheckInMode = CheckIn

        ' ----- Show the results to the user.
        If (Me.ShowDialog() = Windows.Forms.DialogResult.Cancel) Then Return -1

        ' ----- Extract the copy ID number and return it.
        itemDetail = CType(MatchingItems.SelectedItem, CheckItemDetail)
        Return itemDetail.ItemCopyID
    End Function

    Private Function PerformLookup(ByVal asCheckIn As Boolean, _
        ByVal searchText As String) As Boolean
        ' ----- Perform a check-in or check-out title search. Return True
        '       if matches were found.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim matches As Integer
        Dim itemID As Integer
        Dim matchItem As CheckItemDetail = Nothing

        ' ----- Prepare the title.
        sqlText = Trim(searchText)
        If (InStr(sqlText, "*") = 0) Then sqlText &= "*"
        sqlText = Replace(sqlText, "*", "%")

        ' ----- Perform the search.
        sqlText = "SELECT AU.LastName, AU.FirstName, AU.MiddleName, AU.Suffix, " & _
            "AU.BirthYear, AU.DeathYear, NI.Title, NI.SubTitle, NI.Dewey, " & _
            "NI.LC, CMT.FullName AS MediaName, IC.ID, IC.Barcode, IC.CopyNumber " & _
            "FROM ItemCopy AS IC " & _
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " & _
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " & _
            "LEFT JOIN ItemAuthor AS IA ON NI.ID = IA.ItemID " & _
            "LEFT JOIN Author AS AU ON IA.AuthorID = AU.ID " & _
            "WHERE NI.Title LIKE " & DBText(sqlText)
        If (asCheckIn) Then sqlText &= " AND IC.ID IN" _
            Else sqlText &= " AND IC.ID NOT IN"
        sqlText &= " (SELECT ItemCopy FROM PatronCopy WHERE Returned = 0)"

        dbInfo = CreateReader(sqlText)
        If (dbInfo.HasRows = False) Then
            ' ----- No matches.
            dbInfo.Close()
            dbInfo = Nothing
            Return False
        End If

        ' ----- Prepare for item matches.
        MatchingItems.Items.Clear()
        matches = 0
        itemID = -1
        Do While dbInfo.Read
            If (itemID <> CInt(dbInfo!ID)) Then
                ' ----- Create a new matching item entry.
                matchItem = New CheckItemDetail
                matchItem.TitleText = CStr(dbInfo!Title)
                If (IsDBNull(dbInfo!Subtitle) = False) Then _
                    matchItem.TitleText &= ": " & CStr(dbInfo!Subtitle)
                matchItem.AuthorName = FormatAuthorName(dbInfo)
                matchItem.MediaType = CStr(dbInfo!MediaName)
                If (UseLC = True) Then
                    If (IsDBNull(dbInfo!LC) = False) Then _
                        matchItem.CallNumber = GetSubStr(CStr(dbInfo!LC), "/", 1)
                Else
                    If (IsDBNull(dbInfo!Dewey) = False) Then _
                        matchItem.CallNumber = GetSubStr(CStr(dbInfo!Dewey), "/", 1)
                End If
                If (IsDBNull(dbInfo!Barcode) = False) Then _
                    matchItem.Barcode = CStr(dbInfo!Barcode)
                If (IsDBNull(dbInfo!CopyNumber) = False) Then _
                    matchItem.CopyNumber = CInt(dbInfo!CopyNumber)
                matchItem.ItemCopyID = CInt(dbInfo!ID)

                ' ----- Ready to display the item.
                MatchingItems.Items.Add(matchItem)

                ' ----- Limit the search to SearchMatchLimit matches.
                matches += 1
                If (matches >= SearchMatchLimit) Then
                    If (dbInfo.Read) Then matches += 1
                    Exit Do
                End If
            Else
                ' ----- We found another match for the previous item, caused
                '       by the presence of another author record. Modify the
                '       last item by adding "et. al." to the display.
                If (matchItem.MultiAuthor = False) Then
                    matchItem.AuthorName &= ", et. al."
                    matchItem.MultiAuthor = True
                End If
            End If
        Loop
        dbInfo.Close()

        ' ----- Display the results.
        If (matches = 1) Then
            LabelItemsCount.Text = "1 matching item found."
        ElseIf (matches > SearchMatchLimit) Then
            LabelItemsCount.Text = "Limited to first " & SearchMatchLimit & _
                " matching items."
        Else
            LabelItemsCount.Text = matches & " matching items found."
        End If
        dbInfo = Nothing
        Return True

ErrorHandler:
        GeneralError("ItemLookup.PerformLookup", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return False
    End Function

    Private Sub MatchingItems_DrawItem(ByVal sender As Object, ByVal e As System.Windows.Forms.DrawItemEventArgs) Handles MatchingItems.DrawItem
        ' ----- Draw the matching items on two lines.
        Dim useBrush As System.Drawing.Brush
        Dim copyBarcodeText As String
        Dim boldFont As System.Drawing.Font
        Dim itemDetail As CheckItemDetail
        Dim ellipsesText As StringFormat

        ' ----- Draw the background of the item.
        If (CBool(CInt(e.State) And CInt(DrawItemState.Selected))) _
            Then useBrush = SystemBrushes.HighlightText _
            Else useBrush = SystemBrushes.WindowText
        e.DrawBackground()

        ' ----- Make a bold version of the font.
        boldFont = New System.Drawing.Font(e.Font, FontStyle.Bold)

        ' ----- Extract the details from the list.
        itemDetail = CType(MatchingItems.Items(e.Index), CheckItemDetail)

        ' ----- Format the display strings.
        If (itemDetail.CopyNumber = -1) Then copyBarcodeText = "??" Else _
            copyBarcodeText = CStr(itemDetail.CopyNumber)
        copyBarcodeText &= " / " & itemDetail.Barcode
        ellipsesText = New StringFormat
        ellipsesText.Trimming = StringTrimming.EllipsisCharacter

        ' ----- Draw the text of the item.
        e.Graphics.DrawString(itemDetail.TitleText, boldFont, useBrush, _
            New Rectangle(0, e.Bounds.Top, ItemColBarcode.Left - _
            MatchingItems.Left - 5, boldFont.Height), ellipsesText)
        e.Graphics.DrawString(copyBarcodeText, e.Font, useBrush, _
            New Rectangle(ItemColBarcode.Left - ItemColTitle.Left, e.Bounds.Top, _
            ItemColEnd.Left - ItemColBarcode.Left, e.Font.Height), ellipsesText)
        e.Graphics.DrawString(itemDetail.AuthorName, e.Font, useBrush, _
            New Rectangle(ItemColAuthor.Left - ItemColTitle.Left, _
            e.Bounds.Top + SecondItemRow, ItemColCall.Left - ItemColAuthor.Left - 8, _
            e.Font.Height), ellipsesText)
        e.Graphics.DrawString(itemDetail.CallNumber, e.Font, useBrush, _
            New Rectangle(ItemColCall.Left - ItemColTitle.Left, _
            e.Bounds.Top + SecondItemRow, ItemColEnd.Left - ItemColType.Left - 5, _
            e.Font.Height), ellipsesText)
        e.Graphics.DrawString(itemDetail.MediaType, e.Font, useBrush, _
            New Rectangle(ItemColType.Left - ItemColTitle.Left, _
            e.Bounds.Top + SecondItemRow, ItemColType.Left - ItemColCall.Left - 8, _
            e.Font.Height), ellipsesText)

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
        boldFont.Dispose()
    End Sub

    Private Sub CheckLookup_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim formGraphics As System.Drawing.Graphics

        ' ----- Set the default height of items in the matching items listbox.
        formGraphics = Me.CreateGraphics()
        MatchingItems.ItemHeight = _
            CInt(formGraphics.MeasureString("A" & vbCrLf & "g", _
            MatchingItems.Font).Height()) + 3
        SecondItemRow = CInt(formGraphics.MeasureString("Ag", _
            MatchingItems.Font).Height()) + 1
        formGraphics.Dispose()

        ' ----- Adjust the form title.
        If (CheckInMode = True) Then
            Me.Text = "Check-In By Title"
        Else
            Me.Text = "Check-Out By Title"
        End If
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- Make sure a list item is selected.
        If (MatchingItems.SelectedIndex = -1) Then
            MsgBox("Please select an item from the list.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If

        ' ----- Continue with CheckItemByTitle()
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub MatchingItems_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles MatchingItems.DoubleClick
        ' ----- Same as a click on the OK button.
        ActOK.PerformClick()
    End Sub
End Class