' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class GroupName
    Private ActiveID As Integer
    Private StartingOver As Boolean

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageGroups)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageGroups)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageGroups)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the record.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the security group?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM UserName WHERE GroupID = " & recordID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("You cannot delete this record because it is in use.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Delete the record.
        TransactionBegin()
        sqlText = "DELETE FROM GroupActivity WHERE GroupID = " & recordID
        ExecuteSQL(sqlText)
        sqlText = "DELETE FROM GroupName WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        TransactionCommit()
        Return True

ErrorHandler:
        GeneralError("GroupName.DeleteRecord", Err.GetException())
        TransactionRollback()
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT * FROM GroupName ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("GroupName.FillListWithRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM GroupName WHERE ID = " & recordID
        Return CStr(ExecuteSQLReturn(sqlText))

ErrorHandler:
        GeneralError("GroupName.FormatRecordName", Err.GetException())
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use these codes to establish the security groups into which " & _
            "administrative users are placed. Each security group can receive " & _
            "access to different parts of the system."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Security Groups"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim matchPos As Integer

        On Error GoTo ErrorHandler

        ' ----- Load in the list of security groups.
        If (ActivityList.Items.Count = 0) Then
            ' ----- This is the first time. Load from the database.
            ActivityList.Items.Clear()
            sqlText = "SELECT * FROM Activity ORDER BY FullName"
            dbInfo = CreateReader(sqlText)
            Do While (dbInfo.Read)
                ActivityList.Items.Add _
                    (New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
            Loop
            dbInfo.Close()
            dbInfo = Nothing
        Else
            ' ----- Editing a subsequent code. Clear selections. Amazingly,
            '       there's no Clear method to do this.
            Do While (ActivityList.CheckedItems.Count > 0)
                ActivityList.SetItemChecked(ActivityList.CheckedIndices(0), False)
            Loop
            If (ActivityList.Items.Count > 0) Then ActivityList.TopIndex = 0
        End If

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT FullName FROM GroupName WHERE ID = " & ActiveID
        RecordFullName.Text = CStr(ExecuteSQLReturn(sqlText))

        ' ----- Load in the existing security settings.
        sqlText = "SELECT ActivityID FROM GroupActivity WHERE GroupID = " & ActiveID
        dbInfo = CreateReader(sqlText)
        Do While (dbInfo.Read)
            matchPos = ActivityList.Items.IndexOf(CInt(dbInfo!ActivityID))
            If (matchPos <> -1) Then ActivityList.SetItemChecked(matchPos, True)
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("GroupName.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- The user wants to save changes. Return True on success.
        Dim sqlText As String
        Dim newID As Integer = -1

        On Error GoTo ErrorHandler

        ' ----- Prepare to save the data.
        Me.Cursor = Windows.Forms.Cursors.WaitCursor
        TransactionBegin()

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO GroupName (FullName) OUTPUT INSERTED.ID VALUES (" & _
                DBText(Trim(RecordFullName.Text)) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
        Else
            ' ----- Update the existing entry.
            newID = ActiveID
            sqlText = "UPDATE GroupName SET FullName = " & _
                DBText(Trim(RecordFullName.Text)) & " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- Clear any existing security settings.
        sqlText = "DELETE FROM GroupActivity WHERE GroupID = " & newID
        ExecuteSQL(sqlText)

        ' ----- Save the selected security settings.
        For Each itemChecked As ListItemData In ActivityList.CheckedItems
            sqlText = "INSERT INTO GroupActivity (GroupID, ActivityID) VALUES (" & _
                newID & ", " & itemChecked.ItemData & ")"
            ExecuteSQL(sqlText)
        Next itemChecked

        ' ----- Complete all changes.
        TransactionCommit()
        ActiveID = newID

        ' ----- This change may affect this user.
        If (LoggedInGroupID = ActiveID) Then ReprocessSecuritySet()

        ' ----- Success.
        ActiveID = newID
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("GroupName.SaveFormData", Err.GetException())
        TransactionRollback()
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Name is a required field.
        If (Trim(RecordFullName.Text) = "") Then
            MsgBox("Please supply the security group name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM GroupName WHERE UPPER(FullName) = " & _
            DBText(UCase(Trim(RecordFullName.Text)))
        If (ActiveID <> -1) Then sqlText &= " AND ID <> " & ActiveID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("A record already exists with the name '" & Trim(RecordFullName.Text) & _
                    "'.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("GroupName.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub GroupName_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordFullName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordFullName.Enter
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub ActOK_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- Save changes and return to the summary form.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub
End Class
