' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class PatronRecord
    Private ActivePatronID As Integer
    Private SecondItemRow As Integer
    Private AllowPatronEdit As Boolean
    Private SessionPayments As Generic.List(Of PaymentItem)

    Public Sub ViewPatronRecord(ByVal patronID As Integer, ByVal allowEdit As Boolean)
        ' ----- Provide access to the patron record.
        ActivePatronID = patronID
        AllowPatronEdit = allowEdit
        Me.ShowDialog()
    End Sub

    Private Sub RefreshPatronFines(ByVal forcePatronCopy As Integer)
        ' ----- Refresh the list of fines due by and paid by the patron.
        '       Always include the forcePatronCopy item, unless it is -1.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim fineAmount As Decimal
        Dim paidAmount As Decimal
        Dim balanceAmount As Decimal
        Dim totalBalance As Decimal
        Dim boldFont As System.Drawing.Font
        Dim titleText As String
        Dim copyText As String
        Dim newEntry As PatronDetailItem
        Dim canvas As Drawing.Graphics

        On Error GoTo ErrorHandler

        ' ----- Create graphic objects used for measuring.
        boldFont = New System.Drawing.Font(Fines.Font, FontStyle.Bold)
        canvas = Fines.CreateGraphics()

        ' ----- Clear the current list.
        Fines.Items.Clear()
        totalBalance = 0@
        BalanceDue.Text = Format(0@, "Currency")
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Retrieve the fine records for the patron.
        sqlText = "SELECT PC.ID, NI.Title, NI.Subtitle, PC.Fine, PC.Paid, " & _
            "IC.CopyNumber FROM PatronCopy AS PC " & _
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " & _
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " & _
            "WHERE (PC.Patron = " & ActivePatronID & _
            " AND (PC.Fine <> 0 OR PC.Paid <> 0))"
        If (forcePatronCopy <> -1) Then _
            sqlText &= " OR (PC.ID = " & forcePatronCopy & ")"
        sqlText &= " ORDER BY NI.Title"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Collect the values.
            fineAmount = DBGetDecimal(dbInfo!Fine)
            paidAmount = DBGetDecimal(dbInfo!Paid)
            balanceAmount = fineAmount - paidAmount
            If (CInt(dbInfo!ID) <> forcePatronCopy) Then
                If (balanceAmount = 0@) And (ShowPaidFines.Checked = False) Then _
                    Continue Do
            End If
            totalBalance += balanceAmount

            ' ----- Format the display items.
            copyText = " [Copy #" & CStr(dbInfo!CopyNumber) & "]"
            titleText = CStr(dbInfo!Title)
            If Not (TypeOf (dbInfo!Subtitle) Is DBNull) Then _
                titleText &= ": " & CStr(dbInfo!Subtitle)
            titleText = FitTextToWidth(titleText, _
                CInt(FineColEnd.Left - FineColTitle.Left - _
                canvas.MeasureString(copyText, boldFont).Width()), _
                canvas, boldFont)

            ' ----- Prepare a new list item.
            newEntry = New PatronDetailItem
            newEntry.DetailID = CInt(dbInfo!ID)
            newEntry.TitleText = titleText & copyText
            newEntry.FineAmount = fineAmount
            newEntry.PaidAmount = paidAmount
            newEntry.BalanceDue = balanceAmount
            Fines.Items.Add(newEntry)
        Loop
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- Show the total balance.
        BalanceDue.Text = Format(totalBalance, "Currency")
        ActFinePay.Enabled = False

        ' ----- Dispense with the graphics elements.
        canvas.Dispose()
        boldFont.Dispose()
        Me.Cursor = Windows.Forms.Cursors.Default
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("PatronRecord.RefreshPatronFines", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        canvas.Dispose()
        boldFont.Dispose()
        Return
    End Sub

    Private Sub ActRenewItemsOut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActRenewItemsOut.Click
        ' ----- Try to renew a checked-out item.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim checkOutDate As Date
        Dim dueDate As Date
        Dim renewsSoFar As Integer
        Dim maxRenew As Integer
        Dim renewDays As Integer
        Dim itemDetail As PatronDetailItem

        On Error GoTo ErrorHandler

        ' ----- Make sure an item is selected.
        If (ItemsOut.SelectedIndex = -1) Then Return
        itemDetail = CType(ItemsOut.SelectedItem, PatronDetailItem)

        ' ----- Make sure this is not a reference item.
        sqlText = "SELECT IC.Available, IC.Reference FROM PatronCopy AS PC " & _
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " & _
            "WHERE PC.ID = " & itemDetail.DetailID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        If (CBool(dbInfo!Available) = False) Then
            dbInfo.Close()
            dbInfo = Nothing
            MsgBox("You cannot renew this item because it is no longer available " & _
                "for circulation or checkout.", MsgBoxStyle.OkOnly Or _
                MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If
        If (CBool(dbInfo!Reference) = True) Then
            dbInfo.Close()
            dbInfo = Nothing
            MsgBox("You cannot renew this item because it is a reference item.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If
        dbInfo.Close()

        ' ----- Get the status of the checked out item.
        sqlText = "SELECT CheckOut, DueDate, Renewal FROM PatronCopy WHERE ID = " & _
            itemDetail.DetailID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        checkOutDate = CDate(dbInfo!CheckOut)
        dueDate = CDate(dbInfo!DueDate)
        renewsSoFar = CInt(dbInfo!Renewal)
        dbInfo.Close()

        ' ----- Determine the limits on this type of item.
        sqlText = "SELECT * FROM CodeMediaType WHERE ID = (SELECT NI.MediaType " & _
            "FROM PatronCopy AS PC INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " & _
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " & _
            "WHERE PC.ID = " & itemDetail.DetailID & ")"
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        maxRenew = CInt(dbInfo!RenewTimes)
        renewDays = CInt(dbInfo!RenewDays)
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- See if this item has been renewed too often.
        If (renewsSoFar >= maxRenew) Then
            If (maxRenew = 0) Then
                MsgBox("You cannot renew this item because this type of item does " & _
                    "not permit renewals.", MsgBoxStyle.OkOnly Or _
                    MsgBoxStyle.Exclamation, ProgramTitle)
            Else
                MsgBox("You cannot renew this item because you have already renewed it " & _
                    CStr(IIf(renewsSoFar = 1, "1 time", renewsSoFar & " times")) & _
                    ", which is the limit for this type of item.", MsgBoxStyle.OkOnly Or _
                    MsgBoxStyle.Exclamation, ProgramTitle)
            End If
            Return
        End If
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Renew this item by adjusting the due date. While there is room,
        '       keep renewing the item until the item is not overdue, if needed.
        Do While True
            dueDate = DateAdd(DateInterval.Day, renewDays, dueDate)
            renewsSoFar += 1
            If (renewsSoFar >= maxRenew) Then Exit Do
            If (dueDate >= Today) Then Exit Do
        Loop

        ' ----- Don't allow the due date to be on a holiday.
        Do While (IsHolidayDate(dueDate) = True)
            dueDate = DateAdd("d", 1, dueDate)
        Loop

        TransactionBegin()

        ' ----- Update the record.
        sqlText = "UPDATE PatronCopy SET DueDate = " & DBDate(dueDate) & _
            ", Renewal = " & renewsSoFar & " WHERE ID = " & itemDetail.DetailID
        ExecuteSQL(sqlText)

        ' ----- Update the patron record.
        sqlText = "UPDATE Patron SET LastActivity = Now WHERE ID = " & ActivePatronID
        ExecuteSQL(sqlText)

        TransactionCommit()

        ' ----- Update the display.
        itemDetail.DueDate = dueDate
        ItemsOut.Invalidate(ItemsOut.GetItemRectangle(ItemsOut.SelectedIndex))
        Me.Cursor = Windows.Forms.Cursors.Default
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("PatronRecord.ActRenewItemsOut_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        TransactionRollback()
        Return
    End Sub

    Private Sub PatronRecord_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim holdText As String
        Dim counter As Integer
        Dim itemID As Integer
        Dim totalCount As Integer
        Dim totalAhead As Integer
        Dim canvas As Drawing.Graphics
        Dim newEntry As PatronDetailItem

        On Error GoTo ErrorHandler

        ' ----- Adjust the line item height of the fines list.
        canvas = Fines.CreateGraphics()
        Fines.ItemHeight = CInt(canvas.MeasureString("A" & vbCrLf & "g", _
            Fines.Font).Height()) + 3
        SecondItemRow = CInt(canvas.MeasureString("Ag", Fines.Font).Height()) + 1
        canvas.Dispose()

        ' ----- Load in the patron name and barcode.
        sqlText = "SELECT LastName, FirstName, Active, Barcode FROM Patron " & _
            "WHERE ID = " & ActivePatronID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        PatronName.Text = CStr(dbInfo!FirstName) & " " & CStr(dbInfo!LastName)
        If (CBool(dbInfo!Active) = False) Then PatronName.Text &= " [Inactive]"
        If (IsDBNull(dbInfo!Barcode) = True) Then PatronBarcode.Text = "None" Else _
            PatronBarcode.Text = CStr(dbInfo!Barcode)
        dbInfo.Close()

        ' ----- Load in the list of items checked out.
        sqlText = "SELECT PC.ID, NI.Title, NI.Subtitle, PC.DueDate " & _
            "FROM PatronCopy AS PC INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " & _
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " & _
            "WHERE PC.Patron = " & ActivePatronID & _
            " AND PC.Returned = 0 AND PC.Missing = 0 ORDER BY PC.DueDate"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add this checked-out or overdue book to the list.
            holdText = CStr(dbInfo!Title)
            If (IsDBNull(dbInfo!Subtitle) = False) Then _
                holdText &= ": " & CStr(dbInfo!Subtitle)

            newEntry = New PatronDetailItem
            newEntry.DetailID = CInt(dbInfo!ID)
            newEntry.TitleText = holdText
            newEntry.DueDate = CDate(dbInfo!DueDate)
            ItemsOut.Items.Add(newEntry)
        Loop
        dbInfo.Close()

        ' ----- Load in the list of patron fines.
        RefreshPatronFines(-1)

        ' ----- Prepare a list of payments for printing.
        SessionPayments = New Generic.List(Of PaymentItem)

        ' ----- If the user has authorization, show the Edit Patron button.
        ActPasswordPatron.Location = New System.Drawing.Point( _
            ActEditPatron.Location.X, ActEditPatron.Location.Y)
        If (SecurityProfile(LibrarySecurity.ManagePatrons) = True) Then
            ' ----- If this form was called ultimately from the edit patron form,
            '       then hide the Edit Patron button.
            ActEditPatron.Visible = AllowPatronEdit
            ActPasswordPatron.Visible = Not AllowPatronEdit
        Else
            ' ----- This user is not authorized to edit patron records.
            ActEditPatron.Visible = False
            ActPasswordPatron.Visible = True
        End If

        ' ----- Only allow authorized users to adjust fines.
        ActFinePay.Visible = SecurityProfile(LibrarySecurity.ProcessFees)
        Return

ErrorHandler:
        GeneralError("PatronRecord.PatronRecord_Load", Err.GetException())
        Resume Next
    End Sub

    Private Sub ActEditPatron_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActEditPatron.Click
        ' ----- Let the user edit the patron record.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Display the patron form.
        If ((New Patron).EditRecordLimited(ActivePatronID) <> -1) Then
            ' ----- Update the patron's name and barcode number.
            sqlText = "SELECT LastName, FirstName, Active, Barcode FROM Patron " & _
                "WHERE ID = " & ActivePatronID
            dbInfo = CreateReader(sqlText)
            dbInfo.Read()
            PatronName.Text = CStr(dbInfo!FirstName) & " " & CStr(dbInfo!LastName)
            If (CBool(dbInfo!Active) = False) Then PatronName.Text &= " [Inactive]"
            If (IsDBNull(dbInfo!Barcode) = True) Then _
                PatronBarcode.Text = "None" Else _
                PatronBarcode.Text = CStr(dbInfo!Barcode)
            dbInfo.Close()
        End If
        Return

ErrorHandler:
        GeneralError("PatronRecord.ActEditPatron_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub ItemsOut_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ItemsOut.SelectedIndexChanged
        ' ----- Enable the Renew button if needed.
        ActRenewItemsOut.Enabled = CBool(ItemsOut.SelectedIndex <> -1)
    End Sub

    Private Sub ItemsOut_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ItemsOut.Click
        ' ----- Enable the Renew button if needed.
        ActRenewItemsOut.Enabled = CBool(ItemsOut.SelectedIndex <> -1)
    End Sub

    Private Sub ItemsOut_DrawItem(ByVal sender As Object, ByVal e As System.Windows.Forms.DrawItemEventArgs) Handles ItemsOut.DrawItem
        ' ----- Specially format the checked out item.
        Dim useBrush As System.Drawing.Brush
        Dim dueDate As String
        Dim itemDetail As PatronDetailItem
        Dim ellipsesText As StringFormat

        ' ----- Draw the background of the item.
        e.DrawBackground()

        ' ----- Extract the detail from the list item.
        itemDetail = CType(ItemsOut.Items(e.Index), PatronDetailItem)
        dueDate = Format(itemDetail.DueDate, "MMM d, yyyy")

        ' ----- Print the title in the right color.
        If ((e.State And DrawItemState.Selected) = DrawItemState.Selected) Then
            useBrush = SystemBrushes.HighlightText
        Else
            useBrush = SystemBrushes.WindowText
        End If
        ellipsesText = New StringFormat
        ellipsesText.Trimming = StringTrimming.EllipsisCharacter
        e.Graphics.DrawString(itemDetail.TitleText, e.Font, useBrush, _
            New Rectangle(0, e.Bounds.Top, _
            ItemsOutColDue.Left - ItemsOutColTitle.Left - 5, e.Font.Height), _
            ellipsesText)

        ' ----- Print the due date in the right color.
        If ((e.State And DrawItemState.Selected) = DrawItemState.Selected) Then
            useBrush = SystemBrushes.HighlightText
        ElseIf (CDate(dueDate) < Today) Then
            ' ----- Overdue.
            useBrush = Brushes.Red
        Else
            useBrush = SystemBrushes.WindowText
        End If
        e.Graphics.DrawString(dueDate, e.Font, useBrush, _
            ItemsOutColDue.Left - ItemsOutColTitle.Left, e.Bounds.Top)

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
    End Sub

    Private Sub ShowPaidFines_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ShowPaidFines.CheckedChanged
        ' ----- Refresh the list of fines based on the new setting.
        RefreshPatronFines(-1)
    End Sub

    Private Sub Fines_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Fines.SelectedIndexChanged
        ' ----- Enable the buttons if needed.
        ActFinePay.Enabled = CBool(Fines.SelectedIndex <> -1)
    End Sub

    Private Sub Fines_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Fines.Click
        ' ----- Enable the buttons if needed.
        ActFinePay.Enabled = CBool(Fines.SelectedIndex <> -1)
    End Sub

    Private Sub Fines_DrawItem(ByVal sender As Object, ByVal e As System.Windows.Forms.DrawItemEventArgs) Handles Fines.DrawItem
        ' ----- Draw the matching items on two lines.
        Dim useBrush As Drawing.Brush
        Dim useNotice As Drawing.Brush
        Dim titleText As String
        Dim fineText As String
        Dim paidText As String
        Dim balanceText As String
        Dim boldFont As Drawing.Font
        Dim itemDetail As PatronDetailItem

        ' ----- Draw the background of the item.
        If ((e.State And DrawItemState.Selected) = DrawItemState.Selected) Then
            useBrush = SystemBrushes.HighlightText
            useNotice = SystemBrushes.HighlightText
        Else
            useBrush = SystemBrushes.WindowText
            useNotice = Brushes.Red
        End If
        e.DrawBackground()

        ' ----- Make a bold version of the font.
        boldFont = New System.Drawing.Font(e.Font, FontStyle.Bold)

        ' ----- Extract the details from the list item.
        itemDetail = CType(Fines.Items(e.Index), PatronDetailItem)
        titleText = itemDetail.TitleText
        fineText = Format(itemDetail.FineAmount, "Currency")
        paidText = Format(itemDetail.PaidAmount, "Currency")
        balanceText = Format(itemDetail.BalanceDue, "Currency")
        If (itemDetail.BalanceDue = 0@) Then useNotice = useBrush

        ' ----- Draw the text of the item.
        e.Graphics.DrawString(titleText, boldFont, useBrush, 0, e.Bounds.Top)
        e.Graphics.DrawString(fineText, e.Font, useBrush, FineColFine.Left - _
            FineColTitle.Left, e.Bounds.Top + SecondItemRow)
        e.Graphics.DrawString(paidText, e.Font, useBrush, FineColPaid.Left - _
            FineColTitle.Left, e.Bounds.Top + SecondItemRow)
        e.Graphics.DrawString(balanceText, e.Font, useNotice, FineColBalance.Left - _
            FineColTitle.Left, e.Bounds.Top + SecondItemRow)

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
        boldFont.Dispose()
    End Sub

    Private Sub ActFinePay_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActFinePay.Click
        ' ----- Prompt the user to adjust the fines on an item.
        Dim patronCopyID As Integer

        ' ----- An item must be selected.
        If (Fines.SelectedIndex = -1) Then
            MsgBox("Please select a fine from the list.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If
        patronCopyID = CType(Fines.SelectedItem, PatronDetailItem).DetailID

        ' ----- Prompt the user to make changes.
        Call (New PatronPayment).ManagePayments(patronCopyID, SessionPayments)

        ' ----- Refresh the fines list.
        ActFinePay.Enabled = False
        RefreshPatronFines(patronCopyID)
        Fines.SelectedIndex = Fines.Items.IndexOf(patronCopyID)
    End Sub

    Private Sub ActPasswordPatron_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActPasswordPatron.Click
        ' ----- Let the user change their password.
        If ((New PatronPassword).ChangePassword(ActivePatronID) = True) Then
            MsgBox("Password changed successfully.", MsgBoxStyle.OkOnly Or _
                MsgBoxStyle.Information, ProgramTitle)
        End If
    End Sub

    Private Sub ActClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActClose.Click
        ' ----- Print out a ticket if needed.
        If (SessionPayments.Count > 0) Then _
           PrintPaymentTicket(ActivePatronID, SessionPayments)
        SessionPayments.Clear()
        SessionPayments = Nothing

        ' ----- Close the form.
        Me.DialogResult = Windows.Forms.DialogResult.Cancel
    End Sub

    Private Sub ActBalanceTicket_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActBalanceTicket.Click
        ' ----- Print a ticket of all balances.
        PrintBalanceTicket(ActivePatronID, Fines)
    End Sub
End Class
