' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class BarcodeLabel
    Private ActiveLabelID As Integer
    Private CurrentMeasure As Char
    Private StartingOver As Boolean
    Private LabelWidth As Integer
    Private LabelHeight As Integer
    Private IgnoreSizeChange As Boolean
    Private BarcodeFontName As String
    Private Barcode39 As Boolean

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        BarcodeFontName = GetBarcodeFont()
        If (BarcodeFontName = "") Then Return -1
        ActiveLabelID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveLabelID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the record.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete this barcode label template?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Delete the record.
        TransactionBegin()
        sqlText = "DELETE FROM BarcodeLabelItem WHERE BarcodeLabel = " & recordID
        ExecuteSQL(sqlText)
        sqlText = "DELETE FROM BarcodeLabel WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        TransactionCommit()
        Return True

ErrorHandler:
        GeneralError("BarcodeLabel.DeleteRecord", Err.GetException())
        TransactionRollback()
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        BarcodeFontName = GetBarcodeFont()
        If (BarcodeFontName = "") Then Return -1
        ActiveLabelID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveLabelID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT BL.ID, BL.FullName, BS.FullName AS PageName " & _
            "FROM BarcodeLabel AS BL INNER JOIN BarcodeSheet AS BS " & _
            "ON BL.BarcodeSheet = BS.ID ORDER BY BL.FullName, BS.FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(CStr(dbInfo!FullName) & " on " & _
                CStr(dbInfo!PageName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("BarcodeLabel.FillListWithRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close()
        dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim result As String

        On Error GoTo ErrorHandler
        result = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT BL.FullName, BS.FullName AS PageName " & _
            "FROM BarcodeLabel AS BL INNER JOIN BarcodeSheet AS BS " & _
            "ON BL.BarcodeSheet = BS.ID WHERE BL.ID = " & recordID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then result = CStr(dbInfo!FullName) & " on " & _
            CStr(dbInfo!PageName)
        dbInfo.Close()
        dbInfo = Nothing
        Return result

ErrorHandler:
        GeneralError("BarcodeLabel.FormatRecordName", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close()
        dbInfo = Nothing
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return a description of this editor.
        Return "Use these codes to identify the format of printed barcode labels. " & _
            "You can define the layout of a label using text, barcodes, and shapes."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the title-bar text for this editor.
        Return "Barcode Label Templates"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim newBarcodeImage As Library.BarcodeItemBarcodeImage
        Dim newBarcodeText As Library.BarcodeItemBarcodeNumber
        Dim newStaticText As Library.BarcodeItemText
        Dim newStaticLine As Library.BarcodeItemLine
        Dim newStaticRect As Library.BarcodeItemRect

        On Error GoTo ErrorHandler

        ' ----- Load in the list of templates.
        If (RecordTemplate.Items.Count = 0) Then
            RecordTemplate.Items.Add(New ListItemData("<Not Selected>", -1))
            sqlText = "SELECT ID, FullName FROM BarcodeSheet ORDER BY FullName"
            dbInfo = CreateReader(sqlText)
            Do While dbInfo.Read
                RecordTemplate.Items.Add(New ListItemData(CStr(dbInfo!FullName), _
                    CInt(dbInfo!ID)))
            Loop
            dbInfo.Close()
            dbInfo = Nothing
        End If

        ' ----- Add in the measures.
        dbInfo = Nothing
        ' ----- Fill in the list of units.
        If (RecordUnits.Items.Count = 0) Then
            RecordUnits.Items.Add(New ListItemData("Inches", Asc("I")))
            RecordUnits.Items.Add(New ListItemData("Centimeters", Asc("C")))
            RecordUnits.Items.Add(New ListItemData("Points", Asc("P")))
            RecordUnits.Items.Add(New ListItemData("Twips", Asc("T")))
            RecordUnits.SelectedIndex = 0
        End If
        CurrentMeasure = "I"c

        ' ----- Clear the existing data.
        StartingOver = True
        IgnoreSizeChange = True
        LabelWidth = -1
        LabelHeight = -1
        RecordName.Text = ""
        RecordTemplate.SelectedIndex = RecordTemplate.Items.IndexOf(-1)
        If (RecordTemplate.SelectedIndex = -1) Then RecordTemplate.SelectedIndex = 0
        RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(Asc(CurrentMeasure))
        If (RecordUnits.SelectedIndex = -1) Then RecordUnits.SelectedIndex = 0
        DisplayItems.Items.Clear()
        ActUp.Enabled = False
        ActDown.Enabled = False
        ActDelete.Enabled = False
        ItemProperties.SelectedObject = Nothing
        RecordSample.Text = "12345678"
        PreviewArea.Invalidate()
        IgnoreSizeChange = False
        Barcode39 = CBool(Val(GetSystemValue("BarcodeCode39")))
        If (ActiveLabelID = -1) Then Return

        ' ----- Load in the record.
        sqlText = "SELECT * FROM BarcodeLabel WHERE ID = " & ActiveLabelID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        IgnoreSizeChange = True
        RecordName.Text = CStr(dbInfo!FullName)
        RecordTemplate.SelectedIndex = _
            RecordTemplate.Items.IndexOf(CInt(dbInfo!BarcodeSheet))
        If (RecordUnits.SelectedIndex = -1) Then RecordUnits.Items.IndexOf(-1)
        RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(Asc(CChar(dbInfo!UnitType)))
        If (RecordUnits.SelectedIndex = -1) Then RecordUnits.SelectedIndex = 0
        CurrentMeasure = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        IgnoreSizeChange = False
        dbInfo.Close()

        ' ----- Determine the size of the label.
        RefreshLabelTemplate()
        RefreshSizeDisplay()

        ' ----- Load in the list of objects.
        sqlText = "SELECT * FROM BarcodeLabelItem WHERE BarcodeLabel = " & _
            ActiveLabelID & " ORDER BY Priority"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read()
            ' ----- Process this object.
            Select Case UCase(CStr(dbInfo!ItemType))
                Case "B"
                    ' ----- Barcode image.
                    newBarcodeImage = New Library.BarcodeItemBarcodeImage
                    newBarcodeImage.Alignment = CType(CInt(dbInfo!Alignment), _
                        System.Drawing.ContentAlignment)
                    newBarcodeImage.BarcodeColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newBarcodeImage.BarcodeSize = CSng(dbInfo!FontSize)
                    newBarcodeImage.Left = CSng(dbInfo!PosLeft)
                    newBarcodeImage.Top = CSng(dbInfo!PosTop)
                    newBarcodeImage.Width = CSng(dbInfo!PosWidth)
                    newBarcodeImage.Height = CSng(dbInfo!PosHeight)
                    newBarcodeImage.RotationAngle = CShort(dbInfo!Rotation)
                    newBarcodeImage.PadDigits = CByte(DBGetInteger(dbInfo!PadDigits))
                    DisplayItems.Items.Add(newBarcodeImage)
                Case "L"
                    ' ----- Line.
                    newStaticLine = New Library.BarcodeItemLine
                    newStaticLine.LineColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newStaticLine.X1 = CSng(dbInfo!PosLeft)
                    newStaticLine.X2 = CSng(dbInfo!PosWidth)
                    newStaticLine.Y1 = CSng(dbInfo!PosTop)
                    newStaticLine.Y2 = CSng(dbInfo!PosHeight)
                    DisplayItems.Items.Add(newStaticLine)
                Case "N"
                    ' ----- Barcode text.
                    newBarcodeText = New Library.BarcodeItemBarcodeNumber
                    newBarcodeText.Alignment = CType(dbInfo!Alignment, _
                        System.Drawing.ContentAlignment)
                    newBarcodeText.Left = CSng(dbInfo!PosLeft)
                    newBarcodeText.Top = CSng(dbInfo!PosTop)
                    newBarcodeText.Width = CSng(dbInfo!PosWidth)
                    newBarcodeText.Height = CSng(dbInfo!PosHeight)
                    newBarcodeText.RotationAngle = CShort(dbInfo!Rotation)
                    newBarcodeText.TextColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newBarcodeText.Font = New System.Drawing.Font(CStr(dbInfo!FontName), _
                        CSng(dbInfo!FontSize), BuildFontStyle(DBGetText(dbInfo!FontStyle)))
                    newBarcodeText.PadDigits = CByte(DBGetInteger(dbInfo!PadDigits))
                    DisplayItems.Items.Add(newBarcodeText)
                Case "R"
                    ' ----- Rectangle.
                    newStaticRect = New Library.BarcodeItemRect
                    newStaticRect.BorderColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newStaticRect.FillColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color2))
                    newStaticRect.Left = CSng(dbInfo!PosLeft)
                    newStaticRect.Top = CSng(dbInfo!PosTop)
                    newStaticRect.Width = CSng(dbInfo!PosWidth)
                    newStaticRect.Height = CSng(dbInfo!PosHeight)
                    newStaticRect.RotationAngle = CShort(dbInfo!Rotation)
                    DisplayItems.Items.Add(newStaticRect)
                Case "T"
                    ' ----- Static text.
                    newStaticText = New Library.BarcodeItemText
                    newStaticText.DisplayText = CStr(dbInfo!StaticText)
                    newStaticText.Alignment = CType(dbInfo!Alignment, _
                        System.Drawing.ContentAlignment)
                    newStaticText.Left = CSng(dbInfo!PosLeft)
                    newStaticText.Top = CSng(dbInfo!PosTop)
                    newStaticText.Width = CSng(dbInfo!PosWidth)
                    newStaticText.Height = CSng(dbInfo!PosHeight)
                    newStaticText.RotationAngle = CShort(dbInfo!Rotation)
                    newStaticText.TextColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newStaticText.Font = New System.Drawing.Font(CStr(dbInfo!FontName), _
                        CSng(dbInfo!FontSize), BuildFontStyle(DBGetText(dbInfo!FontStyle)))
                    DisplayItems.Items.Add(newStaticText)
            End Select
        Loop
        dbInfo.Close()
        PreviewArea.Invalidate()
        Return

ErrorHandler:
        GeneralError("BarcodeLabel.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close()
        dbInfo = Nothing
        Return
    End Sub

    Private Sub RefreshSizeDisplay()
        ' ----- Show the size of the label.
        Dim toUnit As Char

        toUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        If (LabelWidth = -1) Or (LabelHeight = -1) Then
            LabelPreview.Text = "Label Preview"
        Else
            LabelPreview.Text = "Label Preview: " & _
                ConvertPageUnits("T"c, toUnit, LabelWidth) & " (w) by " & _
                ConvertPageUnits("T"c, toUnit, LabelHeight) & " (h)"
        End If
    End Sub

    Private Function ValidateForm() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Name is a required field.
        If (Trim(RecordName.Text) = "") Then
            MsgBox("Please supply the label template name.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordName.Focus()
            Return False
        End If

        ' ----- Make sure a template is selected.
        If (CInt(CType(RecordTemplate.SelectedItem, ListItemData)) = -1) Then
            MsgBox("Please select a page template.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordTemplate.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM BarcodeLabel WHERE UPPER(FullName) = " & _
            DBText(UCase(Trim(RecordName.Text))) & " AND BarcodeSheet = " & _
            CInt(CType(RecordTemplate.SelectedItem, ListItemData))
        If (ActiveLabelID <> -1) Then sqlText &= " AND ID <> " & ActiveLabelID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            ' ----- Duplicate entry.
            MsgBox("A label template using the page template '" & RecordTemplate.Text & _
                "' already exists with the name '" & Trim(RecordName.Text) & _
                "'.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordName.Focus()
            Return False
        End If

        ' ----- At least one item is required.
        If (DisplayItems.Items.Count = 0) Then
            MsgBox("The label template must include at least one item.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("BarcodeLabel.ValidateForm", Err.GetException())
        Return False
    End Function

    Private Sub RefreshLabelTemplate()
        ' ----- Given the active template, determine the width and height of a
        '       single label, in twips.
        Dim oneWidth As Single
        Dim oneHeight As Single
        Dim sqlText As String
        Dim useMeasure As Char
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- A width and height of -1 indicates an unknown size.
        LabelWidth = -1
        LabelHeight = -1
        If (RecordTemplate.SelectedIndex = -1) Then Return
        If (CType(RecordTemplate.SelectedItem, ListItemData).ItemData = -1) Then Return
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Retrieve the label details.
        sqlText = "SELECT * FROM BarcodeSheet WHERE ID = " & _
            CType(RecordTemplate.SelectedItem, ListItemData).ItemData
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()

        ' ----- The label size is what is left over on the page after the
        '       margins and the space between labels is removed.
        oneWidth = (CSng(dbInfo!PageWidth) - CSng(dbInfo!MarginLeft) - _
            CSng(dbInfo!MarginRight) - ((CSng(dbInfo!ColumnsCount) - 1.0F) * _
            CSng(dbInfo!IntraColumn))) / CSng(dbInfo!ColumnsCount)
        oneHeight = (CSng(dbInfo!PageHeight) - CSng(dbInfo!MarginTop) - _
            CSng(dbInfo!MarginBottom) - ((CSng(dbInfo!RowsCount) - 1.0F) * _
            CSng(dbInfo!IntraRow))) / CSng(dbInfo!RowsCount)
        useMeasure = CChar(dbInfo!UnitType)
        dbInfo.Close()

        ' ----- Convert the result into twips.
        LabelWidth = CInt(ConvertPageUnits(useMeasure, "T"c, oneWidth))
        LabelHeight = CInt(ConvertPageUnits(useMeasure, "T"c, oneHeight))
        Me.Cursor = Windows.Forms.Cursors.Default
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("BarcodeLabel.RefreshLabelTemplate", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close()
        dbInfo = Nothing
        Return
    End Sub

    Private Sub AdjustListItemButtons()
        ' ----- Adjust the display based on the selection.
        If (DisplayItems.SelectedIndex = -1) Then
            ItemProperties.SelectedObject = Nothing
            ActUp.Enabled = False
            ActDown.Enabled = False
            ActDelete.Enabled = False
        Else
            ItemProperties.SelectedObject = DisplayItems.SelectedItem
            ActUp.Enabled = CBool(DisplayItems.SelectedIndex > 0)
            ActDown.Enabled = CBool(DisplayItems.SelectedIndex < _
                (DisplayItems.Items.Count - 1))
            ActDelete.Enabled = True
        End If
    End Sub

    Private Sub RecordName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordName.Enter
        ' ----- Highlight the entire text.
        RecordName.SelectAll()
    End Sub

    Private Sub RecordSample_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordSample.Enter
        ' ----- Highlight the entire text.
        RecordSample.SelectAll()
    End Sub

    Private Sub DisplayItems_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DisplayItems.SelectedIndexChanged
        ' ----- Adjust the display based on the selection.
        AdjustListItemButtons()
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActText.Click
        ' ----- Add a new static text item.
        Dim newStaticText As Library.BarcodeItemText
        Dim fromUnit As Char

        fromUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        newStaticText = New Library.BarcodeItemText
        newStaticText.DisplayText = "Text"
        newStaticText.Alignment = ContentAlignment.TopLeft
        newStaticText.RotationAngle = 0
        newStaticText.TextColor = Color.Black
        newStaticText.Font = New System.Drawing.Font("Arial", 12)
        newStaticText.Left = 0
        newStaticText.Top = 0
        newStaticText.Width = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newStaticText.Height = ConvertPageUnits("I"c, fromUnit, 0.5F)
        DisplayItems.Items.Add(newStaticText)
        DisplayItems.SelectedItem = newStaticText
        ItemProperties.SelectedObject = newStaticText
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActBarcode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActBarcode.Click
        ' ----- Add a new barcode item.
        Dim newBarcodeImage As Library.BarcodeItemBarcodeImage
        Dim fromUnit As Char

        fromUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        newBarcodeImage = New Library.BarcodeItemBarcodeImage
        newBarcodeImage.Alignment = ContentAlignment.TopLeft
        newBarcodeImage.BarcodeColor = Color.Black
        newBarcodeImage.BarcodeSize = 10.0F
        newBarcodeImage.Left = 0
        newBarcodeImage.Top = 0
        newBarcodeImage.Width = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newBarcodeImage.Height = ConvertPageUnits("I"c, fromUnit, 0.5F)
        newBarcodeImage.RotationAngle = 0
        newBarcodeImage.PadDigits = 0
        DisplayItems.Items.Add(newBarcodeImage)
        DisplayItems.SelectedItem = newBarcodeImage
        ItemProperties.SelectedObject = newBarcodeImage
        PreviewArea.Invalidate()
    End Sub
    Private Sub ActBarNumber_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActBarNumber.Click
        ' ----- Add a new barcode text item.
        Dim newBarcodeText As Library.BarcodeItemBarcodeNumber
        Dim fromUnit As Char

        fromUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        newBarcodeText = New Library.BarcodeItemBarcodeNumber
        newBarcodeText.Alignment = ContentAlignment.TopLeft
        newBarcodeText.Left = 0
        newBarcodeText.Top = 0
        newBarcodeText.Width = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newBarcodeText.Height = ConvertPageUnits("I"c, fromUnit, 0.5F)
        newBarcodeText.RotationAngle = 0
        newBarcodeText.TextColor = Color.Black
        newBarcodeText.PadDigits = 0
        newBarcodeText.Font = New System.Drawing.Font("Arial", 12)
        DisplayItems.Items.Add(newBarcodeText)
        DisplayItems.SelectedItem = newBarcodeText
        ItemProperties.SelectedObject = newBarcodeText
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActLine_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActLine.Click
        ' ----- Add a new line item.
        Dim newStaticLine As Library.BarcodeItemLine
        Dim fromUnit As Char

        fromUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        newStaticLine = New Library.BarcodeItemLine
        newStaticLine.LineColor = Color.Black
        newStaticLine.X1 = 0
        newStaticLine.X2 = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newStaticLine.Y1 = 0
        newStaticLine.Y2 = ConvertPageUnits("I"c, fromUnit, 0.5F)
        DisplayItems.Items.Add(newStaticLine)
        DisplayItems.SelectedItem = newStaticLine
        ItemProperties.SelectedObject = newStaticLine
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActRectangle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActRectangle.Click
        ' ----- Add a new rectangle item.
        Dim newStaticRect As Library.BarcodeItemRect
        Dim fromUnit As Char

        fromUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        newStaticRect = New Library.BarcodeItemRect
        newStaticRect.BorderColor = Color.Black
        newStaticRect.FillColor = Color.White
        newStaticRect.Left = 0
        newStaticRect.Top = 0
        newStaticRect.Width = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newStaticRect.Height = ConvertPageUnits("I"c, fromUnit, 0.5F)
        newStaticRect.RotationAngle = 0
        DisplayItems.Items.Add(newStaticRect)
        DisplayItems.SelectedItem = newStaticRect
        ItemProperties.SelectedObject = newStaticRect
        PreviewArea.Invalidate()
    End Sub

    Private Sub ItemProperties_PropertyValueChanged(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PropertyValueChangedEventArgs) Handles ItemProperties.PropertyValueChanged
        ' ----- Update the preview when any property changes.
        PreviewArea.Invalidate()

        ' ----- Update the text of the list box item.
        DisplayItems.Items(DisplayItems.SelectedIndex) = DisplayItems.SelectedItem
    End Sub

    Private Sub ActUp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActUp.Click
        ' ----- Move the selected item up one position.
        Dim holdObject As Object

        On Error Resume Next

        ' ----- Ignore if no item is selected.
        If (DisplayItems.SelectedIndex <= 0) Then Return

        ' ----- Reverse the items.
        holdObject = DisplayItems.Items(DisplayItems.SelectedIndex - 1)
        DisplayItems.Items.RemoveAt(DisplayItems.SelectedIndex - 1)
        DisplayItems.Items.Insert(DisplayItems.SelectedIndex + 1, holdObject)

        ' ----- Update the display.
        AdjustListItemButtons()
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActDown_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ActDown.Click
        ' ----- Move the selected item down one position.
        Dim holdObject As Object

        On Error Resume Next

        ' ----- Ignore if no item is selected.
        If (DisplayItems.SelectedIndex = -1) Then Return
        If (DisplayItems.SelectedIndex = (DisplayItems.Items.Count - 1)) Then Return

        ' ----- Reverse the items.
        holdObject = DisplayItems.Items(DisplayItems.SelectedIndex + 1)
        DisplayItems.Items.RemoveAt(DisplayItems.SelectedIndex + 1)
        DisplayItems.Items.Insert(DisplayItems.SelectedIndex, holdObject)

        ' ----- Update the display.
        AdjustListItemButtons()
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActDelete.Click
        ' ----- Delete the selected item.
        If (DisplayItems.SelectedIndex = -1) Then Return
        DisplayItems.Items.RemoveAt(DisplayItems.SelectedIndex)

        ' ----- Refresh the display.
        ItemProperties.SelectedObject = Nothing
        AdjustListItemButtons()
        PreviewArea.Invalidate()
    End Sub

    Private Sub PreviewArea_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles PreviewArea.Paint
        ' ----- Display a preview of the image.
        Dim pageWidth As Integer
        Dim pageHeight As Integer
        Dim pageLeft As Integer  ' Start of the "white" area.
        Dim pageTop As Integer   ' Start of the "white" area.
        Dim useRatio As Single
        Dim useRatioText As Single
        Dim fromUnit As Char
        Dim counter As Integer
        Dim textMessage As String
        Dim useGeneric As Library.BarcodeItemGeneric
        Dim useBarImage As Library.BarcodeItemBarcodeImage
        Dim useBarText As Library.BarcodeItemBarcodeNumber
        Dim useText As Library.BarcodeItemText
        Dim useLine As Library.BarcodeItemLine
        Dim useRect As Library.BarcodeItemRect
        Dim useFont As System.Drawing.Font
        Dim holdState As System.Drawing.Drawing2D.GraphicsState
        Dim pixelPen As System.Drawing.Pen
        Dim pixelBrush As System.Drawing.SolidBrush
        Dim textColor As System.Drawing.Color
        Dim textAngle As Integer
        Dim textAlign As System.Drawing.ContentAlignment
        Dim textFormat As New Drawing.StringFormat
        Dim X1 As Single
        Dim X2 As Single
        Dim Y1 As Single
        Dim Y2 As Single

        On Error Resume Next

        ' ----- Draw in the background color.
        e.Graphics.Clear(SystemColors.Control)
        If (LabelWidth <= 0) Or (LabelHeight <= 0) Then Return

        ' ----- Determine the page area.
        pageWidth = PreviewArea.Width - 4
        pageHeight = PreviewArea.Height - 4
        useRatio = CSng(pageWidth) / CSng(LabelWidth)
        useRatioText = CSng(pageWidth) / ConvertPageUnits("T"c, "P"c, LabelWidth)
        If (CInt(LabelHeight * useRatio) > pageHeight) Then
            ' ----- Scale based on the height.
            useRatio = CSng(pageHeight) / CSng(LabelHeight)
            useRatioText = CSng(pageHeight) / ConvertPageUnits("T"c, "P"c, LabelHeight)
            pageWidth = CInt(LabelWidth * useRatio)
        Else
            ' ----- Scale based on the width.
            pageHeight = CInt(LabelHeight * useRatio)
        End If
        pageLeft = CInt((PreviewArea.Width - 4 - pageWidth) / 2) + 1
        pageTop = CInt((PreviewArea.Height - 4 - pageHeight) / 2) + 1

        ' ----- Draw the page area.
        e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 1, pageTop + 1, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 2, pageTop + 2, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(Brushes.Black, pageLeft - 1, pageTop - 1, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(Brushes.White, pageLeft, pageTop, pageWidth, pageHeight)

        ' ----- What size of units are we using for item display?
        If (RecordUnits.SelectedIndex = -1) Then Return
        fromUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)

        ' ----- Transform a page of twips into a display page.
        e.Graphics.TranslateTransform(pageLeft, pageTop)
        e.Graphics.ScaleTransform(useRatio, useRatio)

        ' ----- Process each item in the list.
        For counter = 0 To DisplayItems.Items.Count - 1
            ' ----- Save the current state of the graphics area.
            holdState = e.Graphics.Save()

            ' ----- Different code for each type of drawing object.
            useGeneric = CType(DisplayItems.Items(counter), Library.BarcodeItemGeneric)
            Select Case useGeneric.ItemType
                Case "B"
                    ' ----- Barcode image.
                    useBarImage = CType(useGeneric, Library.BarcodeItemBarcodeImage)
                    textMessage = Trim(RecordSample.Text)
                    If (textMessage = "") Then textMessage = "12345678"
                    If (Len(textMessage) < useBarImage.PadDigits) Then
                        textMessage = StrDup(useBarImage.PadDigits - _
                            Len(textMessage), "0") & textMessage
                    End If
                    If (Barcode39 = True) Then textMessage = "*" & textMessage & "*"
                    textColor = useBarImage.BarcodeColor
                    textAngle = useBarImage.RotationAngle
                    textAlign = useBarImage.Alignment

                    ' ----- Get the font to use.
                    useFont = New System.Drawing.Font(BarcodeFontName, _
                        useBarImage.BarcodeSize, System.Drawing.FontStyle.Regular)

                    ' ----- Get the coordinates in points.
                    X1 = ConvertPageUnits(fromUnit, "P"c, useBarImage.Left)
                    X2 = ConvertPageUnits(fromUnit, "P"c, useBarImage.Width)
                    Y1 = ConvertPageUnits(fromUnit, "P"c, useBarImage.Top)
                    Y2 = ConvertPageUnits(fromUnit, "P"c, useBarImage.Height)

                    ' ----- The rest is the same as drawing normal text.
                    GoTo DisplayTheText
                Case "L"
                    ' ----- Line.
                    useLine = CType(useGeneric, Library.BarcodeItemLine)

                    ' ----- Create a one-pixel pen to use.
                    pixelPen = New System.Drawing.Pen(useLine.LineColor, _
                        1 / e.Graphics.DpiX)

                    ' ----- Get the coordinates in twips.
                    X1 = ConvertPageUnits(fromUnit, "T"c, useLine.X1)
                    X2 = ConvertPageUnits(fromUnit, "T"c, useLine.X2)
                    Y1 = ConvertPageUnits(fromUnit, "T"c, useLine.Y1)
                    Y2 = ConvertPageUnits(fromUnit, "T"c, useLine.Y2)

                    ' ----- Draw the line.
                    e.Graphics.DrawLine(pixelPen, X1, Y1, X2, Y2)
                    pixelPen.Dispose()

                    ' ----- If this is the active line object, draw a thin outline
                    '       around the object.
                    If (DisplayItems.SelectedIndex = counter) Then
                        pixelPen = New System.Drawing.Pen(Color.LightGray, _
                            1 / e.Graphics.DpiX)
                        pixelPen.DashStyle = Drawing2D.DashStyle.Dash
                        e.Graphics.DrawRectangle(pixelPen, X1 - 15, Y1 - 15, _
                            X2 - X1 + 30, Y2 - Y1 + 30)
                        pixelPen.Dispose()
                    End If
                Case "N"
                    ' ----- Barcode text.
                    useBarText = CType(useGeneric, Library.BarcodeItemBarcodeNumber)
                    textMessage = Trim(RecordSample.Text)
                    If (textMessage = "") Then textMessage = "12345678"
                    If (Len(textMessage) < useBarText.PadDigits) Then
                        textMessage = StrDup(useBarText.PadDigits - _
                            Len(textMessage), "0") & textMessage
                    End If
                    textColor = useBarText.TextColor
                    textAngle = useBarText.RotationAngle
                    textAlign = useBarText.Alignment

                    ' ----- Get the font to use.
                    useFont = New System.Drawing.Font(useBarText.Font.Name, _
                        useBarText.Font.SizeInPoints, useBarText.Font.Style)

                    ' ----- Get the coordinates in points.
                    X1 = ConvertPageUnits(fromUnit, "P"c, useBarText.Left)
                    X2 = ConvertPageUnits(fromUnit, "P"c, useBarText.Width)
                    Y1 = ConvertPageUnits(fromUnit, "P"c, useBarText.Top)
                    Y2 = ConvertPageUnits(fromUnit, "P"c, useBarText.Height)

                    ' ----- The rest is the same as drawing normal text.
                    GoTo DisplayTheText
                Case "R"
                    ' ----- Rectangle.
                    useRect = CType(useGeneric, Library.BarcodeItemRect)

                    ' ----- Create a one-pixel pen to use.
                    pixelPen = New System.Drawing.Pen(useRect.BorderColor, _
                        1 / e.Graphics.DpiX)
                    pixelBrush = New System.Drawing.SolidBrush(useRect.FillColor)

                    ' ----- Get the coordinates in twips.
                    X1 = ConvertPageUnits(fromUnit, "T"c, useRect.Left)
                    X2 = ConvertPageUnits(fromUnit, "T"c, useRect.Width)
                    Y1 = ConvertPageUnits(fromUnit, "T"c, useRect.Top)
                    Y2 = ConvertPageUnits(fromUnit, "T"c, useRect.Height)

                    ' ----- Rotate the surface if requested.
                    If (useRect.RotationAngle <> 0) Then
                        e.Graphics.TranslateTransform(X1, Y1)
                        e.Graphics.RotateTransform(useRect.RotationAngle)
                        X1 = 0.0F
                        Y1 = 0.0F
                    End If

                    ' ----- Draw the rectangle.
                    e.Graphics.FillRectangle(pixelBrush, X1, Y1, X2, Y2)
                    e.Graphics.DrawRectangle(pixelPen, X1, Y1, X2, Y2)
                    pixelBrush.Dispose()
                    pixelPen.Dispose()

                    ' ----- If this is the active line object, draw a thin outline
                    '       around the object.
                    If (DisplayItems.SelectedIndex = counter) Then
                        pixelPen = New System.Drawing.Pen(Color.LightGray, _
                            1 / e.Graphics.DpiX)
                        pixelPen.DashStyle = Drawing2D.DashStyle.Dash
                        e.Graphics.DrawRectangle(pixelPen, X1 - 1 / e.Graphics.DpiX, _
                            Y1 - 1 / e.Graphics.DpiX, X2 + 2 / e.Graphics.DpiX, _
                            Y2 + 2 / e.Graphics.DpiX)
                        pixelPen.Dispose()
                    End If
                Case "T"
                    ' ----- Text.
                    useText = CType(useGeneric, Library.BarcodeItemText)
                    textMessage = useText.DisplayText
                    textColor = useText.TextColor
                    textAngle = useText.RotationAngle
                    textAlign = useText.Alignment

                    ' ----- Get the font to use.
                    useFont = New System.Drawing.Font(useText.Font.Name, _
                        useText.Font.SizeInPoints, useText.Font.Style)

                    ' ----- Get the coordinates in points.
                    X1 = ConvertPageUnits(fromUnit, "P"c, useText.Left)
                    X2 = ConvertPageUnits(fromUnit, "P"c, useText.Width)
                    Y1 = ConvertPageUnits(fromUnit, "P"c, useText.Top)
                    Y2 = ConvertPageUnits(fromUnit, "P"c, useText.Height)

DisplayTheText:
                    ' ----- For text, we do a new transformation. Everything is
                    '       measured in points instead of twips.
                    e.Graphics.ResetTransform()
                    e.Graphics.TranslateTransform(pageLeft, pageTop)
                    e.Graphics.ScaleTransform(useRatioText, useRatioText)

                    ' ----- Rotate around the upper-left corner of the text box
                    '       if needed.
                    If (textAngle <> 0) Then
                        e.Graphics.TranslateTransform(X1, Y1)
                        e.Graphics.RotateTransform(textAngle)
                        X1 = 0.0F
                        Y1 = 0.0F
                    End If

                    ' ----- If this is the active text object, draw a thin outline
                    '       around the object.
                    If (DisplayItems.SelectedIndex = counter) Then
                        pixelPen = New System.Drawing.Pen(Color.LightGray, _
                            1 / e.Graphics.DpiX)
                        pixelPen.DashStyle = Drawing2D.DashStyle.Dash
                        e.Graphics.DrawRectangle(pixelPen, X1, Y1, X2, Y2)
                        pixelPen.Dispose()
                    End If

                    ' ----- Position the text within the bounding box.
                    Select Case textAlign
                        Case ContentAlignment.TopLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Far
                        Case ContentAlignment.TopCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Far
                        Case ContentAlignment.TopRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Far
                    End Select
                    textFormat.FormatFlags = textFormat.FormatFlags Or _
                        StringFormatFlags.NoWrap

                    ' ----- Print the text.
                    e.Graphics.DrawString(textMessage, useFont, _
                        New System.Drawing.SolidBrush(textColor), _
                        New Drawing.RectangleF(X1, Y1, X2, Y2), textFormat)
                    useFont.Dispose()
            End Select

            ' ----- Restore the original transformed state of the graphics surface.
            e.Graphics.Restore(holdState)
        Next counter

        ' ----- Remove the page-size transformations.
        e.Graphics.ResetTransform()
    End Sub

    Private Sub RecordTemplate_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordTemplate.SelectedIndexChanged
        ' ----- The user selected a different page template.
        If (IgnoreSizeChange = True) Then Return
        RefreshLabelTemplate()
        RefreshSizeDisplay()
        PreviewArea.Invalidate()
    End Sub

    Private Sub RecordSample_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordSample.TextChanged
        ' ----- Refresh the display on a change.
        PreviewArea.Invalidate()
    End Sub

    Private Sub RecordUnits_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordUnits.SelectedIndexChanged
        ' ----- The user is altering the size.
        Dim toUnit As Char
        Dim counter As Integer
        Dim useGeneric As Library.BarcodeItemGeneric
        Dim useBarImage As Library.BarcodeItemBarcodeImage
        Dim useBarText As Library.BarcodeItemBarcodeNumber
        Dim useText As Library.BarcodeItemText
        Dim useLine As Library.BarcodeItemLine
        Dim useRect As Library.BarcodeItemRect

        On Error Resume Next

        RefreshSizeDisplay()
        If (IgnoreSizeChange = True) Then Return
        If (RecordUnits.SelectedIndex = -1) Then Return
        toUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)

        ' ----- Change the units of each object to the new size.
        For counter = 0 To DisplayItems.Items.Count - 1
            useGeneric = CType(DisplayItems.Items(counter), Library.BarcodeItemGeneric)
            Select Case useGeneric.ItemType
                Case "B"
                    ' ----- Barcode image.
                    useBarImage = CType(useGeneric, Library.BarcodeItemBarcodeImage)
                    useBarImage.Left = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Left)
                    useBarImage.Top = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Top)
                    useBarImage.Width = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Width)
                    useBarImage.Height = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Height)
                Case "L"
                    ' ----- Line.
                    useLine = CType(useGeneric, Library.BarcodeItemLine)
                    useLine.X1 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.X1)
                    useLine.X2 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.X2)
                    useLine.Y1 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.Y1)
                    useLine.Y2 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.Y2)
                Case "N"
                    ' ----- Barcode text.
                    useBarText = CType(useGeneric, Library.BarcodeItemBarcodeNumber)
                    useBarText.Left = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Left)
                    useBarText.Top = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Top)
                    useBarText.Width = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Width)
                    useBarText.Height = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Height)
                Case "R"
                    ' ----- Rectangle.
                    useRect = CType(useGeneric, Library.BarcodeItemRect)
                    useRect.Left = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Left)
                    useRect.Top = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Top)
                    useRect.Width = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Width)
                    useRect.Height = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Height)
                Case "T"
                    ' ----- Text.
                    useText = CType(useGeneric, Library.BarcodeItemText)
                    useText.Left = ConvertPageUnits(CurrentMeasure, toUnit, useText.Left)
                    useText.Top = ConvertPageUnits(CurrentMeasure, toUnit, useText.Top)
                    useText.Width = ConvertPageUnits(CurrentMeasure, toUnit, useText.Width)
                    useText.Height = ConvertPageUnits(CurrentMeasure, toUnit, useText.Height)
            End Select

            ' ----- Update the display item in the list.
            DisplayItems.Items(counter) = DisplayItems.Items(counter)
        Next counter

        ' ----- Update the properties area.
        If (DisplayItems.SelectedIndex <> -1) Then _
            ItemProperties.SelectedObject = DisplayItems.SelectedItem
        CurrentMeasure = toUnit
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        Dim sqlText As String
        Dim counter As Integer
        Dim useGeneric As Library.BarcodeItemGeneric
        Dim useBarImage As Library.BarcodeItemBarcodeImage
        Dim useBarText As Library.BarcodeItemBarcodeNumber
        Dim useText As Library.BarcodeItemText
        Dim useLine As Library.BarcodeItemLine
        Dim useRect As Library.BarcodeItemRect

        On Error GoTo ErrorHandler

        ' ----- Confirm the data.
        If (ValidateForm() = False) Then Return
        Me.Cursor = Windows.Forms.Cursors.WaitCursor
        TransactionBegin()

        ' ----- Save the parent record data.
        If (ActiveLabelID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO BarcodeLabel (FullName, BarcodeSheet, UnitType) " & _
                "VALUES (" & DBText(Trim(RecordName.Text)) & ", " & _
                CType(RecordTemplate.SelectedItem, ListItemData).ItemData & ", '" & _
                Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData) & "')"
            ExecuteSQL(sqlText)

            ' ----- Retrieve the new ID.
            sqlText = "SELECT ID FROM BarcodeLabel WHERE FullName = " & _
                DBText(Trim(RecordName.Text)) & " AND BarcodeSheet = " & _
                CType(RecordTemplate.SelectedItem, ListItemData).ItemData & _
                " ORDER BY ID DESC"
            ActiveLabelID = CInt(ExecuteSQLReturn(sqlText))
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE BarcodeLabel SET" & _
                " FullName = " & DBText(Trim(RecordName.Text)) & _
                ", BarcodeSheet = " & CType(RecordTemplate.SelectedItem, ListItemData).ItemData & _
                ", UnitType = '" & Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData) & _
                "' WHERE ID = " & ActiveLabelID
            ExecuteSQL(sqlText)

            ' ----- Remove the detail records.
            sqlText = "DELETE FROM BarcodeLabelItem WHERE BarcodeLabel = " & ActiveLabelID
            ExecuteSQL(sqlText)
        End If

        ' ----- Add the detail records.
        For counter = 0 To DisplayItems.Items.Count - 1
            ' ----- Build the command to add the record.
            useGeneric = CType(DisplayItems.Items(counter), Library.BarcodeItemGeneric)
            sqlText = "INSERT INTO BarcodeLabelItem (Priority, BarcodeLabel, ItemType, " & _
                "PosLeft, PosTop, PosWidth, PosHeight, Rotation, FontName, FontSize, " & _
                "FontStyle, StaticText, Color1, Color2, Alignment, PadDigits) VALUES (" & _
                CStr(counter + 1) & ", " & ActiveLabelID & ", '" & useGeneric.ItemType & "', "
            Select Case useGeneric.ItemType
                Case "B"
                    ' ----- Barcode image.
                    useBarImage = CType(useGeneric, Library.BarcodeItemBarcodeImage)

                    ' ----- PosLeft, PosTop, PosWidth, PosHeight, Rotation, FontName
                    sqlText &= useBarImage.Left & ", " & useBarImage.Top & ", " & _
                        useBarImage.Width & ", " & useBarImage.Height & ", " & _
                        useBarImage.RotationAngle & ", NULL, "
                    ' ----- FontSize, FontStyle, StaticText, Color1, Color2, Alignment, PadDigits
                    sqlText &= useBarImage.BarcodeSize & ", NULL, NULL, " & _
                        useBarImage.BarcodeColor.ToArgb() & ", NULL, " & _
                        CInt(useBarImage.Alignment) & ", " & useBarImage.PadDigits & ")"
                Case "L"
                    ' ----- Line.
                    useLine = CType(useGeneric, Library.BarcodeItemLine)

                    ' ----- PosLeft, PosTop, PosWidth, PosHeight, Rotation, FontName
                    sqlText &= useLine.X1 & ", " & useLine.Y1 & ", " & _
                        useLine.X2 & ", " & useLine.Y2 & ", 0, NULL, "
                    ' ----- FontSize, FontStyle, StaticText, Color1, Color2, Alignment, PadDigits
                    sqlText &= "NULL, NULL, NULL, " & useLine.LineColor.ToArgb() & _
                        ", NULL, NULL, NULL)"
                Case "N"
                    ' ----- Barcode text.
                    useBarText = CType(useGeneric, Library.BarcodeItemBarcodeNumber)

                    ' ----- PosLeft, PosTop, PosWidth, PosHeight, Rotation, FontName
                    sqlText &= useBarText.Left & ", " & useBarText.Top & ", " & _
                        useBarText.Width & ", " & useBarText.Height & ", " & _
                        useBarText.RotationAngle & ", " & DBText(useBarText.Font.Name) & ", "
                    ' ----- FontSize, FontStyle, StaticText, Color1, Color2, Alignment, PadDigits
                    sqlText &= useBarText.Font.SizeInPoints & ", " & _
                        DBText(DBFontStyle(useBarText.Font)) & ", NULL, " & _
                        useBarText.TextColor.ToArgb() & ", NULL, " & _
                        CInt(useBarText.Alignment) & ", " & useBarText.PadDigits & ")"
                Case "R"
                    ' ----- Rectangle.
                    useRect = CType(useGeneric, Library.BarcodeItemRect)

                    ' ----- PosLeft, PosTop, PosWidth, PosHeight, Rotation, FontName
                    sqlText &= useRect.Left & ", " & useRect.Top & ", " & _
                        useRect.Width & ", " & useRect.Height & ", " & _
                        useRect.RotationAngle & ", NULL, "
                    ' ----- FontSize, FontStyle, StaticText, Color1, Color2, Alignment, PadDigits
                    sqlText &= "NULL, NULL, NULL, " & useRect.BorderColor.ToArgb() & _
                        ", " & useRect.FillColor.ToArgb() & ", NULL, NULL)"
                Case "T"
                    ' ----- Text.
                    useText = CType(useGeneric, Library.BarcodeItemText)

                    ' ----- PosLeft, PosTop, PosWidth, PosHeight, Rotation, FontName
                    sqlText &= useText.Left & ", " & useText.Top & ", " & _
                        useText.Width & ", " & useText.Height & ", " & _
                        useText.RotationAngle & ", " & DBText(useText.Font.Name) & ", "
                    ' ----- FontSize, FontStyle, StaticText, Color1, Color2, Alignment, PadDigits
                    sqlText &= useText.Font.SizeInPoints & ", " & _
                        DBText(DBFontStyle(useText.Font)) & ", " & _
                        DBText(useText.DisplayText) & _
                        ", " & useText.TextColor.ToArgb() & ", NULL, " & _
                        CInt(useText.Alignment) & ", NULL)"
            End Select

            ' ----- Add the record.
            ExecuteSQL(sqlText)
        Next counter
        TransactionCommit()

        ' ----- Continue with AddRecord() or EditRecord()
        Me.Cursor = Windows.Forms.Cursors.Default
        Me.DialogResult = Windows.Forms.DialogResult.OK
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("BarcodeLabel.ActOK_Click", Err.GetException())
        TransactionRollback()
        Return
    End Sub
End Class
