' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class ReportSelect
    Private Sub LoadReportGroup(ByVal groupNode As Xml.XmlNode, _
          ByVal indentLevel As Integer)
        ' ----- Add the groups and items at this level,
        '       and recurse as needed.
        Dim scanNode As Xml.XmlNode
        Dim detailNode As Xml.XmlNode
        Dim reportEntry As ReportItem

        On Error GoTo ErrorHandler

        ' ----- Process each item or group.
        For Each scanNode In groupNode.ChildNodes
            ' ----- Build a content item for the list.
            reportEntry = New ReportItem
            reportEntry.Indent = indentLevel

            ' ----- Get the display name.
            detailNode = scanNode.SelectSingleNode("displayText")
            If (detailNode Is Nothing) Then Continue For
            reportEntry.DisplayText = Trim(detailNode.InnerText)

            If (scanNode.Name = "reportGroup") Then
                ' ----- Start a new display group.
                reportEntry.ItemType = ReportItemEnum.GroupLabel
                AllReports.Items.Add(reportEntry)

                ' ----- Recurse to child items.
                LoadReportGroup(scanNode, indentLevel + 1)
            ElseIf (scanNode.Name = "reportItem") Then
                ' ----- This is an item. Record its location.
                detailNode = scanNode.SelectSingleNode("reportPath")
                If Not (detailNode Is Nothing) Then _
                   reportEntry.ReportPath = Trim(detailNode.InnerText)

                ' ----- Get any command-line arguments.
                detailNode = scanNode.SelectSingleNode("reportArgs")
                If Not (detailNode Is Nothing) Then _
                   reportEntry.ReportArgs = Trim(detailNode.InnerText)

                ' ----- Get any item-specific flags.
                detailNode = scanNode.SelectSingleNode("reportFlags")
                If Not (detailNode Is Nothing) Then
                    ' ---- "U" adds "-u loginid" to the command.
                    If (InStr(UCase(detailNode.InnerText), "U") > 0) _
                       And (LoggedInUserName <> "") Then _
                       reportEntry.ReportArgs = _
                       Trim(reportEntry.ReportArgs & " -u " & _
                       LoggedInUserName)
                End If

                ' ----- Store the full description.
                detailNode = scanNode.SelectSingleNode("description")
                If Not (detailNode Is Nothing) Then _
                   reportEntry.Description = Trim(detailNode.InnerText)

                ' ----- So, what type of entry is it?
                If (scanNode.Attributes("type").Value = "built-in") Then
                    ' ----- Built-in program. Check for valid ID.
                    If (IsNumeric(reportEntry.ReportPath) = False) Or _
                       (Val(reportEntry.ReportPath) < 1) Or _
                       (Val(reportEntry.ReportPath) > _
                       CInt(ReportItemEnum.BuiltInStatistics)) Then _
                       Continue For
                    reportEntry.ItemType = CType(CInt( _
                       reportEntry.ReportPath), ReportItemEnum)
                    If (reportEntry.Description = "") Then _
                        reportEntry.Description = GetBuiltinReportDescription( _
                        reportEntry.ItemType)
                    AllReports.Items.Add(reportEntry)
                ElseIf (scanNode.Attributes("type").Value = "program") Then
                    ' ----- EXE program-based report.
                    If (reportEntry.ReportPath = "") Then Continue For
                    reportEntry.ItemType = ReportItemEnum.ExeProgram
                    AllReports.Items.Add(reportEntry)
                ElseIf (scanNode.Attributes("type").Value = "url") Then
                    ' ----- URL-based report.
                    If (reportEntry.ReportPath = "") Then Continue For
                    reportEntry.ItemType = ReportItemEnum.UrlProgram
                    AllReports.Items.Add(reportEntry)
                End If
            End If
        Next scanNode
        Return

ErrorHandler:
        GeneralError("ReportSelect.LoadReportGroup", Err.GetException())
        Resume Next
    End Sub

    Private Sub RefreshReportList()
        ' ----- Load in the list of available reports.
        Dim configFile As String
        Dim configData As Xml.XmlDocument
        Dim reportEntry As ReportItem
        Dim counter As Integer

        On Error GoTo ErrorHandler

        ' ----- Clear the existing list.
        AllReports.Items.Clear()

        ' ----- Get the location of the configuration file.
        ' TODO: Load this from the application's configuration.
        '       For now, just hard-code the value.
        configFile = "c:\ReportConfig.txt"

        ' ----- Load the configuration file.
        If (configFile <> "") Then
            If (System.IO.File.Exists(configFile)) Then
                ' ----- Load in the file.
                configData = New Xml.XmlDocument
                configData.Load(configFile)

                ' ----- Process the configuration file.
                LoadReportGroup(configData.DocumentElement, 0)
            End If
        End If

        ' ----- If the configuration file resulted in no reports
        '       appearing in the list, add the default reports.
        If (AllReports.Items.Count = 0) Then
            For counter = 1 To CInt(ReportItemEnum.BuiltInStatistics)
                ' ----- Build the report entry.
                reportEntry = New ReportItem
                reportEntry.Indent = 0
                reportEntry.ItemType = CType(counter, ReportItemEnum)
                Select Case reportEntry.ItemType
                    Case ReportItemEnum.BuiltInCheckedOut
                        reportEntry.DisplayText = "Items Checked Out"
                    Case ReportItemEnum.BuiltInOverdue
                        reportEntry.DisplayText = "Items Overdue"
                    Case ReportItemEnum.BuiltInMissing
                        reportEntry.DisplayText = "Items Missing"
                    Case ReportItemEnum.BuiltInFinesOwed
                        reportEntry.DisplayText = "Patron Fines Owed"
                    Case ReportItemEnum.BuiltInStatistics
                        reportEntry.DisplayText = "Database Statistics"
                End Select
                reportEntry.Description = GetBuiltinReportDescription( _
                    reportEntry.ItemType)

                ' ----- Add the report entry to the list.
                AllReports.Items.Add(reportEntry)
            Next counter
        End If
        Return

ErrorHandler:
        GeneralError("ReportSelect.RefreshReportList", Err.GetException())
        Resume Next
    End Sub

    Private Sub AllReports_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles AllReports.SelectedIndexChanged
        ' ----- Display a description of the report, if available.
        Dim reportEntry As Library.ReportItem

        ' ----- Clear any previous description.
        FullDescription.Text = "No report selected."
        If (AllReports.SelectedIndex <> -1) Then
            ' ----- Locate the content and display it.
            reportEntry = CType(AllReports.SelectedItem, Library.ReportItem)
            FullDescription.Text = reportEntry.Description
        End If
    End Sub

    Private Sub ReportSelect_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' ----- Display the list of reports.
        RefreshReportList()
    End Sub

    Private Sub ActRun_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ActRun.Click
        ' ----- Run the selected report.
        Dim reportEntry As Library.ReportItem

        On Error GoTo ErrorHandler

        ' ----- Make sure a report is selected.
        If (AllReports.SelectedIndex = -1) Then
            MsgBox("Please select a report from the list.", _
               MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, _
               ProgramTitle)
            Return
        End If

        ' ----- Different code for each type of entry.
        reportEntry = CType(AllReports.SelectedItem, Library.ReportItem)
        Me.Cursor = Windows.Forms.Cursors.WaitCursor
        Select Case reportEntry.ItemType
            Case ReportItemEnum.GroupLabel
                ' ----- No report for group entries.
                MsgBox("Please select a report from the list.", _
                   MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, _
                   ProgramTitle)
            Case ReportItemEnum.BuiltInCheckedOut
                ' ----- Items Checked Out
                BasicReportCheckedOut()
            Case ReportItemEnum.BuiltInOverdue
                ' ----- Items Overdue
                BasicReportOverdue()
            Case ReportItemEnum.BuiltInMissing
                ' ----- Items Missing
                BasicReportMissing()
            Case ReportItemEnum.BuiltInFinesOwed
                ' ----- Fines Owed by Patrons
                BasicReportFines()
            Case ReportItemEnum.BuiltInStatistics
                ' ----- Library Database Statistics
                BasicReportStatistics()
            Case ReportItemEnum.ExeProgram
                ' ----- Start a program.
                Process.Start("""" & reportEntry.ReportPath & """ " & _
                    reportEntry.ReportArgs)
            Case ReportItemEnum.UrlProgram
                ' ----- Start a URL.
                Process.Start(reportEntry.ReportPath)
        End Select
        Me.Cursor = Windows.Forms.Cursors.Default
        Return

ErrorHandler:
        GeneralError("ReportSelect.ActRun_Click", Err.GetException())
        Resume Next
    End Sub

    Private Sub BasicReportCheckedOut()
        ' ----- Run built-in report #1: Items checked out report.
        Dim sqlText As String
        Dim reportData As Data.DataTable
        Dim reportForm As ReportBuiltinViewer

        On Error GoTo ErrorHandler

        ' ----- Retrieve the data as a dataset.
        sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " & _
            "PA.Barcode AS PatronBarcode, " & _
            "PC.DueDate, IC.CopyNumber, IC.Barcode AS ItemBarcode, " & _
            "NI.Title, CMT.FullName AS MediaName " & _
            "FROM Patron AS PA " & _
            "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " & _
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " & _
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " & _
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " & _
            "WHERE PC.Returned = 0 " & _
            "AND PC.Missing = 0 " & _
            "AND IC.Missing = 0 " & _
            "ORDER BY NI.Title, IC.CopyNumber, PA.LastName, PA.FirstName"
        reportData = CreateDataTable(sqlText)

        ' ----- Check for no data.
        If (reportData.Rows.Count = 0) Then
            reportData.Dispose()
            MsgBox("No items are checked out.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        reportForm.StartReport("Library.ReportCheckedOut.rdlc", _
            "Library_ReportSchemaPatronItems", reportData)
        Return

ErrorHandler:
        GeneralError("ReportSelect.BasicReportCheckedOut", Err.GetException())
        Return
    End Sub

    Private Sub BasicReportOverdue()
        ' ----- Run built-in report #2: Items overdue report.
        Dim sqlText As String
        Dim reportData As Data.DataTable
        Dim reportForm As ReportBuiltinViewer

        On Error GoTo ErrorHandler

        ' ----- Retrieve the data as a dataset.
        sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " & _
            "PA.Barcode AS PatronBarcode, " & _
            "PC.DueDate, IC.CopyNumber, IC.Barcode AS ItemBarcode, " & _
            "NI.Title, CMT.FullName AS MediaName " & _
            "FROM Patron AS PA " & _
            "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " & _
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " & _
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " & _
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " & _
            "WHERE PC.Returned = 0 " & _
            "AND PC.Missing = 0 " & _
            "AND IC.Missing = 0 " & _
            "AND PC.DueDate < GETDATE() " & _
            "ORDER BY NI.Title, IC.CopyNumber, PA.LastName, PA.FirstName"
        reportData = CreateDataTable(sqlText)

        ' ----- Check for no data.
        If (reportData.Rows.Count = 0) Then
            reportData.Dispose()
            MsgBox("No items are overdue.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        reportForm.StartReport("Library.ReportOverdue.rdlc", _
            "Library_ReportSchemaPatronItems", reportData)
        Return

ErrorHandler:
        GeneralError("ReportSelect.BasicReportOverdue", Err.GetException())
        Return
    End Sub

    Private Sub BasicReportMissing()
        ' ----- Run built-in report #3: Items missing report.
        Dim sqlText As String
        Dim reportData As Data.DataTable
        Dim reportForm As ReportBuiltinViewer

        On Error GoTo ErrorHandler

        ' ----- Retrieve the data as a dataset.
        sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " & _
            "PA.Barcode AS PatronBarcode, " & _
            "PC.DueDate, IC.CopyNumber, IC.Barcode AS ItemBarcode, " & _
            "NI.Title, CMT.FullName AS MediaName " & _
            "FROM Patron AS PA " & _
            "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " & _
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " & _
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " & _
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " & _
            "WHERE PC.Missing = 1 " & _
            "OR IC.Missing = 1 " & _
            "ORDER BY NI.Title, IC.CopyNumber, PA.LastName, PA.FirstName"
        reportData = CreateDataTable(sqlText)

        ' ----- Check for no data.
        If (reportData.Rows.Count = 0) Then
            reportData.Dispose()
            MsgBox("No items are overdue.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        reportForm.StartReport("Library.ReportMissing.rdlc", _
            "Library_ReportSchemaPatronItems", reportData)
        Return

ErrorHandler:
        GeneralError("ReportSelect.BasicReportMissing", Err.GetException())
        Return
    End Sub

    Private Sub BasicReportFines()
        ' ----- Run built-in report #4: Fines owed by patrons report.
        Dim sqlText As String
        Dim reportData As Data.DataTable
        Dim reportForm As ReportBuiltinViewer

        On Error GoTo ErrorHandler

        ' ----- Retrieve the data as a dataset.
        sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " & _
            "PA.Barcode AS PatronBarcode, " & _
            "SUM(PC.Fine - PC.Paid) AS FinesDue " & _
            "FROM Patron AS PA " & _
            "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " & _
            "GROUP BY PA.LastName + ', ' + PA.FirstName, PA.Barcode " & _
            "HAVING SUM(PC.Fine - PC.Paid) > 0 " & _
            "ORDER BY PatronName"
        reportData = CreateDataTable(sqlText)

        ' ----- Check for no data.
        If (reportData.Rows.Count = 0) Then
            reportData.Dispose()
            MsgBox("There are no patrons with unpaid fines.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        reportForm.StartReport("Library.ReportPatronFines.rdlc", _
            "Library_ReportSchemaPatronFines", reportData)
        Return

ErrorHandler:
        GeneralError("ReportSelect.BasicReportFines", Err.GetException())
        Return
    End Sub

    Private Sub BasicReportStatistics()
        ' ----- Run built-in report #5: Library database statistics report.
        Dim sqlText As String
        Dim reportData As Collections.Generic.List(Of ReportSchemaStatistics)
        Dim oneEntry As ReportSchemaStatistics
        Dim reportForm As ReportBuiltinViewer
        Dim resultValue As Integer
        Dim counter As Integer
        Const tableSets As String = "Author,Publisher,Subject," & _
            "NamedItem,ItemCopy,Patron"
        Const tableTitles As String = "Authors,Publishers,Subject Headings," & _
            "Items,Item Copies,Patrons"

        On Error GoTo ErrorHandler

        ' ----- Build the report data. It's all counts from different tables.
        reportData = New Collections.Generic.List(Of ReportSchemaStatistics)
        For counter = 1 To CountSubStr(tableSets, ",") + 1
            ' ----- Process one table.
            sqlText = "SELECT COUNT(*) FROM " & GetSubStr(tableSets, ",", counter)
            resultValue = DBGetInteger(ExecuteSQLReturn(sqlText))

            ' ----- Add it to the report data.
            oneEntry = New ReportSchemaStatistics
            oneEntry.EntryName = GetSubStr(tableTitles, ",", counter)
            oneEntry.EntryValue = CStr(resultValue)
            reportData.Add(oneEntry)
        Next counter

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        reportForm.StartReport("Library.ReportStatistics.rdlc", _
            "Library_ReportSchemaStatistics", reportData)
        Return

ErrorHandler:
        GeneralError("ReportSelect.BasicReportStatistics", Err.GetException())
        Return
    End Sub

    Private Function GetBuiltinReportDescription(ByVal whichReport As ReportItemEnum) As String
        ' ----- Return a predefined description for the built-in reports.
        Select Case whichReport
            Case ReportItemEnum.BuiltInCheckedOut
                Return "Displays all items currently checked out, sorted by name."
            Case ReportItemEnum.BuiltInOverdue
                Return "Displays all overdue items, sorted by name."
            Case ReportItemEnum.BuiltInMissing
                Return "Displays all missing items, sorted by name."
            Case ReportItemEnum.BuiltInFinesOwed
                Return "Displays all unpaid fines owed by patrons, " & _
                    "sorted by patron name."
            Case ReportItemEnum.BuiltInStatistics
                Return "Displays some record counts from the Library database."
            Case Else
                Return "There is no description for this report."
        End Select
    End Function
End Class
