' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class ReportSelect
    ' ***** Insert Code Snippet #5 here.

    ' ***** Insert Code Snippet #8 here.

    ' ***** Insert Code Snippet #9 here.

    ' ***** Insert Code Snippet #10 here.

    ' ***** Insert Code Snippet #11 here.

    Private Sub LoadReportGroup(ByVal groupNode As Xml.XmlNode, _
          ByVal indentLevel As Integer)
        ' ----- Add the groups and items at this level,
        '       and recurse as needed.
        Dim scanNode As Xml.XmlNode
        Dim detailNode As Xml.XmlNode
        Dim reportEntry As ReportItem

        On Error GoTo ErrorHandler

        ' ----- Process each item or group.
        For Each scanNode In groupNode.ChildNodes
            ' ----- Build a content item for the list.
            reportEntry = New ReportItem
            reportEntry.Indent = indentLevel

            ' ----- Get the display name.
            detailNode = scanNode.SelectSingleNode("displayText")
            If (detailNode Is Nothing) Then Continue For
            reportEntry.DisplayText = Trim(detailNode.InnerText)

            If (scanNode.Name = "reportGroup") Then
                ' ----- Start a new display group.
                reportEntry.ItemType = ReportItemEnum.GroupLabel
                AllReports.Items.Add(reportEntry)

                ' ----- Recurse to child items.
                LoadReportGroup(scanNode, indentLevel + 1)
            ElseIf (scanNode.Name = "reportItem") Then
                ' ----- This is an item. Record its location.
                detailNode = scanNode.SelectSingleNode("reportPath")
                If Not (detailNode Is Nothing) Then _
                   reportEntry.ReportPath = Trim(detailNode.InnerText)

                ' ----- Get any command-line arguments.
                detailNode = scanNode.SelectSingleNode("reportArgs")
                If Not (detailNode Is Nothing) Then _
                   reportEntry.ReportArgs = Trim(detailNode.InnerText)

                ' ----- Get any item-specific flags.
                detailNode = scanNode.SelectSingleNode("reportFlags")
                If Not (detailNode Is Nothing) Then
                    ' ---- "U" adds "-u loginid" to the command.
                    If (InStr(UCase(detailNode.InnerText), "U") > 0) _
                       And (LoggedInUserName <> "") Then _
                       reportEntry.ReportArgs = _
                       Trim(reportEntry.ReportArgs & " -u " & _
                       LoggedInUserName)
                End If

                ' ----- Store the full description.
                detailNode = scanNode.SelectSingleNode("description")
                If Not (detailNode Is Nothing) Then _
                   reportEntry.Description = Trim(detailNode.InnerText)

                ' ----- So, what type of entry is it?
                If (scanNode.Attributes("type").Value = "built-in") Then
                    ' ----- Built-in program. Check for valid ID.
                    If (IsNumeric(reportEntry.ReportPath) = False) Or _
                       (Val(reportEntry.ReportPath) < 1) Or _
                       (Val(reportEntry.ReportPath) > _
                       CInt(ReportItemEnum.BuiltInStatistics)) Then _
                       Continue For
                    reportEntry.ItemType = CType(CInt( _
                       reportEntry.ReportPath), ReportItemEnum)

                    ' ***** Insert Code Snippet #6 here.

                    AllReports.Items.Add(reportEntry)
                ElseIf (scanNode.Attributes("type").Value = "program") Then
                    ' ----- EXE program-based report.
                    If (reportEntry.ReportPath = "") Then Continue For
                    reportEntry.ItemType = ReportItemEnum.ExeProgram
                    AllReports.Items.Add(reportEntry)
                ElseIf (scanNode.Attributes("type").Value = "url") Then
                    ' ----- URL-based report.
                    If (reportEntry.ReportPath = "") Then Continue For
                    reportEntry.ItemType = ReportItemEnum.UrlProgram
                    AllReports.Items.Add(reportEntry)
                End If
            End If
        Next scanNode
        Return

ErrorHandler:
        GeneralError("ReportSelect.LoadReportGroup", Err.GetException())
        Resume Next
    End Sub

    Private Sub RefreshReportList()
        ' ----- Load in the list of available reports.
        Dim configFile As String
        Dim configData As Xml.XmlDocument
        Dim reportEntry As ReportItem
        Dim counter As Integer

        On Error GoTo ErrorHandler

        ' ----- Clear the existing list.
        AllReports.Items.Clear()

        ' ----- Get the location of the configuration file.
        ' TODO: Load this from the application's configuration.
        '       For now, just hard-code the value.
        configFile = "c:\ReportConfig.txt"

        ' ----- Load the configuration file.
        If (configFile <> "") Then
            If (System.IO.File.Exists(configFile)) Then
                ' ----- Load in the file.
                configData = New Xml.XmlDocument
                configData.Load(configFile)

                ' ----- Process the configuration file.
                LoadReportGroup(configData.DocumentElement, 0)
            End If
        End If

        ' ----- If the configuration file resulted in no reports
        '       appearing in the list, add the default reports.
        If (AllReports.Items.Count = 0) Then
            For counter = 1 To CInt(ReportItemEnum.BuiltInStatistics)
                ' ----- Build the report entry.
                reportEntry = New ReportItem
                reportEntry.Indent = 0
                reportEntry.ItemType = CType(counter, ReportItemEnum)
                Select Case reportEntry.ItemType
                    Case ReportItemEnum.BuiltInCheckedOut
                        reportEntry.DisplayText = "Items Checked Out"
                    Case ReportItemEnum.BuiltInOverdue
                        reportEntry.DisplayText = "Items Overdue"
                    Case ReportItemEnum.BuiltInMissing
                        reportEntry.DisplayText = "Items Missing"
                    Case ReportItemEnum.BuiltInFinesOwed
                        reportEntry.DisplayText = "Patron Fines Owed"
                    Case ReportItemEnum.BuiltInStatistics
                        reportEntry.DisplayText = "Database Statistics"
                End Select

                ' ***** Insert Code Snippet #7 here.

                ' ----- Add the report entry to the list.
                AllReports.Items.Add(reportEntry)
            Next counter
        End If
        Return

ErrorHandler:
        GeneralError("ReportSelect.RefreshReportList", Err.GetException())
        Resume Next
    End Sub

    Private Sub AllReports_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles AllReports.SelectedIndexChanged
        ' ----- Display a description of the report, if available.
        Dim reportEntry As Library.ReportItem

        ' ----- Clear any previous description.
        FullDescription.Text = "No report selected."
        If (AllReports.SelectedIndex <> -1) Then
            ' ----- Locate the content and display it.
            reportEntry = CType(AllReports.SelectedItem, Library.ReportItem)
            FullDescription.Text = reportEntry.Description
        End If
    End Sub

    Private Sub ReportSelect_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' ----- Display the list of reports.
        RefreshReportList()
    End Sub

    Private Sub ActRun_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ActRun.Click
        ' ----- Run the selected report.
        Dim reportEntry As Library.ReportItem

        On Error GoTo ErrorHandler

        ' ----- Make sure a report is selected.
        If (AllReports.SelectedIndex = -1) Then
            MsgBox("Please select a report from the list.", _
               MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, _
               ProgramTitle)
            Return
        End If

        ' ----- Different code for each type of entry.
        reportEntry = CType(AllReports.SelectedItem, Library.ReportItem)
        Me.Cursor = Windows.Forms.Cursors.WaitCursor
        Select Case reportEntry.ItemType
            Case ReportItemEnum.GroupLabel
                ' ----- No report for group entries.
                MsgBox("Please select a report from the list.", _
                   MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, _
                   ProgramTitle)
            Case ReportItemEnum.BuiltInCheckedOut
                ' ----- Items Checked Out

                ' ***** Remove the "' TODO: Write" text from the next line.

                ' TODO: Write BasicReportCheckedOut()
            Case ReportItemEnum.BuiltInOverdue
                ' ----- Items Overdue

                ' ***** Remove the "' TODO: Write" text from the next line.

                ' TODO: Write BasicReportOverdue()
            Case ReportItemEnum.BuiltInMissing
                ' ----- Items Missing

                ' ***** Remove the "' TODO: Write" text from the next line.

                ' TODO: Write BasicReportMissing()
            Case ReportItemEnum.BuiltInFinesOwed
                ' ----- Fines Owed by Patrons

                ' ***** Remove the "' TODO: Write" text from the next line.

                ' TODO: Write BasicReportFines()
            Case ReportItemEnum.BuiltInStatistics
                ' ----- Library Database Statistics

                ' ***** Remove the "' TODO: Write" text from the next line.

                ' TODO: Write BasicReportStatistics()
            Case ReportItemEnum.ExeProgram
                ' ----- Start a program.
                Process.Start("""" & reportEntry.ReportPath & """ " & _
                    reportEntry.ReportArgs)
            Case ReportItemEnum.UrlProgram
                ' ----- Start a URL.
                Process.Start(reportEntry.ReportPath)
        End Select
        Me.Cursor = Windows.Forms.Cursors.Default
        Return

ErrorHandler:
        GeneralError("ReportSelect.ActRun_Click", Err.GetException())
        Resume Next
    End Sub
End Class
