' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class BarcodePrint
    Private Enum BarcodeOutput
        Item = 1
        Patron = 2
        Misc = 3
    End Enum

    Private PreviewMode As Boolean
    Private LabelsX As Integer
    Private LabelsY As Integer
    Private PageWidth As Integer
    Private PageHeight As Integer
    Private PageMarginTop As Integer
    Private PageMarginBottom As Integer
    Private PageMarginLeft As Integer
    Private PageMarginRight As Integer
    Private PageIntraColumn As Integer
    Private PageIntraRow As Integer
    Private PageLabelWidth As Integer
    Private PageLabelHeight As Integer
    Private PageSoFar As Integer
    Private LabelElements As Collections.ArrayList
    Private BarcodeFontName As String
    Private Barcode39 As Boolean
    Private WithEvents BarcodeDoc As Drawing.Printing.PrintDocument

    Private Sub BarcodePrint_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Initialize.
        LabelsX = -1
        LabelsY = -1

        ' ----- Load in the list of number types.
        RecordType.Items.Add(New ListItemData("Item Barcodes", BarcodeOutput.Item))
        RecordType.Items.Add(New ListItemData("Patron Barcodes", BarcodeOutput.Patron))
        RecordType.Items.Add(New ListItemData("Miscellaneous Barcodes", _
            BarcodeOutput.Misc))
        RecordType.SelectedIndex = 0

        ' ----- Load in the available page templates.
        RecordPage.Items.Add(New ListItemData("<Not Selected>", -1))
        sqlText = "SELECT ID, FullName FROM BarcodeSheet ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            RecordPage.Items.Add(New ListItemData(CStr(dbInfo!FullName), _
                CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        RecordPage.SelectedIndex = 0

        ' ----- Don't load in the label templates at this time.
        RecordTemplate.Items.Add(New ListItemData("<Not Selected>", -1))
        RecordTemplate.SelectedIndex = 0
        Return

ErrorHandler:
        GeneralError("BarcodePrint.BarcodePrint_Load", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub RecordPage_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordPage.SelectedIndexChanged
        ' ----- Adjust the display based on the selected page.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim whichPage As Integer

        On Error GoTo ErrorHandler

        ' ----- Clear the previous details selection.
        RecordTemplate.Items.Clear()
        RecordTemplate.Items.Add(New ListItemData("<Not Selected>", -1))
        RecordTemplate.SelectedIndex = 0
        RecordTemplate.Enabled = False
        LabelTemplate.Enabled = False
        RecordCountPerPage.Text = "No template selected."
        RefreshEndCount()

        ' ----- Check for no selection.
        If (RecordPage.SelectedIndex = -1) Then Return
        whichPage = CInt(CType(RecordPage.SelectedItem, ListItemData))
        If (whichPage = -1) Then Return
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Load in the count of the labels.
        sqlText = "SELECT ColumnsCount, RowsCount FROM BarcodeSheet WHERE ID = " & _
            whichPage
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read = False) Then
            Me.Cursor = Windows.Forms.Cursors.Default
            dbInfo.Close()
            Return
        End If
        LabelsX = CInt(dbInfo!ColumnsCount)
        LabelsY = CInt(dbInfo!RowsCount)
        dbInfo.Close()

        ' ----- Update the count of the labels per sheet.
        If ((LabelsX * LabelsY) = 0) Then
            RecordCountPerPage.Text = "No labels per page."
        ElseIf ((LabelsX * LabelsY) = 1) Then
            RecordCountPerPage.Text = "1 label per page (1 by 1)"
        Else
            RecordCountPerPage.Text = CStr(LabelsX * LabelsY) & " labels per page (" & _
                LabelsX & " by " & LabelsY & ")"
        End If
        RefreshEndCount()

        ' ----- Load in the list of templates.
        sqlText = "SELECT ID, FullName FROM BarcodeLabel WHERE BarcodeSheet = " & _
            whichPage & " ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        If (dbInfo.HasRows = False) Then
            Me.Cursor = Windows.Forms.Cursors.Default
            dbInfo.Close()
            Return
        End If
        Do While dbInfo.Read
            RecordTemplate.Items.Add(New ListItemData(CStr(dbInfo!FullName), _
                CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        RecordTemplate.Enabled = True
        LabelTemplate.Enabled = True
        Me.Cursor = Windows.Forms.Cursors.Default
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("BarcodePrint.RecordPage_SelectedIndexChanged", _
            Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub RecordType_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordType.SelectedIndexChanged
        ' ----- Load in the starting value of the labels based on the selection.
        Dim startCount As Decimal
        Dim whichTarget As Integer

        On Error Resume Next

        ' ----- What type of counting will we use?
        If (RecordType.SelectedIndex = -1) Then Return
        whichTarget = CInt(CType(RecordType.SelectedItem, ListItemData))
        If (whichTarget = BarcodeOutput.Item) Then
            ' ----- Item labels.
            startCount = CDec(Val(GetSystemValue("NextBarcodeItem")))
        ElseIf (whichTarget = BarcodeOutput.Patron) Then
            ' ----- Patron labels.
            startCount = CDec(Val(GetSystemValue("NextBarcodePatron")))
        Else
            ' ----- Miscellaneous labels.
            startCount = CDec(Val(GetSystemValue("NextBarcodeMisc")))
        End If
        If (IsNumeric(startCount) = False) Then startCount = 1D
        If (Val(startCount) <= 0) Then startCount = 1D

        ' ----- Update the display.
        RecordStart.Text = CStr(startCount)
        RefreshEndCount()
    End Sub

    Private Sub RecordStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordStart.TextChanged
        ' ----- Update the ending value.
        RefreshEndCount()
    End Sub

    Private Sub RecordStart_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordStart.GotFocus
        ' ----- Highlight the entire text.
        RecordStart.SelectAll()
    End Sub

    Private Sub RecordPages_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordPages.TextChanged
        ' ----- Update the ending value.
        RefreshEndCount()
    End Sub

    Private Sub RecordPages_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordPages.GotFocus
        ' ----- Highlight the entire text.
        RecordPages.SelectAll()
    End Sub

    Private Sub RefreshEndCount()
        ' ----- Update the end count based on the start count.
        Dim endCount As Decimal

        On Error Resume Next

        ' ----- Assume no end point.
        RecordStop.Text = "Not defined."
        If (LabelsX <= 0) Or (LabelsY <= 0) Then Return
        If (IsNumeric(RecordStart.Text) = False) Then Return
        If (IsNumeric(RecordPages.Text) = False) Then Return

        ' ----- Update the display.
        endCount = CDec(RecordStart.Text) + _
            (LabelsX * LabelsY * CDec(RecordPages.Text)) - 1
        RecordStop.Text = CStr(endCount)
    End Sub

    Private Function VerifyFields() As Boolean
        ' ----- Get the name of the barcode font.
        If (BarcodeFontName = "") Then
            BarcodeFontName = GetBarcodeFont()
            If (BarcodeFontName = "") Then Return False
            Barcode39 = CBool(Val(GetSystemValue("BarcodeCode39")))
        End If

        ' ----- A valid page template is required.
        If (CInt(CType(RecordPage.SelectedItem, ListItemData)) = -1) Then
            MsgBox("Please select a Page Template.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordPage.Focus()
            Return False
        End If
        If (LabelsX <= 0) Or (LabelsY <= 0) Then
            MsgBox("The selected Page Template will generate no valid labels.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordPage.Focus()
            Return False
        End If

        ' ----- A valid label template is required.
        If (CInt(CType(RecordTemplate.SelectedItem, ListItemData)) = -1) Then
            MsgBox("Please select a Label Template.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordTemplate.Focus()
            Return False
        End If

        ' ----- The starting value must be valid.
        If (IsNumeric(RecordStart.Text) = False) Then
            MsgBox("Start Value must be numeric.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordStart.Focus()
            Return False
        End If

        ' ----- The pages value must be valid.
        If (IsNumeric(RecordPages.Text) = False) Then
            MsgBox("Pages must be numeric.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordPages.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActPreview.Click
        ' ----- The user wants to preview the labels.
        On Error Resume Next

        ' ----- Make sure the user supplied valid data.
        If (VerifyFields() = False) Then Return

        ' ----- Load in all of the page-specific details to be used in printing.
        If (LoadPageDetails() = False) Then Return

        ' ----- Create the preview dialog.
        Me.Cursor = Windows.Forms.Cursors.WaitCursor
        PageSoFar = 0
        PreviewMode = True
        BarcodeDoc = New System.Drawing.Printing.PrintDocument

        ' ----- Display the preview.
        BarcodePreview.Document = BarcodeDoc
        BarcodePreview.ShowDialog()
        BarcodeDoc = Nothing
        Me.Cursor = Windows.Forms.Cursors.Default
    End Sub

    Private Sub ActPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActPrint.Click
        ' ----- The user wants to print the labels.
        Dim whichTarget As Integer

        On Error Resume Next

        ' ----- Make sure the user supplied valid data.
        If (VerifyFields() = False) Then Return

        ' ----- Load in all of the page-specific details to be used in printing.
        If (LoadPageDetails() = False) Then Return

        ' ----- Prompt to print the labels, and then print.
        Me.Cursor = Windows.Forms.Cursors.WaitCursor
        PageSoFar = 0
        PreviewMode = False
        BarcodeDoc = New System.Drawing.Printing.PrintDocument
        BarcodePrintDialog.Document = BarcodeDoc
        If (BarcodePrintDialog.ShowDialog() <> Windows.Forms.DialogResult.OK) Then
            BarcodeDoc = Nothing
            Me.Cursor = Windows.Forms.Cursors.Default
            Return
        End If
        BarcodeDoc.Print()
        BarcodeDoc = Nothing
        Me.Cursor = Windows.Forms.Cursors.Default

        ' ----- Ask the user about the success of the print.
        If (MsgBox("Did the labels print successfully?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) = _
            MsgBoxResult.No) Then Return

        ' ----- Update the database with the print values.
        whichTarget = CInt(CType(RecordType.SelectedItem, ListItemData))
        If (whichTarget = BarcodeOutput.Item) Then
            ' ----- Item labels.
            SetSystemValue("NextBarcodeItem", CStr(Val(RecordStop.Text) + 1))
        ElseIf (whichTarget = BarcodeOutput.Patron) Then
            ' ----- Patron labels.
            SetSystemValue("NextBarcodePatron", CStr(Val(RecordStop.Text) + 1))
        Else
            ' ----- Miscellaneous labels.
            SetSystemValue("NextBarcodeMisc", CStr(Val(RecordStop.Text) + 1))
        End If

        ' ----- Close the form.
        ActClose.PerformClick()
    End Sub

    Private Function LoadPageDetails() As Boolean
        ' ----- Retrieve the label formatting details from the database.
        '       Return True on success.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim fromUnit As Char
        Dim newBarcodeImage As Library.BarcodeItemBarcodeImage
        Dim newBarcodeText As Library.BarcodeItemBarcodeNumber
        Dim newStaticText As Library.BarcodeItemText
        Dim newStaticLine As Library.BarcodeItemLine
        Dim newStaticRect As Library.BarcodeItemRect

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Load in the page structure details. Store all values in Twips.
        sqlText = "SELECT * FROM BarcodeSheet WHERE ID = " & _
            CInt(CType(RecordPage.SelectedItem, ListItemData))
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        fromUnit = CChar(dbInfo!UnitType)
        PageWidth = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!PageWidth)))
        PageHeight = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!PageHeight)))
        PageMarginTop = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!MarginTop)))
        PageMarginBottom = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!MarginBottom)))
        PageMarginLeft = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!MarginLeft)))
        PageMarginRight = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!MarginRight)))
        PageIntraColumn = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!IntraColumn)))
        PageIntraRow = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!IntraRow)))
        dbInfo.Close()
        PageLabelWidth = CInt((PageWidth - PageMarginLeft - PageMarginRight - _
            ((LabelsX - 1) * PageIntraColumn)) / LabelsX)
        PageLabelHeight = CInt((PageHeight - PageMarginTop - PageMarginBottom - _
            ((LabelsY - 1) * PageIntraRow)) / LabelsY)

        ' ----- Get the units to be used for label items.
        sqlText = "SELECT UnitType FROM BarcodeLabel WHERE ID = " & _
            CInt(CType(RecordTemplate.SelectedItem, ListItemData))
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        fromUnit = CChar(dbInfo!UnitType)
        dbInfo.Close()

        ' ----- Load in the items that appear on each label.
        LabelElements = New System.Collections.ArrayList
        sqlText = "SELECT * FROM BarcodeLabelItem WHERE BarcodeLabel = " & _
            CInt(CType(RecordTemplate.SelectedItem, ListItemData)) & _
            " ORDER BY Priority"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read()
            ' ----- Process this object.
            Select Case UCase(CStr(dbInfo!ItemType))
                Case "B"
                    ' ----- Barcode image.
                    newBarcodeImage = New Library.BarcodeItemBarcodeImage
                    newBarcodeImage.Alignment = CType(CInt(dbInfo!Alignment), _
                        System.Drawing.ContentAlignment)
                    newBarcodeImage.BarcodeColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newBarcodeImage.BarcodeSize = CSng(dbInfo!FontSize)
                    newBarcodeImage.Left = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosLeft))
                    newBarcodeImage.Top = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosTop))
                    newBarcodeImage.Width = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosWidth))
                    newBarcodeImage.Height = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosHeight))
                    newBarcodeImage.RotationAngle = CShort(dbInfo!Rotation)
                    newBarcodeImage.PadDigits = CByte(DBGetInteger(dbInfo!PadDigits))
                    LabelElements.Add(newBarcodeImage)
                Case "L"
                    ' ----- Line.
                    newStaticLine = New Library.BarcodeItemLine
                    newStaticLine.LineColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newStaticLine.X1 = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosLeft))
                    newStaticLine.X2 = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosWidth))
                    newStaticLine.Y1 = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosTop))
                    newStaticLine.Y2 = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosHeight))
                    LabelElements.Add(newStaticLine)
                Case "N"
                    ' ----- Barcode text.
                    newBarcodeText = New Library.BarcodeItemBarcodeNumber
                    newBarcodeText.Alignment = CType(dbInfo!Alignment, _
                        System.Drawing.ContentAlignment)
                    newBarcodeText.Left = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosLeft))
                    newBarcodeText.Top = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosTop))
                    newBarcodeText.Width = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosWidth))
                    newBarcodeText.Height = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosHeight))
                    newBarcodeText.RotationAngle = CShort(dbInfo!Rotation)
                    newBarcodeText.TextColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newBarcodeText.Font = New System.Drawing.Font(CStr(dbInfo!FontName), _
                        CSng(dbInfo!FontSize), BuildFontStyle(DBGetText(dbInfo!FontStyle)))
                    newBarcodeText.PadDigits = CByte(DBGetInteger(dbInfo!PadDigits))
                    LabelElements.Add(newBarcodeText)
                Case "R"
                    ' ----- Rectangle.
                    newStaticRect = New Library.BarcodeItemRect
                    newStaticRect.BorderColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newStaticRect.FillColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color2))
                    newStaticRect.Left = _
                        ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!PosLeft))
                    newStaticRect.Top = _
                        ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!PosTop))
                    newStaticRect.Width = _
                        ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!PosWidth))
                    newStaticRect.Height = _
                        ConvertPageUnits(fromUnit, "T"c, CSng(dbInfo!PosHeight))
                    newStaticRect.RotationAngle = CShort(dbInfo!Rotation)
                    LabelElements.Add(newStaticRect)
                Case "T"
                    ' ----- Static text.
                    newStaticText = New Library.BarcodeItemText
                    newStaticText.DisplayText = CStr(dbInfo!StaticText)
                    newStaticText.Alignment = CType(dbInfo!Alignment, _
                        System.Drawing.ContentAlignment)
                    newStaticText.Left = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosLeft))
                    newStaticText.Top = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosTop))
                    newStaticText.Width = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosWidth))
                    newStaticText.Height = ConvertPageUnits(fromUnit, "T"c, _
                        CSng(dbInfo!PosHeight))
                    newStaticText.RotationAngle = CShort(dbInfo!Rotation)
                    newStaticText.TextColor = _
                        System.Drawing.Color.FromArgb(CInt(dbInfo!Color1))
                    newStaticText.Font = New System.Drawing.Font(CStr(dbInfo!FontName), _
                        CSng(dbInfo!FontSize), BuildFontStyle(DBGetText(dbInfo!FontStyle)))
                    LabelElements.Add(newStaticText)
            End Select
        Loop
        dbInfo.Close()

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("BarcodePrint.LoadPageDetails", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return False
    End Function

    Private Sub BarcodeDoc_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles BarcodeDoc.PrintPage
        ' ----- Print one page of labels.
        Dim pixelPen As System.Drawing.Pen
        Dim rowScan As Integer
        Dim colScan As Integer
        Dim leftOffset As Integer
        Dim topOffset As Integer
        Dim hardLeft As Integer
        Dim hardTop As Integer
        Dim barcodeNumber As Integer
        Dim holdState As System.Drawing.Drawing2D.GraphicsState

        ' ----- Display the outline of labels.
        If (PreviewMode = True) Then
            pixelPen = New System.Drawing.Pen(Color.Cyan, 1 / e.Graphics.DpiX)
            For rowScan = 1 To LabelsY
                For colScan = 1 To LabelsX
                    leftOffset = CInt(ConvertPageUnits("T"c, "H"c, PageMarginLeft + _
                        ((colScan - 1) * (PageIntraColumn + PageLabelWidth))))
                    topOffset = CInt(ConvertPageUnits("T"c, "H"c, PageMarginTop + _
                        ((rowScan - 1) * (PageIntraRow + PageLabelHeight))))
                    e.Graphics.DrawRectangle(Pens.Cyan, leftOffset, topOffset, _
                        ConvertPageUnits("T"c, "H"c, PageLabelWidth), _
                        ConvertPageUnits("T"c, "H"c, PageLabelHeight))
                Next colScan
            Next rowScan
            pixelPen.Dispose()
        End If

        ' ----- Are we printing to a real printer? If so, we need to take into
        '       account the unprintable area. In preview mode, the VisibleClipBound
        '       has a really huge width and height.
        If (e.PageBounds.Width > e.Graphics.VisibleClipBounds.Width) Or _
                (PreviewMode = False) Then
            ' ----- Real print.
            hardLeft = CInt(ConvertPageUnits("H"c, "T"c, _
                (e.PageBounds.Width - e.Graphics.VisibleClipBounds.Width) / 2.0F))
            hardTop = CInt(ConvertPageUnits("H"c, "T"c, _
                (e.PageBounds.Height - e.Graphics.VisibleClipBounds.Height) / 2.0F))
        Else
            ' ----- Print preview.
            hardLeft = 0
            hardTop = 0
        End If

        ' ----- Determine the starting barcode number on this page.
        barcodeNumber = CInt(Val(RecordStart.Text) + (PageSoFar * (LabelsX * LabelsY)))

        ' ----- Process each label.
        For rowScan = 1 To LabelsY
            For colScan = 1 To LabelsX
                ' ----- Find the upper-left corner of the label.
                leftOffset = PageMarginLeft + ((colScan - 1) * _
                    (PageIntraColumn + PageLabelWidth)) - hardLeft
                topOffset = PageMarginTop + ((rowScan - 1) * _
                    (PageIntraRow + PageLabelHeight)) - hardTop

                ' ----- Save the current graphics state. Move into twips mode, and
                '       position the origin to the upper-left corner of the label.
                holdState = e.Graphics.Save()
                e.Graphics.ScaleTransform(100.0F / 1440.0F, 100.0F / 1440.0F)
                e.Graphics.TranslateTransform(leftOffset, topOffset)

                ' ----- Print the label.
                PrintOneLabel(e.Graphics, barcodeNumber)
                barcodeNumber += 1

                ' ----- Restore the original graphics state.
                e.Graphics.Restore(holdState)
            Next colScan
        Next rowScan

        ' ----- Move to the next page.
        PageSoFar += 1
        If (PageSoFar >= CInt(RecordPages.Text)) Then
            ' ----- If this was preview mode, the user might want to click on
            '       the "Print" button included on the preview dialog. Since
            '       we print preview screens differently, we need to ensure
            '       the a true print through this button will print a non-
            '       preview set of labels.
            PreviewMode = False
            e.HasMorePages = False
        Else
            e.HasMorePages = True
        End If
    End Sub

    Private Sub PrintOneLabel(ByVal g As System.Drawing.Graphics, ByVal barcodeNumber As Decimal)
        ' ----- Print out one barcode label.
        Dim counter As Integer
        Dim textMessage As String
        Dim useGeneric As Library.BarcodeItemGeneric
        Dim useBarImage As Library.BarcodeItemBarcodeImage
        Dim useBarText As Library.BarcodeItemBarcodeNumber
        Dim useText As Library.BarcodeItemText
        Dim useLine As Library.BarcodeItemLine
        Dim useRect As Library.BarcodeItemRect
        Dim useFont As System.Drawing.Font
        Dim holdState As System.Drawing.Drawing2D.GraphicsState
        Dim pixelPen As System.Drawing.Pen
        Dim pixelBrush As System.Drawing.SolidBrush
        Dim textColor As System.Drawing.Color
        Dim textAngle As Integer
        Dim textAlign As System.Drawing.ContentAlignment
        Dim textFormat As New Drawing.StringFormat
        Dim X1 As Single
        Dim X2 As Single
        Dim Y1 As Single
        Dim Y2 As Single
        Dim textSize As System.Drawing.SizeF
        Dim scanLabel As System.Collections.IEnumerator

        On Error Resume Next

        ' ----- NOTE: The calling procedure has adjusted the graphics display.
        '       All output is measured in twips, and the origin is at the upper-
        '       left corner of the label to be printed.

        ' ----- Process each item in the list.
        scanLabel = LabelElements.GetEnumerator()
        Do While scanLabel.MoveNext()
            ' ----- Save the current state of the graphics area.
            holdState = g.Save()

            ' ----- Different code for each type of drawing object.
            useGeneric = CType(scanLabel.Current, Library.BarcodeItemGeneric)
            Select Case useGeneric.ItemType
                Case "B"
                    ' ----- Barcode image.
                    useBarImage = CType(useGeneric, Library.BarcodeItemBarcodeImage)
                    textMessage = CStr(barcodeNumber)
                    If (Len(textMessage) < useBarImage.PadDigits) Then
                        textMessage = StrDup(useBarImage.PadDigits - _
                            Len(textMessage), "0") & textMessage
                    End If
                    If (Barcode39 = True) Then textMessage = "*" & textMessage & "*"
                    textColor = useBarImage.BarcodeColor
                    textAngle = useBarImage.RotationAngle
                    textAlign = useBarImage.Alignment

                    ' ----- Get the font to use.
                    useFont = New System.Drawing.Font(BarcodeFontName, _
                        CSng(useBarImage.BarcodeSize * 14.4), _
                        System.Drawing.FontStyle.Regular)

                    ' ----- Get the coordinates in twips.
                    X1 = useBarImage.Left
                    X2 = useBarImage.Width
                    Y1 = useBarImage.Top
                    Y2 = useBarImage.Height

                    ' ----- The rest is the same as drawing normal text.
                    GoTo DisplayTheText
                Case "L"
                    ' ----- Line.
                    useLine = CType(useGeneric, Library.BarcodeItemLine)

                    ' ----- Create a one-pixel pen to use.
                    pixelPen = New System.Drawing.Pen(useLine.LineColor, 1 / g.DpiX)

                    ' ----- Get the coordinates in twips.
                    X1 = useLine.X1
                    X2 = useLine.X2
                    Y1 = useLine.Y1
                    Y2 = useLine.Y2

                    ' ----- Draw the line.
                    g.DrawLine(pixelPen, X1, Y1, X2, Y2)
                    pixelPen.Dispose()
                Case "N"
                    ' ----- Barcode text.
                    useBarText = CType(useGeneric, Library.BarcodeItemBarcodeNumber)
                    textMessage = CStr(barcodeNumber)
                    If (Len(textMessage) < useBarText.PadDigits) Then
                        textMessage = StrDup(useBarText.PadDigits - _
                            Len(textMessage), "0") & textMessage
                    End If
                    textColor = useBarText.TextColor
                    textAngle = useBarText.RotationAngle
                    textAlign = useBarText.Alignment

                    ' ----- Get the font to use.
                    useFont = New System.Drawing.Font(useBarText.Font.Name, _
                        CSng(useBarText.Font.SizeInPoints * 14.4), useBarText.Font.Style)

                    ' ----- Get the coordinates in twips.
                    X1 = useBarText.Left
                    X2 = useBarText.Width
                    Y1 = useBarText.Top
                    Y2 = useBarText.Height

                    ' ----- The rest is the same as drawing normal text.
                    GoTo DisplayTheText
                Case "R"
                    ' ----- Rectangle.
                    useRect = CType(useGeneric, Library.BarcodeItemRect)

                    ' ----- Create a one-pixel pen to use.
                    pixelPen = New System.Drawing.Pen(useRect.BorderColor, 1 / g.DpiX)
                    pixelBrush = New System.Drawing.SolidBrush(useRect.FillColor)

                    ' ----- Get the coordinates in twips.
                    X1 = useRect.Left
                    X2 = useRect.Width
                    Y1 = useRect.Top
                    Y2 = useRect.Height

                    ' ----- Rotate the surface if requested.
                    If (useRect.RotationAngle <> 0) Then
                        g.TranslateTransform(X1, Y1)
                        g.RotateTransform(useRect.RotationAngle)
                        X1 = 0.0F
                        Y1 = 0.0F
                    End If

                    ' ----- Draw the rectangle.
                    g.FillRectangle(pixelBrush, X1, Y1, X2, Y2)
                    g.DrawRectangle(pixelPen, X1, Y1, X2, Y2)
                    pixelBrush.Dispose()
                    pixelPen.Dispose()
                Case "T"
                    ' ----- Text.
                    useText = CType(useGeneric, Library.BarcodeItemText)
                    textMessage = useText.DisplayText
                    textColor = useText.TextColor
                    textAngle = useText.RotationAngle
                    textAlign = useText.Alignment

                    ' ----- Get the font to use.
                    useFont = New System.Drawing.Font(useText.Font.Name, _
                        CSng(useText.Font.SizeInPoints * 14.4), useText.Font.Style)

                    ' ----- Get the coordinates in points.
                    X1 = useText.Left
                    X2 = useText.Width
                    Y1 = useText.Top
                    Y2 = useText.Height

DisplayTheText:
                    ' ----- Rotate around the upper-left corner of the text box
                    '       if needed.
                    If (textAngle <> 0) Then
                        g.TranslateTransform(X1, Y1)
                        g.RotateTransform(textAngle)
                        X1 = 0.0F
                        Y1 = 0.0F
                    End If

                    ' ----- Set the clipping region.
                    g.SetClip(New System.Drawing.RectangleF(X1, Y1, X2, Y2))

                    ' ----- Position the text within the bounding box.
                    Select Case textAlign
                        Case ContentAlignment.TopLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Far
                        Case ContentAlignment.TopCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Far
                        Case ContentAlignment.TopRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Far
                    End Select
                    textFormat.FormatFlags = textFormat.FormatFlags Or _
                        StringFormatFlags.NoWrap

                    ' ----- Print the text.
                    g.DrawString(textMessage, useFont, _
                        New System.Drawing.SolidBrush(textColor), _
                        New Drawing.RectangleF(X1, Y1, X2, Y2), textFormat)
                    useFont.Dispose()
            End Select

            ' ----- Restore the original transformed state of the graphics surface.
            g.Restore(holdState)
        Loop
    End Sub
End Class
