' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class ItemCopy
    Private ActiveItem As Integer
    Private ActiveCopy As Integer

    Public Function AddCopy(ByVal itemID As Integer, ByVal useItemName As String) As Integer
        ' ----- A default location must be defined.
        If (ConfirmDefaultLocation() = False) Then Return -1

        ' ----- Prompt the user to add a new item copy.
        ActiveItem = itemID
        ActiveCopy = -1
        ItemName.Text = useItemName
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveCopy Else Return -1
    End Function

    Public Function EditCopy(ByVal itemID As Integer, ByVal copyID As Integer, _
            ByVal useItemName As String) As Integer
        ' ----- A default location must be defined.
        If (ConfirmDefaultLocation() = False) Then Return -1

        ' ----- Prompt the user to edit an existing item copy.
        ActiveItem = itemID
        ActiveCopy = copyID
        ItemName.Text = useItemName
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveCopy Else Return -1
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save the record to the database.
        Dim sqlText As String
        Dim newID As Integer

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Different code for new and existing copies.
        If (ActiveCopy = -1) Then
            ' ----- Add a new record.
            sqlText = "INSERT INTO ItemCopy (ItemID, CopyNumber, Description, " & _
                "Barcode, Available, Reference, Missing, Acquired, Cost, Status, " & _
                "Condition, Location) OUTPUT INSERTED.ID VALUES (" & ActiveItem & _
                ", " & RecordCopy.Text & _
                ", " & DBText(Trim(RecordDescription.Text)) & _
                ", " & DBText(Trim(RecordBarcode.Text)) & _
                ", " & CInt(RecordAvailable.Checked) & _
                ", " & CInt(RecordReference.Checked) & _
                ", " & CInt(RecordMissing.Checked) & _
                ", " & DBDate(RecordAcquired.Text) & _
                ", " & DBNum(RecordCost.Text) & _
                ", " & DBCombo(RecordGeneralStatus) & _
                ", " & DBText(Trim(RecordCondition.Text)) & _
                ", " & DBCombo(RecordLocation) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveCopy = newID
        Else
            ' ----- Update an existing record.
            sqlText = "UPDATE ItemCopy SET" & _
                " CopyNumber = " & RecordCopy.Text & _
                ", Description = " & DBText(Trim(RecordDescription.Text)) & _
                ", Barcode = " & DBText(Trim(RecordBarcode.Text)) & _
                ", Available = " & CInt(RecordAvailable.Checked) & _
                ", Reference = " & CInt(RecordReference.Checked) & _
                ", Missing = " & CInt(RecordMissing.Checked) & _
                ", Acquired = " & DBDate(RecordAcquired.Text) & _
                ", Cost = " & DBNum(RecordCost.Text) & _
                ", Status = " & DBCombo(RecordGeneralStatus) & _
                ", Condition = " & DBText(Trim(RecordCondition.Text)) & _
                ", Location = " & DBCombo(RecordLocation) & _
                " WHERE ID = " & ActiveCopy
            ExecuteSQL(sqlText)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("ItemCopy.SaveFormData", Err.GetException())
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Confirm the fields on the form before saving.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Copy number is required, and it must be numeric.
        If (Trim(RecordCopy.Text) = "") Then
            MsgBox("Copy Number is required.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordCopy.Focus()
            Return False
        End If
        If (DigitsOnly(RecordCopy.Text) <> Trim(RecordCopy.Text)) Then
            MsgBox("Copy Number must be numeric.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordCopy.Focus()
            Return False
        End If

        ' ----- Make sure the copy number is unique.
        sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE CopyNumber = " & RecordCopy.Text & _
            " AND ItemID = " & ActiveItem
        If (ActiveCopy <> -1) Then sqlText &= " AND ID <> " & ActiveCopy
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            ' ----- Duplicate entry.
            MsgBox("A record already exists with the copy number '" & _
                Trim(RecordCopy.Text) & "'.", MsgBoxStyle.OkOnly Or _
                MsgBoxStyle.Exclamation, ProgramTitle)
            RecordCopy.Focus()
            Return False
        End If

        ' ----- Barcode, if supplied, must be numeric.
        If (Trim(RecordBarcode.Text) <> "") Then
            If (DigitsOnly(RecordBarcode.Text) <> Trim(RecordBarcode.Text)) Then
                MsgBox("Barcode, if supplied, must be numeric.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                RecordBarcode.Focus()
                Return False
            End If
        End If

        ' ----- Make sure the bar code is not already in use.
        If (Trim(RecordBarcode.Text) <> "") Then
            sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE UPPER(Barcode) = " & _
                DBText(UCase(Trim(RecordBarcode.Text)))
            If (ActiveCopy <> -1) Then sqlText &= " AND ID <> " & ActiveCopy
            If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
                ' ----- Duplicate entry.
                MsgBox("A record already exists with the barcode'" & _
                    Trim(RecordBarcode.Text) & "'.", MsgBoxStyle.OkOnly Or _
                    MsgBoxStyle.Exclamation, ProgramTitle)
                RecordBarcode.Focus()
                Return False
            End If
        End If

        ' ----- Date acquired, if supplied, must be a valid date.
        If (Trim(RecordAcquired.Text) <> "") Then
            If (IsDate(RecordAcquired.Text) = False) Then
                MsgBox("Date Acquired, if supplied, must be a valid date.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                RecordAcquired.Focus()
                Return False
            End If
        End If

        ' ----- Item cost, if supplied, must be numeric.
        If (Trim(RecordCost.Text) <> "") Then
            If (IsNumeric(RecordCost.Text) = False) Then
                MsgBox("Item Cost, if supplied, must be numeric.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                RecordCost.Focus()
                Return False
            End If
        End If

        ' ----- General status is required.
        If (CInt(CType(RecordGeneralStatus.SelectedItem, ListItemData)) = -1) Then
            MsgBox("General Status is required.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordGeneralStatus.Focus()
            Return False
        End If

        ' ----- Location is required.
        If (CInt(CType(RecordLocation.SelectedItem, ListItemData)) = -1) Then
            MsgBox("Location is required.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordLocation.Focus()
            Return False
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("ItemCopy.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save the record.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub ItemCopy_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Load in the list of status codes.
        RecordGeneralStatus.Items.Add(New ListItemData("<Not Selected>", -1))
        RecordGeneralStatus.SelectedIndex = 0
        sqlText = "SELECT * FROM CodeCopyStatus ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            RecordGeneralStatus.Items.Add(New ListItemData( _
                CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- Load in the list of locations.
        RecordLocation.Items.Add(New ListItemData("<Not Selected>", -1))
        RecordLocation.SelectedIndex = 0
        sqlText = "SELECT * FROM CodeLocation ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            RecordLocation.Items.Add(New ListItemData( _
                CStr(dbInfo!FullName), CInt(dbInfo!ID)))
            If (CInt(dbInfo!ID) = DefaultItemLocation) Then _
                RecordLocation.SelectedIndex = RecordLocation.Items.Count - 1
        Loop
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- For new items, that's it.
        If (ActiveCopy = -1) Then Return

        ' ----- Load in the existing item.
        sqlText = "SELECT * FROM ItemCopy WHERE ID = " & ActiveCopy
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        RecordCopy.Text = CStr(dbInfo!CopyNumber)
        RecordDescription.Text = DBGetText(dbInfo!Description)
        RecordBarcode.Text = DBGetText(dbInfo!Barcode)
        RecordAvailable.Checked = CBool(dbInfo!Available)
        RecordReference.Checked = CBool(dbInfo!Reference)
        RecordMissing.Checked = CBool(dbInfo!Missing)
        If (IsDBNull(dbInfo!Acquired) = False) Then _
            RecordAcquired.Text = Format(CDate(dbInfo!Acquired), "M/d/yyyy")
        If (IsDBNull(dbInfo!Cost) = False) Then _
            RecordCost.Text = Format(CDec(dbInfo!Cost), "0.00")
        RecordGeneralStatus.SelectedIndex = _
            RecordGeneralStatus.Items.IndexOf(CInt(dbInfo!Status))
        If (RecordGeneralStatus.SelectedIndex = -1) Then _
            RecordGeneralStatus.SelectedIndex = _
            RecordGeneralStatus.Items.IndexOf(-1)
        RecordCondition.Text = DBGetText(dbInfo!Condition)
        If (IsDBNull(dbInfo!Location) = False) Then _
            RecordLocation.SelectedIndex = _
            RecordLocation.Items.IndexOf(CInt(dbInfo!Location))
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- Display the disposition of the item.
        RecordStatus.Text = GetCopyDisposition(ActiveCopy)
        Return

ErrorHandler:
        GeneralError("ItemCopy.ItemCopy_Load", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub RecordCopy_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordCopy.Enter
        ' ----- Highlight the entire text.
        RecordCopy.SelectAll()
    End Sub

    Private Sub RecordAcquired_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordAcquired.Enter
        ' ----- Highlight the entire text.
        RecordAcquired.SelectAll()
    End Sub

    Private Sub RecordCost_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordCost.Enter
        ' ----- Highlight the entire text.
        RecordCost.SelectAll()
    End Sub

    Private Sub RecordCondition_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordCondition.Enter
        ' ----- Highlight the entire text.
        RecordCondition.SelectAll()
    End Sub

    Private Sub StatusDetails_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StatusDetails.Click
        ' ----- Show details about the disposition of the item.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim response As String
        Dim amountDue As Decimal

        On Error GoTo ErrorHandler

        ' ----- Do a quick check for new items.
        If (ActiveCopy = -1) Then
            MsgBox("This is a new item.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
            Return
        End If
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Retrieve information about the book.
        sqlText = "SELECT IC.Reference, IC.Missing, PC.Missing AS PatronMissing, " & _
            "PC.DueDate, PC.Patron, PT.LastName, PT.FirstName, PC.Fine, PC.Paid, " & _
            "PC.CheckOut FROM ItemCopy AS IC LEFT JOIN PatronCopy AS PC " & _
            "ON IC.ID = PC.ItemCopy LEFT JOIN Patron AS PT " & _
            "ON PC.Patron = PT.ID WHERE IC.ID = " & ActiveCopy & _
            " AND PC.Returned = 0"
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read = False) Then
            ' ----- Perhaps the database, due to the JOIN clause, could not return
            '       the base record of a checked-in book.
            dbInfo.Close()
            sqlText = "SELECT Reference, Missing, NULL AS PatronMissing, " & _
                "NULL AS DueDate, NULL AS Patron, NULL AS LastName, " & _
                "NULL AS FirstName, NULL AS Fine, NULL AS Paid, " & _
                "NULL AS CheckOut FROM ItemCopy WHERE ID = " & ActiveCopy
            dbInfo = CreateReader(sqlText)
            If (dbInfo.Read = False) Then
                ' ----- What happened to the record?
                response = "An error occurred while trying to determine the " & _
                    "disposition of the item."
                GoTo ShowResponse
            End If
        End If

        ' ----- We found the record. Show its details.
        If (IsDBNull(dbInfo!Patron) = False) Then
            ' ----- This item is checked out.
            If (CBool(dbInfo!Missing) = True) Or (CBool(dbInfo!PatronMissing) = True) Then
                response = "This item was reported missing by patron "
            ElseIf (CDate(dbInfo!DueDate) < Today) Then
                response = "This item is overdue. It was last due on " & _
                    Format(CDate(dbInfo!DueDate), "MMMM d, yyyy") & " by patron "
            Else
                response = "This item is currently checked out. It is due on " & _
                    Format(CDate(dbInfo!DueDate), "MMMM d, yyyy") & " by patron "
            End If
            response &= CStr(dbInfo!FirstName) & " " & CStr(dbInfo!LastName) & ". " & _
                "It was originally checked out on " & Format(CDate(dbInfo!CheckOut), _
                "MMMM d, yyyy") & "."
            amountDue = 0D
            If (IsDBNull(dbInfo!Fine) = False) Then amountDue += CDec(dbInfo!Fine)
            If (IsDBNull(dbInfo!Paid) = False) Then amountDue -= CDec(dbInfo!Paid)
            If (amountDue > 0D) Then response &= " The patron owes " & Format(amountDue, _
                "$0.00") & " in fines on this item."
        Else
            ' ----- This book is currently checked in.
            If (CBool(dbInfo!Missing) = True) Then
                response = "This item has been reported as missing."
            ElseIf (CBool(dbInfo!Reference) = True) Then
                response = "This item is in the reference collection."
            Else
                response = "This item is currently checked in."
            End If
        End If
        dbInfo.Close()
        dbInfo = Nothing

ShowResponse:
        ' ----- Display the disposition.
        Me.Cursor = Windows.Forms.Cursors.Default
        MsgBox(response, MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("ItemCopy.StatusDetails_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub RecordBarcode_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordBarcode.Enter
        ' ----- Highlight the entire text.
        RecordBarcode.SelectAll()

        ' ----- Don't allow Enter to close the form.
        Me.AcceptButton = Nothing
    End Sub

    Private Sub RecordBarcode_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordBarcode.Leave
        ' ----- Allow Enter to close the form again.
        Me.AcceptButton = ActOK
    End Sub

    Private Sub RecordBarcode_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles RecordBarcode.KeyPress
        ' ----- Ignore the enter key.
        If (e.KeyChar = ChrW(Keys.Return)) Then e.Handled = True
    End Sub
End Class
