' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports System.Text
Imports System.Security.Cryptography

' ***** Insert Code Snippet #1 here.

Friend Module General
    ' ***** Insert Code Snippet #2 here.

    ' ***** Insert Code Snippet #3 here.

    ' ***** Insert Code Snippet #4 here.

    ' ----- Public constants.
    Public Const ProgramTitle As String = "The Library Project"
    Public Const NotAuthorizedMessage As String = _
        "You are not authorized to perform this task."
    Public Const UseDBVersion As Integer = 1

    ' ----- Constants for the MatchingImages image list.
    Public Const MatchPresent As Integer = 0
    Public Const MatchNone As Integer = 1

    Public Enum LookupMethods As Integer
        ByTitle = 1
        ByAuthor = 2
        BySubject = 3
        ByKeywordAny = 4
        ByKeywordAll = 5
        ByPublisher = 6
        BySeries = 7
        BySeriesID = 8
        ByBarcode = 9
        ByDatabaseID = 10
        ByAuthorID = 11
        BySubjectID = 12
        ByPublisherID = 13
    End Enum

    ' ----- Security values.
    Public Enum LibrarySecurity As Integer
        ManageAuthors = 1
        ManageAuthorTypes = 2
        ManageCopyStatus = 3
        ManageMediaTypes = 4
        ManageSeries = 5
        ManageGroups = 6
        ManageItems = 7
        ManagePatrons = 8
        ManagePublishers = 9
        ManageValues = 10
        ManageUsers = 11
        ProcessFees = 12
        ManageLocations = 13
        CheckOutItems = 14
        CheckInItems = 15
        AdminFeatures = 16
        DailyProcessing = 17
        RunReports = 18
        PatronOverride = 19
        ManageBarcodeTemplates = 20
        ManageHolidays = 21
        ManagePatronGroups = 22
        ViewAdminPatronMessages = 23
    End Enum
    Public Const MaxLibrarySecurity As LibrarySecurity = LibrarySecurity.ViewAdminPatronMessages

    ' ----- Global variables.
    Public LibraryDB As System.Data.SqlClient.SqlConnection
    Public HoldTransaction As System.Data.SqlClient.SqlTransaction
    Public LoggedInUserID As Integer
    Public LoggedInUserName As String
    Public LoggedInGroupID As Integer
    Public SecurityProfile(MaxLibrarySecurity) As Boolean
    Public DefaultItemLocation As Integer
    Public SearchMatchLimit As Integer
    Public MainHelpFile As String
    Public MainAdminHelpFile As String
    Public FineGraceDays As Integer
    Public AllHolidays As Collections.Generic.List(Of Library.HolidaySet)

    ' ***** Insert Code Snippet #5 here.

    ' ***** Insert Code Snippet #8 here.

    Public Function BuildFontStyle(ByVal textStyle As String) As System.Drawing.FontStyle
        ' ----- Given a string with the possible letters "BIUK", convert it to a
        '       font style object with Bold, Italic, Underline, or Strikeout.
        '       See the DBFontStyle function for the reverse of this function.
        Dim resultStyle As New System.Drawing.FontStyle

        ' ----- Process the string.
        resultStyle = FontStyle.Regular
        If (InStr(UCase(textStyle), "B") > 0) Then _
            resultStyle = resultStyle Or FontStyle.Bold
        If (InStr(UCase(textStyle), "I") > 0) Then _
            resultStyle = resultStyle Or FontStyle.Italic
        If (InStr(UCase(textStyle), "U") > 0) Then _
            resultStyle = resultStyle Or FontStyle.Underline
        If (InStr(UCase(textStyle), "K") > 0) Then _
            resultStyle = resultStyle Or FontStyle.Strikeout
        Return resultStyle
    End Function

    Public Function CalculatePatronFines(ByVal patronID As Integer) As Decimal
        ' ----- Given a patron ID, calculate the fines due.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Retrieve the fine records for the patron.
        sqlText = "SELECT SUM(Fine - Paid) FROM PatronCopy WHERE Patron = " & patronID
        Return DBGetDecimal(ExecuteSQLReturn(sqlText))

ErrorHandler:
        GeneralError("CalculatePatronFines", Err.GetException())
        Return 0@
    End Function

    Public Function CenterText(ByVal origText As String, ByVal textWidth As Integer) As String
        ' ----- Center a piece of text in a field width. If the text is too
        '       wide, truncate it.
        Dim resultText As String

        resultText = Trim(origText)
        If (Len(resultText) >= textWidth) Then
            ' ----- Truncate as needed.
            Return Trim(Left(origText, textWidth))
        Else
            ' ----- Start with extra spaces.
            Return Space((textWidth - Len(origText)) \ 2) & resultText
        End If
    End Function

    Public Sub CleanUpProgram()
        ' ----- Prepare to exit the program.
        On Error Resume Next
        LibraryDB.Close()
    End Sub

    Public Function ConfirmDefaultLocation() As Boolean
        ' ----- Return true if a default location is defined.
        Dim locationID As String

        ' ----- Obtain the default location if needed.
        If (DefaultItemLocation = -1) Then
            locationID = GetSystemValue("DefaultLocation")
            If (locationID = "") Then locationID = "-1"
            DefaultItemLocation = CInt(locationID)
        End If

        ' ----- Return the status result.
        If (DefaultItemLocation = -1) Then
            MsgBox("You must first define the default item location.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        Else
            Return True
        End If
    End Function

    Public Function ConnectDatabase() As Boolean
        ' ----- Connect to the database. Return True on success.
        Dim connectionString As String
        Dim configChanged As Boolean

        ' ----- Initialize.
        HoldTransaction = Nothing
        configChanged = False

        ' ----- Obtain the connection string.
        If (Trim(My.Settings.DBConnection & "") = "") Then
            ' ----- Inform the user about the need to configure the database.
            If (MsgBox("This copy of the application has not been configured to " & _
                "connect to the library database. If you know the database settings, " & _
                "you can configure it now. Would you like to proceed?", _
                MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
                <> MsgBoxResult.Yes) Then Return False

            ' ----- Prompt for the new connection details.
            connectionString = LocateDatabase.PromptUser()
            If (connectionString = "") Then Return False
            configChanged = True
        Else
            connectionString = My.Settings.DBConnection
        End If

TryConnectingAgain:
        ' ----- Attempt to open the database.
        Try
            LibraryDB = New SqlClient.SqlConnection(connectionString)
            LibraryDB.Open()
        Catch ex As Exception
            ' ----- Some database failure.
            GeneralError("ConnectDatabase", ex)

            ' ----- Perhaps it is just a configuration issue.
            If (MsgBox("The connection to the database may have failed due to " & _
                "invalid configuration settings. Would you like to change the " & _
                "database configuration at this time?", _
                MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
                <> MsgBoxResult.Yes) Then Return False

            ' ----- Prompt for new details.
            connectionString = LocateDatabase.PromptUser()
            If (connectionString = "") Then Return False
            configChanged = True
            GoTo TryConnectingAgain
        End Try

        ' ----- Save the udpated configuration if needed.
        If (configChanged = True) Then _
            My.Settings.DBConnection = connectionString

        ' ----- Success.
        Return True
    End Function

    Public Function ConvertPageUnits(ByVal fromUnit As Char, ByVal toUnit As Char, _
        ByVal baseSize As Single) As Single
        ' ----- Convert a unit from one size to another. The units are:
        '       I = Inches
        '       C = Centimeters
        '       P = Points
        '       T = Twips
        '       H = 1/100 of an inch
        Dim workSize As Single

        On Error Resume Next

        ' ----- First, convert everything to twips.
        Select Case UCase(fromUnit)
            Case "C"c : workSize = baseSize * (1440.0F / 2.54F)
            Case "H"c : workSize = baseSize * (1440.0F / 100.0F)
            Case "I"c : workSize = baseSize * 1440.0F
            Case "P"c : workSize = baseSize * (1440.0F / 72.0F)
            Case "T"c : workSize = baseSize
            Case Else : Return baseSize
        End Select

        ' ----- Now convert it to the requested units.
        Select Case UCase(toUnit)
            Case "C"c : Return workSize / (1440.0F / 2.54F)
            Case "H"c : Return workSize / (1440.0F / 100.0F)
            Case "I"c : Return workSize / 1440.0F
            Case "P"c : Return workSize / (1440.0F / 72.0F)
            Case "T"c : Return workSize
            Case Else : Return baseSize
        End Select
    End Function

    Public Function CountSubStr(ByVal mainText As String, ByVal subText As String) As Integer
        ' ----- Return a count of the number of times that a subText occurs in
        '       a string (mainText).
        Dim totalTimes As Integer
        Dim startPos As Integer
        Dim foundPos As Integer

        totalTimes = 0
        startPos = 1

        ' ----- Keep searching until we don't find it no more!
        Do
            ' ----- Search for the subText.
            foundPos = InStr(startPos, mainText, subText)
            If (foundPos = 0) Then Exit Do
            totalTimes = totalTimes + 1

            ' ----- Move to just after the occurrence.
            startPos = foundPos + Len(subText)
        Loop

        ' ----- Return the count.
        Return totalTimes
    End Function

    Public Function CreateDataTable(ByVal sqlText As String) As Data.DataTable
        ' ----- Given a SQL statement, return a data table.
        Dim dbCommand As SqlClient.SqlCommand
        Dim dbAdapter As SqlClient.SqlDataAdapter
        Dim dbTable As Data.DataTable

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        dbCommand = New SqlClient.SqlCommand(sqlText, LibraryDB)
        If Not (HoldTransaction Is Nothing) Then dbCommand.Transaction = HoldTransaction
        dbAdapter = New SqlClient.SqlDataAdapter(dbCommand)
        dbTable = New Data.DataTable
        dbAdapter.Fill(dbTable)
        dbAdapter = Nothing
        dbCommand = Nothing
        Return dbTable
    End Function

    Public Function CreateReader(ByVal sqlText As String) As SqlClient.SqlDataReader
        ' ----- Given a SQL statement, return a data reader.
        Dim dbCommand As SqlClient.SqlCommand
        Dim dbScan As SqlClient.SqlDataReader

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        dbCommand = New SqlClient.SqlCommand(sqlText, LibraryDB)
        If Not (HoldTransaction Is Nothing) Then dbCommand.Transaction = HoldTransaction
        dbScan = dbCommand.ExecuteReader()
        dbCommand = Nothing
        Return dbScan
    End Function

    Public Function DBCombo(ByRef whichField As ComboBox) As String
        ' ----- Prepare the value of a combobox for use in the database.
        Dim listItem As Library.ListItemData

        ' ----- Get the stored data for this item.
        If (whichField.SelectedItem Is Nothing) Then Return "NULL"
        listItem = CType(whichField.SelectedItem, Library.ListItemData)

        ' ----- An item value of "-1" has special meaning: unselected.
        If (listItem.ItemData = -1) Then
            Return "NULL"
        Else
            Return CStr(listItem.ItemData)
        End If
    End Function

    Public Function DBDate(ByVal origText As String) As String
        ' ----- Prepare a date for insertion in a SQL statement.
        If (Trim(origText) = "") Then
            Return "NULL"
        ElseIf (IsDate(origText)) Then
            Return "'" & Format(CDate(origText), "d-MMM-yyyy") & "'"
        Else
            Return "NULL"
        End If
    End Function

    Public Function DBDate(ByVal origDate As Date) As String
        ' ----- Prepare a date for insertion in a SQL statement.
        Return "'" & Format(origDate, "d-MMM-yyyy") & "'"
    End Function

    Public Function DBFontStyle(ByVal useFont As System.Drawing.Font) As String
        ' ----- Given a font, extract to a string the possible letters "BIUK"
        '      (bold, italic, underline, strikeout). See the BuildFontStyle
        '      function for the reverse of this function.
        Dim resultStyle As String

        ' ----- Process the string.
        resultStyle = ""
        If (useFont.Bold = True) Then resultStyle &= "B"
        If (useFont.Italic = True) Then resultStyle &= "I"
        If (useFont.Underline = True) Then resultStyle &= "U"
        If (useFont.Strikeout = True) Then resultStyle &= "K"
        Return resultStyle
    End Function

    Public Function DBGetDecimal(ByRef dataField As Object) As Decimal
        ' ----- Return the decimal equivalent of an optional database field.
        If (IsDBNull(dataField) = True) Then
            Return 0@
        Else
            Return CDec(dataField)
        End If
    End Function

    Public Function DBGetInteger(ByRef dataField As Object) As Integer
        ' ----- Return the integer equivalent of an optional database field.
        If (IsDBNull(dataField) = True) Then
            Return 0
        Else
            Return CInt(dataField)
        End If
    End Function

    Public Function DBGetText(ByRef dataField As Object) As String
        ' ----- Return the text equivalent of an optional database field.
        If (IsDBNull(dataField) = True) Then
            Return ""
        Else
            Return CStr(dataField)
        End If
    End Function

    Public Function DBNum(ByVal origText As String) As String
        ' ----- Prepare a number for insertion in a SQL statement.
        If (Trim(origText) = "") Then
            Return "NULL"
        Else
            Return Trim(origText)
        End If
    End Function

    Public Function DBText(ByVal origText As String) As String
        ' ----- Prepare a string for insertion in a SQL statement.
        If (Trim(origText) = "") Then
            Return "NULL"
        Else
            Return "'" & Replace(origText, "'", "''") & "'"
        End If
    End Function

    Public Function DigitsOnly(ByVal origText As String) As String
        ' ----- Return only the digits found in a string.
        Dim destText As String
        Dim counter As Integer

        ' ----- Examine each character.
        destText = ""
        For counter = 1 To Len(origText)
            If (IsNumeric(Mid(origText, counter, 1))) Then _
                destText &= Mid(origText, counter, 1)
        Next counter
        Return destText
    End Function

    Public Function EncryptPassword(ByVal loginID As String, _
            ByVal passwordText As String) As String
        ' ----- Given a user name and a password, encrypt the password so that it
        '       is not easy to decrypt. There is no limit on password length since
        '       it is going to be hashed anyway.
        Dim hashingFunction As HMACSHA1
        Dim secretKey() As Byte
        Dim hashValue() As Byte
        Dim counter As Integer
        Dim result As String = ""

        ' ----- Prepare the secret key. Force it to uppercase for consistency,
        '       and then stuff it in a byte array.
        secretKey = (New UnicodeEncoding).GetBytes(UCase(loginID))

        ' ----- Create the hashing component using Managed SHA-1.
        hashingFunction = New HMACSHA1(secretKey, True)

        ' ----- Calculate the hash value. One simple line of code.
        hashValue = hashingFunction.ComputeHash( _
            (New UnicodeEncoding).GetBytes(passwordText))

        ' ----- The hash value is ready, but I like things in plaintext when
        '       possible. Let's convert it to a long hex string.
        For counter = 0 To hashValue.Length - 1
            result &= Hex(hashValue(counter))
        Next counter

        ' ----- Stored passwords are limited to 20 characters.
        Return Left(result, 20)
    End Function

    Public Sub ExecuteSQL(ByVal sqlText As String)
        ' ----- Given a SQL statement, run it.
        Dim DBCommand As SqlClient.SqlCommand

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        DBCommand = New SqlClient.SqlCommand(sqlText, LibraryDB)
        If Not (HoldTransaction Is Nothing) Then DBCommand.Transaction = HoldTransaction
        DBCommand.ExecuteNonQuery()
    End Sub

    Public Function ExecuteSQLReturn(ByVal sqlText As String) As Object
        ' ----- Given a SQL statement, run it, and return a related result.
        Dim DBCommand As SqlClient.SqlCommand

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        DBCommand = New SqlClient.SqlCommand(sqlText, LibraryDB)
        If Not (HoldTransaction Is Nothing) Then DBCommand.Transaction = HoldTransaction
        Return DBCommand.ExecuteScalar()
    End Function

    Public Function FitTextToWidth(ByVal origText As String, ByVal pixelWidth As Integer, _
            ByVal canvas As System.Drawing.Graphics, _
            ByVal useFont As System.Drawing.Font) As String
        ' ----- Given a text string, make sure it fits in the specified 
        '       pixel width. Truncate and add an ellipsis if needed.
        Dim newText As String

        newText = origText
        If (canvas.MeasureString(newText, useFont).Width() > pixelWidth) Then
            Do While (canvas.MeasureString(newText & "...", useFont).Width() _
                    > pixelWidth)
                newText = Left(newText, newText.Length - 1)
                If (newText = "") Then Exit Do
            Loop
            If (newText <> "") Then newText &= "..."
        End If
        Return newText
    End Function

    Public Function FormatAuthorName(ByRef dbInfo As SqlClient.SqlDataReader) As String
        ' ----- Given an author record, return the formatted name.
        Dim authorName As String

        On Error Resume Next

        ' ----- Format the name.
        authorName = CStr(dbInfo!LastName)
        If (IsDBNull(dbInfo!FirstName) = False) Then
            authorName &= ", " & CStr(dbInfo!FirstName)
            If (IsDBNull(dbInfo!MiddleName) = False) Then _
                authorName &= " " & CStr(dbInfo!MiddleName)
        End If
        If (IsDBNull(dbInfo!Suffix) = False) Then _
            authorName &= ", " & CStr(dbInfo!Suffix)

        ' ----- Add in the birth and death years.
        If (IsDBNull(dbInfo!BirthYear) = False) Or _
                (IsDBNull(dbInfo!DeathYear) = False) Then
            authorName &= " ("
            If (IsDBNull(dbInfo!BirthYear) = True) Then
                authorName &= "????"
            Else
                authorName &= CStr(Math.Abs(CInt(dbInfo!BirthYear)))
                If (CInt(dbInfo!BirthYear) < 0) Then authorName &= "BC"
            End If
            authorName &= "-"
            If (IsDBNull(dbInfo!DeathYear) = False) Then
                authorName &= CStr(Math.Abs(CInt(dbInfo!DeathYear)))
                If (CInt(dbInfo!DeathYear) < 0) Then authorName &= "BC"
            End If
            authorName &= ")"
        End If

        ' ----- Finished.
        Return authorName
    End Function

    Public Sub GeneralError(ByVal routineName As String, _
            ByVal theError As System.Exception)
        ' ----- Report an error to the user.
        On Error Resume Next

        MsgBox("The following error occurred at location '" & routineName & _
            "':" & vbCrLf & vbCrLf & theError.Message, _
            MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
        My.Application.Log.WriteException(theError)
    End Sub

    Public Function GetBarcodeFont() As String
        ' ----- Retrieve the barcode font to use.
        Dim barcodeFontName As String

        barcodeFontName = Trim(GetSystemValue("BarcodeFont"))
        If (barcodeFontName = "") Then MsgBox("The font used to print barcodes has not " & _
            "been identified. You must identify the barcode font through the " & _
            "Application Maintenance features before editing label templates.", _
            MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
        Return barcodeFontName
    End Function

    Public Function GetCopyDisposition(ByVal copyID As Integer) As String
        ' ----- Given the ID of an item copy, return it's current status.
        '       The available results are:
        '         New Item Copy      Error or missing record
        '         Checked In         Copy is checked in and available
        '         Checked Out        Copy is checked out by a patron
        '         Overdue            Copy is checked out and late
        '         Missing            Copy has been reported missing
        '         Reference          Copy is a reference copy
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim response As String

        On Error GoTo ErrorHandler

        ' ----- Retrieve information about the book.
        sqlText = "SELECT IC.Reference, IC.Missing, PC.Missing AS PatronMissing, " & _
            "PC.DueDate, PC.Patron FROM ItemCopy AS IC LEFT JOIN PatronCopy AS PC " & _
            "ON IC.ID = PC.ItemCopy WHERE IC.ID = " & copyID & _
            " AND PC.Returned = 0"
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read = False) Then
            ' ----- Perhaps the database, due to the JOIN clause, could not return
            '       the base record of a checked-in book.
            dbInfo.Close()
            sqlText = "SELECT Reference, Missing, NULL AS PatronMissing, " & _
                "NULL AS DueDate, NULL AS Patron FROM ItemCopy WHERE ID = " & copyID
            dbInfo = CreateReader(sqlText)
            If (dbInfo.Read = False) Then
                ' ----- What happened to the record?
                dbInfo.Close()
                dbInfo = Nothing
                Return "New Item Copy"
            End If
        End If

        ' ----- We found the record. Check its details.
        If (IsDBNull(dbInfo!Patron) = False) Then
            ' ----- This item is checked out.
            If (CBool(dbInfo!Missing) = True) Or (CBool(dbInfo!PatronMissing) = True) Then
                response = "Missing"
            ElseIf (CDate(dbInfo!DueDate) < Today) Then
                response = "Overdue"
            Else
                response = "Checked Out"
            End If
        Else
            ' ----- This book is currently checked in.
            If (CBool(dbInfo!Missing) = True) Then
                response = "Missing"
            ElseIf (CBool(dbInfo!Reference) = True) Then
                response = "Reference"
            Else
                response = "Checked In"
            End If
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return response

ErrorHandler:
        GeneralError("GetCopyDisposition", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return "New Item Copy"
    End Function

    Public Function GetSubStr(ByVal origString As String, ByVal delim As String, _
            ByVal whichField As Integer) As String
        ' ----- Extracts a delimited string from another larger string.
        Dim stringParts() As String

        ' ----- Handle some errors.
        If (whichField < 0) Then Return ""
        If (Len(origString) < 1) Then Return ""
        If (Len(delim) = 0) Then Return ""

        ' ----- Break the string up into delimited parts.
        stringParts = Split(origString, delim)

        ' ----- See if the part we want exists and return it.
        If (whichField > UBound(stringParts) + 1) Then Return "" _
            Else Return stringParts(whichField - 1)
    End Function

    Public Function GetSystemValue(ByVal valueName As String) As String
        ' ----- Return the data portion of a system value name-data pair.
        Dim sqlText As String
        Dim returnValue As String

        ' ----- Retrieve the value.
        returnValue = ""
        sqlText = "SELECT ValueData FROM SystemValue WHERE UPPER(ValueName) = " & _
            DBText(UCase(valueName))
        Try
            returnValue = DBGetText(ExecuteSQLReturn(sqlText))
        Catch ex As Exception
            GeneralError("GetSystemValue", ex)
        End Try
        Return returnValue
    End Function

    Public Sub InitializeSystem()
        ' ----- Initialize global variables here.
        Dim counter As Integer

        ' ----- Ensure we have the latest settings.
        If (My.Settings.SettingsUpgraded = False) Then
            My.Settings.Upgrade()
            My.Settings.SettingsUpgraded = True
        End If

        ' ----- Clear security-related values.
        LoggedInUserID = -1
        LoggedInUserName = ""
        LoggedInGroupID = -1
        DefaultItemLocation = -1
        SearchMatchLimit = 250
        FineGraceDays = -1
        AllHolidays = New Collections.Generic.List(Of HolidaySet)

        For counter = 1 To MaxLibrarySecurity
            SecurityProfile(counter) = False
        Next counter

        ' ----- Locate the online help files.
        MainHelpFile = My.Settings.HelpFile & ""
        MainAdminHelpFile = My.Settings.HelpFileAdmin & ""
    End Sub

    Public Function IsHolidayDate(ByVal whatDate As Date) As Boolean
        ' ----- See if the given date is a holiday.
        Dim oneHoliday As Library.HolidaySet

        ' ----- Scan through the holidays, looking for a match.
        For Each oneHoliday In AllHolidays
            If (oneHoliday.IsHoliday(whatDate)) Then Return True
        Next oneHoliday

        ' ----- Not a holiday.
        Return False
    End Function

    Public Function LeftAndRightText(ByVal origLeft As String, ByVal origRight As String, _
            ByVal textWidth As Integer) As String
        ' ----- Build a string of the indicated width that has both left-justified
        '       and right-justified text.
        Dim leftText As String
        Dim rightText As String

        leftText = Trim(origLeft)
        rightText = Trim(origRight)
        If (Len(leftText & rightText) >= textWidth) Then
            Return Trim(Left(leftText & rightText, textWidth))
        Else
            Return leftText & Space(textWidth - Len(leftText & rightText)) & rightText
        End If
    End Function

    Public Sub LoadDatabaseSettings()
        ' ----- Get some system-level values from database storage.
        Dim holdText As String

        On Error Resume Next

        ' ----- Get the default location.
        holdText = GetSystemValue("DefaultLocation")
        If (holdText = "") Then holdText = "-1"
        DefaultItemLocation = CInt(holdText)

        ' ----- Get the maximum number of search maches.
        holdText = GetSystemValue("SearchLimit")
        If (holdText = "") Then holdText = "-1"
        SearchMatchLimit = CInt(holdText)

        ' ----- Get the number of days to wait before charging fines.
        holdText = GetSystemValue("FineGrace")
        If (holdText = "") Then holdText = "-1"
        FineGraceDays = CInt(holdText)
    End Sub

    Public Sub RefreshHolidays()
        ' ----- Load in the list of holidays.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim newHoliday As Library.HolidaySet

        On Error GoTo ErrorHandler

        ' ----- Clear the current list of holidays.
        AllHolidays.Clear()

        ' ----- Get the holidays from the database.
        sqlText = "SELECT * FROM Holiday"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            newHoliday = New Library.HolidaySet(CStr(dbInfo!EntryType), _
                CStr(dbInfo!EntryDetail))
            AllHolidays.Add(newHoliday)
        Loop
        dbInfo.Close()
        Return

ErrorHandler:
        GeneralError("RefreshHolidays", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Sub ReprocessSecuritySet()
        ' ----- Reload in the security set for the current user. If no user
        '       is logged in, clear all settings.
        Dim counter As Integer
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader = Nothing

        ' ----- Clear out the existing items.
        For counter = 1 To MaxLibrarySecurity
            SecurityProfile(counter) = False
        Next counter

        ' ----- Exit if there is no user logged in.
        If (LoggedInUserID = -1) Or (LoggedInGroupID = -1) Then Return

        Try
            ' ----- Load in the security elements for this user.
            sqlText = "SELECT ActivityID FROM GroupActivity WHERE GroupID = " & _
                LoggedInGroupID
            dbInfo = CreateReader(sqlText)
            Do While (dbInfo.Read)
                SecurityProfile(CInt(dbInfo!ActivityID)) = True
            Loop
            dbInfo.Close()
        Catch ex As Exception
            ' ----- Some database-related error.
            GeneralError("ReprocessSecuritySet", ex)
            If (dbInfo IsNot Nothing) Then dbInfo.Close()

            ' ----- Un-login the administrator through recursion.
            LoggedInUserID = -1
            LoggedInGroupID = -1
            ReprocessSecuritySet()
        Finally
            dbInfo = Nothing
        End Try
    End Sub

    Public Sub SetSystemValue(ByVal valueName As String, ByVal valueData As String)
        ' ----- Update a record in the SystemValue table.
        Dim sqlText As String

        Try
            ' ----- See if the entry already exists.
            sqlText = "SELECT COUNT(*) FROM SystemValue WHERE UPPER(ValueName) = " & _
                DBText(UCase(valueName))
            If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
                ' ----- Value already exists.
                sqlText = "UPDATE SystemValue SET ValueData = " & DBText(valueData) & _
                    " WHERE UPPER(ValueName) = " & DBText(UCase(valueName))
            Else
                ' ----- Need to create value.
                sqlText = "INSERT INTO SystemValue (ValueName, ValueData) VALUES (" & _
                    DBText(valueName) & ", " & DBText(valueData) & ")"
            End If

            ' ----- Update the value.
            ExecuteSQL(sqlText)
        Catch ex As System.Exception
            GeneralError("SetSystemValue", ex)
        End Try
    End Sub

    Public Sub TransactionBegin()
        ' ----- Create a new database transaction.
        On Error Resume Next
        HoldTransaction = LibraryDB.BeginTransaction()
    End Sub

    Public Sub TransactionCommit()
        ' ----- Ignore if there is no transaction.
        On Error Resume Next
        If (HoldTransaction Is Nothing) Then Return

        ' ----- Commit the transaction.
        HoldTransaction.Commit()
        HoldTransaction = Nothing
    End Sub

    Public Sub TransactionRollback()
        ' ----- Ignore if there is no transaction.
        On Error Resume Next
        If (HoldTransaction Is Nothing) Then Return

        ' ----- Rollback the transaction.
        HoldTransaction.Rollback()
        HoldTransaction = Nothing
    End Sub
End Module
