' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class PatronPayment
    Private ActivePatronCopyID As Integer
    Private OneLineHeight As Integer
    Private TwoLineHeight As Integer
    Private SecondItemRow As Integer
    Private PaymentsOnly As Generic.List(Of PaymentItem)

    Private Enum EventEntryType
        NotDefined
        PatronPayment
        FineAdded
        FineDismissal
        RefundToPatron
        OverdueFines
    End Enum

    Private Class EventHistoryItem
        Public PaymentID As Integer
        Public EntryDate As Date
        Public PaymentAmount As Decimal
        Public Comments As String
        Public EntryType As EventEntryType
    End Class

    Public Sub ManagePayments(ByVal patronCopyID As Integer, _
            ByVal sessionPayments As Generic.List(Of PaymentItem))
        ' ----- Manage the payments for an item.
        ActivePatronCopyID = patronCopyID
        PaymentsOnly = sessionPayments
        Me.ShowDialog()
    End Sub

    Private Sub PatronPayment_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim balanceDue As Decimal
        Dim originalFine As Decimal
        Dim canvas As Drawing.Graphics
        Dim historyItem As EventHistoryItem

        On Error GoTo ErrorHandler

        ' ----- Get the height of the history list lines.
        canvas = EventHistory.CreateGraphics()
        OneLineHeight = EventHistory.ItemHeight
        TwoLineHeight = CInt(canvas.MeasureString("A" & vbCrLf & "g", _
            EventHistory.Font).Height()) + 3
        SecondItemRow = CInt(canvas.MeasureString("Ag", _
            EventHistory.Font).Height()) + 1
        canvas.Dispose()

        ' ----- Retrieve the summary details for this patron/item entry.
        sqlText = "SELECT PA.FirstName, PA.LastName, NI.Title, PC.Fine, " & _
            "PC.Paid FROM PatronCopy AS PC INNER JOIN Patron AS PA ON " & _
            "PC.Patron = PA.ID INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " & _
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " & _
            "WHERE PC.ID = " & ActivePatronCopyID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        RecordPatron.Text = CStr(dbInfo!FirstName) & " " & CStr(dbInfo!LastName)
        RecordItem.Text = CStr(dbInfo!Title)
        originalFine = CDec(dbInfo!Fine)
        RecordFine.Text = Format(originalFine, "Currency")
        RecordPayments.Text = Format(CDec(dbInfo!Paid), "Currency")
        balanceDue = originalFine - CDec(dbInfo!Paid)
        RecordBalance.Text = Format(balanceDue, "Currency")
        dbInfo.Close()

        ' ----- Load in the history for this item.
        EventHistory.Items.Clear()
        sqlText = "SELECT * FROM PatronPayment WHERE PatronCopy = " & _
            ActivePatronCopyID & " ORDER BY EntryDate"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Build the history detail entry.
            historyItem = New EventHistoryItem
            historyItem.PaymentID = CInt(dbInfo!ID)
            historyItem.EntryDate = CDate(dbInfo!EntryDate)
            historyItem.PaymentAmount = CDec(dbInfo!Amount)
            historyItem.Comments = DBGetText(dbInfo!Comment)
            Select Case UCase(CStr(dbInfo!EntryType))
                ' ----- The database stores letter codes.
                Case "D"
                    historyItem.EntryType = EventEntryType.FineDismissal
                    originalFine += CDec(dbInfo!Amount)
                Case "F"
                    historyItem.EntryType = EventEntryType.FineAdded
                    originalFine -= CDec(dbInfo!Amount)
                Case "P"
                    historyItem.EntryType = EventEntryType.PatronPayment
                Case "R"
                    historyItem.EntryType = EventEntryType.RefundToPatron
            End Select
            EventHistory.Items.Add(historyItem)
        Loop
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- Add in the original fine if needed.
        If (originalFine > 0) Then
            historyItem = New EventHistoryItem
            historyItem.PaymentID = -1
            historyItem.EntryDate = Date.MinValue
            historyItem.PaymentAmount = originalFine
            historyItem.Comments = ""
            historyItem.EntryType = EventEntryType.OverdueFines
            EventHistory.Items.Insert(0, historyItem)
        End If
        Return

ErrorHandler:
        GeneralError("PatronPayment.PatronPayment_Load", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function SaveEventData() As Boolean
        ' ----- Add a new payment entry. Return True on success.
        Dim entryType As EventEntryType
        Dim entryCode As String
        Dim fineAmount As Decimal
        Dim paidAmount As Decimal
        Dim sqlText As String
        Dim newID As Integer
        Dim holdText As String
        Dim scanPayment As PaymentItem
        Dim foundPayment As Boolean
        Dim historyItem As EventHistoryItem

        On Error GoTo ErrorHandler

        ' ----- Determine the type of entry.
        entryType = EventEntryType.NotDefined
        If (OptionPayment.Checked = True) Then
            entryType = EventEntryType.PatronPayment
            entryCode = "P"
        ElseIf (OptionFine.Checked = True) Then
            entryType = EventEntryType.FineAdded
            entryCode = "F"
        ElseIf (OptionReduce.Checked = True) Then
            entryType = EventEntryType.FineDismissal
            entryCode = "D"
        Else
            ' ----- Refund entry (OptionRefund.Checked = True).
            entryType = EventEntryType.RefundToPatron
            entryCode = "R"
        End If

        ' ----- Determine the new totals.
        fineAmount = CDec(RecordFine.Text)
        paidAmount = CDec(RecordPayments.Text)
        Select Case entryType
            Case EventEntryType.PatronPayment
                paidAmount += CDec(RecordAmount.Text)
            Case EventEntryType.FineAdded
                fineAmount += CDec(RecordAmount.Text)
            Case EventEntryType.FineDismissal
                fineAmount -= CDec(RecordAmount.Text)
            Case EventEntryType.RefundToPatron
                paidAmount -= CDec(RecordAmount.Text)
        End Select
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Add the entry to the database.
        TransactionBegin()
        sqlText = "INSERT INTO PatronPayment (PatronCopy, EntryDate, EntryType, " & _
            "Amount, Comment, UserID) OUTPUT INSERTED.ID VALUES (" & _
            ActivePatronCopyID & ", GETDATE(), " & DBText(entryCode) & ", " & _
            RecordAmount.Text & ", " & DBText(Trim(RecordComment.Text)) & _
            ", " & LoggedInUserID & ")"
        newID = CInt(ExecuteSQLReturn(sqlText))

        sqlText = "UPDATE PatronCopy SET Fine = " & fineAmount & _
            ", Paid = " & paidAmount & " WHERE ID = " & ActivePatronCopyID
        ExecuteSQL(sqlText)
        TransactionCommit()

        ' ----- Add an item to the entry list.
        historyItem = New EventHistoryItem
        historyItem.PaymentID = newID
        historyItem.EntryDate = Today
        historyItem.PaymentAmount = CDec(RecordAmount.Text)
        historyItem.Comments = Trim(RecordComment.Text)
        historyItem.EntryType = entryType
        EventHistory.Items.Add(historyItem)

        ' ----- Update the payment in the payments list.
        foundPayment = False
        For Each scanPayment In PaymentsOnly
            If (scanPayment.PatronCopyID = ActivePatronCopyID) Then
                ' ----- Update an existing payment.
                foundPayment = True
                If (entryType = EventEntryType.PatronPayment) Then _
                    scanPayment.FeesPaid += paidAmount
                scanPayment.BalanceDue = fineAmount - paidAmount
                Exit For
            End If
        Next scanPayment
        If (foundPayment = False) And (entryType = EventEntryType.PatronPayment) Then
            ' ----- Add a new payment.
            scanPayment = New PaymentItem
            scanPayment.PatronCopyID = ActivePatronCopyID
            scanPayment.ItemTitle = RecordItem.Text
            scanPayment.FeesPaid = paidAmount
            scanPayment.BalanceDue = fineAmount - paidAmount
            PaymentsOnly.Add(scanPayment)
        End If

        ' ----- Update the on-screen values.
        RecordFine.Text = Format(fineAmount, "Currency")
        RecordPayments.Text = Format(paidAmount, "Currency")
        RecordBalance.Text = Format(fineAmount - paidAmount, "Currency")

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("PatronPayment.SaveEventData", Err.GetException())
        TransactionRollback()
        Return False
    End Function

    Private Function VerifyEventData() As Boolean
        ' ----- The user is trying to add a new event. Make sure all of the
        '       fields are valid. Return True on success.

        ' ----- An entry type is required.
        If (OptionPayment.Checked = False) And (OptionFine.Checked = False) And _
                (OptionReduce.Checked = False) And (OptionRefund.Checked = False) Then
            MsgBox("Please indicate the type of payment entry.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Some entry types require a comment.
        If (Trim(RecordComment.Text) = "") Then
            If (OptionFine.Checked = True) Or (OptionRefund.Checked = True) Then
                MsgBox("Comment is required for this type of payment entry.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                Return False
            End If
        End If

        ' ----- Check for a valid amount.
        If (IsNumeric(RecordAmount.Text) = False) Then
            MsgBox("Please supply a valid amount.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordAmount.Focus()
            Return False
        End If
        If (CDec(RecordAmount.Text) <= 0@) Or (CDec(RecordAmount.Text) >= 1000@) Then
            MsgBox("Amount must be between $0.01 and $999.99.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordAmount.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAdd.Click
        ' ----- Add a new payment entry.
        If (VerifyEventData() = False) Then Return
        If (SaveEventData() = False) Then Return

        ' ----- Prepare for a new entry.
        RecordAmount.Text = ""
        RecordComment.Text = ""
        OptionPayment.Checked = False
        OptionFine.Checked = False
        OptionReduce.Checked = False
        OptionRefund.Checked = False
        Return
    End Sub

    Private Sub EventHistory_MeasureItem(ByVal sender As Object, ByVal e As System.Windows.Forms.MeasureItemEventArgs) Handles EventHistory.MeasureItem
        ' ----- Is the active item one or two lines high?
        Dim historyItem As EventHistoryItem

        historyItem = CType(EventHistory.Items(e.Index), EventHistoryItem)
        If (historyItem.Comments = "") Then
            ' ----- One line.
            e.ItemHeight = OneLineHeight
        Else
            ' ----- Two lines.
            e.ItemHeight = TwoLineHeight
        End If
    End Sub

    Private Sub EventHistory_DrawItem(ByVal sender As Object, ByVal e As System.Windows.Forms.DrawItemEventArgs) Handles EventHistory.DrawItem
        ' ----- Draw the matching items on two lines.
        Dim useBrush As Drawing.Brush
        Dim historyItem As EventHistoryItem
        Dim dateText As String
        Dim amountText As String
        Dim reasonText As String
        Dim commentText As String
        Dim italicFont As Drawing.Font

        ' ----- Draw the background of the item.
        If ((e.State And DrawItemState.Selected) = DrawItemState.Selected) Then
            useBrush = SystemBrushes.HighlightText
        Else
            useBrush = SystemBrushes.WindowText
        End If
        e.DrawBackground()

        ' ----- Make an italic version of the font.
        italicFont = New System.Drawing.Font(e.Font, FontStyle.Italic)

        ' ----- Get the record that contains the display details.
        historyItem = CType(EventHistory.Items(e.Index), EventHistoryItem)

        ' ----- Format the display components.
        If (historyItem.EntryDate = Date.MinValue) Then
            dateText = "--/--/----"
        Else
            dateText = Format(historyItem.EntryDate, "M/d/yyyy")
        End If
        amountText = Format(historyItem.PaymentAmount, "Currency")
        Select Case historyItem.EntryType
            Case EventEntryType.PatronPayment
                reasonText = "Payment by patron"
            Case EventEntryType.FineAdded
                reasonText = "Fine added"
            Case EventEntryType.FineDismissal
                reasonText = "Dismissal of fine"
            Case EventEntryType.RefundToPatron
                reasonText = "Refund to patron"
            Case EventEntryType.OverdueFines
                reasonText = "Overdue fines"
            Case Else
                reasonText = "Undefined entry"
        End Select
        If (historyItem.Comments = "") Then commentText = "" Else _
            commentText = ">> " & historyItem.Comments

        ' ----- Draw the text of the item.
        e.Graphics.DrawString(dateText, e.Font, useBrush, 0, e.Bounds.Top)
        e.Graphics.DrawString(amountText, e.Font, useBrush, _
            LabelHistoryCol2.Left - EventHistory.Left + 2, e.Bounds.Top)
        e.Graphics.DrawString(reasonText, e.Font, useBrush, _
            LabelHistoryCol3.Left - EventHistory.Left + 2, e.Bounds.Top)
        If (commentText <> "") Then
            e.Graphics.DrawString(commentText, italicFont, useBrush, _
                LabelHistoryCol3.Left - EventHistory.Left + 2, _
                e.Bounds.Top + SecondItemRow)
        End If

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
        italicFont.Dispose()
    End Sub

    Private Sub RecordAmount_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordAmount.Enter
        ' ----- Highlight the entire text.
        RecordAmount.SelectAll()
    End Sub

    Private Sub RecordComment_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordComment.Enter
        ' ----- Highlight the entire text.
        RecordComment.SelectAll()
    End Sub
End Class
