' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class LocateDatabase
    Private Const AuthenticationTypeWindows As Integer = 1
    Private Const AuthenticationTypeSQL As Integer = 2

    Public Function PromptUser() As String
        ' ----- Prompt the user for database connection details. Return a
        '       valid ADO.NET connection string for SQL Server, or a blank
        '       string if the user clicks Cancel.
        Dim newConnection As String

        ' ----- Prompt the user.
        Me.ShowDialog()

        ' ----- Build the new connection string.
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then
            newConnection = "Data Source=" & Trim(RecordServer.Text) & _
                ";Initial Catalog=" & Trim(RecordDatabase.Text)
            If (CInt(CType(RecordAuthentication.SelectedItem, ListItemData)) = _
                    AuthenticationTypeWindows) Then
                ' ----- Use Windows security.
                newConnection &= ";Integrated Security=true"
            Else
                ' ----- Use SQL Server security.
                newConnection &= ";User ID=" & Trim(RecordUser.Text) & _
                    ";Password=" & Trim(RecordPassword.Text)
            End If
            Return newConnection
        Else
            Return ""
        End If
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Check the supplied data, and return True if it is all valid.
        On Error Resume Next

        ' ----- Server name is required.
        If (Trim(RecordServer.Text) = "") Then
            MsgBox("The Server or Host name is required.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordServer.Focus()
            Return False
        End If

        ' ----- Database name is required.
        If (Trim(RecordDatabase.Text) = "") Then
            MsgBox("The Database Name is required.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordDatabase.Focus()
            Return False
        End If

        ' ----- For SQL Server authentication, the User ID is required.
        If (CInt(CType(RecordAuthentication.SelectedItem, ListItemData)) = _
                AuthenticationTypeSQL) Then
            If (Trim(RecordUser.Text) = "") Then
                MsgBox("The User Name is required for SQL Server authentication.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                RecordUser.Focus()
                Return False
            End If
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub LocateDatabase_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "LocateDatabase.htm")
    End Sub

    Private Sub LocateDatabase_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim counter As Integer
        Dim connectionString As String
        Dim oneKey As String
        Dim oneValue As String

        On Error Resume Next

        ' ----- Add the available authentication schemes.
        RecordAuthentication.Items.Add(New ListItemData("Microsoft Windows", _
            AuthenticationTypeWindows))
        RecordAuthentication.SelectedIndex = 0
        RecordAuthentication.Items.Add(New ListItemData("SQL Server", _
            AuthenticationTypeSQL))

        ' ----- Load in the existing data.
        connectionString = My.Settings.DBConnection & ""
        For counter = 1 To CountSubStr(connectionString, ";") + 1
            ' ----- Each comma-delimited part has the format "key=value".
            oneKey = GetSubStr(connectionString, ";", counter)
            oneValue = Trim(GetSubStr(oneKey, "=", 2))
            oneKey = Replace(UCase(Trim(GetSubStr(oneKey, "=", 1))), " ", "")

            ' ----- Process each part.
            Select Case oneKey
                Case "DATASOURCE"
                    ' ----- Show the server host.
                    RecordServer.Text = oneValue
                Case "INITIALCATALOG"
                    ' ----- Show the default database name.
                    RecordDatabase.Text = oneValue
                Case "INTEGRATEDSECURITY"
                    ' ----- Only check for "true". False is assumed.
                    If (UCase(oneValue) = "TRUE") Then _
                        RecordAuthentication.SelectedIndex = _
                        RecordAuthentication.Items.IndexOf(AuthenticationTypeWindows)
                Case "USERID"
                    ' ----- A user ID forces SQL authentication.
                    RecordAuthentication.SelectedIndex = _
                        RecordAuthentication.Items.IndexOf(AuthenticationTypeSQL)
                    RecordUser.Text = oneValue
                Case "PASSWORD"
                    ' ----- A password forces SQL authentication.
                    RecordAuthentication.SelectedIndex = _
                        RecordAuthentication.Items.IndexOf(AuthenticationTypeSQL)
                    RecordPassword.Text = oneValue
            End Select
        Next counter
    End Sub

    Private Sub RecordDatabase_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordDatabase.Enter
        ' ----- Highlight the entire text.
        RecordDatabase.SelectAll()
    End Sub

    Private Sub RecordPassword_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordPassword.Enter
        ' ----- Highlight the entire text.
        RecordPassword.SelectAll()
    End Sub

    Private Sub RecordServer_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordServer.Enter
        ' ----- Highlight the entire text.
        RecordServer.SelectAll()
    End Sub

    Private Sub RecordUser_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordUser.Enter
        ' ----- Highlight the entire text.
        RecordUser.SelectAll()
    End Sub

    Private Sub RecordAuthentication_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordAuthentication.SelectedIndexChanged
        ' ----- Enable dependent fields as needed.
        If (CInt(CType(RecordAuthentication.SelectedItem, ListItemData)) = _
                AuthenticationTypeWindows) Then
            ' ----- Windows authentication requires no user/password.
            LabelUser.Enabled = False
            RecordUser.Enabled = False
            LabelPassword.Enabled = False
            RecordPassword.Enabled = False
        Else
            ' ----- Requires specific SQL Server user and password.
            LabelUser.Enabled = True
            RecordUser.Enabled = True
            LabelPassword.Enabled = True
            RecordPassword.Enabled = True
        End If
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub
End Class
