' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports MVB = Microsoft.VisualBasic

Public Class NamedItem
    Private ActiveID As Integer
    Private StartingOver As Boolean
    Private PublisherID As Integer
    Private SeriesID As Integer
    Private SessionSaved As Boolean

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Or (SessionSaved = True) _
            Then Return ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageItems)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageItems)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageItems)
    End Function

    Private Function DeleteOneKeyword(ByVal keywordID As Integer) As Boolean
        ' ----- Remove a keyword from this record. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Delete the record.
        sqlText = "DELETE FROM ItemKeyword WHERE ItemID = " & ActiveID & _
            " AND KeywordID = " & keywordID
        ExecuteSQL(sqlText)

        ' ----- If no books are using this keyword, remove it.
        sqlText = "SELECT COUNT(*) FROM ItemKeyword WHERE KeywordID = " & keywordID
        If (CInt(ExecuteSQLReturn(sqlText)) = 0) Then
            sqlText = "DELETE FROM Keyword WHERE ID = " & keywordID
            ExecuteSQL(sqlText)
        End If
        Return True

ErrorHandler:
        GeneralError("NamedItem.DeleteOneKeyword", Err.GetException())
        Return False
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String
        Dim counter As Integer

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the item?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE ItemID = " & recordID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("You cannot delete this record because it has copies.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Delete each of the keywords.
        For counter = 0 To KeywordList.Items.Count - 1
            DeleteOneKeyword(CInt(CType(KeywordList.Items(counter), ListItemData)))
        Next counter

        ' ----- Delete the records.
        TransactionBegin()
        sqlText = "DELETE FROM ItemAuthor WHERE ItemID = " & recordID
        ExecuteSQL(sqlText)
        sqlText = "DELETE FROM NamedItem WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        TransactionCommit()
        Return True

ErrorHandler:
        GeneralError("NamedItem.DeleteRecord", Err.GetException())
        TransactionRollback()
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Or (SessionSaved = True) _
            Then Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items. Prompt the
        '       user first.
        destList.Items.Clear()
        SearchForRecord(destList, exceededMatches)
    End Sub

    Private Function FormatItemName(ByRef dbInfo As SqlClient.SqlDataReader) As String
        ' ----- Given an item record, return the formatted name.
        Dim nameText As String

        On Error Resume Next

        ' ----- Format the name.
        nameText = CStr(dbInfo!Title)
        If (IsDBNull(dbInfo!Subtitle) = False) Then _
            nameText &= ": " & CStr(dbInfo!Subtitle)
        If (IsDBNull(dbInfo!Edition) = False) Then _
            nameText &= ", " & CStr(dbInfo!Edition)
        nameText &= " [" & CStr(dbInfo!MediaName) & "]"

        ' ----- Finished.
        Return nameText
    End Function

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim result As String

        On Error GoTo ErrorHandler
        result = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT NI.ID, NI.Title, NI.Subtitle, NI.Edition, CMT.FullName " & _
            "AS MediaName FROM NamedItem AS NI INNER JOIN CodeMediaType AS CMT " & _
            "ON NI.MediaType = CMT.ID WHERE NI.ID = " & recordID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then result = FormatItemName(dbInfo)
        dbInfo.Close()
        dbInfo = Nothing
        Return result

ErrorHandler:
        GeneralError("NamedItem.FormatRecordName", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use this list to identify the items found in the library. " & _
            "Each item may include more than one copy."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Named Items"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim newItem As System.Windows.Forms.ListViewItem
        Dim holdText As String
        Dim counter As Integer

        On Error GoTo ErrorHandler

        ' ----- Load in the list of security groups.
        RecordMediaType.Items.Clear()
        RecordMediaType.Items.Add(New ListItemData("<Not Selected>", -1))
        sqlText = "SELECT * FROM CodeMediaType ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While (dbInfo.Read)
            RecordMediaType.Items.Add(New ListItemData( _
                CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- Clear the existing data.
        StartingOver = True
        RecordID.Text = "Not Available"
        RecordTitle.Text = ""
        RecordSubtitle.Text = ""
        RecordDescription.Text = ""
        RecordCopyright.Text = ""
        RecordEdition.Text = ""
        RecordPublisher.Text = "Not Available"
        PublisherID = -1
        RecordDewey.Text = ""
        RecordLC.Text = ""
        RecordLCCN.Text = ""
        RecordISxN.Text = ""
        RecordMediaType.SelectedIndex = RecordMediaType.Items.IndexOf(-1)
        RecordOutOfPrint.Checked = False
        RecordSeries.Text = "No Series"
        SeriesID = -1
        AuthorsList.Items.Clear()
        ActAuthorProperties.Enabled = False
        ActAuthorDelete.Enabled = False
        ActAuthorMoveUp.Enabled = False
        ActAuthorMoveDown.Enabled = False
        SubjectList.Items.Clear()
        ActSubjectDelete.Enabled = False
        ActSubjectProperties.Enabled = False
        KeywordList.Items.Clear()
        ActKeywordDelete.Enabled = False
        ActKeywordProperties.Enabled = False
        CopyList.Items.Clear()
        ActCopyEdit.Enabled = False
        ActCopyDelete.Enabled = False
        ActCancel.Text = "Cancel"
        TabNamedItem.SelectedTab = TabGeneral
        SessionSaved = False
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT NI.*, PB.FullName AS PubName, CS.FullName As SeriesName " & _
            "FROM NamedItem AS NI LEFT JOIN Publisher AS PB ON NI.Publisher = PB.ID " & _
            "LEFT JOIN CodeSeries AS CS ON NI.Series = CS.ID " & _
            "WHERE NI.ID = " & ActiveID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            RecordID.Text = CStr(dbInfo!ID)
            RecordTitle.Text = CStr(dbInfo!Title)
            RecordSubtitle.Text = DBGetText(dbInfo!Subtitle)
            RecordDescription.Text = DBGetText(dbInfo!Description)
            RecordCopyright.Text = DBGetText(dbInfo!Copyright)
            RecordEdition.Text = DBGetText(dbInfo!Edition)
            If (isdbnull(dbInfo!PubName) = false) Then _
                RecordPublisher.Text = CStr(dbInfo!PubName)
            If (IsDBNull(dbInfo!Publisher) = False) Then _
                PublisherID = CInt(dbInfo!Publisher)
            RecordDewey.Text = DBGetText(dbInfo!Dewey)
            RecordLC.Text = DBGetText(dbInfo!LC)
            RecordLCCN.Text = DBGetText(dbInfo!LCCN)
            RecordISxN.Text = DBGetText(dbInfo!ISxN)
            RecordMediaType.SelectedIndex = RecordMediaType.Items.IndexOf( _
                CInt(dbInfo!MediaType))
            RecordOutOfPrint.Checked = CBool(dbInfo!OutOfPrint)
            If (IsDBNull(dbInfo!SeriesName) = False) Then _
                RecordSeries.Text = CStr(dbInfo!SeriesName)
            If (IsDBNull(dbInfo!Series) = False) Then _
                SeriesID = CInt(dbInfo!Series)
        End If
        dbInfo.Close()

        ' ----- Load in the authors for this item.
        sqlText = "SELECT AU.*, CT.FullName AS TypeName, CT.ID AS TypeID " & _
            "FROM ItemAuthor AS IA INNER JOIN Author AS AU ON IA.AuthorID = AU.ID " & _
            "INNER JOIN CodeAuthorType AS CT ON IA.AuthorType = CT.ID " & _
            "WHERE IA.ItemID = " & ActiveID & " ORDER BY IA.Sequence"
        dbInfo = CreateReader(sqlText)
        Do While (dbInfo.Read)
            newItem = AuthorsList.Items.Add(FormatAuthorName(dbInfo))
            newItem.SubItems.Add(CStr(dbInfo!TypeName))
            newItem.Tag = CStr(dbInfo!ID) & "," & CStr(dbInfo!TypeID)
        Loop
        dbInfo.Close()

        ' ----- Load in the subjects for this item.
        sqlText = "SELECT SB.* FROM ItemSubject AS IJ INNER JOIN Subject AS SB " & _
            "ON IJ.SubjectID = SB.ID WHERE IJ.ItemID = " & ActiveID & _
            " ORDER BY SB.FullName"
        dbInfo = CreateReader(sqlText)
        Do While (dbInfo.Read)
            SubjectList.Items.Add(New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()

        ' ----- Load in the keywords for this item.
        sqlText = "SELECT KW.* FROM ItemKeyword AS IK INNER JOIN Keyword AS KW " & _
            "ON IK.KeywordID = KW.ID WHERE IK.ItemID = " & ActiveID & _
            " ORDER BY KW.FullName"
        dbInfo = CreateReader(sqlText)
        Do While (dbInfo.Read)
            KeywordList.Items.Add(New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()

        ' ----- Load in the copies for this item.
        sqlText = "SELECT ID, CopyNumber, Barcode FROM ItemCopy WHERE ItemID = " & _
            ActiveID & " ORDER BY CopyNumber"
        dbInfo = CreateReader(sqlText)
        Do While (dbInfo.Read)
            newItem = CopyList.Items.Add(CStr(dbInfo!CopyNumber))
            newItem.SubItems.Add("")  ' Add disposition later
            holdText = DBGetText(dbInfo!Barcode)
            If (holdText = "") Then holdText = "None"
            newItem.SubItems.Add(holdText)
            newItem.Tag = CStr(dbInfo!ID)
        Loop
        dbInfo.Close()

        ' ----- Add the disposition for each copy. We can't do it in the
        '       earlier loop since that would cause two OleDB readers
        '       to be opened at once, and that doesn't work.
        For counter = 0 To CopyList.Items.Count - 1
            newItem = CopyList.Items(counter)
            newItem.SubItems(1).Text = GetCopyDisposition(CInt(newItem.Tag))
        Next counter

        ' ----- Finished.
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("NamedItem.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Save the changes to the database.
        Dim sqlText As String
        Dim newID As Integer

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO NamedItem (Title, Subtitle, Description, " & _
                "Copyright, Edition, Publisher, Dewey, LC, LCCN, ISxN, MediaType, " & _
                "OutOfPrint, Series) OUTPUT INSERTED.ID VALUES (" & _
                DBText(Trim(RecordTitle.Text)) & _
                ", " & DBText(Trim(RecordSubtitle.Text)) & _
                ", " & DBText(Trim(RecordDescription.Text)) & _
                ", " & DBNum(Trim(RecordCopyright.Text)) & _
                ", " & DBText(Trim(RecordEdition.Text)) & _
                ", " & CStr(IIf(PublisherID = -1, "NULL", PublisherID)) & _
                ", " & DBText(Trim(RecordDewey.Text)) & _
                ", " & DBText(Trim(RecordLC.Text)) & _
                ", " & DBText(Trim(RecordLCCN.Text)) & _
                ", " & DBText(Trim(RecordISxN.Text)) & _
                ", " & DBCombo(RecordMediaType) & _
                ", " & CInt(RecordOutOfPrint.Checked) & _
                ", " & CStr(IIf(SeriesID = -1, "NULL", SeriesID)) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE NamedItem SET" & _
                " Title = " & DBText(Trim(RecordTitle.Text)) & _
                ", Subtitle = " & DBText(Trim(RecordSubtitle.Text)) & _
                ", Description = " & DBText(Trim(RecordDescription.Text)) & _
                ", Copyright = " & DBNum(Trim(RecordCopyright.Text)) & _
                ", Edition = " & DBText(Trim(RecordEdition.Text)) & _
                ", Publisher = " & CStr(IIf(PublisherID = -1, "NULL", PublisherID)) & _
                ", Dewey = " & DBText(Trim(RecordDewey.Text)) & _
                ", LC = " & DBText(Trim(RecordLC.Text)) & _
                ", LCCN = " & DBText(Trim(RecordLCCN.Text)) & _
                ", ISxN = " & DBText(Trim(RecordISxN.Text)) & _
                ", MediaType = " & DBCombo(RecordMediaType) & _
                ", OutOfPrint = " & CInt(RecordOutOfPrint.Checked) & _
                ", Series = " & CStr(IIf(SeriesID = -1, "NULL", SeriesID)) & _
                " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        ActCancel.Text = "Close"
        SessionSaved = True
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("NamedItem.SaveFormData", Err.GetException())
        Return False
    End Function

    Public Overrides Sub SearchForRecord(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Search for items using user-specified criteria.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim userLimit As String
        Dim resultCount As Integer

        On Error GoTo ErrorHandler

        ' ----- Prompt the user for the limited item title.
        exceededMatches = False
        userLimit = (New ItemLimit).PromptUser()
        If (userLimit = "") Then Return
        If (InStr(userLimit, "*") = 0) Then userLimit &= "*"

        ' ----- Clear any existing values.
        destList.Items.Clear()
        resultCount = 0

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT NI.ID, NI.Title, NI.Subtitle, NI.Edition, CMT.FullName " & _
            "AS MediaName FROM NamedItem AS NI INNER JOIN CodeMediaType AS CMT " & _
            "ON NI.MediaType = CMT.ID WHERE NI.Title LIKE " & _
            DBText(Replace(Trim(userLimit), "*", "%")) & _
            " ORDER BY NI.Title, NI.Subtitle, NI.Edition"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            resultCount += 1
            If (resultCount > SearchMatchLimit) Then
                exceededMatches = True
                Exit Do
            End If
            destList.Items.Add(New ListItemData(FormatItemName(dbInfo), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("NamedItem.SearchForRecord", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function UsesSearch() As Boolean
        ' ----- NamedItem records are not searchable.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        On Error GoTo ErrorHandler

        ' ----- Title is a required field.
        If (Trim(RecordTitle.Text) = "") Then
            MsgBox("Please supply the Title.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            TabNamedItem.SelectedTab = TabGeneral
            RecordTitle.Focus()
            Return False
        End If

        ' ----- Media Type is a required field.
        If (CInt(CType(RecordMediaType.SelectedItem, ListItemData)) = -1) Then
            MsgBox("Please supply the Media Type.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            TabNamedItem.SelectedTab = TabClassification
            RecordMediaType.Focus()
            Return False
        End If

        ' ----- Copyright Year, if supplied, must be numeric.
        If (Trim(RecordCopyright.Text) <> "") Then
            If (IsNumeric(RecordCopyright.Text) = False) Or _
                    (InStr(RecordCopyright.Text, ".") > 0) Then
                MsgBox("Copyright Year, if supplied, must be a valid integer.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                TabNamedItem.SelectedTab = TabGeneral
                RecordCopyright.Focus()
                Return False
            End If
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("NamedItem.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub NamedItem_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then
            TabNamedItem.SelectedTab = TabGeneral
            RecordTitle.Focus()
        End If
        StartingOver = False
    End Sub

    Private Sub NamedItem_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "NamedItem.htm")
    End Sub

    Private Sub NamedItem_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Adjust the author list columns.
        ColAuthorName.Width = LabelColNameType.Left - LabelColAuthor.Left
        ColAuthorNameType.Width = (AuthorsList.Left + AuthorsList.Width) - _
            LabelColNameType.Left - 12

        ' ----- Adjust the copy list columns.
        ColCopyNumber.Width = LabelColCopyStatus.Left - LabelColCopy.Left
        ColCopyStatus.Width = LabelColCopyBarcode.Left - LabelColCopyStatus.Left
        ColCopyBarcode.Width = (CopyList.Left + CopyList.Width) - _
            LabelColCopyBarcode.Left - 12
    End Sub

    Private Sub RecordCopyright_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordCopyright.Enter
        ' ----- Highlight the entire text.
        RecordCopyright.SelectAll()
    End Sub

    Private Sub RecordDewey_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordDewey.Enter
        ' ----- Highlight the entire text.
        RecordDewey.SelectAll()
    End Sub

    Private Sub RecordEdition_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordEdition.Enter
        ' ----- Highlight the entire text.
        RecordEdition.SelectAll()
    End Sub

    Private Sub RecordISxN_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordISxN.Enter
        ' ----- Highlight the entire text.
        RecordISxN.SelectAll()
    End Sub

    Private Sub RecordLC_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordLC.Enter
        ' ----- Highlight the entire text.
        RecordLC.SelectAll()
    End Sub

    Private Sub RecordLCCN_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordLCCN.Enter
        ' ----- Highlight the entire text.
        RecordLCCN.SelectAll()
    End Sub

    Private Sub RecordSubtitle_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordSubtitle.Enter
        ' ----- Highlight the entire text.
        RecordSubtitle.SelectAll()
    End Sub

    Private Sub RecordTitle_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordTitle.Enter
        ' ----- Highlight the entire text.
        RecordTitle.SelectAll()
    End Sub

    Private Sub RecordCopyright_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles RecordCopyright.KeyPress
        ' ----- Only allow digits and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Then Return
        e.Handled = True
    End Sub

    Private Sub ActSelectPublisher_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActSelectPublisher.Click
        ' ----- Get a new publisher.
        Dim newPublisher As Integer
        Dim displayText As String

        ' ----- Prompt the user.
        newPublisher = (New PublisherAddLocate).PromptUser()
        If (newPublisher = -1) Then Return

        ' ----- Check to clear the publisher.
        If (newPublisher = -2) Then
            RecordPublisher.Text = "Not Available"
            PublisherID = -1
            Return
        End If

        ' ----- Save and display the publisher.
        PublisherID = newPublisher
        displayText = (New Publisher).FormatRecordName(PublisherID)
        RecordPublisher.Text = displayText
    End Sub

    Private Sub ActSelectSeries_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActSelectSeries.Click
        ' ----- Get a new series.
        Dim newSeries As Integer
        Dim displayText As String

        ' ----- Prompt the user.
        newSeries = (New SeriesAddLocate).PromptUser()
        If (newSeries = -1) Then Return

        ' ----- Check to clear the series.
        If (newSeries = -2) Then
            RecordSeries.Text = "No Series"
            SeriesID = -1
            Return
        End If

        ' ----- Save and display the series.
        SeriesID = newSeries
        displayText = (New CodeSeries).FormatRecordName(SeriesID)
        RecordSeries.Text = displayText
    End Sub

    Private Sub ActKeywordDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActKeywordDelete.Click
        ' ----- Delete the selected keyword.
        If (KeywordList.SelectedIndex = -1) Then Return

        ' ----- Confirm with the user.
        If (MsgBox("Really delete the keyword '" & KeywordList.Text & _
            "' from this item?", MsgBoxStyle.YesNo Or MsgBoxStyle.Question, _
            ProgramTitle) <> MsgBoxResult.Yes) Then Return

        ' ----- Proceed with the removal.
        If (DeleteOneKeyword(CInt(CType(KeywordList.SelectedItem, ListItemData))) _
            = False) Then Return

        ' ----- Remove this item from the display.
        KeywordList.Items.Remove(KeywordList.SelectedItem)
        KeywordList.SelectedIndex = -1
        ActKeywordDelete.Enabled = False
        ActKeywordProperties.Enabled = False
        ActCancel.Text = "Close"
        SessionSaved = True
    End Sub

    Private Sub ActKeywordAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActKeywordAdd.Click
        ' ----- Let the user add a new keyword.
        Dim keywordID As Integer
        Dim sqlText As String
        Dim keywordText As String

        On Error GoTo ErrorHandler

        ' ----- The record must be saved first.
        If (ActiveID = -1) Then
            ' ----- Confirm with the user.
            If (MsgBox("The item must be saved to the database before keywords " & _
                "can be added. Would you like to save the record now?", _
                MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
                <> MsgBoxResult.Yes) Then Return

            ' ----- Verify and save the data.
            If (ValidateFormData() = False) Then Return
            If (SaveFormData() = False) Then Return
        End If

        ' ----- Prompt the user to add a keyword.
        keywordID = (New KeywordAdd).AddRecord()
        If (keywordID = -1) Then Return

        ' ----- See if the record already exists.
        If (KeywordList.Items.IndexOf(keywordID) <> -1) Then Return

        ' ----- Add the keyword to the database record.
        sqlText = "INSERT INTO ItemKeyword (ItemID, KeywordID) VALUES (" & _
            ActiveID & ", " & keywordID & ")"
        ExecuteSQL(sqlText)
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Add this item to the display.
        sqlText = "SELECT FullName FROM Keyword WHERE ID = " & keywordID
        KeywordList.Items.Add(New ListItemData(CStr(ExecuteSQLReturn(sqlText)), keywordID))
        Return

ErrorHandler:
        GeneralError("NamedItem.ActKeywordAdd_Click", Err.GetException())
        Return
    End Sub

    Private Sub KeywordList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles KeywordList.SelectedIndexChanged
        ' ----- Enable buttons as needed.
        ActKeywordDelete.Enabled = (KeywordList.SelectedIndex <> -1)
        ActKeywordProperties.Enabled = ActKeywordDelete.Enabled
    End Sub

    Private Sub KeywordList_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles KeywordList.Click
        ' ----- Enable buttons as needed.
        ActKeywordDelete.Enabled = (KeywordList.SelectedIndex <> -1)
        ActKeywordProperties.Enabled = ActKeywordDelete.Enabled
    End Sub

    Private Sub SubjectList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SubjectList.SelectedIndexChanged
        ' ----- Enable buttons as needed.
        ActSubjectDelete.Enabled = (SubjectList.SelectedIndex <> -1)
        ActSubjectProperties.Enabled = ActSubjectDelete.Enabled
    End Sub

    Private Sub SubjectList_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles SubjectList.Click
        ' ----- Enable buttons as needed.
        ActSubjectDelete.Enabled = (SubjectList.SelectedIndex <> -1)
        ActSubjectProperties.Enabled = ActSubjectDelete.Enabled
    End Sub

    Private Sub ActSubjectDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActSubjectDelete.Click
        ' ----- Delete the selected subject.
        If (SubjectList.SelectedIndex = -1) Then Return

        ' ----- Confirm with the user.
        If (MsgBox("Really delete the subject '" & SubjectList.Text & _
            "' from this item?", MsgBoxStyle.YesNo Or MsgBoxStyle.Question, _
            ProgramTitle) <> MsgBoxResult.Yes) Then Return

        ' ----- Proceed with the removal.
        If (DeleteOneSubject(CInt(CType(SubjectList.SelectedItem, ListItemData))) _
            = False) Then Return

        ' ----- Remove this item from the display.
        SubjectList.Items.Remove(SubjectList.SelectedItem)
        SubjectList.SelectedIndex = -1
        ActSubjectDelete.Enabled = False
        ActSubjectProperties.Enabled = False
        ActCancel.Text = "Close"
        SessionSaved = True
    End Sub

    Private Function DeleteOneSubject(ByVal subjectID As Integer) As Boolean
        ' ----- Remove a subject from this record. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Delete the record.
        sqlText = "DELETE FROM ItemSubject WHERE ItemID = " & ActiveID & _
            " AND SubjectID = " & subjectID
        ExecuteSQL(sqlText)

        ' ----- If no books are using this subject, remove it.
        sqlText = "SELECT COUNT(*) FROM ItemSubject WHERE SubjectID = " & subjectID
        If (CInt(ExecuteSQLReturn(sqlText)) = 0) Then
            sqlText = "DELETE FROM Subject WHERE ID = " & subjectID
            ExecuteSQL(sqlText)
        End If
        Return True

ErrorHandler:
        GeneralError("NamedItem.DeleteOneSubject", Err.GetException())
        Return False
    End Function

    Private Sub ActKeywordProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActKeywordProperties.Click
        ' ----- Display some information about the selected keyword.
        Dim sqlText As String
        Dim inUse As Integer
        Dim sWork As String

        On Error GoTo ErrorHandler

        ' ----- Ignore if no item selected.
        If (KeywordList.SelectedIndex = -1) Then Return

        ' ----- Collect some statistics.
        sqlText = "SELECT COUNT(*) FROM ItemKeyword WHERE KeywordID = " & _
            CInt(CType(KeywordList.SelectedItem, ListItemData))
        inUse = CInt(ExecuteSQLReturn(sqlText))

        ' ----- Display an informational message.
        sWork = "Keyword: " & KeywordList.Text & vbCrLf & vbCrLf & _
            "This keyword is in use by " & inUse & " library item"
        If (inUse = 1) Then sWork &= "." Else sWork &= "s."
        MsgBox(sWork, MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
        Return

ErrorHandler:
        GeneralError("NamedItem.ActKeywordProperties_Click", Err.GetException())
        Return
    End Sub

    Private Sub ActSubjectAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActSubjectAdd.Click
        ' ----- Let the user add a new subject.
        Dim subjectID As Integer
        Dim sqlText As String
        Dim subjectText As String

        On Error GoTo ErrorHandler

        ' ----- The record must be saved first.
        If (ActiveID = -1) Then
            ' ----- Confirm with the user.
            If (MsgBox("The item must be saved to the database before subjects " & _
                "can be added. Would you like to save the record now?", _
                MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
                <> MsgBoxResult.Yes) Then Return

            ' ----- Verify and save the data.
            If (ValidateFormData() = False) Then Return
            If (SaveFormData() = False) Then Return
        End If

        ' ----- Prompt the user to add a subject.
        subjectID = (New SubjectAdd).AddRecord()
        If (subjectID = -1) Then Return

        ' ----- See if the record already exists.
        If (SubjectList.Items.IndexOf(subjectID) <> -1) Then Return

        ' ----- Add the subject to the database record.
        sqlText = "INSERT INTO ItemSubject (ItemID, SubjectID) VALUES (" & _
            ActiveID & ", " & subjectID & ")"
        ExecuteSQL(sqlText)
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Add this item to the display.
        sqlText = "SELECT FullName FROM Subject WHERE ID = " & subjectID
        SubjectList.Items.Add(New ListItemData(CStr(ExecuteSQLReturn(sqlText)), subjectID))
        Return

ErrorHandler:
        GeneralError("NamedItem.ActSubjectAdd_Click", Err.GetException())
        Return
    End Sub

    Private Sub ActSubjectProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActSubjectProperties.Click
        ' ----- Display some information about the selected subject.
        Dim sqlText As String
        Dim inUse As Integer
        Dim message As String

        On Error GoTo ErrorHandler

        ' ----- Ignore if no item selected.
        If (SubjectList.SelectedIndex = -1) Then Return

        ' ----- Collect some statistics.
        sqlText = "SELECT COUNT(*) FROM ItemSubject WHERE SubjectID = " & _
            CInt(CType(SubjectList.SelectedItem, ListItemData))
        inUse = CInt(ExecuteSQLReturn(sqlText))

        ' ----- Display an informational message.
        message = "Subject: " & SubjectList.Text & vbCrLf & vbCrLf & _
            "This subject is in use by " & inUse & " library item"
        If (inUse = 1) Then message &= "." Else message &= "s."
        MsgBox(message, MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
        Return

ErrorHandler:
        GeneralError("NamedItem.ActSubjectProperties_Click", Err.GetException())
        Return
    End Sub

    Private Sub SubjectList_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles SubjectList.DoubleClick
        ' ----- Same as the properties button.
        If (ActSubjectProperties.Enabled = True) Then ActSubjectProperties.PerformClick()
    End Sub

    Private Sub KeywordList_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles KeywordList.DoubleClick
        ' ----- Same as the properties button.
        If (ActKeywordProperties.Enabled = True) Then ActKeywordProperties.PerformClick()
    End Sub

    Private Sub ActAuthorAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAuthorAdd.Click
        ' ----- Let the user add a new author.
        Dim authorID As Integer
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim authorText As String
        Dim nameTypeID As Integer
        Dim typeText As String
        Dim sequenceID As Integer
        Dim newItem As System.Windows.Forms.ListViewItem

        On Error GoTo ErrorHandler

        ' ----- The record must be saved first.
        If (ActiveID = -1) Then
            ' ----- Confirm with the user.
            If (MsgBox("The item must be saved to the database before authors " & _
                "or names can be added. Would you like to save the record now?", _
                MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
                <> MsgBoxResult.Yes) Then Return

            ' ----- Verify and save the data.
            If (ValidateFormData() = False) Then Return
            If (SaveFormData() = False) Then Return
        End If

        ' ----- Prompt the user to add a author.
        authorID = (New AuthorAddLocate).PromptUser(nameTypeID)
        If (authorID = -1) Then Return

        ' ----- Jump out early if the record already exists.
        sqlText = "SELECT COUNT(*) FROM ItemAuthor WHERE ItemID = " & ActiveID & _
            " AND AuthorID = " & authorID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then Return
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Get the next sequence number.
        sequenceID = 1
        sqlText = "SELECT MAX(Sequence) AS TheMax FROM ItemAuthor WHERE ItemID = " & _
            ActiveID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read() = True) Then
            If (IsDBNull(dbInfo!TheMax) = False) Then _
                sequenceID = CInt(dbInfo!TheMax) + 1
        End If
        dbInfo.Close()

        ' ----- Add the author to the database record.
        sqlText = "INSERT INTO ItemAuthor (ItemID, AuthorID, Sequence, AuthorType) " & _
            "VALUES (" & ActiveID & ", " & authorID & ", " & sequenceID & _
            ", " & nameTypeID & ")"
        ExecuteSQL(sqlText)
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Get the text of the name type.
        sqlText = "SELECT FullName FROM CodeAuthorType WHERE ID = " & nameTypeID
        typeText = CStr(ExecuteSQLReturn(sqlText))

        ' ----- Get the author name.
        sqlText = "SELECT * FROM Author WHERE ID = " & authorID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        authorText = FormatAuthorName(dbInfo)
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- Add this item to the display.
        newItem = AuthorsList.Items.Add(authorText)
        newItem.SubItems.Add(typeText)
        newItem.Tag = CStr(authorID) & "," & CStr(nameTypeID)
        UpdateAuthorButtons()
        Me.Cursor = Windows.Forms.Cursors.Default
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("NamedItem.ActSubjectAdd_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub ActAuthorDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAuthorDelete.Click
        ' ----- Allow the user to delete an author entry.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Make sure an item is selected.
        If (AuthorsList.SelectedItems.Count = 0) Then Return

        ' ----- Confirm with the user.
        If (MsgBox("Really disassociate the author '" & _
            AuthorsList.SelectedItems(0).Text & "' from the library item?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
            <> MsgBoxResult.Yes) Then Return

        ' ----- Remove the selection from the database.
        sqlText = "DELETE FROM ItemAuthor WHERE ItemID = " & ActiveID & _
            " AND AuthorID = " & GetSubStr(CStr(AuthorsList.SelectedItems(0).Tag), _
            ",", 1)
        ExecuteSQL(sqlText)
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Remove the item from the display.
        AuthorsList.Items.Remove(AuthorsList.SelectedItems(0))
        UpdateAuthorButtons()
        Return

ErrorHandler:
        GeneralError("NamedItem.ActAuthorDelete_Click", Err.GetException())
        Return
    End Sub

    Private Sub ActAuthorMoveUp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAuthorMoveUp.Click
        ' ----- Move the selected author up in priority.
        If (AuthorsList.SelectedItems.Count = 0) Then Return
        If (AuthorsList.SelectedItems(0).Index = 0) Then Return

        SwitchTwoAuthors(AuthorsList.SelectedItems(0).Index, _
            AuthorsList.SelectedItems(0).Index - 1)
    End Sub

    Private Sub ActAuthorMoveDown_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAuthorMoveDown.Click
        ' ----- Move the selected author down in priority.
        If (AuthorsList.SelectedItems.Count = 0) Then Return
        If (AuthorsList.SelectedItems(0).Index = (AuthorsList.Items.Count - 1)) _
            Then Return

        SwitchTwoAuthors(AuthorsList.SelectedItems(0).Index, _
            AuthorsList.SelectedItems(0).Index + 1)
    End Sub

    Private Sub SwitchTwoAuthors(ByVal activeIndex As Integer, ByVal inactiveIndex As Integer)
        ' ----- Given the indexes of two items in the AuthorsList control,
        '       switch them in the database and on-screen.
        Dim activeAuthor As Integer
        Dim inactiveAuthor As Integer
        Dim activeSequence As Integer
        Dim inactiveSequence As Integer
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim activeItem As System.Windows.Forms.ListViewItem
        Dim inactiveItem As System.Windows.Forms.ListViewItem
        Dim holdText As String

        On Error GoTo ErrorHandler

        ' ----- Get the author IDs from the two items.
        activeItem = AuthorsList.Items(activeIndex)
        inactiveItem = AuthorsList.Items(inactiveIndex)
        activeAuthor = CInt(GetSubStr(CStr(activeItem.Tag), ",", 1))
        inactiveAuthor = CInt(GetSubStr(CStr(inactiveItem.Tag), ",", 1))

        ' ----- Get the sequence numbers of the two records.
        activeSequence = -1
        inactiveSequence = -1
        sqlText = "SELECT AuthorID, Sequence FROM ItemAuthor WHERE ItemID = " & _
            ActiveID & " AND AuthorID IN (" & activeAuthor & ", " & _
            inactiveAuthor & ")"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            If (CInt(dbInfo!AuthorID) = activeAuthor) Then
                activeSequence = CInt(dbInfo!Sequence)
            ElseIf (CInt(dbInfo!AuthorID) = inactiveAuthor) Then
                inactiveSequence = CInt(dbInfo!Sequence)
            End If
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        If (activeSequence = -1) Or (inactiveSequence = -1) Then
            MsgBox("Error occurred while trying to access the author or name " & _
                "records.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If

        ' ----- Update the database records.
        Me.Cursor = Windows.Forms.Cursors.WaitCursor
        TransactionBegin()
        sqlText = "UPDATE ItemAuthor SET Sequence = " & inactiveSequence & _
            " WHERE ItemID = " & ActiveID & " AND AuthorID = " & activeAuthor
        ExecuteSQL(sqlText)
        sqlText = "UPDATE ItemAuthor SET Sequence = " & activeSequence & _
            " WHERE ItemID = " & ActiveID & " AND AuthorID = " & inactiveAuthor
        ExecuteSQL(sqlText)
        TransactionCommit()
        Me.Cursor = Windows.Forms.Cursors.Default

        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Reverse the elements on the display.
        holdText = activeItem.Text
        activeItem.Text = inactiveItem.Text
        inactiveItem.Text = holdText

        holdText = activeItem.SubItems(1).Text
        activeItem.SubItems(1).Text = inactiveItem.SubItems(1).Text
        inactiveItem.SubItems(1).Text = holdText

        holdText = CStr(activeItem.Tag)
        activeItem.Tag = inactiveItem.Tag
        inactiveItem.Tag = holdText

        ' ----- Highlight the correct item.
        AuthorsList.SelectedItems.Clear()
        inactiveItem.Selected = True
        UpdateAuthorButtons()
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("NamedItem.SwitchTwoAuthors", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        TransactionRollback()
        Return
    End Sub

    Private Sub ActAuthorProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAuthorProperties.Click
        ' ----- Allow the use to change the name type.
        Dim sqlText As String
        Dim newNameType As Integer
        Dim oldNameType As Integer
        Dim authorID As Integer
        Dim holdText As String

        On Error GoTo ErrorHandler

        ' ----- Check for invalid data.
        If (AuthorsList.SelectedItems.Count = 0) Then Return

        ' ----- Get the IDs of the author and author type.
        holdText = CStr(AuthorsList.SelectedItems(0).Tag)
        authorID = CInt(GetSubStr(holdText, ",", 1))
        oldNameType = CInt(GetSubStr(holdText, ",", 2))

        ' ----- Prompt the user.
        newNameType = (New ItemAuthorEdit).EditRecord(ActiveID, authorID, _
            BuildItemName(), AuthorsList.SelectedItems(0).Text, oldNameType)
        If (newNameType = -1) Then Return
        ActCancel.Text = "Close"
        SessionSaved = True
        If (newNameType = oldNameType) Then Return

        ' ----- Update the internal value of the display item.
        AuthorsList.SelectedItems(0).Tag = authorID & "," & newNameType

        ' ----- Get the text for the new name type and update the display.
        sqlText = "SELECT FullName FROM CodeAuthorType WHERE ID = " & newNameType
        AuthorsList.SelectedItems(0).SubItems(1).Text = CStr(ExecuteSQLReturn(sqlText))
        Return

ErrorHandler:
        GeneralError("NamedItem.ActAuthorProperties_Click", Err.GetException())
        Return
    End Sub

    Private Sub UpdateAuthorButtons()
        ' ----- Update the author panel buttons based on selection.
        If (AuthorsList.SelectedItems.Count = 0) Then
            ' ----- No selection.
            ActAuthorDelete.Enabled = False
            ActAuthorProperties.Enabled = False
            ActAuthorMoveUp.Enabled = False
            ActAuthorMoveDown.Enabled = False
        Else
            ' ----- Selection.
            ActAuthorDelete.Enabled = True
            ActAuthorProperties.Enabled = True
            ActAuthorMoveUp.Enabled = (AuthorsList.SelectedItems(0).Index > 0)
            ActAuthorMoveDown.Enabled = (AuthorsList.SelectedItems(0).Index < _
                (AuthorsList.Items.Count - 1))
        End If
    End Sub

    Private Sub AuthorsList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AuthorsList.SelectedIndexChanged
        ' ----- Refresh the buttons.
        UpdateAuthorButtons()
    End Sub

    Private Sub AuthorsList_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles AuthorsList.Click
        ' ----- Refresh the buttons.
        UpdateAuthorButtons()
    End Sub

    Private Sub ActCopyAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActCopyAdd.Click
        ' ----- Let the user add a new author.
        Dim copyID As Integer
        Dim sqlText As String
        Dim barcode As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim copyNumber As String
        Dim newItem As System.Windows.Forms.ListViewItem

        On Error GoTo ErrorHandler

        ' ----- The record must be saved first.
        If (ActiveID = -1) Then
            ' ----- Confirm with the user.
            If (MsgBox("The item must be saved to the database before copies " & _
                "can be added. Would you like to save the record now?", _
                MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
                <> MsgBoxResult.Yes) Then Return

            ' ----- Verify and save the data.
            If (ValidateFormData() = False) Then Return
            If (SaveFormData() = False) Then Return
        End If

        ' ----- Prompt the user to add a subject.
        copyID = (New ItemCopy).AddCopy(ActiveID, BuildItemName())
        If (copyID = -1) Then Return
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Get some information about the copy.
        sqlText = "SELECT CopyNumber, Barcode FROM ItemCopy WHERE ID = " & copyID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        copyNumber = CStr(dbInfo!CopyNumber)
        barcode = DBGetText(dbInfo!Barcode)
        dbInfo.Close()
        dbInfo = Nothing
        If (barcode = "") Then barcode = "None"

        ' ----- Add this item to the display.
        newItem = CopyList.Items.Add(copyNumber)
        newItem.SubItems.Add(GetCopyDisposition(copyID))
        newItem.SubItems.Add(barcode)
        newItem.Tag = CStr(copyID)
        UpdateCopyButtons()
        Return

ErrorHandler:
        GeneralError("NamedItem.ActCopyAdd_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function BuildItemName() As String
        ' ----- Build the text of the item name used by some subordinate
        '       forms.
        Dim itemName As String

        ' ----- The name includes title, subtitle, edition, and media type.
        itemName = RecordTitle.Text
        If (Trim(RecordSubtitle.Text) <> "") Then itemName &= ": " & RecordSubtitle.Text
        If (Trim(RecordEdition.Text) <> "") Then itemName &= ", " & RecordEdition.Text
        If (CInt(CType(RecordMediaType.SelectedItem, ListItemData)) <> -1) Then _
            itemName &= " [" & RecordMediaType.Text & "]"
        Return itemName
    End Function

    Private Sub ActCopyEdit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActCopyEdit.Click
        ' ----- Let the user edit an existing author.
        Dim copyID As Integer
        Dim sqlText As String
        Dim barcode As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim copyNumber As String
        Dim newItem As System.Windows.Forms.ListViewItem

        On Error GoTo ErrorHandler

        ' ----- Check for invalid data.
        If (CopyList.SelectedItems.Count = 0) Then Return
        newItem = CopyList.SelectedItems(0)
        copyID = CInt(newItem.Tag)

        ' ----- Prompt the user to add a subject.
        copyID = (New ItemCopy).EditCopy(ActiveID, copyID, BuildItemName())
        If (copyID = -1) Then Return
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Get some information about the copy.
        sqlText = "SELECT CopyNumber, Barcode FROM ItemCopy WHERE ID = " & copyID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        copyNumber = CStr(dbInfo!CopyNumber)
        barcode = DBGetText(dbInfo!Barcode)
        dbInfo.Close()
        dbInfo = Nothing
        If (barcode = "") Then barcode = "None"

        ' ----- Update this item in the display.
        newItem.SubItems(0).Text = copyNumber
        newItem.SubItems(1).Text = GetCopyDisposition(copyID)
        newItem.SubItems(2).Text = barcode
        UpdateCopyButtons()
        Return

ErrorHandler:
        GeneralError("NamedItem.ActCopyEdit_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub ActCopyDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActCopyDelete.Click
        ' ----- Allow the user to delete an item copy.
        Dim sqlText As String
        Dim holdText As String
        Dim copyID As Integer

        On Error GoTo ErrorHandler

        ' ----- Make sure an item is selected.
        If (CopyList.SelectedItems.Count = 0) Then Return
        copyID = CInt(CopyList.SelectedItems(0).Tag)

        ' ----- Make sure this item is not checked out.
        holdText = CopyList.SelectedItems(0).SubItems(1).Text
        If (holdText = "Checked Out") Or (holdText = "Overdue") Then
            MsgBox("You cannot delete this item because it is still checked out.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End If

        ' ----- Confirm with the user.
        If (MsgBox("Deleting an item copy from the library is a permanent " & _
            "process; it cannot be undone without re-entering the item." & _
            vbCrLf & vbCrLf & "Also, any patron history associated with this " & _
            "item copy will be lost. This includes any unpaid fines associated " & _
            "with the item copy." & vbCrLf & vbCrLf & "Modifying the item copy " & _
            "and setting it to 'Not Available' is often a better choice." & _
            vbCrLf & vbCrLf & "Really delete copy '" & _
            CopyList.SelectedItems(0).Text & "' from the database?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
            <> MsgBoxResult.Yes) Then Return

        ' ----- Remove the selection from the database.
        TransactionBegin()
        sqlText = "DELETE FROM PatronPayment WHERE PatronCopy IN (SELECT ID " & _
            "FROM PatronCopy WHERE ItemCopy = " & copyID & ")"
        ExecuteSQL(sqlText)
        sqlText = "DELETE FROM PatronCopy WHERE ItemCopy = " & copyID
        ExecuteSQL(sqlText)
        sqlText = "DELETE FROM ItemCopy WHERE ID = " & copyID
        ExecuteSQL(sqlText)
        TransactionCommit()
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Remove the item from the display.
        CopyList.Items.Remove(CopyList.SelectedItems(0))
        UpdateCopyButtons()
        Return

ErrorHandler:
        GeneralError("NamedItem.ActCopyDelete_Click", Err.GetException())
        TransactionRollback()
        Return
    End Sub

    Private Sub UpdateCopyButtons()
        ' ----- Update the author panel buttons based on selection.
        If (CopyList.SelectedItems.Count = 0) Then
            ' ----- No selection.
            ActCopyEdit.Enabled = False
            ActCopyDelete.Enabled = False
        Else
            ' ----- Selection.
            ActCopyEdit.Enabled = True
            ActCopyDelete.Enabled = True
        End If
    End Sub

    Private Sub CopyList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyList.SelectedIndexChanged
        ' ----- Refresh the buttons.
        UpdateCopyButtons()
    End Sub

    Private Sub CopyList_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles CopyList.Click
        ' ----- Refresh the buttons.
        UpdateCopyButtons()
    End Sub

    Private Sub AuthorsList_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles AuthorsList.DoubleClick
        ' ----- Same as a click on the edit button.
        If (ActAuthorProperties.Enabled = True) Then ActAuthorProperties.PerformClick()
    End Sub

    Private Sub CopyList_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles CopyList.DoubleClick
        ' ----- Same as a click on the edit button.
        If (ActCopyEdit.Enabled = True) Then ActCopyEdit.PerformClick()
    End Sub

    Private Sub RecordTitle_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordTitle.TextChanged
        ' ----- Update the large display title when the title changes.
        If (Trim(RecordTitle.Text) = "") Then
            MainTitle.Text = "Untitled"
        Else
            MainTitle.Text = Trim(RecordTitle.Text)
        End If
    End Sub
End Class
