' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class PatronAccess
    Private SelectedPatron As Integer
    Private InListMode As Boolean

    Public Function SelectPatron() As Integer
        ' ----- Let the user connect as a patron. Return the patron ID,
        '       or -1 if not selected.
        InListMode = False
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.Cancel) Then _
            Return -1 Else Return SelectedPatron
    End Function

    Private Sub RecordBarcode_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordBarcode.Enter
        ' ----- Highlight the entire text.
        RecordBarcode.SelectAll()
    End Sub

    Private Sub RecordPassword_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordPassword.Enter
        ' ----- Highlight the entire text.
        RecordPassword.SelectAll()
    End Sub

    Private Sub PatronAccess_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "PatronAccess.htm")
    End Sub

    Private Sub PatronAccess_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.

        ' ----- Move the list box to a reasonable position.
        PatronList.Location = New Point(LabelBarcode.Left, RecordBarcode.Top)
        PatronList.Size = New Size(RecordBarcode.Width + (RecordBarcode.Left - _
            LabelBarcode.Left), PatronList.Height)

        ' ----- Reposition the password field.
        LabelNotRequired.Location = New Point(RecordPassword.Left, _
            LabelPassword.Top)

        ' ----- Display the right password field.
        RecordPassword.Visible = _
                Not SecurityProfile(LibrarySecurity.PatronOverride)
        LabelNotRequired.Visible = _
            SecurityProfile(LibrarySecurity.PatronOverride)
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to access a patron.
        Dim barcodeText As String
        Dim lastName As String
        Dim firstName As String
        Dim passwordText As String
        Dim holdText As String
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim patronID As Integer

        On Error GoTo ErrorHandler

        ' ----- Special code for list mode.
        If (InListMode = True) Then
            ' ----- A patron selection is required.
            If (PatronList.SelectedIndex = -1) Then
                MsgBox("Please select a patron from the list.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                PatronList.Focus()
                Return
            End If

            ' ----- Save the selection and close the form.
            SelectedPatron = CInt(CType(PatronList.SelectedItem, ListItemData))
            Me.DialogResult = Windows.Forms.DialogResult.OK
            Return
        End If

        ' ----- A patron name or barcode is required.
        If (Trim(RecordBarcode.Text) = "") Then
            MsgBox("Please supply a barcode or patron name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordBarcode.Focus()
            Return
        End If

        ' ----- Get the name or barcode text components.
        If (IsNumeric(RecordBarcode.Text)) Then
            ' ----- Perform a barcode search.
            barcodeText = DigitsOnly(RecordBarcode.Text)
            lastName = ""
            firstName = ""
        Else
            ' ----- Perform a name search.
            barcodeText = ""
            lastName = Trim(GetSubStr(RecordBarcode.Text, ",", 1))
            firstName = Trim(GetSubStr(RecordBarcode.Text, ",", 2))

            ' ----- A last name is required.
            If (lastName = "") Then
                MsgBox("Please supply a patron name in 'Last, First' format.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                RecordBarcode.Focus()
                Return
            End If

            ' ----- No wildcards allowed in the last name.
            If (InStr(lastName, "*") > 0) Then
                MsgBox("The last name may not include wildcard characters.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                RecordBarcode.Focus()
                Return
            End If

            ' ----- First name may contain wildcards.
            If (InStr(firstName, "*") = 0) Then firstName &= "*"
        End If

        ' ----- Check the password, if required.
        If (SecurityProfile(LibrarySecurity.PatronOverride) = True) Then
            passwordText = ""
        Else
            passwordText = EncryptPassword("patron", RecordPassword.Text)
        End If

        ' ----- Try to access the record.
        Me.Cursor = Windows.Forms.Cursors.WaitCursor
        sqlText = "SELECT ID, AdminMessage FROM Patron WHERE "
        If (barcodeText <> "") Then
            ' ----- Barcode search.
            sqlText &= "Barcode = " & DBText(barcodeText)
        Else
            ' ----- Name search.
            sqlText &= "LastName LIKE " & DBText(lastName) & _
                " AND FirstName LIKE " & DBText(Replace(firstName, "*", "%"))
        End If
        If (passwordText <> "") Then _
            sqlText &= " AND [Password] = " & DBText(passwordText)
        dbInfo = CreateReader(sqlText)
        If (dbInfo.HasRows = False) Then
            Me.Cursor = Windows.Forms.Cursors.Default
            dbInfo.Close()
            dbInfo = Nothing

            ' ----- Perhaps a barcode was being scanned, and the enter key added to
            '       the end caused the OK button to trigger. If so, silently move
            '       to the password field.
            If (barcodeText <> "") And (RecordPassword.Text = "") And _
                    (Me.ActiveControl.Name = RecordBarcode.Name) Then
                On Error Resume Next
                RecordPassword.Focus()
            Else
                MsgBox("Invalid patron or password.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            End If
            Return
        End If
        Me.Cursor = Windows.Forms.Cursors.Default

        ' ----- We found at least one matching patron.
        dbInfo.Read()
        patronID = CInt(dbInfo!ID)
        If (dbInfo.Read) Then
            ' ----- We found a second record. If in patron mode, then the user
            '       was too vague.
            dbInfo.Close()
            dbInfo = Nothing
            If (passwordText <> "") Then
                MsgBox("Invalid patron or password.", _
                    MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                Return
            End If

            ' ----- Allow the user to choose the patron.
            sqlText = Replace(sqlText, "SELECT ID", _
                "SELECT ID, LastName, FirstName, BarCode")
            dbInfo = CreateReader(sqlText)
            Do While dbInfo.Read
                ' ----- Add one patron to the list.
                holdText = CStr(dbInfo!LastName) & ", " & CStr(dbInfo!FirstName)
                If Not (TypeOf (dbInfo!Barcode) Is DBNull) Then _
                    holdText &= " (" & CStr(dbInfo!Barcode) & ")"
                PatronList.Items.Add(New ListItemData(holdText, CInt(dbInfo!ID)))
            Loop
            dbInfo.Close()
            dbInfo = Nothing

            ' ----- Change into "list" mode.
            InfoText.Text = "Your criteria matched more than one patron. " & _
                "Please select the correct patron from the list."
            LabelBarcode.Visible = False
            RecordBarcode.Visible = False
            LabelPassword.Visible = False
            RecordPassword.Visible = False
            LabelNotRequired.Visible = False
            PatronList.Visible = True
            PatronList.Focus()
            InListMode = True
            Return
        Else
            ' ----- Found a valid match.
            dbInfo.Close()
            dbInfo = Nothing
        End If

        ' ----- If authorized, show the administrative message for this patron.
        If (SecurityProfile(LibrarySecurity.ViewAdminPatronMessages) = True) Then
            sqlText = "SELECT FirstName, LastName, AdminMessage FROM Patron " & _
                "WHERE ID = " & patronID
            dbInfo = CreateReader(sqlText)
            dbInfo.Read()
            If Not (TypeOf (dbInfo!AdminMessage) Is DBNull) Then
                MsgBox("Message concerning " & DBGetText(dbInfo!FirstName) & " " & _
                    DBGetText(dbInfo!LastName) & ":" & vbCrLf & vbCrLf & _
                    DBGetText(dbInfo!AdminMessage), _
                    MsgBoxStyle.OKOnly Or MsgBoxStyle.Information, ProgramTitle)
            End If
            dbInfo.Close()
        End If

        ' ----- Finished.
        SelectedPatron = patronID
        Me.DialogResult = Windows.Forms.DialogResult.OK
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("PatronAccess.ActOK_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub PatronList_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles PatronList.DoubleClick
        ' ----- Same as a click on the OK button.
        If (PatronList.SelectedIndex = -1) Then Return
        ActOK.PerformClick()
    End Sub
End Class