' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class Publisher
    Private ActiveID As Integer
    Private StartingOver As Boolean

    Public Overloads Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overloads Function AddRecord(ByVal pubText As String) As Integer
        ' ----- Add a new record, but use a starting value previously entered
        '       by the user.
        ActiveID = -1
        PrepareFormFields()
        RecordFullName.Text = pubText
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then Return _
            ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManagePublishers)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManagePublishers)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManagePublishers)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the publisher?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM NamedItem WHERE Publisher = " & recordID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("You cannot delete this record because it is in use.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Delete the record.
        sqlText = "DELETE FROM Publisher WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        Return True

ErrorHandler:
        GeneralError("Publisher.DeleteRecord", Err.GetException())
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items. Prompt the
        '       user first.
        destList.Items.Clear()
        SearchForRecord(destList, exceededMatches)
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM Publisher WHERE ID = " & recordID
        Return CStr(ExecuteSQLReturn(sqlText))

ErrorHandler:
        GeneralError("Publisher.FormatRecordName", Err.GetException())
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "This form lets you defined the list of publishers that distribute " & _
            "items found in the library."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Publishers"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim inUse As Integer

        On Error GoTo ErrorHandler

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        RecordWeb.Text = ""
        RecordStatus.Text = "New publisher record"
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM Publisher WHERE ID = " & ActiveID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            RecordFullName.Text = CStr(dbInfo!FullName)
            RecordWeb.Text = DBGetText(dbInfo!WebSite)
        End If
        dbInfo.Close()

        ' ----- Find out how many records link to this publisher.
        sqlText = "SELECT COUNT(*) FROM NamedItem WHERE Publisher = " & ActiveID
        inUse = CInt(ExecuteSQLReturn(sqlText))
        RecordStatus.Text = "Linked to " & CStr(inUse) & " library item"
        If (inUse <> 1) Then RecordStatus.Text &= "s"
        Return

ErrorHandler:
        GeneralError("Publisher.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Sub SearchForRecord(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Search for publishers using user-specified criteria.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim sLimit As String
        Dim resultCount As Integer

        On Error GoTo ErrorHandler

        ' ----- Prompt the user for the limited publisher name.
        exceededMatches = False
        sLimit = (New PublisherLimit).PromptUser()
        If (sLimit = "") Then Return
        If (InStr(sLimit, "*") = 0) Then sLimit &= "*"

        ' ----- Clear any existing values.
        destList.Items.Clear()
        resultCount = 0

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, FullName FROM Publisher WHERE FullName LIKE " & _
            DBText(Replace(Trim(sLimit), "*", "%")) & " ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            resultCount += 1
            If (resultCount > SearchMatchLimit) Then
                exceededMatches = True
                Exit Do
            End If
            destList.Items.Add(New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("Publisher.SearchForRecord", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function UsesSearch() As Boolean
        ' ----- Publisher records are searchable.
        Return True
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the record.
        Dim sqlText As String
        Dim newID As Integer

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO Publisher (FullName, WebSite) OUTPUT INSERTED.ID " & _
                "VALUES (" & DBText(Trim(RecordFullName.Text)) & ", " & _
                DBText(Trim(RecordWeb.Text)) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE Publisher SET " & _
                "FullName = " & DBText(Trim(RecordFullName.Text)) & _
                ", WebSite = " & DBText(Trim(RecordWeb.Text)) & _
                " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("Publisher.SaveFormData", Err.GetException())
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Name is a required field.
        If (Trim(RecordFullName.Text) = "") Then
            MsgBox("Please supply the publisher name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM Publisher WHERE UPPER(FullName) = " & _
            DBText(UCase(Trim(RecordFullName.Text)))
        If (ActiveID <> -1) Then sqlText &= " AND ID <> " & ActiveID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            ' ----- Duplicate entry.
            MsgBox("A record already exists with the name '" & Trim(RecordFullName.Text) & _
                "'.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("Publisher.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub Publisher_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordFullName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordFullName.Enter
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub RecordWeb_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordWeb.Enter
        ' ----- Highlight the entire text.
        RecordWeb.SelectAll()
    End Sub

    Private Sub ShowWeb_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ShowWeb.Click
        ' ----- Show the web site displayed in the field.
        Dim newProcess As ProcessStartInfo

        On Error Resume Next
        If (Trim(RecordWeb.Text) = "") Then Return
        newProcess = New ProcessStartInfo(Trim(RecordWeb.Text))
        Process.Start(newProcess)
    End Sub

    Private Sub RecordWeb_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordWeb.TextChanged
        ' ----- Enable the web site button as needed.
        ShowWeb.Enabled = CBool(Trim(RecordWeb.Text) <> "")
    End Sub

    Private Sub Publisher_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "Publisher.htm")
    End Sub
End Class
