' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class PublisherAddLocate
    Private ActivePanel As Integer

    Private Const PanelCriteria As Integer = 1
    Private Const PanelResults As Integer = 2

    Private Function FindUserRecords(ByVal searchText As String) As Integer
        ' ----- Search for publishers using user-specified criteria.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim matches As Integer

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        SearchResults.Items.Clear()
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Retrieve the values from the database.
        matches = 0
        sqlText = "SELECT ID, FullName FROM Publisher WHERE FullName LIKE " & _
            DBText(searchText) & " ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            SearchResults.Items.Add(New ListItemData(CStr(dbInfo!FullName), _
                CInt(dbInfo!ID)))
            matches += 1
            If (matches >= SearchMatchLimit) Then Exit Do
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Me.Cursor = Windows.Forms.Cursors.Default
        Return matches

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("PublisherAddLocate.FindUserRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return -1
    End Function

    Public Function PromptUser() As Integer
        ' ----- Prompt the user for a publisher. Return the matching ID. Return
        '       -1 if no change made, or -2 to clear the active setting.
        ActivePanel = PanelCriteria
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then
            If (CInt(CType(SearchResults.SelectedItem, ListItemData)) = -2) Then
                ' ----- Clear the current publisher.
                Return -2
            ElseIf (CInt(CType(SearchResults.SelectedItem, ListItemData)) = -1) Then
                ' ----- Prompt for a new publisher using the original
                '       typed name.
                Return (New Publisher).AddRecord(Trim(PublisherName.Text))
            Else
                ' ----- The user selected a match.
                Return CInt(CType(SearchResults.SelectedItem, ListItemData))
            End If
        Else
            ' ----- Cancelled by user.
            Return -1
        End If
    End Function

    Private Sub ActBack_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ActBack.Click
        ' ----- Go back to the first panel.
        ActivePanel = PanelCriteria
        SecondPanel.Visible = False
        FirstPanel.Visible = True
        ActBack.Enabled = False
        ActNext.Text = "&Next >"
        PublisherName.Focus()
    End Sub

    Private Sub ActNext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActNext.Click
        ' ----- Take action on the user input.
        Dim searchText As String
        Dim matches As Integer

        ' ----- Different code for each of the panels.
        If (ActivePanel = PanelCriteria) Then
            ' ----- Determine the search string.
            searchText = Trim(PublisherName.Text)
            If (searchText = "") Then
                MsgBox("Please supply a publisher name.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                PublisherName.Focus()
                Return
            End If
            If (InStr(searchText, "*") = 0) Then searchText &= "*"
            searchText = Replace(searchText, "*", "%")

            ' ----- Search for matches. 
            matches = FindUserRecords(searchText)
            If (matches = -1) Then Return

            ' ----- If there are no matches, then either error out, or
            '       add a new record, depending on the user's security.
            '       If there are matches, show them.
            If (matches = 0) Then
                If (SecurityProfile(LibrarySecurity.ManagePublishers)) Then
                    ' ----- Prompt to add a new record.
                    SearchResults.ClearSelected()
                    SearchResults.Items.Add(New ListItemData( _
                        "<No matches. Add new publisher.>", -1))
                    SearchResults.SelectedIndex = 0

                    ' ----- Return to the PromptUser() routine.
                    Me.DialogResult = Windows.Forms.DialogResult.OK
                Else
                    ' ----- Give an error.
                    MsgBox("No matches.", _
                        MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                    Return
                End If
            Else
                ' ----- If the user can add new publishers, add that option.
                If (SecurityProfile(LibrarySecurity.ManagePublishers)) Then _
                    SearchResults.Items.Insert(0, New ListItemData( _
                        "<None of these matches. Add new publisher.>", -1))
                ActivePanel = PanelResults
                SecondPanel.Visible = True
                FirstPanel.Visible = False
                ActBack.Enabled = True
                ActNext.Text = "Finish"
                SearchResults.Focus()
            End If
        Else
            ' ----- Check for a selected result.
            If (SearchResults.SelectedIndex = -1) Then
                MsgBox("Please select a match from the list.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                SearchResults.Focus()
                Return
            End If

            ' ----- Return to the PromptUser() routine.
            Me.DialogResult = Windows.Forms.DialogResult.OK
        End If
    End Sub

    Private Sub PublisherName_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PublisherName.Enter
        ' ----- Highlight the entire text.
        PublisherName.SelectAll()
    End Sub

    Private Sub SearchResults_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles SearchResults.DoubleClick
        ' ----- Same as the finish button.
        If (SearchResults.SelectedIndex = -1) Then Return
        ActNext.PerformClick()
    End Sub

    Private Sub ActClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActClear.Click
        ' ----- Clear the current publisher.
        SearchResults.ClearSelected()
        SearchResults.Items.Add(New ListItemData("Clear item", -2))
        SearchResults.SelectedIndex = 0

        ' ----- Return to the PromptUser() routine.
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub PublisherAddLocate_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "PublisherAddLocate.htm")
    End Sub

    Private Sub PublisherAddLocate_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        SecondPanel.Location = New System.Drawing.Point(FirstPanel.Location.X, _
            FirstPanel.Location.Y)
    End Sub
End Class
