' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Module TicketPrinting
    Public Sub PrintCheckoutTicket(ByVal patronID As Integer, _
            ByVal checkedOutItems As ListBox)
        ' ----- Print out a ticket of what the patron checked out. The supplied
        '       ListBox control contains objects of type CheckedOutItem.
        Dim ticketWidth As Integer
        Dim ticketText As System.Text.StringBuilder
        Dim counter As Integer
        Dim patronFines As Decimal
        Dim itemDetail As CheckedOutItem

        On Error GoTo ErrorHandler

        ' ----- Ignore if there is nothing to print.
        If (patronID = -1) Or (checkedOutItems.Items.Count = 0) Then Return

        ' ----- Get the width of the ticket.
        ticketWidth = My.Settings.ReceiptWidth
        If (ticketWidth <= 0) Then ticketWidth = 40

        ' ----- Build the heading.
        ticketText = GetTicketHeader(patronID, ticketWidth)
        If (ticketText Is Nothing) Then Return

        ' ----- Process each checked-out item.
        For counter = 0 To checkedOutItems.Items.Count - 1
            ' ----- Extract the detail from the list.
            itemDetail = CType(checkedOutItems.Items(counter), CheckedOutItem)

            ' ----- Add the item name.
            ticketText.AppendLine(Left(itemDetail.ItemTitle, ticketWidth))

            ' ----- Add the barcode number and due date.
            ticketText.AppendLine(LeftAndRightText(itemDetail.Barcode, _
                "Due: " & Format(itemDetail.DueDate, "MMM d, yyyy"), ticketWidth))
            ticketText.AppendLine()
        Next counter

        ' ----- If there are fines due, print them here.
        patronFines = CalculatePatronFines(patronID)
        If (patronFines > 0@) Then
            ticketText.AppendLine("Fines Due: " & Format(patronFines, "$#,##0.00"))
            ticketText.AppendLine()
        End If

        ' ----- Add the bottom display text.
        ticketText.Append(GetTicketFooter(ticketWidth))

        ' ----- Send the ticket to the printer.
        RawPrinterHelper.SendStringToPrinter(My.Settings.ReceiptPrinter, _
            ticketText.ToString(), "Checkout Receipt")
        Return

ErrorHandler:
        GeneralError("TicketPrinting.PrintCheckoutTicket", Err.GetException())
        Return
    End Sub

    Public Sub PrintBalanceTicket(ByVal patronID As Integer, _
            ByRef finesList As ListBox)
        ' ----- Print out a ticket of unpaid balances. The supplied ListBox
        '       control contains objects of type PatronDetialItem.
        Dim ticketWidth As Integer
        Dim ticketText As System.Text.StringBuilder
        Dim holdText As String
        Dim counter As Integer
        Dim sqlText As String
        Dim itemDetail As PatronDetailItem
        Dim included As Integer = 0

        On Error GoTo ErrorHandler

        ' ----- Ignore if there is nothing to print.
        If (patronID = -1) Or (finesList.Items.Count = 0) Then Return

        ' ----- Get the width of the ticket.
        ticketWidth = My.Settings.ReceiptWidth
        If (ticketWidth <= 0) Then ticketWidth = 40

        ' ----- Build the heading.
        ticketText = GetTicketHeader(patronID, ticketWidth)
        If (ticketText Is Nothing) Then Return

        ' ----- Process each checked-out item.
        For counter = 0 To finesList.Items.Count - 1
            ' ----- Extract the detail from the list.
            itemDetail = CType(finesList.Items(counter), PatronDetailItem)

            ' ----- Skip entries with a zero balance.
            If (itemDetail.BalanceDue = 0@) Then Continue For
            included += 1

            ' ----- Add the item name.
            ticketText.AppendLine(Left(itemDetail.TitleText, ticketWidth))

            ' ----- Print the remaining balance.
            ticketText.AppendLine(LeftAndRightText("Balance Due:", _
                Format(itemDetail.BalanceDue, "$#,##0.00"), ticketWidth))
            ticketText.AppendLine()
        Next counter

        ' ----- Warn if there is nothing to print.
        If (included = 0) Then
            MsgBox("There are no unpaid balances to print.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
            Return
        End If

        ' ----- If there are fines due, print them here.
        ticketText.AppendLine("Fines Due: " & Format( _
            CalculatePatronFines(patronID), "$#,##0.00"))
        ticketText.AppendLine()

        ' ----- Add the bottom display text.
        ticketText.Append(GetTicketFooter(ticketWidth))

        ' ----- Send the ticket to the printer.
        RawPrinterHelper.SendStringToPrinter(My.Settings.ReceiptPrinter, _
            ticketText.ToString(), "Balance Receipt")
        Return

ErrorHandler:
        GeneralError("TicketPrinting.PrintBalanceTicket", Err.GetException())
        Return
    End Sub

    Public Sub PrintPaymentTicket(ByVal patronID As Integer, _
            ByRef paymentList As Generic.List(Of PaymentItem))
        ' ----- Print out a ticket of unpaid balances. The supplied ListBox
        '       control contains objects of type PatronDetialItem.
        Dim ticketWidth As Integer
        Dim ticketText As System.Text.StringBuilder
        Dim holdText As String
        Dim counter As Integer
        Dim sqlText As String
        Dim itemDetail As PaymentItem
        Dim included As Integer = 0

        On Error GoTo ErrorHandler

        ' ----- Ignore if there is nothing to print.
        If (patronID = -1) Or (paymentList.Count = 0) Then Return

        ' ----- Get the width of the ticket.
        ticketWidth = My.Settings.ReceiptWidth
        If (ticketWidth <= 0) Then ticketWidth = 40

        ' ----- Build the heading.
        ticketText = GetTicketHeader(patronID, ticketWidth)
        If (ticketText Is Nothing) Then Return

        ' ----- Process each checked-out item.
        For counter = 0 To paymentList.Count - 1
            ' ----- Extract the detail from the list.
            itemDetail = paymentList(counter)

            ' ----- Ignore items with no payment amount.
            If (itemDetail.FeesPaid = 0@) Then Continue For
            included += 1

            ' ----- Add the item name.
            ticketText.AppendLine(Left(itemDetail.ItemTitle, ticketWidth))

            ' ----- Show the payment amount.
            ticketText.AppendLine(LeftAndRightText("Payments:", _
                Format(itemDetail.FeesPaid, "$#,##0.00"), ticketWidth))

            ' ----- Print the remaining balance.
            ticketText.AppendLine(LeftAndRightText("Balance Due:", _
                Format(itemDetail.BalanceDue, "$#,##0.00"), ticketWidth))
            ticketText.AppendLine()
        Next counter
        If (included = 0) Then Return

        ' ----- If there are fines due, print them here.
        ticketText.AppendLine("Fines Due: " & Format( _
            CalculatePatronFines(patronID), "$#,##0.00"))
        ticketText.AppendLine()

        ' ----- Add the bottom display text.
        ticketText.Append(GetTicketFooter(ticketWidth))

        ' ----- Send the ticket to the printer.
        RawPrinterHelper.SendStringToPrinter(My.Settings.ReceiptPrinter, _
            ticketText.ToString(), "Balance Receipt")
        Return

ErrorHandler:
        GeneralError("TicketPrinting.PrintPaymentTicket", Err.GetException())
        Return
    End Sub

    Private Function GetTicketHeader(ByVal patronID As Integer, ByVal ticketWidth As Integer) As System.Text.StringBuilder
        ' ----- Print out a ticket of what the patron checked out. The supplied
        '       ListBox control contains objects of type CheckedOutItem.
        Dim ticketText As System.Text.StringBuilder = Nothing
        Dim holdText As String
        Dim counter As Integer
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Check for an active printer.
        If (My.Settings.UseReceipts = False) Then Return Nothing

        ' ----- Get the printer path.
        If (My.Settings.ReceiptPrinter Is Nothing) Then Return Nothing
        If (Trim(My.Settings.ReceiptPrinter) = "") Then Return Nothing

        ' ----- Create the storage area for the content.
        ticketText = New System.Text.StringBuilder

        ' ----- Start with the heading.
        holdText = Trim(GetSystemValue("TicketHeading"))
        If (holdText <> "") Then
            For counter = 1 To CountSubStr(holdText, "|") + 1
                ' ----- Center each line of the heading.
                ticketText.AppendLine(CenterText( _
                    GetSubStr(holdText, "|", counter), ticketWidth))
            Next counter
        End If

        ' ----- Add today's date.
        ticketText.AppendLine(CenterText(Format(Today, "MMMM d, yyyy"), ticketWidth))
        ticketText.AppendLine()

        ' ----- Add the patron name.
        sqlText = "SELECT FirstName + ' ' + LastName FROM Patron " & _
            "WHERE ID = " & patronID
        ticketText.AppendLine(CenterText("Patron: " & _
            CStr(ExecuteSQLReturn(sqlText)), ticketWidth))
        ticketText.AppendLine()

        ' ----- That's the full header.
        Return ticketText

ErrorHandler:
        GeneralError("TicketPrinting.GetTicketHeader", Err.GetException())
        Return ticketText
    End Function

    Private Function GetTicketFooter(ByVal ticketWidth As Integer) As String
        ' ----- Build the content for the end of the ticket.
        Dim ticketText As String = ""
        Dim holdText As String
        Dim counter As Integer

        ' ----- Add the bottom display text.
        holdText = Trim(GetSystemValue("TicketFooting"))
        If (holdText <> "") Then
            For counter = 1 To CountSubStr(holdText, "|") + 1
                ' ----- Center each line of the heading.
                ticketText &= CenterText(GetSubStr(holdText, "|", counter), _
                    ticketWidth) & vbCrLf
            Next counter
        End If

        ' ----- Include any final characters.
        ticketText &= GetPrinterPostlude()
        Return ticketText
    End Function

    Private Function GetPrinterPostlude() As String
        ' ----- Retrieve the postlude and process any escape sequences.
        Dim origText As String
        Dim oneChar As String
        Dim resultText As String
        Dim position As Integer
        Dim hexCode As Integer

        ' ----- Retrieve the raw postlude.
        origText = My.Settings.ReceiptPostlude
        If (origText Is Nothing) Then Return ""
        If (Trim(origText) = "") Then Return ""

        ' ----- Scan through the text and process any backslash characters.
        resultText = ""
        position = 1
        Do While (position <= Len(origText))
            ' ----- Peek at the next character.
            oneChar = Mid(origText, position, 1)
            If (oneChar = "\") Then
                ' ----- Handle the escape character.
                oneChar = Mid(origText, position + 1, 1)
                Select Case UCase(oneChar)
                    Case "E"
                        ' ----- Add the escape character.
                        resultText &= CStr(Chr(Keys.Escape))
                        position += 2
                    Case "N"
                        ' ----- Add the newline character.
                        resultText &= CStr(Chr(Keys.LineFeed))
                        position += 2
                    Case "R"
                        ' ----- Add the carriage return character.
                        resultText &= CStr(Chr(Keys.Return))
                        position += 2
                    Case "X"
                        ' ----- Add in an ASCII value identified by the next
                        '       two hex digits.
                        oneChar = Mid(origText, position + 2, 2)
                        Try
                            hexCode = CInt("&H" & oneChar)
                            resultText &= CStr(Chr(hexCode))
                        Catch
                            ' ----- Ignore this character.
                        Finally
                            position += 4
                        End Try
                    Case Else
                        ' ----- Simply add this character to the final output.
                        '       oneChar may be the "\" character.
                        resultText &= oneChar
                        position += 2
                End Select
            Else
                ' ----- Simply add this character to the final output.
                resultText &= oneChar
                position += 1
            End If
        Loop

        ' ----- Return the final text.
        Return resultText
    End Function
End Module
