' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class UserName
    Private ActiveID As Integer
    Private StartingOver As Boolean
    Private OrigLoginID As String

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageUsers)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageUsers)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageUsers)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the record.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Do not delete the current user.
        If (LoggedInUserID = recordID) Then
            MsgBox("You cannot delete the active user record.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the selected user record?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Delete the record.
        TransactionBegin()
        sqlText = "UPDATE PatronPayment SET UserID = NULL WHERE UserID = " & recordID
        ExecuteSQL(sqlText)
        sqlText = "DELETE FROM UserName WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        TransactionCommit()
        Return True

ErrorHandler:
        GeneralError("UserName.DeleteRecord", Err.GetException())
        TransactionRollback()
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim itemText As String

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, FullName, Active FROM UserName ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            itemText = CStr(dbInfo!FullName)
            If (CBool(dbInfo!Active) = False) Then itemText &= " (Inactive)"
            destList.Items.Add(New ListItemData(itemText, CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("UserName.FillListWithRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim itemText As String

        On Error GoTo ErrorHandler
        itemText = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName, Active FROM UserName WHERE ID = " & recordID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            itemText = CStr(dbInfo!FullName)
            If (CBool(dbInfo!Active) = False) Then itemText &= " (Inactive)"
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return itemText

ErrorHandler:
        GeneralError("UserName.FormatRecordName", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use these codes to identify the administrative uses who have " & _
            "access to the system."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Administrative Users"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Load in the list of security groups.
        If (RecordGroup.Items.Count = 0) Then
            ' ----- This is the first time. Load from the database.
            RecordGroup.Items.Clear()
            RecordGroup.Items.Add(New ListItemData("<Not Selected>", -1))
            sqlText = "SELECT * FROM GroupName ORDER BY FullName"
            dbInfo = CreateReader(sqlText)
            Do While (dbInfo.Read)
                RecordGroup.Items.Add _
                    (New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
            Loop
            dbInfo.Close()
            dbInfo = Nothing
        End If

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        RecordLoginID.Text = ""
        RecordPassword.Text = ""
        RecordConfirm.Text = ""
        RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(-1)
        RecordActive.Checked = True
        OrigLoginID = ""
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM UserName WHERE ID = " & ActiveID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            ' ----- We never reload the password.
            RecordFullName.Text = CStr(dbInfo!FullName)
            RecordLoginID.Text = CStr(dbInfo!LoginID)
            OrigLoginID = CStr(dbInfo!LoginID)
            RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(CInt(dbInfo!GroupID))
            RecordActive.Checked = CBool(dbInfo!Active)
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("UserName.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- The user wants to save changes. Return True on success.
        Dim sqlText As String
        Dim passwordResult As String
        Dim savePassword As Boolean
        Dim newID As Integer = -1

        On Error GoTo ErrorHandler

        ' ----- Prepare to save the data.
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Encrypt the password text.
            passwordResult = EncryptPassword(Trim(RecordLoginID.Text), _
                Trim(RecordPassword.Text))

            ' ----- Create a new entry.
            sqlText = "INSERT INTO UserName(FullName, LoginID, [Password], Active, " & _
                "GroupID) OUTPUT INSERTED.ID VALUES (" & _
                DBText(Trim(RecordFullName.Text)) & ", " & _
                DBText(Trim(RecordLoginID.Text)) & ", " & _
                DBText(passwordResult) & ", " & _
                CStr(CInt(RecordActive.Checked)) & ", " & _
                DBCombo(RecordGroup) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE UserName SET FullName = " & _
                DBText(Trim(RecordFullName.Text)) & _
                ", LoginID = " & DBText(Trim(RecordLoginID.Text)) & _
                ", Active = " & CStr(CInt(RecordActive.Checked)) & _
                ", GroupID = " & DBCombo(RecordGroup)

            ' ----- See if the password should be saved.
            savePassword = False
            If (OrigLoginID = "") Then savePassword = True
            If (UCase(Trim(OrigLoginID)) <> UCase(Trim(RecordLoginID.Text))) Then _
                savePassword = True
            If (Trim(RecordPassword.Text) <> "") Then savePassword = True
            If (savePassword = True) Then
                If (UCase(Trim(RecordPassword.Text)) = "NP") Then
                    ' ----- NP removes a password.
                    passwordResult = EncryptPassword(Trim(RecordLoginID.Text), "")
                Else
                    passwordResult = EncryptPassword(Trim(RecordLoginID.Text), _
                        Trim(RecordPassword.Text))
                End If
                sqlText &= ", [Password] = " & DBText(passwordResult)
            End If
            sqlText &= " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- This change may affect this user.
        If (LoggedInGroupID = ActiveID) Then ReprocessSecuritySet()

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("UserName.SaveFormData", Err.GetException())
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Name is a required field.
        If (Trim(RecordFullName.Text) = "") Then
            MsgBox("Please supply the User Name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Login ID is a required field.
        If (Trim(RecordLoginID.Text) = "") Then
            MsgBox("Please supply the Login ID.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordLoginID.Focus()
            Return False
        End If

        ' ----- Password and confirm must match.
        If (RecordPassword.Text <> RecordConfirm.Text) Then
            MsgBox("The Password and Confirm fields must match.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordPassword.Focus()
            Return False
        End If

        ' ----- If the login ID changed and there is no password, warn the user that
        '       the new login ID will have no password.
        If (UCase(Trim(OrigLoginID)) <> UCase(Trim(RecordLoginID.Text))) And _
                (OrigLoginID <> "") And (Trim(RecordPassword.Text) = "") Then
            If (MsgBox("When changing the Login ID, the password must also be " & _
                "supplied. If you leave the Password field blank, no password " & _
                "will be assigned to this user.  Do you wish to continue with a " & _
                "blank password?", MsgBoxStyle.YesNo Or MsgBoxStyle.Question, _
                ProgramTitle) <> MsgBoxResult.Yes) Then Return False
        End If

        ' ----- Make sure the login ID is not already in use.
        sqlText = "SELECT COUNT(*) FROM UserName WHERE UPPER(LoginID) = " & _
            DBText(UCase(Trim(RecordLoginID.Text)))
        If (ActiveID <> -1) Then sqlText &= " AND ID <> " & ActiveID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("A record already exists with the Login ID '" & _
                Trim(RecordLoginID.Text) & "'.", MsgBoxStyle.OkOnly Or _
                MsgBoxStyle.Exclamation, ProgramTitle)
            RecordLoginID.Focus()
            Return False
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("UserName.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub ActOK_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- Save changes and return to the summary form.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub RecordConfirm_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordConfirm.Enter
        ' ----- Highlight the entire text.
        RecordConfirm.SelectAll()
    End Sub

    Private Sub RecordFullName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordFullName.Enter
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub RecordLoginID_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordLoginID.Enter
        ' ----- Highlight the entire text.
        RecordLoginID.SelectAll()
    End Sub

    Private Sub RecordPassword_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordPassword.Enter
        ' ----- Highlight the entire text.
        RecordPassword.SelectAll()
    End Sub

    Private Sub UserName_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub

    Private Sub UserName_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "UserName.htm")
    End Sub
End Class
