' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports MVB = Microsoft.VisualBasic

Public Class Author
    Private ActiveID As Integer
    Private StartingOver As Boolean

    Public Overloads Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overloads Function AddRecord(ByVal lastText As String, _
            ByVal firstText As String) As Integer
        ' ----- Add a new record, but use a starting value previously entered
        '       by the user.
        ActiveID = -1
        PrepareFormFields()
        RecordLastName.Text = lastText
        RecordFirstName.Text = firstText
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageAuthors)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageAuthors)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageAuthors)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the author?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM ItemAuthor WHERE AuthorID = " & recordID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("You cannot delete this record because it is in use.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Delete the record.
        sqlText = "DELETE FROM Author WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        Return True

ErrorHandler:
        GeneralError("Author.DeleteRecord", Err.GetException())
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items. Prompt the
        '       user first.
        destList.Items.Clear()
        SearchForRecord(destList, exceededMatches)
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim result As String

        On Error GoTo ErrorHandler
        result = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT * FROM Author WHERE ID = " & recordID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            ' ----- Format the display name.
            result = FormatAuthorName(dbInfo)
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return result

ErrorHandler:
        GeneralError("Author.FormatRecordName", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use this form to enter authors, editors, performers, and " & _
            "others that are primariliy responsible for the content of an item " & _
            "in the library."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Authors and Names"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear the existing data.
        StartingOver = True
        RecordLastName.Text = ""
        RecordFirstName.Text = ""
        RecordMiddleName.Text = ""
        RecordSuffix.Text = ""
        RecordBirthYear.Text = ""
        RecordDeathYear.Text = ""
        RecordComments.Text = ""
        ShowMatches.Text = "      None"
        ShowMatches.ImageIndex = MatchNone
        LabelMatches.Visible = True
        ShowMatches.Visible = True
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM Author WHERE ID = " & ActiveID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            RecordLastName.Text = CStr(dbInfo!LastName)
            RecordFirstName.Text = DBGetText(dbInfo!FirstName)
            RecordMiddleName.Text = DBGetText(dbInfo!MiddleName)
            RecordSuffix.Text = DBGetText(dbInfo!Suffix)
            RecordBirthYear.Text = DBGetText(dbInfo!BirthYear)
            RecordDeathYear.Text = DBGetText(dbInfo!DeathYear)
            RecordComments.Text = DBGetText(dbInfo!Comments)
        End If
        dbInfo.Close()
        dbInfo = Nothing
        LabelMatches.Visible = False
        ShowMatches.Visible = False
        Return

ErrorHandler:
        GeneralError("Author.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub RefreshMatchingAuthors()
        ' ----- See if there are name matches for this author.
        Dim sqlText As String
        Dim matchCount As Integer
        Dim messageText As String

        On Error GoTo ErrorHandler

        ' ----- Only performed for new records.
        If (ActiveID <> -1) Then Return

        ' ----- Reset the display.
        ShowMatches.Text = "      None"
        ShowMatches.ImageIndex = MatchNone

        ' ----- Ignore if we have insufficient data.
        If (Trim(RecordLastName.Text) = "") Then Return

        ' ----- Retrieve a count of matching records.
        sqlText = "SELECT COUNT(*) AS TheCount FROM Author WHERE LastName LIKE " & _
            DBText(Trim(RecordLastName.Text))
        If (Trim(RecordFirstName.Text) <> "") Then
            sqlText &= " AND FirstName LIKE " & _
                DBText(MVB.Left(Trim(RecordFirstName.Text), 1) & "%")
        End If
        matchCount = CInt(ExecuteSQLReturn(sqlText))
        If (matchCount > 0) Then
            ' ----- There are matches.
            If (matchCount = 1) Then
                messageText = "1 match"
            Else
                messageText = matchCount & " matches"
            End If
            messageText &= ". Click for details."
            ShowMatches.Text = "      " & messageText
            ShowMatches.ImageIndex = MatchPresent
        End If
        Return

ErrorHandler:
        GeneralError("Author.RefreshMatchingAuthors", Err.GetException())
        Return
    End Sub

    Public Overrides Sub SearchForRecord(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Search for authors using user-specified criteria.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim userLimit As String
        Dim limitFirst As String
        Dim limitLast As String
        Dim hasWhere As Boolean
        Dim resultCount As Integer

        On Error GoTo ErrorHandler

        ' ----- Prompt the user for the limited author name.
        exceededMatches = False
        userLimit = (New AuthorLimit).PromptUser()
        If (userLimit = "") Then Return

        ' ----- Use the limits to help prepare the search text.
        limitLast = Trim(GetSubStr(userLimit, ",", 1))
        limitFirst = Trim(GetSubStr(userLimit, ",", 2))
        If ((limitLast & limitFirst) = "") Then Return
        If (InStr(limitLast, "*") = 0) Then limitLast &= "*"
        If (InStr(limitFirst, "*") = 0) Then limitFirst &= "*"
        limitLast = Replace(limitLast, "*", "%")
        limitFirst = Replace(limitFirst, "*", "%")

        ' ----- Clear any existing values.
        destList.Items.Clear()

        ' ----- Prepare the full SQL statement based on the user's limits.
        hasWhere = False
        sqlText = "SELECT * FROM Author"
        If (limitLast <> "") Then
            sqlText &= " WHERE LastName LIKE " & DBText(limitLast)
            hasWhere = True
        End If
        If (limitFirst <> "") Then
            If (hasWhere) Then sqlText &= " AND" Else sqlText &= " WHERE"
            sqlText &= " FirstName LIKE " & DBText(limitFirst)
        End If
        sqlText &= " ORDER BY LastName, FirstName"

        ' ----- Retrieve and process the values from the database.
        dbInfo = CreateReader(sqlText)
        resultCount = 0
        Do While dbInfo.Read
            ' ----- Add this item.
            resultCount += 1
            If (resultCount > SearchMatchLimit) Then
                ' ----- Too many matches found.
                exceededMatches = True
                Exit Do
            End If
            destList.Items.Add(New ListItemData(FormatAuthorName(dbInfo), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("Author.SearchForRecord", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function UsesSearch() As Boolean
        ' ----- Author records are searchable.
        Return True
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save the record to the database.
        Dim sqlText As String
        Dim newID As Integer

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO Author (LastName, FirstName, MiddleName, " & _
                "Suffix, BirthYear, DeathYear, Comments) OUTPUT INSERTED.ID " & _
                "VALUES (" & DBText(Trim(RecordLastName.Text)) & _
                ", " & DBText(Trim(RecordFirstName.Text)) & _
                ", " & DBText(Trim(RecordMiddleName.Text)) & _
                ", " & DBText(Trim(RecordSuffix.Text)) & _
                ", " & DBNum(RecordBirthYear.Text) & _
                ", " & DBNum(RecordDeathYear.Text) & _
                ", " & DBText(Trim(RecordComments.Text)) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE Author SET" & _
                "  LastName = " & DBText(Trim(RecordLastName.Text)) & _
                ", FirstName = " & DBText(Trim(RecordFirstName.Text)) & _
                ", MiddleName = " & DBText(Trim(RecordMiddleName.Text)) & _
                ", Suffix = " & DBText(Trim(RecordSuffix.Text)) & _
                ", BirthYear = " & DBNum(RecordBirthYear.Text) & _
                ", DeathYear = " & DBNum(RecordDeathYear.Text) & _
                ", Comments = " & DBText(Trim(RecordComments.Text)) & _
                " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("Author.SaveFormData", Err.GetException())
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Last name is a required field.
        If (Trim(RecordLastName.Text) = "") Then
            MsgBox("Please supply the last name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordLastName.Focus()
            Return False
        End If

        ' ----- Birth year, if supplied, must be valid.
        If (Trim(RecordBirthYear.Text) <> "") Then
            If (IsNumeric(RecordBirthYear.Text) = False) Or _
                    (InStr(RecordBirthYear.Text, ".") > 0) Then
                MsgBox("Year of Birth, if supplied, must be a valid integer.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                RecordBirthYear.Focus()
                Return False
            End If
        End If

        ' ----- Death year, if supplied, must be valid.
        If (Trim(RecordDeathYear.Text) <> "") Then
            If (IsNumeric(RecordDeathYear.Text) = False) Or _
                    (InStr(RecordDeathYear.Text, ".") > 0) Then
                MsgBox("Year of Death, if supplied, must be a valid integer.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                RecordDeathYear.Focus()
                Return False
            End If
        End If

        ' ----- Check the matching authors, just in case.
        If (ActiveID = -1) Then
            RefreshMatchingAuthors()
            If (ShowMatches.ImageIndex = MatchPresent) Then
                If (MsgBox("The name you supplied is similar to one or more names " & _
                    "already in the database. Do you still want to save this record?", _
                    MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
                    <> MsgBoxResult.Yes) Then Return False
            End If
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("Author.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub Author_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the first field.
        If (StartingOver) Then RecordLastName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordLastName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordLastName.Enter
        ' ----- Highlight the entire text.
        RecordLastName.SelectAll()
    End Sub

    Private Sub RecordFirstName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordFirstName.Enter
        ' ----- Highlight the entire text.
        RecordFirstName.SelectAll()
    End Sub

    Private Sub RecordMiddleName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordMiddleName.Enter
        ' ----- Highlight the entire text.
        RecordMiddleName.SelectAll()
    End Sub

    Private Sub RecordSuffix_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordSuffix.Enter
        ' ----- Highlight the entire text.
        RecordSuffix.SelectAll()
    End Sub

    Private Sub RecordBirthYear_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordBirthYear.Enter
        ' ----- Highlight the entire text.
        RecordBirthYear.SelectAll()
    End Sub

    Private Sub RecordDeathYear_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordDeathYear.Enter
        ' ----- Highlight the entire text.
        RecordDeathYear.SelectAll()
    End Sub

    Private Sub RecordBirthYear_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles RecordBirthYear.KeyPress
        ' ----- Only allow digits, hyphens, and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Or (e.KeyChar = "-") Then Return
        e.Handled = True
    End Sub

    Private Sub RecordDeathYear_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles RecordDeathYear.KeyPress
        ' ----- Only allow digits, hyphens, and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Or (e.KeyChar = "-") Then Return
        e.Handled = True
    End Sub

    Private Sub RecordLastName_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordLastName.Leave
        ' ----- Update the matching display.
        RefreshMatchingAuthors()
    End Sub

    Private Sub RecordFirstName_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordFirstName.Leave
        ' ----- Update the matching display.
        RefreshMatchingAuthors()
    End Sub

    Private Sub ShowMatches_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ShowMatches.Click
        ' ----- The user wants more information on the matches.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim nameList As String
        Dim message As String
        Dim matches As Integer
        Dim counter As Integer

        On Error GoTo ErrorHandler

        ' ----- Only performed for new records.
        If (ActiveID <> -1) Then Return

        ' ----- Ignore if there are no matches.
        If (ShowMatches.ImageIndex = MatchNone) Then
            MsgBox("No matches.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
            Return
        End If

        ' ----- Retrieve a count of matching records.
        matches = CInt(Val(ShowMatches.Text))

        ' ----- Retrieve the matching records.
        sqlText = "SELECT * FROM Author WHERE LastName LIKE " & _
            DBText(Trim(RecordLastName.Text))
        If (Trim(RecordFirstName.Text) <> "") Then
            sqlText &= " AND FirstName LIKE " & _
                DBText(MVB.Left(Trim(RecordFirstName.Text), 1) & "%")
        End If
        sqlText &= " ORDER BY LastName, FirstName"
        dbInfo = CreateReader(sqlText)
        counter = 0
        nameList = ""
        Do While (dbInfo.Read)
            ' ----- Add this author.
            nameList &= vbCrLf & vbTab & FormatAuthorName(dbInfo)

            ' ----- Limit to ten matches.
            counter += 1
            If (counter >= 10) Then Exit Do
        Loop
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- Format and display the message.
        message = "The database contains " & CStr(matches) & " match"
        If (matches > 1) Then message &= "es"
        message &= " to the name:" & vbCrLf & vbCrLf & vbTab & _
            Trim(RecordLastName.Text)
        If (Trim(RecordFirstName.Text) <> "") Then message &= ", " & _
            Trim(RecordFirstName.Text)
        If (matches > 10) Then
            message &= vbCrLf & vbCrLf & "The first ten matches are:"
        Else
            message &= vbCrLf & vbCrLf & "The matches are:"
        End If
        message &= vbCrLf & nameList
        MsgBox(message, MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
        Return

ErrorHandler:
        GeneralError("Author.ShowMatches_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub LabelMatches_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles LabelMatches.Click
        ' ----- Same as a click on the related data label.
        ShowMatches_Click(sender, e)
    End Sub

    Private Sub Author_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "Author.htm")
    End Sub
End Class
