' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class BarcodePage
    Private Const MaxPageInInches As Single = 22.0F
    Private Const TwipsPerInch As Single = 1440.0F

    Private ActiveSheetID As Integer
    Private StartingOver As Boolean
    Private CurrentMeasure As Char

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveSheetID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveSheetID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the record.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete this barcode template?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question Or MsgBoxStyle.DefaultButton2, _
            ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM BarcodeLabel WHERE BarcodeSheet = " & recordID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            MsgBox("You cannot delete this template because it is in use by one or " & _
                "more labels templates.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, _
                ProgramTitle)
            Return False
        End If

        ' ----- Delete the record.
        sqlText = "DELETE FROM BarcodeSheet WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        Return True

ErrorHandler:
        GeneralError("BarcodePage.DeleteRecord", Err.GetException())
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveSheetID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveSheetID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, FullName FROM BarcodeSheet ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("BarcodePage.FillListWithRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close()
        dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
    
        On Error GoTo ErrorHandler

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM BarcodeSheet WHERE ID = " & recordID
        Return CStr(ExecuteSQLReturn(sqlText))

ErrorHandler:
        GeneralError("BarcodePage.FormatRecordName", Err.GetException())
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return a description of this editor.
        Return "Use these codes to identify the format of pages onto wihch " & _
            "barcodes are printed. You can define the layout of the page and the " & _
            "number and position of barcode labels."
    End Function
    Public Overrides Function GetEditTitle() As String
        ' ----- Return the title-bar text for this editor.
        Return "Barcode Sheet Templates"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Fill in the list of units.
        If (RecordUnits.Items.Count = 0) Then
            RecordUnits.Items.Add(New ListItemData("Inches", Asc("I")))
            RecordUnits.Items.Add(New ListItemData("Centimeters", Asc("C")))
            RecordUnits.Items.Add(New ListItemData("Points", Asc("P")))
            RecordUnits.Items.Add(New ListItemData("Twips", Asc("T")))
            RecordUnits.SelectedIndex = 0
        End If

        ' ----- Default to inches.
        CurrentMeasure = "I"c
        RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(Asc(CurrentMeasure))

        ' ----- Clear the existing data.
        StartingOver = True
        RecordName.Text = ""
        BCPageWidth.Text = ""
        BCPageHeight.Text = ""
        BCLeftMargin.Text = ""
        BCRightMargin.Text = ""
        BCTopMargin.Text = ""
        BCBottomMargin.Text = ""
        BCInterColumn.Text = ""
        BCInterRow.Text = ""
        BCColumns.Text = ""
        BCRows.Text = ""
        PreviewArea.Invalidate()
        If (ActiveSheetID = -1) Then Return

        ' ----- Load in the record.
        sqlText = "SELECT * FROM BarcodeSheet WHERE ID = " & ActiveSheetID
        dbInfo = CreateReader(sqlText)
        dbInfo.Read()
        RecordName.Text = CStr(dbInfo!FullName)
        RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(Asc(CChar(dbInfo!UnitType)))
        If (RecordUnits.SelectedIndex = -1) Then RecordUnits.SelectedIndex = 0
        CurrentMeasure = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        BCPageWidth.Text = CStr(CSng(dbInfo!PageWidth))
        BCPageHeight.Text = CStr(CSng(dbInfo!PageHeight))
        BCLeftMargin.Text = CStr(CSng(dbInfo!MarginLeft))
        BCRightMargin.Text = CStr(CSng(dbInfo!MarginRight))
        BCTopMargin.Text = CStr(CSng(dbInfo!MarginTop))
        BCBottomMargin.Text = CStr(CSng(dbInfo!MarginBottom))
        BCInterColumn.Text = CStr(CSng(dbInfo!IntraColumn))
        BCInterRow.Text = CStr(CSng(dbInfo!IntraRow))
        BCColumns.Text = CStr(dbInfo!ColumnsCount)
        BCRows.Text = CStr(dbInfo!RowsCount)
        dbInfo.Close()
        PreviewArea.Invalidate()
        Return

ErrorHandler:
        GeneralError("BarcodePage.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Sub BarcodePage_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordName.Enter
        ' ----- Highlight the entire text.
        RecordName.SelectAll()
    End Sub

    Private Sub BCPageWidth_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCPageWidth.Enter
        ' ----- Highlight the entire text.
        BCPageWidth.SelectAll()
    End Sub

    Private Sub BCPageHeight_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCPageHeight.Enter
        ' ----- Highlight the entire text.
        BCPageHeight.SelectAll()
    End Sub

    Private Sub BCLeftMargin_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCLeftMargin.Enter
        ' ----- Highlight the entire text.
        BCLeftMargin.SelectAll()
    End Sub

    Private Sub BCRightMargin_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCRightMargin.Enter
        ' ----- Highlight the entire text.
        BCRightMargin.SelectAll()
    End Sub

    Private Sub BCTopMargin_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCTopMargin.Enter
        ' ----- Highlight the entire text.
        BCTopMargin.SelectAll()
    End Sub

    Private Sub BCBottomMargin_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCBottomMargin.Enter
        ' ----- Highlight the entire text.
        BCBottomMargin.SelectAll()
    End Sub

    Private Sub BCInterColumn_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCInterColumn.Enter
        ' ----- Highlight the entire text.
        BCInterColumn.SelectAll()
    End Sub

    Private Sub BCInterRow_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCInterRow.Enter
        ' ----- Highlight the entire text.
        BCInterRow.SelectAll()
    End Sub

    Private Sub BCColumns_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCColumns.Enter
        ' ----- Highlight the entire text.
        BCColumns.SelectAll()
    End Sub

    Private Sub BCRows_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BCRows.Enter
        ' ----- Highlight the entire text.
        BCRows.SelectAll()
    End Sub

    Private Function ValidateForm() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Name is a required field.
        If (Trim(RecordName.Text) = "") Then
            MsgBox("Please supply a template name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordName.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM BarcodeSheet WHERE UPPER(FullName) = " & _
            DBText(UCase(Trim(RecordName.Text)))
        If (ActiveSheetID <> -1) Then sqlText &= " AND ID <> " & ActiveSheetID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            ' ----- Duplicate entry.
            MsgBox("A record already exists with the name '" & Trim(RecordName.Text) & _
                "'.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordName.Focus()
            Return False
        End If

        ' ----- Check the other fields.
        If (VerifyPageSize(False) = False) Then Return False
        If (VerifyLayout(False) = False) Then Return False

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("BarcodePage.ValidateForm", Err.GetException())
        Return False
    End Function

    Private Function VerifyLayout(ByVal silent As Boolean) As Boolean
        ' ----- Check the values supplied by the user to see if they are valid.
        '       This routine assumes that VerifyPageSize() has been called.
        If (IsNumeric(BCLeftMargin.Text) = False) Or (Val(BCLeftMargin.Text) < 0) Then
            If (silent = False) Then MsgBox("Left Margin is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If (IsNumeric(BCRightMargin.Text) = False) Or (Val(BCRightMargin.Text) < 0) Then
            If (silent = False) Then MsgBox("Right Margin is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If (IsNumeric(BCTopMargin.Text) = False) Or (Val(BCTopMargin.Text) < 0) Then
            If (silent = False) Then MsgBox("Top Margin is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If (IsNumeric(BCBottomMargin.Text) = False) Or (Val(BCBottomMargin.Text) < 0) Then
            If (silent = False) Then MsgBox("Bottom Margin is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If (IsNumeric(BCInterColumn.Text) = False) Or (Val(BCInterColumn.Text) < 0) Then
            If (silent = False) Then MsgBox("Intra-Column Spacing is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If (IsNumeric(BCInterRow.Text) = False) Or (Val(BCInterRow.Text) < 0) Then
            If (silent = False) Then MsgBox("Intra-Row Spacing is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If (IsNumeric(BCColumns.Text) = False) Or (Val(BCColumns.Text) <= 0) Then
            If (silent = False) Then MsgBox("Column Count is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If (IsNumeric(BCRows.Text) = False) Or (Val(BCRows.Text) <= 0) Then
            If (silent = False) Then MsgBox("Row Count is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Make sure there is space for the labels.
        If ((Val(BCPageWidth.Text) - ((Val(BCColumns.Text) - 1) * _
                Val(BCInterColumn.Text))) <= 0) Then
            If (silent = False) Then MsgBox("Insufficient horizontal space provided.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If ((Val(BCPageHeight.Text) - ((Val(BCRows.Text) - 1) * _
                Val(BCInterRow.Text))) <= 0) Then
            If (silent = False) Then MsgBox("Insufficient vertical space provided.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function VerifyPageSize(ByVal silent As Boolean) As Boolean
        ' ----- Check the values supplied by the user to see if they are valid.
        Dim pageWidth As Single
        Dim pageHeight As Single
        Dim fromUnit As Char

        ' ----- Check for valid width and height.
        If (IsNumeric(BCPageWidth.Text) = False) Or (Val(BCPageWidth.Text) <= 0) Then
            If (silent = False) Then MsgBox("Page Width is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If (IsNumeric(BCPageHeight.Text) = False) Or (Val(BCPageHeight.Text) <= 0) Then
            If (silent = False) Then MsgBox("Page Height is required.", _
                MsgBoxStyle.OKOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- The width and height are limited to a maximum size.
        fromUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        pageWidth = ConvertPageUnits(fromUnit, "T"c, CSng(BCPageWidth.Text))
        pageHeight = ConvertPageUnits(fromUnit, "T"c, CSng(BCPageHeight.Text))
        If (pageWidth > (TwipsPerInch * MaxPageInInches)) Then
            If (silent = False) Then MsgBox("The width of the page is limited to " & _
                ConvertPageUnits("T"c, fromUnit, MaxPageInInches) & " " & _
                RecordUnits.Text & ".", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        If (pageHeight > (TwipsPerInch * MaxPageInInches)) Then
            If (silent = False) Then MsgBox("The height of the page is limited to " & _
                ConvertPageUnits("T"c, fromUnit, MaxPageInInches) & " " & _
                RecordUnits.Text & ".", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- Success.
        Return True
    End Function
    Private Sub PreviewArea_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles PreviewArea.Paint
        ' ----- Display a preview of the image.
        Dim pageWidth As Integer
        Dim pageHeight As Integer
        Dim trueWidth As Integer
        Dim trueHeight As Integer
        Dim pageLeft As Integer  ' Start of the "white" area.
        Dim pageTop As Integer   ' Start of the "white" area.
        Dim oneWidth As Single
        Dim oneHeight As Single
        Dim oneWidthTwips As Integer
        Dim oneHeightTwips As Integer
        Dim rowScan As Integer
        Dim colScan As Integer
        Dim leftOffset As Integer
        Dim topOffset As Integer
        Dim useRatio As Single
        Dim fromUnit As Char

        On Error Resume Next

        ' ----- Draw in the background color.
        e.Graphics.Clear(SystemColors.Control)

        ' ----- Make sure that a valid page size is supplied.
        If (VerifyPageSize(True) = False) Then Return

        ' ----- Determine the page area.
        fromUnit = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        trueWidth = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(BCPageWidth.Text)))
        trueHeight = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(BCPageHeight.Text)))
        If (trueWidth = 0) Or (trueHeight = 0) Then Return
        pageWidth = PreviewArea.Width - 4
        pageHeight = PreviewArea.Height - 4
        useRatio = CSng(pageWidth) / CSng(trueWidth)
        If (CInt(trueHeight * useRatio) > pageHeight) Then
            ' ----- Scale based on the height.
            useRatio = CSng(pageHeight) / CSng(trueHeight)
            pageWidth = CInt(trueWidth * useRatio)
        Else
            ' ----- Scale based on the width.
            pageHeight = CInt(trueHeight * useRatio)
        End If
        pageLeft = CInt((PreviewArea.Width - 4 - pageWidth) / 2) + 1
        pageTop = CInt((PreviewArea.Height - 4 - pageHeight) / 2) + 1

        ' ----- Draw the page area.
        e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 1, _
            pageTop + 1, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 2, _
            pageTop + 2, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(Brushes.Black, pageLeft - 1, pageTop - 1, _
            pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(Brushes.White, pageLeft, pageTop, pageWidth, pageHeight)

        ' ----- Check the rest of the fields.
        If (VerifyLayout(True) = False) Then Return

        ' ----- Transform a page of twips into a display page.
        e.Graphics.TranslateTransform(pageLeft, pageTop)
        e.Graphics.ScaleTransform(useRatio, useRatio)

        ' ----- Figure out the size of one label.
        oneWidth = (CSng(BCPageWidth.Text) - CSng(BCLeftMargin.Text) - _
            CSng(BCRightMargin.Text) - ((CSng(BCColumns.Text) - 1.0F) * _
            CSng(BCInterColumn.Text))) / CSng(BCColumns.Text)
        oneHeight = (CSng(BCPageHeight.Text) - CSng(BCTopMargin.Text) - _
            CSng(BCBottomMargin.Text) - ((CSng(BCRows.Text) - 1.0F) * _
            CSng(BCInterRow.Text))) / CSng(BCRows.Text)
        oneWidthTwips = CInt(ConvertPageUnits(fromUnit, "T"c, oneWidth))
        oneHeightTwips = CInt(ConvertPageUnits(fromUnit, "T"c, oneHeight))

        ' ----- Process each row and column.
        For rowScan = 1 To CInt(BCRows.Text)
            For colScan = 1 To CInt(BCColumns.Text)
                leftOffset = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(BCLeftMargin.Text) + _
                    ((colScan - 1) * (CSng(BCInterColumn.Text) + oneWidth))))
                topOffset = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(BCTopMargin.Text) + _
                    ((rowScan - 1) * (CSng(BCInterRow.Text) + oneHeight))))
                e.Graphics.DrawRectangle(Pens.Cyan, leftOffset, topOffset, _
                    oneWidthTwips, oneHeightTwips)
            Next colScan
        Next rowScan

        ' ----- Finished.
        e.Graphics.ResetTransform()
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Confirm the data.
        If (ValidateForm() = False) Then Return
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveSheetID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO BarcodeSheet (FullName, UnitType, PageWidth, " & _
                "PageHeight, MarginLeft, MarginRight, MarginTop, MarginBottom, " & _
                "IntraColumn, IntraRow, ColumnsCount, RowsCount) VALUES ("
            ' ----- FullName, UnitType, PageWidth, PageHeight
            sqlText &= DBText(Trim(RecordName.Text)) & ", '" & _
                Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData) & "', " & _
                BCPageWidth.Text & ", " & BCPageHeight.Text & ", "
            ' ----- MarginLeft, MarginRight, MarginTop, MarginBottom
            sqlText &= BCLeftMargin.Text & ", " & BCRightMargin.Text & ", " & _
                BCTopMargin.Text & ", " & BCBottomMargin.Text & ", "
            ' ----- IntraColumn, IntraRow, ColumnsCount, RowsCount
            sqlText &= BCInterColumn.Text & ", " & BCInterRow.Text & ", " & _
                BCColumns.Text & ", " & BCRows.Text & ")"
            ExecuteSQL(sqlText)

            ' ----- Retrieve the new ID.
            sqlText = "SELECT ID FROM BarcodeSheet WHERE FullName = " & _
                DBText(Trim(RecordName.Text)) & " ORDER BY ID DESC"
            dbInfo = CreateReader(sqlText)
            dbInfo.Read()
            ActiveSheetID = CInt(dbInfo!ID)
            dbInfo.Close()
            dbInfo = Nothing
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE BarcodeSheet SET" & _
                " FullName = " & DBText(Trim(RecordName.Text)) & _
                ", UnitType = '" & Chr(CType(RecordUnits.SelectedItem, _
                    ListItemData).ItemData) & _
                "', PageWidth = " & BCPageWidth.Text & _
                ", PageHeight = " & BCPageHeight.Text & _
                ", MarginLeft = " & BCLeftMargin.Text & _
                ", MarginRight = " & BCRightMargin.Text & _
                ", MarginTop = " & BCTopMargin.Text & _
                ", MarginBottom = " & BCBottomMargin.Text & _
                ", IntraColumn = " & BCInterColumn.Text & _
                ", IntraRow = " & BCInterRow.Text & _
                ", ColumnsCount = " & BCColumns.Text & _
                ", RowsCount = " & BCRows.Text & _
                " WHERE ID = " & ActiveSheetID
            ExecuteSQL(sqlText)
        End If

        ' ----- Continue with AddRecord() or EditRecord()
        Me.Cursor = Windows.Forms.Cursors.Default
        Me.DialogResult = Windows.Forms.DialogResult.OK
        Return

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("BarcodePage.ActOK_Click", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close()
        dbInfo = Nothing
        Return
    End Sub

    Private Sub BCPageWidth_TextChanged(ByVal sender As System.Object, _
            ByVal e As System.EventArgs) Handles BCPageWidth.TextChanged, _
            BCPageHeight.TextChanged, BCLeftMargin.TextChanged, _
            BCRightMargin.TextChanged, BCTopMargin.TextChanged, _
            BCBottomMargin.TextChanged, BCInterColumn.TextChanged, _
            BCInterRow.TextChanged, BCRows.TextChanged, BCColumns.TextChanged
        ' ----- Refresh the preview.
        PreviewArea.Invalidate()
    End Sub

    Private Sub RecordUnits_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordUnits.SelectedIndexChanged
        ' ----- Refresh the preview.
        Dim newMeasure As Char

        ' ----- See if the measure changed.
        newMeasure = Chr(CType(RecordUnits.SelectedItem, ListItemData).ItemData)
        If (CurrentMeasure <> newMeasure) Then
            ' ----- Convert the measure.
            If (IsNumeric(BCPageWidth.Text) = True) Then BCPageWidth.Text = _
                CStr(ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCPageWidth.Text)))
            If (IsNumeric(BCPageHeight.Text) = True) Then BCPageHeight.Text = _
                CStr(ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCPageHeight.Text)))
            If (IsNumeric(BCLeftMargin.Text) = True) Then BCLeftMargin.Text = _
                CStr(ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCLeftMargin.Text)))
            If (IsNumeric(BCRightMargin.Text) = True) Then BCRightMargin.Text = _
                CStr(ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCRightMargin.Text)))
            If (IsNumeric(BCTopMargin.Text) = True) Then BCTopMargin.Text = _
                CStr(ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCTopMargin.Text)))
            If (IsNumeric(BCBottomMargin.Text) = True) Then BCBottomMargin.Text = _
                CStr(ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCBottomMargin.Text)))
            If (IsNumeric(BCInterColumn.Text) = True) Then BCInterColumn.Text = _
                CStr(ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCInterColumn.Text)))
            If (IsNumeric(BCInterRow.Text) = True) Then BCInterRow.Text = _
                CStr(ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCInterRow.Text)))
            CurrentMeasure = newMeasure
        End If
        PreviewArea.Invalidate()
    End Sub

    Private Sub BarcodePage_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "BarcodePage.htm")
    End Sub
End Class
