' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports System.Drawing.Printing

Public Class Maintenance
    ' ----- Store the connection string off-screen.
    Private LibraryConnection As String

    Private Const ClassifyDewey As Integer = 1
    Private Const ClassifyLC As Integer = 2

    Private Const DefaultReceiptPrinterWidth As Integer = 40

    Private Sub Maintenance_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "Maintenance.htm")
    End Sub

    Private Sub Maintenance_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim allFonts As New System.Drawing.Text.InstalledFontCollection
        Dim counter As Integer
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Fill in the list of available system fonts.
        RecordFontName.Items.Add(New ListItemData("<Not Selected>", -1))
        For counter = 0 To allFonts.Families.Length - 1
            RecordFontName.Items.Add(New ListItemData(allFonts.Families(counter).Name, _
                counter))
        Next counter

        ' ----- Fill in the list of locations.
        RecordDefaultLocation.Items.Add(New ListItemData("<Not Selected>", -1))
        sqlText = "SELECT * FROM CodeLocation ORDER BY FullName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            RecordDefaultLocation.Items.Add(New ListItemData( _
                CStr(dbInfo!FullName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing

        ' ----- Fill in the list of classifications.
        RecordClassification.Items.Add(New ListItemData("Dewey Decimal", ClassifyDewey))
        RecordClassification.Items.Add(New ListItemData("Library of Congress", ClassifyLC))

        ' ----- Fill in the list of local printers.
        For Each installedPrinter As String In PrinterSettings.InstalledPrinters
            RecordPrinterLocation.Items.Add(installedPrinter)
        Next installedPrinter

        ' ----- Fill in the current values.
        PopulateCurrentValues()
        Return

ErrorHandler:
        GeneralError("Maintenance.Maintenance_Load", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function GetDBDisplayText(ByVal connectionString As String) As String
        ' ----- Given a connection string, return just an informational portion
        '       for display purposes.
        Dim displayText As String = ""
        Dim counter As Integer
        Dim oneKey As String
        Dim oneValue As String

        For counter = 1 To CountSubStr(connectionString, ";") + 1
            ' ----- Each comma-delimited part has the format "key=value".
            oneKey = GetSubStr(connectionString, ";", counter)
            oneValue = Trim(GetSubStr(oneKey, "=", 2))
            oneKey = Replace(UCase(Trim(GetSubStr(oneKey, "=", 1))), " ", "")

            If (oneKey = "DATASOURCE") Then
                If (displayText <> "") Then displayText = "\" & displayText
                displayText = oneValue & displayText
            ElseIf (oneKey = "INITIALCATALOG") Then
                If (displayText <> "") Then displayText &= "\"
                displayText &= oneValue
            End If
        Next counter

        ' ----- Check for no configuration string.
        If (displayText = "") Or (displayText = "\") Then displayText = "Not Specified"
        Return displayText
    End Function

    Private Sub PopulateCurrentValues()
        ' ----- Prepare the form.
        Dim holdText As String

        On Error GoTo ErrorHandler

        ' ===== System-wide fields.

        ' ----- Fill in the current default location.
        holdText = GetSystemValue("DefaultLocation")
        RecordDefaultLocation.SelectedIndex = _
            RecordDefaultLocation.Items.IndexOf(CInt(Val(holdText)))
        If (RecordDefaultLocation.SelectedIndex = -1) Then _
            RecordDefaultLocation.SelectedIndex = RecordDefaultLocation.Items.IndexOf(-1)

        RecordGraceDays.Text = GetSystemValue("FineGrace")
        RecordSearchLimit.Text = GetSystemValue("SearchLimit")

        ' ----- Fill in the classification type.
        If (CBool(Val(GetSystemValue("UseLC"))) = True) Then
            RecordClassification.SelectedIndex = _
                RecordClassification.Items.IndexOf(ClassifyLC)
        Else
            RecordClassification.SelectedIndex = _
                RecordClassification.Items.IndexOf(ClassifyDewey)
        End If

        RecordSelfCheckout.Checked = CBool(Val(GetSystemValue("PatronCheckOut")))
        RecordBarcodeItem.Text = GetSystemValue("NextBarcodeItem")
        RecordBarcodePatron.Text = GetSystemValue("NextBarcodePatron")
        RecordBarcodeMisc.Text = GetSystemValue("NextBarcodeMisc")

        ' ----- Fill in the current font name.
        holdText = GetSystemValue("BarcodeFont")
        RecordFontName.SelectedIndex = RecordFontName.FindStringExact(holdText)
        If (RecordFontName.SelectedIndex = -1) Then _
            RecordFontName.SelectedIndex = RecordFontName.Items.IndexOf(-1)

        RecordBarcode39.Checked = CBool(Val(GetSystemValue("BarcodeCode39")))
        RecordTicketTop.Text = GetSystemValue("TicketHeading")
        RecordTicketBottom.Text = GetSystemValue("TicketFooting")

        ' ===== Workstation-specific fields.

        LibraryConnection = My.Settings.DBConnection & ""
        RecordDBLocation.Text = GetDBDisplayText(LibraryConnection)
        RecordConfigLocation.Text = My.Settings.ReportConfig & ""
        RecordBasicHelp.Text = My.Settings.HelpFile & ""
        RecordAdminHelp.Text = My.Settings.HelpFileAdmin & ""
        EnableReceipts.Checked = My.Settings.UseReceipts
        RecordPrinterLocation.Text = My.Settings.ReceiptPrinter & ""
        RecordPrinterWidth.Text = CStr(My.Settings.ReceiptWidth)
        RecordPostlude.Text = My.Settings.ReceiptPostlude & ""
        HideLogin.Checked = My.Settings.HideLogin
        Return

ErrorHandler:
        GeneralError("Maintenance.PopulateCurrentValues", Err.GetException())
        Resume Next
    End Sub

    Private Sub ActAllValues_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAllValues.Click
        ' ----- Let the user edit the list of system values.
        Dim RecordsForm As Library.ListEditRecords

        ' ----- Edit the records.
        RecordsForm = New Library.ListEditRecords
        RecordsForm.ManageRecords(New Library.SystemValue)
        RecordsForm = Nothing

        ' ----- Refresh the display elements.
        PopulateCurrentValues()
    End Sub

    Private Sub RecordGraceDays_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordGraceDays.Enter
        ' ----- Highlight the entire text.
        RecordGraceDays.SelectAll()
    End Sub

    Private Sub RecordSearchLimit_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordSearchLimit.Enter
        ' ----- Highlight the entire text.
        RecordSearchLimit.SelectAll()
    End Sub

    Private Sub RecordBarcodeItem_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordBarcodeItem.Enter
        ' ----- Highlight the entire text.
        RecordBarcodeItem.SelectAll()
    End Sub

    Private Sub RecordBarcodePatron_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordBarcodePatron.Enter
        ' ----- Highlight the entire text.
        RecordBarcodePatron.SelectAll()
    End Sub

    Private Sub RecordBarcodeMisc_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordBarcodeMisc.Enter
        ' ----- Highlight the entire text.
        RecordBarcodeMisc.SelectAll()
    End Sub

    Private Sub RecordTicketTop_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordTicketTop.Enter
        ' ----- Highlight the entire text.
        RecordTicketTop.SelectAll()
    End Sub

    Private Sub RecordTicketBottom_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordTicketBottom.Enter
        ' ----- Highlight the entire text.
        RecordTicketBottom.SelectAll()
    End Sub

    Private Sub RecordConfigLocation_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordConfigLocation.Enter
        ' ----- Highlight the entire text.
        RecordConfigLocation.SelectAll()
    End Sub

    Private Sub RecordBasicHelp_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordBasicHelp.Enter
        ' ----- Highlight the entire text.
        RecordBasicHelp.SelectAll()
    End Sub

    Private Sub RecordAdminHelp_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordAdminHelp.Enter
        ' ----- Highlight the entire text.
        RecordAdminHelp.SelectAll()
    End Sub

    Private Sub RecordPrinterLocation_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordPrinterLocation.Enter
        ' ----- Highlight the entire text.
        RecordPrinterLocation.SelectAll()
    End Sub

    Private Sub RecordPrinterWidth_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordPrinterWidth.Enter
        ' ----- Highlight the entire text.
        RecordPrinterWidth.SelectAll()
    End Sub

    Private Sub RecordPostlude_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RecordPostlude.Enter
        ' ----- Highlight the entire text.
        RecordPostlude.SelectAll()
    End Sub

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- Check the user-supplied data.
        If (ValidateFormData() = False) Then Return

        ' ----- Save the updated values.
        If (SaveFormData() = False) Then Return

        ' ----- Update some internal values immediately.
        MainHelpFile = RecordBasicHelp.Text
        MainAdminHelpFile = RecordAdminHelp.Text
        DefaultItemLocation = CInt(CType(RecordDefaultLocation.SelectedItem, _
            ListItemData))
        FineGraceDays = CInt(Val(RecordGraceDays.Text))

        ' ----- Close the form.
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Function ValidateFormData() As Boolean
        ' ----- Check the selections. Although the program will not run very
        '       well if any of these values are missing, there are other
        '       places in the program that impose the limits.

        ' ----- If grace days is supplied, it must be numeric.
        If (Trim(RecordGraceDays.Text) <> "") Then
            If (IsNumeric(RecordGraceDays.Text) = False) Then
                MsgBox("Fine Grace Days, if supplied, must be numeric.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
                TabMaintenance.SelectedTab = TabSystem
                RecordGraceDays.Focus()
                Return False
            End If
        End If

        ' ----- If search limit is supplied, it must be numeric.
        If (Trim(RecordSearchLimit.Text) <> "") Then
            If (IsNumeric(RecordSearchLimit.Text) = False) Then
                MsgBox("Search Result Limit, if supplied, must be numeric.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
                TabMaintenance.SelectedTab = TabSystem
                RecordSearchLimit.Focus()
                Return False
            End If
        End If

        ' ----- If barcode item number is supplied, it must be numeric.
        If (Trim(RecordBarcodeItem.Text) <> "") Then
            If (IsNumeric(RecordBarcodeItem.Text) = False) Then
                MsgBox("Barcode Item Number, if supplied, must be numeric.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
                TabMaintenance.SelectedTab = TabSystem
                RecordBarcodeItem.Focus()
                Return False
            End If
        End If

        ' ----- If barcode patron number is supplied, it must be numeric.
        If (Trim(RecordBarcodePatron.Text) <> "") Then
            If (IsNumeric(RecordBarcodePatron.Text) = False) Then
                MsgBox("Barcode Patron Number, if supplied, must be numeric.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
                TabMaintenance.SelectedTab = TabSystem
                RecordBarcodePatron.Focus()
                Return False
            End If
        End If

        ' ----- If barcode misc number is supplied, it must be numeric.
        If (Trim(RecordBarcodeMisc.Text) <> "") Then
            If (IsNumeric(RecordBarcodeMisc.Text) = False) Then
                MsgBox("Barcode Miscellaneous Number, if supplied, must be numeric.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
                TabMaintenance.SelectedTab = TabSystem
                RecordBarcodeMisc.Focus()
                Return False
            End If
        End If

        ' ----- If printer width is supplied, it must be numeric.
        If (EnableReceipts.Checked = True) And (Trim(RecordPrinterWidth.Text) <> "") Then
            If (IsNumeric(RecordPrinterWidth.Text) = False) Then
                MsgBox("Receipt Width, if supplied, must be numeric.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
                TabMaintenance.SelectedTab = TabWorkstation
                RecordPrinterWidth.Focus()
                Return False
            End If
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save all of the field values.
        Dim holdText As String

        On Error Resume Next

        ' ===== System-wide fields.
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the default location.
        If (CInt(CType(RecordDefaultLocation.SelectedItem, ListItemData)) = -1) Then _
            holdText = "" Else holdText = CStr(CInt(CType( _
            RecordDefaultLocation.SelectedItem, ListItemData)))
        SetSystemValue("DefaultLocation", holdText)

        SetSystemValue("FineGrace", Trim(RecordGraceDays.Text))
        SetSystemValue("SearchLimit", Trim(RecordSearchLimit.Text))

        ' ----- Save the classification type.
        If (CInt(CType(RecordClassification.SelectedItem, ListItemData)) = _
            ClassifyLC) Then holdText = "-1" Else holdText = "0"
        SetSystemValue("UseLC", holdText)

        ' ----- Save the self-checkout flag.
        If (RecordSelfCheckout.Checked = True) Then holdText = "-1" Else holdText = "0"
        SetSystemValue("PatronCheckOut", holdText)

        SetSystemValue("NextBarcodeItem", Trim(RecordBarcodeItem.Text))
        SetSystemValue("NextBarcodePatron", Trim(RecordBarcodePatron.Text))
        SetSystemValue("NextBarcodeMisc", Trim(RecordBarcodeMisc.Text))

        ' ----- Save the barcode font.
        If (CInt(CType(RecordFontName.SelectedItem, ListItemData)) = -1) Then _
            holdText = "" Else holdText = RecordFontName.Text
        SetSystemValue("BarcodeFont", holdText)

        ' ----- Save the code-39 barcode flag.
        If (RecordBarcode39.Checked = True) Then holdText = "-1" Else holdText = "0"
        SetSystemValue("BarcodeCode39", holdText)

        SetSystemValue("TicketHeading", Trim(RecordTicketTop.Text))
        SetSystemValue("TicketFooting", Trim(RecordTicketBottom.Text))

        ' ===== Workstation-specific fields.

        My.Settings.DBConnection = LibraryConnection
        My.Settings.ReportConfig = Trim(RecordConfigLocation.Text)
        My.Settings.HelpFile = Trim(RecordBasicHelp.Text)
        My.Settings.HelpFileAdmin = Trim(RecordAdminHelp.Text)
        My.Settings.HideLogin = HideLogin.Checked
        My.Settings.UseReceipts = EnableReceipts.Checked
        My.Settings.ReceiptPrinter = Trim(RecordPrinterLocation.Text)
        My.Settings.ReceiptPostlude = RecordPostlude.Text

        ' ----- Save the receipt printer width.
        If (Trim(RecordPrinterWidth.Text) = "") Then
            My.Settings.ReceiptWidth = DefaultReceiptPrinterWidth
        Else
            My.Settings.ReceiptWidth = CInt(RecordPrinterWidth.Text)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True
    End Function

    Private Sub ActDBLocation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActDBLocation.Click
        ' ----- Prompt for the database connection details.
        Dim newConnection As String

        ' ----- Prompt the user for the new setting.
        newConnection = LocateDatabase.PromptUser()
        If (newConnection = "") Then Return

        ' ----- Store the new value.
        LibraryConnection = newConnection
        RecordDBLocation.Text = GetDBDisplayText(LibraryConnection)
    End Sub

    Private Sub ActConfigLocation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActConfigLocation.Click
        ' ----- Let the user browse for the report configuration file.
        On Error Resume Next

        ' ----- Set up the file structure.
        LocateFile.DefaultExt = "xml"
        LocateFile.FileName = RecordConfigLocation.Text
        LocateFile.Filter = "XML Files (*.xml)|*.xml|" & _
            "All Files (*.*)|*.*"
        LocateFile.FilterIndex = 1
        LocateFile.Title = "Locate Configuration File"

        ' ----- Prompt the user.
        If (LocateFile.ShowDialog() <> Windows.Forms.DialogResult.OK) Then Return

        ' ----- Save the file path.
        RecordConfigLocation.Text = LocateFile.FileName
    End Sub

    Private Sub ActBasicHelp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActBasicHelp.Click
        ' ----- Let the user browse for the basic help file.
        On Error Resume Next

        ' ----- Set up the file structure.
        LocateFile.DefaultExt = "chm"
        LocateFile.FileName = RecordBasicHelp.Text
        LocateFile.Filter = "Help Files (*.chm)|*.chm|" & _
            "All Files (*.*)|*.*"
        LocateFile.FilterIndex = 1
        LocateFile.Title = "Locate Help"

        ' ----- Prompt the user.
        If (LocateFile.ShowDialog() <> Windows.Forms.DialogResult.OK) Then Return

        ' ----- Save the file path.
        RecordBasicHelp.Text = LocateFile.FileName
    End Sub

    Private Sub ActAdminHelp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActAdminHelp.Click
        ' ----- Let the user browse for the administrative help file.
        On Error Resume Next

        ' ----- Set up the file structure.
        LocateFile.DefaultExt = "chm"
        LocateFile.FileName = RecordAdminHelp.Text
        LocateFile.Filter = "Help Files (*.chm)|*.chm|" & _
            "All Files (*.*)|*.*"
        LocateFile.FilterIndex = 1
        LocateFile.Title = "Locate Help"

        ' ----- Prompt the user.
        If (LocateFile.ShowDialog() <> Windows.Forms.DialogResult.OK) Then Return

        ' ----- Save the file path.
        RecordAdminHelp.Text = LocateFile.FileName
    End Sub

    Private Sub EnableReceipts_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EnableReceipts.CheckedChanged
        ' ----- Toggle the availability of receipt-specific fields.
        LabelPrinterLocation.Enabled = EnableReceipts.Checked
        RecordPrinterLocation.Enabled = EnableReceipts.Checked
        LabelPrinterWidth.Enabled = EnableReceipts.Checked
        RecordPrinterWidth.Enabled = EnableReceipts.Checked
        LabelPostlude.Enabled = EnableReceipts.Checked
        RecordPostlude.Enabled = EnableReceipts.Checked
        LabelPostludeInfo.Enabled = EnableReceipts.Checked
    End Sub
End Class
