' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Public Class SystemValue
    Private ActiveID As Integer
    Private StartingOver As Boolean

    Public Overrides Function AddRecord() As Integer
        ' ----- Add a new record.
        ActiveID = -1
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageValues)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageValues)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageValues)
    End Function

    Public Overrides Function DeleteRecord(ByVal recordID As Integer) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Confirm with the user.
        If (MsgBox("Do you really wish to delete the system value?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) <> MsgBoxResult.Yes) _
            Then Return False

        ' ----- Delete the record.
        sqlText = "DELETE FROM SystemValue WHERE ID = " & recordID
        ExecuteSQL(sqlText)
        Return True

ErrorHandler:
        GeneralError("SystemValue.DeleteRecord", Err.GetException())
        Return False
    End Function

    Public Overrides Function EditRecord(ByVal recordID As Integer) As Integer
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = Windows.Forms.DialogResult.OK) Then _
            Return ActiveID Else Return -1
    End Function

    Public Overrides Sub FillListWithRecords(ByRef destList As ListBox, _
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT * FROM SystemValue ORDER BY ValueName"
        dbInfo = CreateReader(sqlText)
        Do While dbInfo.Read
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(CStr(dbInfo!ValueName), CInt(dbInfo!ID)))
        Loop
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("SystemValue.FillListWithRecords", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Public Overrides Function FormatRecordName(ByVal recordID As Integer) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT ValueName FROM SystemValue WHERE ID = " & recordID
        Return CStr(ExecuteSQLReturn(sqlText))

ErrorHandler:
        GeneralError("SystemValue.FormatRecordName", Err.GetException())
        Return "Error"
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use these codes to configure extended system features. These values " & _
            "are defined and managed by the system administrator."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "System Values"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader

        On Error GoTo ErrorHandler

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        RecordData.Text = ""
        If (ActiveID = -1) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM SystemValue WHERE ID = " & ActiveID
        dbInfo = CreateReader(sqlText)
        If (dbInfo.Read) Then
            RecordFullName.Text = CStr(dbInfo!ValueName)
            RecordData.Text = DBGetText(dbInfo!ValueData)
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return

ErrorHandler:
        GeneralError("SystemValue.PrepareFormFields", Err.GetException())
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the database.
        Dim sqlText As String
        Dim newID As Integer

        On Error GoTo ErrorHandler
        Me.Cursor = Windows.Forms.Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO SystemValue (ValueName, ValueData) " & _
                "OUTPUT INSERTED.ID VALUES (" & _
                DBText(Trim(RecordFullName.Text)) & ", " & _
                DBText(Trim(RecordData.Text)) & ")"
            newID = CInt(ExecuteSQLReturn(sqlText))
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE SystemValue SET " & _
                "ValueName = " & DBText(Trim(RecordFullName.Text)) & _
                ", ValueData = " & DBText(Trim(RecordData.Text)) & _
                " WHERE ID = " & ActiveID
            ExecuteSQL(sqlText)
        End If

        ' ----- Success.
        Me.Cursor = Windows.Forms.Cursors.Default
        Return True

ErrorHandler:
        Me.Cursor = Windows.Forms.Cursors.Default
        GeneralError("SystemValue.SaveFormData", Err.GetException())
        Return False
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String

        On Error GoTo ErrorHandler

        ' ----- Name is a required field.
        If (Trim(RecordFullName.Text) = "") Then
            MsgBox("Please supply the system value name.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM SystemValue WHERE UPPER(ValueName) = " & _
            DBText(UCase(Trim(RecordFullName.Text)))
        If (ActiveID <> -1) Then sqlText &= " AND ID <> " & ActiveID
        If (CInt(ExecuteSQLReturn(sqlText)) > 0) Then
            ' ----- Duplicate entry.
            MsgBox("A record already exists with the name '" & Trim(RecordFullName.Text) & _
                "'.", MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Success.
        Return True

ErrorHandler:
        GeneralError("SystemValue.ValidateFormData", Err.GetException())
        Return False
    End Function

    Private Sub ActOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = Windows.Forms.DialogResult.OK
    End Sub

    Private Sub SystemValue_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordFullName_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordFullName.Enter
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub RecordData_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles RecordData.Enter
        ' ----- Highlight the entire text.
        RecordData.SelectAll()
    End Sub

    Private Sub SystemValue_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "SystemValue.htm")
    End Sub
End Class
