/* ----- ACME Library Project
 *       Database creation script.
 *       Start-to-Finish Visual Basic 2005
 *       Copyright (c) 2006 by Tim Patrick
 */

CREATE TABLE Activity
(
   ID         bigint        PRIMARY KEY,
   FullName   varchar(50)   NOT NULL
);
GO

INSERT INTO Activity (ID, FullName) VALUES (1, 'Manage authors and names');
INSERT INTO Activity (ID, FullName) VALUES (2, 'Manage author and name types');
INSERT INTO Activity (ID, FullName) VALUES (3, 'Manage copy status codes');
INSERT INTO Activity (ID, FullName) VALUES (4, 'Manage media types');
INSERT INTO Activity (ID, FullName) VALUES (5, 'Manage series');
INSERT INTO Activity (ID, FullName) VALUES (6, 'Manage security groups');
INSERT INTO Activity (ID, FullName) VALUES (7, 'Manage library materials');
INSERT INTO Activity (ID, FullName) VALUES (8, 'Manage patrons');
INSERT INTO Activity (ID, FullName) VALUES (9, 'Manage publishers');
INSERT INTO Activity (ID, FullName) VALUES (10, 'Manage system values');
INSERT INTO Activity (ID, FullName) VALUES (11, 'Manage administrative users');
INSERT INTO Activity (ID, FullName) VALUES (12, 'Process and accept fees');
INSERT INTO Activity (ID, FullName) VALUES (13, 'Manage locations');
INSERT INTO Activity (ID, FullName) VALUES (14, 'Check out library items');
INSERT INTO Activity (ID, FullName) VALUES (15, 'Check in library items');
INSERT INTO Activity (ID, FullName) VALUES (16, 'Access administrative features');
INSERT INTO Activity (ID, FullName) VALUES (17, 'Perform daily processing');
INSERT INTO Activity (ID, FullName) VALUES (18, 'Run system reports');
INSERT INTO Activity (ID, FullName) VALUES (19, 'Access patrons without patron password');
INSERT INTO Activity (ID, FullName) VALUES (20, 'Manage barcodes');
INSERT INTO Activity (ID, FullName) VALUES (21, 'Manage holidays');
INSERT INTO Activity (ID, FullName) VALUES (22, 'Manage patron groups');
INSERT INTO Activity (ID, FullName) VALUES (23, 'View administrative patron messages');
GO

CREATE TABLE Author
(
   ID           bigint         IDENTITY PRIMARY KEY,
   LastName     varchar(50)    NOT NULL,
   FirstName    varchar(30)    NULL,
   MiddleName   varchar(30)    NULL,
   Suffix       varchar(10)    NULL,
   BirthYear    smallint       NULL,
   DeathYear    smallint       NULL,
   Comments     varchar(250)   NULL
);
GO

CREATE TABLE BarcodeLabel
(
   ID             bigint        IDENTITY PRIMARY KEY,
   FullName       varchar(50)   NOT NULL,
   BarcodeSheet   bigint        NOT NULL,
   UnitType       varchar(1)    NOT NULL
);
GO

CREATE TABLE BarcodeLabelItem
(
   ID             bigint         IDENTITY PRIMARY KEY,
   Priority       smallint       NOT NULL,
   BarcodeLabel   bigint         NOT NULL,
   ItemType       varchar(1)     NOT NULL,
   PosLeft        decimal(10,4)  NOT NULL,
   PosTop         decimal(10,4)  NOT NULL,
   PosWidth       decimal(10,4)  NOT NULL,
   PosHeight      decimal(10,4)  NOT NULL,
   Rotation       smallint       NULL,
   FontName       varchar(50)    NULL,
   FontSize       decimal(10,4)  NULL,
   StaticText     varchar(100)   NULL,
   FontStyle      varchar(4)     NULL,
   Color1         bigint         NULL,
   Color2         bigint         NULL,
   Alignment      smallint       NULL,
   PadDigits      smallint       NULL
);
GO

CREATE TABLE BarcodeSheet
(
   ID             bigint         IDENTITY PRIMARY KEY,
   FullName       varchar(50)    NOT NULL,
   UnitType       varchar(1)     NOT NULL,
   PageWidth      decimal(10,4)  NOT NULL,
   PageHeight     decimal(10,4)  NOT NULL,
   MarginLeft     decimal(10,4)  NOT NULL,
   MarginRight    decimal(10,4)  NOT NULL,
   MarginTop      decimal(10,4)  NOT NULL,
   MarginBottom   decimal(10,4)  NOT NULL,
   IntraColumn    decimal(10,4)  NOT NULL,
   IntraRow       decimal(10,4)  NOT NULL,
   ColumnsCount   smallint       NOT NULL,
   RowsCount      smallint       NOT NULL
);
GO

CREATE TABLE CodeAuthorType
(
   ID         bigint        IDENTITY PRIMARY KEY,
   FullName   varchar(50)   NOT NULL
);
GO

CREATE TABLE CodeCopyStatus
(
   ID         bigint        IDENTITY PRIMARY KEY,
   FullName   varchar(50)   NOT NULL
);
GO

CREATE TABLE CodeLocation
(
   ID               bigint         IDENTITY PRIMARY KEY,
   FullName         varchar(50)    NOT NULL,
   LastProcessing   datetime       NULL
);
GO

CREATE TABLE CodeMediaType
(
   ID             bigint        IDENTITY PRIMARY KEY,
   FullName       varchar(50)   NOT NULL,
   CheckoutDays   smallint      NOT NULL,
   RenewDays      smallint      NOT NULL,
   RenewTimes     smallint      NOT NULL,
   DailyFine      money         NOT NULL
);
GO

CREATE TABLE CodePatronGroup
(
   ID         bigint        IDENTITY PRIMARY KEY,
   FullName   varchar(50)   NOT NULL
);
GO

CREATE TABLE CodeSeries
(
   ID         bigint        IDENTITY PRIMARY KEY,
   FullName   varchar(50)   NOT NULL
);
GO

CREATE TABLE GroupActivity
(
   GroupID      bigint   NOT NULL,
   ActivityID   bigint   NOT NULL,
   PRIMARY KEY (GroupID, ActivityID)
);
GO

CREATE TABLE GroupName
(
   ID         bigint        IDENTITY PRIMARY KEY,
   FullName   varchar(50)   NOT NULL
);
GO

CREATE TABLE Holiday
(
   ID            bigint        IDENTITY PRIMARY KEY,
   FullName      varchar(50)   NOT NULL,
   EntryType     varchar(1)    NOT NULL,
   EntryDetail   varchar(10)   NOT NULL
);
GO

CREATE TABLE ItemAuthor
(
   ItemID       bigint     NOT NULL,
   AuthorID     bigint     NOT NULL,
   Sequence     smallint   NOT NULL,
   AuthorType   bigint     NOT NULL,
   PRIMARY KEY (ItemID, AuthorID)
);
GO

CREATE TABLE ItemCopy
(
   ID           bigint        IDENTITY PRIMARY KEY,
   ItemID       bigint        NOT NULL,
   CopyNumber   smallint      NOT NULL,
   Description  varchar(max)  NULL,
   Available    bit           NOT NULL,
   Missing      bit           NOT NULL,
   Reference    bit           NOT NULL,
   Condition    varchar(30)   NULL,
   Acquired     datetime      NULL,
   Cost         money         NULL,
   Status       bigint        NOT NULL,
   Barcode      varchar(20)   NULL,
   Location     bigint        NULL
);
GO

CREATE TABLE ItemKeyword
(
   ItemID      bigint   NOT NULL,
   KeywordID   bigint   NOT NULL,
   PRIMARY KEY (ItemID, KeywordID)
);
GO

CREATE TABLE ItemSubject
(
   ItemID      bigint   NOT NULL,
   SubjectID   bigint   NOT NULL,
   PRIMARY KEY (ItemID, SubjectID)
);
GO

CREATE TABLE Keyword
(
   ID         bigint        IDENTITY PRIMARY KEY,
   FullName   varchar(50)   NOT NULL
);
GO

CREATE TABLE NamedItem
(
   ID            bigint         IDENTITY PRIMARY KEY,
   Title         varchar(150)   NOT NULL,
   Subtitle      varchar(150)   NULL,
   Description   varchar(max)   NULL,
   Edition       varchar(10)    NULL,
   Publisher     bigint         NULL,
   Dewey         varchar(20)    NULL,
   LC            varchar(25)    NULL,
   ISxN          varchar(20)    NULL,
   LCCN          varchar(12)    NULL,
   Copyright     smallint       NULL,
   Series        bigint         NULL,
   MediaType     bigint         NOT NULL,
   OutOfPrint    bit            NOT NULL
);
GO

CREATE TABLE Patron
(
   ID             bigint         IDENTITY PRIMARY KEY,
   LastName       varchar(30)    NOT NULL,
   FirstName      varchar(30)    NOT NULL,
   LastActivity   datetime       NULL,
   Active         bit            NOT NULL,
   Comments       varchar(max)   NULL,
   AdminMessage   varchar(max)   NULL,
   Barcode        varchar(20)    NULL,
   Password       varchar(20)    NOT NULL,
   Email          varchar(100)   NULL,
   Phone          varchar(20)    NULL,
   Address        varchar(50)    NULL,
   City           varchar(20)    NULL,
   State          varchar(2)     NULL,
   Postal         varchar(10)    NULL,
   PatronGroup    bigint   NULL
);
GO

CREATE TABLE PatronCopy
(
   ID            bigint     IDENTITY PRIMARY KEY,
   Patron        bigint     NOT NULL,
   ItemCopy      bigint     NOT NULL,
   CheckOut      datetime   NOT NULL,
   Renewal       smallint   NOT NULL,
   DueDate       datetime   NOT NULL,
   CheckIn       datetime   NULL,
   Returned      bit        NOT NULL,
   Missing       bit        NOT NULL,
   Fine          money      NOT NULL,
   Paid          money      NOT NULL,
   ProcessDate   datetime   NULL
);
GO

CREATE TABLE PatronPayment
(
   ID           bigint        IDENTITY PRIMARY KEY,
   PatronCopy   bigint        NOT NULL,
   EntryDate    datetime      NOT NULL,
   EntryType    varchar(1)    NOT NULL,
   Amount       money         NOT NULL,
   Comment      varchar(50)   NULL,
   UserID       bigint        NULL
);
GO

CREATE TABLE Publisher
(
   ID         bigint         IDENTITY PRIMARY KEY,
   FullName   varchar(100)   NOT NULL,
   WebSite    varchar(255)   NULL
);
GO

CREATE TABLE Subject
(
   ID         bigint         IDENTITY PRIMARY KEY,
   FullName   varchar(150)   NOT NULL
);
GO

CREATE TABLE SystemValue
(
   ID          bigint         IDENTITY PRIMARY KEY,
   ValueName   varchar(50)    NOT NULL,
   ValueData   varchar(100)   NULL
);
GO

INSERT INTO SystemValue (ValueName, ValueData) VALUES ('BarcodeCode39', '0');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('BarcodeFont', NULL);
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('DatabaseVersion', '1');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('DefaultLocation', NULL);
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('FineGrace', '3');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('Licensee', NULL);
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('LicenseCode', NULL);
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('NextBarcodeItem', '2000001');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('NextBarcodeMisc', '1000001');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('NextBarcodePatron', '1');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('PatronCheckOut', '-1');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('SearchLimit', '250');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('TicketHeading', 'Library System');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('TicketFooting', 'Thank you.');
INSERT INTO SystemValue (ValueName, ValueData) VALUES ('UseLC', '0');
GO

CREATE TABLE UserName
(
   ID         bigint        IDENTITY PRIMARY KEY,
   FullName   varchar(50)   NOT NULL,
   LoginID    varchar(20)   NOT NULL,
   Password   varchar(20)   NULL,
   Active     bit           NOT NULL,
   GroupID    bigint        NOT NULL
);
GO

ALTER TABLE GroupActivity    ADD FOREIGN KEY (GroupID)      REFERENCES GroupName       (ID);
ALTER TABLE GroupActivity    ADD FOREIGN KEY (ActivityID)   REFERENCES Activity        (ID);
ALTER TABLE UserName         ADD FOREIGN KEY (GroupID)      REFERENCES GroupName       (ID);
ALTER TABLE NamedItem        ADD FOREIGN KEY (Publisher)    REFERENCES Publisher       (ID);
ALTER TABLE NamedItem        ADD FOREIGN KEY (Series)       REFERENCES CodeSeries      (ID);
ALTER TABLE NamedItem        ADD FOREIGN KEY (MediaType)    REFERENCES CodeMediaType   (ID);
ALTER TABLE ItemCopy         ADD FOREIGN KEY (ItemID)       REFERENCES NamedItem       (ID);
ALTER TABLE ItemCopy         ADD FOREIGN KEY (Status)       REFERENCES CodeCopyStatus  (ID);
ALTER TABLE ItemCopy         ADD FOREIGN KEY (Location)     REFERENCES CodeLocation    (ID);
ALTER TABLE ItemAuthor       ADD FOREIGN KEY (ItemID)       REFERENCES NamedItem       (ID);
ALTER TABLE ItemAuthor       ADD FOREIGN KEY (AuthorID)     REFERENCES Author          (ID);
ALTER TABLE ItemAuthor       ADD FOREIGN KEY (AuthorType)   REFERENCES CodeAuthorType  (ID);
ALTER TABLE ItemKeyword      ADD FOREIGN KEY (ItemID)       REFERENCES NamedItem       (ID);
ALTER TABLE ItemKeyword      ADD FOREIGN KEY (KeywordID)    REFERENCES Keyword         (ID);
ALTER TABLE ItemSubject      ADD FOREIGN KEY (ItemID)       REFERENCES NamedItem       (ID);
ALTER TABLE ItemSubject      ADD FOREIGN KEY (SubjectID)    REFERENCES Subject         (ID);
ALTER TABLE Patron           ADD FOREIGN KEY (PatronGroup)  REFERENCES CodePatronGroup (ID);
ALTER TABLE PatronCopy       ADD FOREIGN KEY (Patron)       REFERENCES Patron          (ID);
ALTER TABLE PatronCopy       ADD FOREIGN KEY (ItemCopy)     REFERENCES ItemCopy        (ID);
ALTER TABLE PatronPayment    ADD FOREIGN KEY (PatronCopy)   REFERENCES PatronCopy      (ID);
ALTER TABLE PatronPayment    ADD FOREIGN KEY (UserID)       REFERENCES UserName        (ID);
ALTER TABLE BarcodeLabel     ADD FOREIGN KEY (BarcodeSheet) REFERENCES BarcodeSheet    (ID);
ALTER TABLE BarcodeLabelItem ADD FOREIGN KEY (BarcodeLabel) REFERENCES BarcodeLabel    (ID);
GO

/* ----- Create the basic security account. */
INSERT INTO GroupName (FullName)
  VALUES ('Administrators');

INSERT INTO GroupActivity (GroupID, ActivityID)
  SELECT 1, ID FROM Activity;

INSERT INTO UserName (FullName, LoginID, Active, GroupID)
  VALUES ('Administrator', 'admin', 1, 1);
GO
