' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Friend Module General
    ' ----- Public constants.
    Public Const ProgramTitle As String = "The Library Project"
    Public Const NotAuthorizedMessage As String = _
        "You are not authorized to perform this task."
    Public Const UseDBVersion As Integer = 1

    ' ----- Constants for the MatchingImages image list.
    Public Const MatchPresent As Integer = 0
    Public Const MatchNone As Integer = 1

    Public Enum LookupMethods As Integer
        ByTitle = 1
        ByAuthor = 2
        BySubject = 3
        ByKeywordAny = 4
        ByKeywordAll = 5
        ByPublisher = 6
        BySeries = 7
        BySeriesID = 8
        ByBarcode = 9
        ByDatabaseID = 10
        ByAuthorID = 11
        BySubjectID = 12
        ByPublisherID = 13
    End Enum

    ' ----- Security values.
    Public Enum LibrarySecurity As Integer
        ManageAuthors = 1
        ManageAuthorTypes = 2
        ManageCopyStatus = 3
        ManageMediaTypes = 4
        ManageSeries = 5
        ManageGroups = 6
        ManageItems = 7
        ManagePatrons = 8
        ManagePublishers = 9
        ManageValues = 10
        ManageUsers = 11
        ProcessFees = 12
        ManageLocations = 13
        CheckOutItems = 14
        CheckInItems = 15
        AdminFeatures = 16
        DailyProcessing = 17
        RunReports = 18
        PatronOverride = 19
        ManageBarcodeTemplates = 20
        ManageHolidays = 21
        ManagePatronGroups = 22
        ViewAdminPatronMessages = 23
    End Enum
    Public Const MaxLibrarySecurity As LibrarySecurity = LibrarySecurity.ViewAdminPatronMessages

    Public Function CenterText(ByVal origText As String, ByVal textWidth As Integer) As String
        ' ----- Center a piece of text in a field width. If the text is too
        '       wide, truncate it.
        Dim resultText As String

        resultText = Trim(origText)
        If (Len(resultText) >= textWidth) Then
            ' ----- Truncate as needed.
            Return Trim(Left(origText, textWidth))
        Else
            ' ----- Start with extra spaces.
            Return Space((textWidth - Len(origText)) \ 2) & resultText
        End If
    End Function

    Public Function CountSubStr(ByVal mainText As String, ByVal subText As String) As Integer
        ' ----- Return a count of the number of times that a subText occurs in
        '       a string (mainText).
        Dim totalTimes As Integer
        Dim startPos As Integer
        Dim foundPos As Integer

        totalTimes = 0
        startPos = 1

        ' ----- Keep searching until we don't find it no more!
        Do
            ' ----- Search for the subText.
            foundPos = InStr(startPos, mainText, subText)
            If (foundPos = 0) Then Exit Do
            totalTimes = totalTimes + 1

            ' ----- Move to just after the occurrence.
            startPos = foundPos + Len(subText)
        Loop

        ' ----- Return the count.
        Return totalTimes
    End Function

    Public Function DigitsOnly(ByVal origText As String) As String
        ' ----- Return only the digits found in a string.
        Dim destText As String
        Dim counter As Integer

        ' ----- Examine each character.
        destText = ""
        For counter = 1 To Len(origText)
            If (IsNumeric(Mid(origText, counter, 1))) Then _
                destText &= Mid(origText, counter, 1)
        Next counter
        Return destText
    End Function

    Public Sub GeneralError(ByVal routineName As String, _
            ByVal theError As System.Exception)
        ' ----- Report an error to the user.
        On Error Resume Next

        MsgBox("The following error occurred at location '" & routineName & _
            "':" & vbCrLf & vbCrLf & theError.Message, _
            MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
        My.Application.Log.WriteException(theError)
    End Sub

    Public Function GetSubStr(ByVal origString As String, ByVal delim As String, _
            ByVal whichField As Integer) As String
        ' ----- Extracts a delimited string from another larger string.
        Dim stringParts() As String

        ' ----- Handle some errors.
        If (whichField < 0) Then Return ""
        If (Len(origString) < 1) Then Return ""
        If (Len(delim) = 0) Then Return ""

        ' ----- Break the string up into delimited parts.
        stringParts = Split(origString, delim)

        ' ----- See if the part we want exists and return it.
        If (whichField > UBound(stringParts) + 1) Then Return "" _
            Else Return stringParts(whichField - 1)
    End Function

    Public Function LeftAndRightText(ByVal origLeft As String, ByVal origRight As String, _
            ByVal textWidth As Integer) As String
        ' ----- Build a string of the indicated width that has both left-justified
        '       and right-justified text.
        Dim leftText As String
        Dim rightText As String

        leftText = Trim(origLeft)
        rightText = Trim(origRight)
        If (Len(leftText & rightText) >= textWidth) Then
            Return Trim(Left(leftText & rightText, textWidth))
        Else
            Return leftText & Space(textWidth - Len(leftText & rightText)) & rightText
        End If
    End Function
End Module
