' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports Microsoft.VisualBasic
Imports System.Data

Public Module General
    Public Function CreateDataTable(ByVal sqlText As String, _
            ByVal whichDatabase As SqlClient.SqlConnection) As Data.DataTable
        ' ----- Given a SQL statement, return a data table.
        Dim dbCommand As SqlClient.SqlCommand
        Dim dbAdapter As SqlClient.SqlDataAdapter
        Dim dbTable As Data.DataTable

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        dbCommand = New SqlClient.SqlCommand(sqlText, whichDatabase)
        dbAdapter = New SqlClient.SqlDataAdapter(dbCommand)
        dbTable = New Data.DataTable
        dbAdapter.Fill(dbTable)
        dbAdapter = Nothing
        dbCommand = Nothing
        Return dbTable
    End Function

    Public Function CreateReader(ByVal sqlText As String, _
            ByVal whichDatabase As SqlClient.SqlConnection) _
            As SqlClient.SqlDataReader
        ' ----- Given a SQL statement, return a data reader.
        Dim dbCommand As SqlClient.SqlCommand
        Dim dbScan As SqlClient.SqlDataReader

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        dbCommand = New SqlClient.SqlCommand(sqlText, whichDatabase)
        dbScan = dbCommand.ExecuteReader()
        dbCommand = Nothing
        Return dbScan
    End Function

    Public Function DBText(ByVal origText As String) As String
        ' ----- Prepare a string for insertion in a SQL statement.
        If (Trim(origText) = "") Then
            Return "NULL"
        Else
            Return "'" & Replace(origText, "'", "''") & "'"
        End If
    End Function

    Public Function ExecuteSQLReturn(ByVal sqlText As String, _
            ByVal whichDatabase As SqlClient.SqlConnection) As Object
        ' ----- Given a SQL statement, run it, and return a related result.
        Dim DBCommand As SqlClient.SqlCommand

        ' ----- Try to run the statement. Note that no error trapping is done
        '       here. It is up to the calling routine to set up error checking.
        DBCommand = New SqlClient.SqlCommand(sqlText, whichDatabase)
        Return DBCommand.ExecuteScalar()
    End Function

    Public Function FormatAuthorName(ByRef dbInfo As SqlClient.SqlDataReader) As String
        ' ----- Given an author record, return the formatted name.
        Dim authorName As String

        On Error Resume Next

        ' ----- Format the name.
        authorName = CStr(dbInfo!LastName)
        If (IsDBNull(dbInfo!FirstName) = False) Then
            authorName &= ", " & CStr(dbInfo!FirstName)
            If (IsDBNull(dbInfo!MiddleName) = False) Then _
                authorName &= " " & CStr(dbInfo!MiddleName)
        End If
        If (IsDBNull(dbInfo!Suffix) = False) Then _
            authorName &= ", " & CStr(dbInfo!Suffix)

        ' ----- Add in the birth and death years.
        If (IsDBNull(dbInfo!BirthYear) = False) Or _
                (IsDBNull(dbInfo!DeathYear) = False) Then
            authorName &= " ("
            If (IsDBNull(dbInfo!BirthYear) = True) Then
                authorName &= "????"
            Else
                authorName &= CStr(Math.Abs(CInt(dbInfo!BirthYear)))
                If (CInt(dbInfo!BirthYear) < 0) Then authorName &= "BC"
            End If
            authorName &= "-"
            If (IsDBNull(dbInfo!DeathYear) = False) Then
                authorName &= CStr(Math.Abs(CInt(dbInfo!DeathYear)))
                If (CInt(dbInfo!DeathYear) < 0) Then authorName &= "BC"
            End If
            authorName &= ")"
        End If

        ' ----- Finished.
        Return authorName
    End Function

    Public Function GetCopyDisposition(ByVal copyID As Integer, _
            ByVal whichDatabase As SqlClient.SqlConnection) As String
        ' ----- Given the ID of an item copy, return it's current status.
        '       The available results are:
        '         New Item Copy      Error or missing record
        '         Checked In         Copy is checked in and available
        '         Checked Out        Copy is checked out by a patron
        '         Overdue            Copy is checked out and late
        '         Missing            Copy has been reported missing
        '         Reference          Copy is a reference copy
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim response As String

        On Error GoTo ErrorHandler

        ' ----- Retrieve information about the book.
        sqlText = "SELECT IC.Reference, IC.Missing, PC.Missing AS PatronMissing, " & _
            "PC.DueDate, PC.Patron FROM ItemCopy AS IC LEFT JOIN PatronCopy AS PC " & _
            "ON IC.ID = PC.ItemCopy WHERE IC.ID = " & copyID & _
            " AND PC.Returned = 0"
        dbInfo = CreateReader(sqlText, whichDatabase)
        If (dbInfo.Read = False) Then
            ' ----- Perhaps the database, due to the JOIN clause, could not return
            '       the base record of a checked-in book.
            dbInfo.Close()
            sqlText = "SELECT Reference, Missing, NULL AS PatronMissing, " & _
                "NULL AS DueDate, NULL AS Patron FROM ItemCopy WHERE ID = " & copyID
            dbInfo = CreateReader(sqlText, whichDatabase)
            If (dbInfo.Read = False) Then
                ' ----- What happened to the record?
                dbInfo.Close()
                dbInfo = Nothing
                Return "New Item Copy"
            End If
        End If

        ' ----- We found the record. Check its details.
        If (IsDBNull(dbInfo!Patron) = False) Then
            ' ----- This item is checked out.
            If (CBool(dbInfo!Missing) = True) Or (CBool(dbInfo!PatronMissing) = True) Then
                response = "Missing"
            ElseIf (CDate(dbInfo!DueDate) < Today) Then
                response = "Overdue"
            Else
                response = "Checked Out"
            End If
        Else
            ' ----- This book is currently checked in.
            If (CBool(dbInfo!Missing) = True) Then
                response = "Missing"
            ElseIf (CBool(dbInfo!Reference) = True) Then
                response = "Reference"
            Else
                response = "Checked In"
            End If
        End If
        dbInfo.Close()
        dbInfo = Nothing
        Return response

ErrorHandler:
        On Error Resume Next
        If Not (dbInfo Is Nothing) Then dbInfo.Close() : dbInfo = Nothing
        Return "New Item Copy"
    End Function
End Module
