' ----- The Library Project: License Generator
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Module General
    Public Const ProgramTitle As String = "Library License Generator"
    Public Const ApplicationProductName As String = "Library Project"

    ' ----- The filenames for the two keys are static.
    Public Const PublicKeyFilename As String = "LibraryPublicKey.xml"
    Public Const PrivateKeyFilename As String = "LibraryPrivateKey.xml"

    ' ----- Indicates the status of the directory containing the 
    '       public and private keys.
    Public Enum KeyFolderStatus
        NoFolder
        InvalidFolder
        MissingBothFiles
        MissingPrivateFile
        MissingPublicFile
        ValidFolder
    End Enum

    Public Function AttemptCreateFolder(ByVal whichFolder As String) As Boolean
        ' ----- Ask the user to create a folder, and create it.
        '       Return True on success.
        If (Trim(whichFolder) = "") Then Return False
        If (My.Computer.FileSystem.DirectoryExists(whichFolder) = True) Then _
            Return True

        ' ----- Prompt the user.
        If (MsgBox("The directory" & vbCrLf & vbCrLf & vbTab & whichFolder & _
            vbCrLf & vbCrLf & "does not exist. Do you wish to create it?", _
            MsgBoxStyle.YesNo Or MsgBoxStyle.Question, ProgramTitle) _
            <> MsgBoxResult.Yes) Then Return False

        ' ----- Create the directory.
        Try
            My.Computer.FileSystem.CreateDirectory(whichFolder)
        Catch ex As Exception
            MsgBox("Could not create the directory due to error: " & ex.Message, _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End Try

        ' ----- Success.
        Return True
    End Function

    Public Function GetFolderStatus(ByVal whichFolder As String) As KeyFolderStatus
        ' ----- Given a folder, return the status.
        Dim usePath As String
        Dim privateExists As Boolean
        Dim publicExists As Boolean

        ' ----- Was anything even supplied?
        If (Trim(whichFolder) = "") Then Return KeyFolderStatus.NoFolder

        ' ----- Is this a valid directory.
        If (My.Computer.FileSystem.DirectoryExists(whichFolder) = False) Then _
            Return KeyFolderStatus.InvalidFolder

        ' ----- Check for the files.
        usePath = My.Computer.FileSystem.CombinePath(whichFolder, PublicKeyFilename)
        publicExists = My.Computer.FileSystem.FileExists(usePath)
        usePath = My.Computer.FileSystem.CombinePath(whichFolder, PrivateKeyFilename)
        privateExists = My.Computer.FileSystem.FileExists(usePath)

        ' ----- Return the file status.
        If (publicExists = False) And (privateExists = False) Then
            Return KeyFolderStatus.MissingBothFiles
        ElseIf (publicExists = False) Then
            Return KeyFolderStatus.MissingPublicFile
        ElseIf (privateExists = False) Then
            Return KeyFolderStatus.MissingPrivateFile
        Else
            Return KeyFolderStatus.ValidFolder
        End If
    End Function

    Public Function GetFolderStatusText(ByVal whichStatus As KeyFolderStatus) As String
        ' ----- Return a description of the current status.
        Select Case whichStatus
            Case KeyFolderStatus.NoFolder
                Return "No directory specified."
            Case KeyFolderStatus.InvalidFolder
                Return "Invaid directory specified."
            Case KeyFolderStatus.MissingPublicFile
                Return "The directly lacks a public key file."
            Case KeyFolderStatus.MissingPrivateFile
                Return "The directly lacks a private key file."
            Case KeyFolderStatus.MissingBothFiles
                Return "The directly lacks the two key files."
            Case KeyFolderStatus.ValidFolder
                Return "The keys are ready to use."
            Case Else
                Return "Unknown."
        End Select
    End Function
End Module
