' ----- The Library Project: License Generator
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports System.Security.Cryptography

Public Class KeyLocationForm
    Public Function GetKeyFolder(ByRef folderPath As String) As Boolean
        ' ----- Prompt the user for the key folder.
        KeyDirectory.Text = folderPath
        RefreshFolderStatus(GetFolderStatus(folderPath))
        If (Me.ShowDialog() = Windows.Forms.DialogResult.OK) Then
            ' ----- Return the new user selection.
            folderPath = KeyDirectory.Text
            Return True
        Else
            Return False
        End If
    End Function

    Private Sub RefreshFolderStatus(ByVal whichStatus As KeyFolderStatus)
        ' ----- Display a description of the current status.
        If (whichStatus = KeyFolderStatus.ValidFolder) Then
            StatusDetailColor.BackColor = Color.Green
        Else
            StatusDetailColor.BackColor = Color.Red
        End If
        Me.StatusDetailText.Text = GetFolderStatusText(whichStatus)
    End Sub

    Private Sub ActGenerate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActGenerate.Click
        ' ----- Generate and save the keys.
        Dim twoPartKey As RSA
        Dim currentStatus As KeyFolderStatus
        Dim publicFile As String
        Dim privateFile As String

        On Error GoTo ErrorHandler

        ' ----- Determine the directory and file status.
        currentStatus = GetFolderStatus(KeyDirectory.Text)
        If (currentStatus = KeyFolderStatus.NoFolder) Then
            MsgBox("Please supply a directory path.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            KeyDirectory.Focus()
            Return
        End If

        ' ----- Make sure the directory exists.
        If (currentStatus = KeyFolderStatus.InvalidFolder) Then
            If (AttemptCreateFolder(KeyDirectory.Text) = False) Then Return
            currentStatus = KeyFolderStatus.MissingBothFiles
            RefreshFolderStatus(currentStatus)
        End If

        ' ----- Warn if either file exists.
        If (currentStatus = KeyFolderStatus.ValidFolder) Then
            If (MsgBox("One or both of the key files already exists. Do you " & _
                "really want to overwrite the existing file(s)? This action " & _
                "cannot be undone.", MsgBoxStyle.YesNo Or MsgBoxStyle.Question Or _
                MsgBoxStyle.DefaultButton2, ProgramTitle) <> MsgBoxResult.Yes) _
                Then Return
        End If

        ' ----- Delete the existing files.
        publicFile = My.Computer.FileSystem.CombinePath( _
            KeyDirectory.Text, PublicKeyFilename)
        privateFile = My.Computer.FileSystem.CombinePath( _
            KeyDirectory.Text, PrivateKeyFilename)
        On Error Resume Next
        Kill(publicFile)
        Kill(privateFile)
        On Error GoTo ErrorHandler
        RefreshFolderStatus(KeyFolderStatus.MissingBothFiles)

        ' ----- Generate the keys.
        twoPartKey = New RSACryptoServiceProvider
        twoPartKey = RSA.Create()

        ' ----- Save the public key.
        My.Computer.FileSystem.WriteAllText(publicFile, _
            twoPartKey.ToXmlString(False), False)
        RefreshFolderStatus(KeyFolderStatus.MissingPrivateFile)

        ' ----- Save the private key.
        My.Computer.FileSystem.WriteAllText(privateFile, _
            twoPartKey.ToXmlString(True), False)
        RefreshFolderStatus(KeyFolderStatus.ValidFolder)

        MsgBox("Key files generated successfully.", _
            MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
        Return

ErrorHandler:
        MsgBox("Error occurred while generating files:" & vbCrLf & vbCrLf & _
            Err.Description, MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, _
            ProgramTitle)
        Return
    End Sub

    Private Sub ActConfirm_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActConfirm.Click
        ' ----- Update the status.
        RefreshFolderStatus(GetFolderStatus(KeyDirectory.Text))
    End Sub

    Private Sub ActBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActBrowse.Click
        ' ----- Let the user browse for the folder through a system dialog.
        If (BrowseKeyFolder.ShowDialog() = Windows.Forms.DialogResult.OK) Then
            KeyDirectory.Text = BrowseKeyFolder.SelectedPath
            ActConfirm.PerformClick()
        End If
    End Sub
End Class