' ----- The Library Project: License Generator
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports System.Xml
Imports System.Security.Cryptography
Imports System.Security.Cryptography.Xml

Public Class MainForm
    Private ActiveStatus As KeyFolderStatus

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim useFolder As String

        ' ----- Licensing defaults to today.
        LicenseDate.Value = Today

        ' ----- Display the folder with the key.
        useFolder = My.Settings.PathForKeyPair
        KeyLocation.Text = useFolder
        ActiveStatus = GetFolderStatus(useFolder)
        If (ActiveStatus = KeyFolderStatus.ValidFolder) Then
            StatusDetailColor.BackColor = Color.Green
        Else
            StatusDetailColor.BackColor = Color.Red
        End If
        StatusDetailText.Text = GetFolderStatusText(ActiveStatus)

        ' ----- If the keys are not valid, prompt immediately.
        If (ActiveStatus <> KeyFolderStatus.ValidFolder) Then
            ActLocation.PerformClick()
        End If
    End Sub

    Private Sub ActLocation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActLocation.Click
        ' ----- Prompt the user to alter the key pair.
        Dim useFolder As String
        Dim result As Boolean

        ' ----- Promt the user.
        useFolder = KeyLocation.Text
        result = KeyLocationForm.GetKeyFolder(useFolder)
        KeyLocationForm = Nothing
        If (result = True) Then
            ' ----- Save the changes.
            My.Settings.PathForKeyPair = useFolder
            ActiveStatus = GetFolderStatus(useFolder)

            ' ----- Update the display.
            KeyLocation.Text = useFolder
            If (ActiveStatus = KeyFolderStatus.ValidFolder) Then
                StatusDetailColor.BackColor = Color.Green
            Else
                StatusDetailColor.BackColor = Color.Red
            End If
            StatusDetailText.Text = GetFolderStatusText(ActiveStatus)
        End If
    End Sub

    Private Sub ActGenerate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActGenerate.Click
        ' ----- Generate the license key.
        Dim licenseXML As XmlDocument

        ' ----- Make sure we have valid data for generation.
        If (VerifyFormData() = False) Then Return

        ' ----- Generate the XML content.
        licenseXML = BuildXmlLicenseContent()
        If (licenseXML Is Nothing) Then Return

        ' ----- Digitally sign the file.
        If (SignXmlLicenseContent(licenseXML) = False) Then Return

        ' ----- Prompt the user for the output location.
        If (LicenseSaveLocation.ShowDialog() <> _
            Windows.Forms.DialogResult.OK) Then Return

        ' ----- Delete any existing license file.
        If (My.Computer.FileSystem.FileExists(LicenseSaveLocation.FileName) = True) Then
            Try
                Kill(LicenseSaveLocation.FileName)
            Catch ex As Exception
                MsgBox("The license could not be saved because an existing " & _
                    "license file with the same name could not be replaced.", _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                Return
            End Try
        End If

        ' ----- Save the license to the file.
        Try
            licenseXML.Save(LicenseSaveLocation.FileName)
        Catch ex As Exception
            MsgBox("The license could not be saved due to the following error:" & _
                vbCrLf & vbCrLf & ex.Message, _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return
        End Try

        ' ----- Finished.
        MsgBox("License file saved successfully.", _
            MsgBoxStyle.OkOnly Or MsgBoxStyle.Information, ProgramTitle)
        LicenseeName.Focus()
    End Sub

    Private Function VerifyFormData() As Boolean
        ' ----- Confirm that all form data is valid. Return True on success.
        Dim versionParts() As String
        Dim scanVersion As String
        Const VersionMessage As String = _
            "The Covered Version must contain between two and four " & _
            "dot-delimited components, and each component must be a " & _
            "number between 0 and 9999, or the '*' character."

        ' ----- The folder status must be valid.
        If (ActiveStatus <> KeyFolderStatus.ValidFolder) Then
            MsgBox("The public-private key location and status must both be valid.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If

        ' ----- The version number must contain between two and four
        '       dot-delimited parts, and the parts must be numbers 
        '       (0 through 9999) or "*".
        versionParts = Split(Trim(CoveredVersion.Text), ".")
        If (versionParts.Length < 2) Or (versionParts.Length > 4) Then
            MsgBox(VersionMessage, _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            CoveredVersion.Focus()
            Return False
        End If
        For Each scanVersion In versionParts
            If (IsNumeric(scanVersion) = True) Then
                If (Val(scanVersion) < 0) Or (Val(scanVersion) > 9999) Then
                    MsgBox(VersionMessage, _
                        MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                    CoveredVersion.Focus()
                    Return False
                End If
            ElseIf (scanVersion <> "*") Then
                MsgBox(VersionMessage, _
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
                CoveredVersion.Focus()
                Return False
            End If
        Next scanVersion

        ' ----- Licensee Name is required.
        If (Trim(LicenseeName.Text) = "") Then
            MsgBox("Licensee Name is required.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            LicenseeName.Focus()
            Return False
        End If

        ' ----- Serial Number is required.
        If (Trim(SerialNumber.Text) = "") Then
            MsgBox("Serial Number is required.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            SerialNumber.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function BuildXmlLicenseContent() As XmlDocument
        ' ----- Create the XML license content from the user-supplied data.
        '       The final XML content will look something like this.
        '         <?xml version="1.0" encoding="utf-8"?>
        '         <License>
        '           <Product>Library Project</Product>
        '           <LicenseDate>1/1/2007</LicenseDate>
        '           <ExpireDate>12/31/2999</ExpireDate>
        '           <CoveredVersion>1.*</CoveredVersion>
        '           <Licensee>Your Name Here</Licensee>
        '           <SerialNumber>LIB-123456789</SerialNumber>
        '         </License>
        Dim result As XmlDocument
        Dim contentHeader As XmlDeclaration
        Dim rootElement As XmlElement
        Dim dataElement As XmlElement

        ' ----- Build the XML header.
        result = New XmlDocument
        contentHeader = result.CreateXmlDeclaration("1.0", _
            Nothing, String.Empty)
        result.InsertBefore(contentHeader, result.DocumentElement)

        ' ----- Add the license root element.
        rootElement = result.CreateElement("License")
        result.InsertAfter(rootElement, contentHeader)

        ' ----- Add the product name.
        dataElement = result.CreateElement("Product")
        dataElement.InnerText = ApplicationProductName
        rootElement.AppendChild(dataElement)

        ' ----- Add the license date.
        dataElement = result.CreateElement("LicenseDate")
        dataElement.InnerText = Format(LicenseDate.Value, "M/d/yyyy")
        rootElement.AppendChild(dataElement)

        ' ----- Add the expiration date.
        dataElement = result.CreateElement("ExpireDate")
        dataElement.InnerText = Format(ExpirationDate.Value, "M/d/yyyy")
        rootElement.AppendChild(dataElement)

        ' ----- Add the covered version number.
        dataElement = result.CreateElement("CoveredVersion")
        dataElement.InnerText = Trim(CoveredVersion.Text)
        rootElement.AppendChild(dataElement)

        ' ----- Add the licensee name.
        dataElement = result.CreateElement("Licensee")
        dataElement.InnerText = Trim(LicenseeName.Text)
        rootElement.AppendChild(dataElement)

        ' ----- Add the serial number.
        dataElement = result.CreateElement("SerialNumber")
        dataElement.InnerText = Trim(SerialNumber.Text)
        rootElement.AppendChild(dataElement)

        ' ----- Finished.
        Return result
    End Function

    Private Function SignXmlLicenseContent(ByVal sourceXML As XmlDocument) As Boolean
        ' ----- Add a digital signature to an XML document.
        Dim privateKeyFile As String
        Dim privateKey As RSA
        Dim signature As SignedXml
        Dim referenceMethod As Reference

        ' ----- Load in the private key.
        privateKeyFile = My.Computer.FileSystem.CombinePath( _
            KeyLocation.Text, PrivateKeyFilename)
        If (My.Computer.FileSystem.FileExists(privateKeyFile) = False) Then
            MsgBox("Could not locate the private key file.", _
                MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
            Return False
        End If
        privateKey = RSA.Create()
        privateKey.FromXmlString(My.Computer.FileSystem.ReadAllText(privateKeyFile))

        ' ----- Create the object that generates the signature.
        signature = New SignedXml(sourceXML)
        signature.SignedInfo.CanonicalizationMethod = _
            SignedXml.XmlDsigCanonicalizationUrl
        signature.SigningKey = privateKey

        ' ----- The signature will appear as a <reference> element in the
        '       XML. The signature object can generate that automatically
        '       if we tell it to. It's more important later when trying to
        '       verify the signature. This transform says to ingore the
        '       signature itself when comparing the signature.
        referenceMethod = New Reference("")
        referenceMethod.AddTransform(New _
            XmlDsigEnvelopedSignatureTransform(False))
        signature.AddReference(referenceMethod)

        ' ----- Add the signature to the XML content.
        signature.ComputeSignature()
        sourceXML.DocumentElement.AppendChild(signature.GetXml())

        ' ----- Finished.
        Return True

ErrorHandler:
        MsgBox("The license could not be digitally signed due to the following " & _
            "error:" & vbCrLf & vbCrLf & Err.Description, _
            MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation, ProgramTitle)
        Return False
    End Function

    Private Sub TextFields_Enter(ByVal sender As Object, ByVal e As System.EventArgs) _
            Handles CoveredVersion.Enter, LicenseeName.Enter, SerialNumber.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub
End Class
