' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports System.Data

Partial Class SearchDetail
    Inherits System.Web.UI.Page

    Public LibraryDB As System.Data.SqlClient.SqlConnection

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' ----- Prepare the detail page.
        Dim itemID As Integer
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim dbTable As DataTable
        Dim counter As Integer
        Dim rowScan As DataRow
        Dim holdText As String
        Dim useLC As Boolean
        Dim copyTable As System.Text.StringBuilder

        ' ----- Get the item ID.
        itemID = Val(Page.Request.QueryString("ID"))
        If (itemID <= 0) Then
            Response.Redirect("Default.aspx")
            Return
        End If

        ' ----- Connect to the database.
        LibraryDB = New SqlClient.SqlConnection( _
            ConfigurationManager.ConnectionStrings( _
            "LibraryConnection").ConnectionString)
        LibraryDB.Open()

        ' ----- Add in the title.
        sqlText = "SELECT Title, Subtitle FROM NamedItem WHERE ID = " & itemID
        dbInfo = CreateReader(sqlText, LibraryDB)
        dbInfo.Read()
        ContentTitle.Text = CStr(dbInfo!Title)

        ' ----- Add in the subtitle if available.
        If (IsDBNull(dbInfo!Subtitle) = False) Then
            ContentSubtitle.Text = CStr(dbInfo!Subtitle)
        Else
            ContentSubtitle.Visible = False
        End If
        dbInfo.Close()

        ' ----- Add in the authors and names.
        sqlText = "SELECT AU.*, IA.Sequence, CA.FullName FROM ItemAuthor AS IA " & _
            "INNER JOIN Author AS AU ON IA.AuthorID = AU.ID INNER JOIN " & _
            "CodeAuthorType AS CA ON IA.AuthorType = CA.ID WHERE IA.ItemID = " & _
            itemID & " ORDER BY IA.Sequence"
        dbInfo = CreateReader(sqlText, LibraryDB)
        If (dbInfo.HasRows) Then
            ' ----- There are authors or names.
            holdText = ""
            Do While dbInfo.Read
                ' ----- Add in this one author name.
                If (holdText <> "") Then holdText &= vbCrLf
                holdText &= FormatAuthorName(dbInfo)
            Loop
        Else
            ContentAuthors.Text = "None listed."
        End If
        dbInfo.Close()

        ' ----- Does this library use LC or Dewey?
        sqlText = "SELECT ValueData FROM SystemValue WHERE UPPER(ValueName) = " & _
            DBText("USELC")
        useLC = CBool(Val(ExecuteSQLReturn(sqlText, LibraryDB)))

        ' ----- Retrieve most of the remaining fields.
        sqlText = "SELECT NI.Dewey, NI.Dewey, NI.LC, NI.Description, NI.Publisher, " & _
            "PB.FullName AS PubName, NI.Edition, NI.Copyright, NI.ISxN, NI.Series, " & _
            "CS.FullName AS SeriesName, CMT.FullName AS MediaName " & _
            "FROM NamedItem AS NI LEFT JOIN Publisher AS PB ON NI.Publisher = PB.ID " & _
            "LEFT JOIN CodeSeries AS CS ON NI.Series = CS.ID " & _
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " & _
            "WHERE NI.ID = " & itemID
        dbInfo = CreateReader(sqlText, LibraryDB)
        dbInfo.Read()

        ' ----- Add in the call number information.
        holdText = "No call number available."
        If (useLC) Then
            ' ----- Show the Library of Congress number.
            If (IsDBNull(dbInfo!LC) = False) Then holdText = CStr(dbInfo!LC)
        Else
            ' ----- Show the Dewey Decimal number.
            If (IsDBNull(dbInfo!Dewey) = False) Then holdText = CStr(dbInfo!Dewey)
        End If
        holdText = Replace(holdText, "/", vbCrLf)
        ContentCall.Text = holdText

        ' ----- Display the media type name.
        ContentMedia.Text = CStr(dbInfo!MediaName)

        ' ----- Add in the item description.
        If (IsDBNull(dbInfo!Description) = False) Then
            ContentDescription.Text = CStr(dbInfo!Description)
        Else
            ContentDescription.Text = "Not supplied."
        End If

        ' ----- Add in the publishing information.
        holdText = "None available."
        counter = 0
        If (IsDBNull(dbInfo!PubName) = False) Then counter += 1
        If (IsDBNull(dbInfo!Edition) = False) Then counter += 1
        If (IsDBNull(dbInfo!Copyright) = False) Then counter += 1
        If (IsDBNull(dbInfo!ISxN) = False) Then counter += 1
        If (counter > 0) Then
            ' ----- There is some publishing information to display.
            holdText = ""

            ' ----- Add the publisher name, if available.
            If (IsDBNull(dbInfo!PubName) = False) Then _
                holdText = CStr(dbInfo!PubName)

            ' ----- Add the edition number, if available.
            If (IsDBNull(dbInfo!Edition) = False) Then
                If (holdText <> "") Then holdText &= vbCrLf
                holdText &= "Edition: " & CStr(dbInfo!Edition)
            End If

            ' ----- Add the copyright year, if available.
            If (IsDBNull(dbInfo!Copyright) = False) Then
                If (holdText <> "") Then holdText &= vbCrLf
                holdText &= "Copyright: " & CStr(dbInfo!Copyright)
            End If

            ' ----- Add the ISBN/ISSN number, if available.
            If (IsDBNull(dbInfo!ISxN) = False) Then
                If (holdText <> "") Then holdText &= vbCrLf
                holdText &= "ISBN / ISSN: " & CStr(dbInfo!ISxN)
            End If
        End If
        ContentPublish.Text = holdText

        ' ----- Add in the series information.
        If (IsDBNull(dbInfo!SeriesName) = False) Then
            ContentSeries.Text = CStr(dbInfo!SeriesName)
        Else
            ContentSeries.Text = "None listed."
        End If
        dbInfo.Close()

        ' ----- Add in the related subjects.
        holdText = "None listed."
        sqlText = "SELECT SB.* FROM ItemSubject AS IB INNER JOIN Subject AS SB " & _
            "ON IB.SubjectID = SB.ID WHERE IB.ItemID = " & _
            itemID & " ORDER BY SB.FullName"
        dbInfo = CreateReader(sqlText, LibraryDB)
        If (dbInfo.HasRows) Then
            ' ----- There are subjects. Display a heading.
            holdText = ""

            Do While dbInfo.Read
                ' ----- Add in this one subject name.
                If (holdText <> "") Then holdText &= vbCrLf
                holdText &= CStr(dbInfo!FullName)
            Loop
        End If
        dbInfo.Close()
        ContentSubject.Text = holdText

        ' ----- See how many copies there are.
        sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE ItemID = " & itemID
        counter = CInt(ExecuteSQLReturn(sqlText, LibraryDB))

        ' ----- Add in the copies information if needed.
        If (counter > 0) Then
            ' ----- Display the right copy count.
            If (counter = 1) Then
                ContentCopies.Text = "1 copy"
            Else
                ContentCopies.Text = counter & " copies"
            End If

            ' ----- Build up a table by hand.
            copyTable = New System.Text.StringBuilder
            copyTable.AppendLine("<table cellspacing=""0"" cellpadding=""3"">")
            copyTable.AppendLine("<tr><th>Copy</th><th>Location</th>" & _
                "<th>Status</th></tr>")
            sqlText = "SELECT IC.ID, IC.CopyNumber, CL.FullName FROM ItemCopy AS IC " & _
                "LEFT JOIN CodeLocation AS CL ON IC.Location = CL.ID " & _
                "WHERE IC.ItemID = " & itemID & " ORDER BY IC.CopyNumber"
            dbTable = CreateDataTable(sqlText, LibraryDB)
            For Each rowScan In dbTable.Rows
                ' ----- Add the copy number.
                copyTable.Append("<tr><td>" & CStr(CInt(rowScan!CopyNumber)) & "</td>")

                ' ----- Add the location.
                If (IsDBNull(rowScan!FullName) = True) Then holdText = "Not listed" _
                    Else holdText = Server.HtmlEncode(CStr(rowScan!FullName))
                copyTable.Append("<td>" & holdText & "</td>")

                ' ----- Add the disposition.
                copyTable.AppendLine("<td>" & Server.HtmlEncode(GetCopyDisposition( _
                    CInt(rowScan!ID), LibraryDB)) & "</td></tr>")
            Next rowScan
            dbTable.Dispose()
            dbTable = Nothing
            copyTable.AppendLine("</table>")

            ' ----- Add the table to the output.
            PutTableHere.Text = copyTable.ToString()
        End If

        ' ----- Finished with the connection.
        LibraryDB.Close()
    End Sub
End Class
