' ----- The Library Project
'       Start-to-Finish Visual Basic 2005
'       Copyright (c) 2006 by Tim Patrick

Imports System.Data

Partial Class SearchResults
    Inherits System.Web.UI.Page

    Public LibraryDB As System.Data.SqlClient.SqlConnection

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' ----- Prepare the data for the report.
        Dim sqlText As String
        Dim dbInfo As SqlClient.SqlDataReader
        Dim reportData As Collections.Generic.List(Of BoundSchemaSearchResults)
        Dim oneEntry As BoundSchemaSearchResults
        Dim useTitle As String = ""
        Dim useLastName As String = ""
        Dim useFirstName As String = ""
        Dim useMediaType As Integer = -1
        Dim sourceTextBox As TextBox
        Dim sourceMediaType As DropDownList
        Dim needsAnd As Boolean

        ' ----- This page is only valid from the Default.aspx page.
        If (Page.PreviousPage Is Nothing) Then
            Response.Redirect("Default.aspx")
            Return
        End If

        ' ----- Get the title search text.
        sourceTextBox = CType(PreviousPage.FindControl("SearchTitle"), TextBox)
        If (sourceTextBox IsNot Nothing) Then useTitle = Trim(sourceTextBox.Text)

        ' ----- Get the last name search text.
        sourceTextBox = CType(PreviousPage.FindControl("SearchLastName"), TextBox)
        If (sourceTextBox IsNot Nothing) Then useLastName = Trim(sourceTextBox.Text)

        ' ----- Get the first name search text.
        sourceTextBox = CType(PreviousPage.FindControl("SearchFirstName"), TextBox)
        If (sourceTextBox IsNot Nothing) Then useFirstName = Trim(sourceTextBox.Text)

        ' ----- Get the media type value.
        sourceMediaType = CType(PreviousPage.FindControl("SearchMediaType"), _
            DropDownList)
        If (sourceMediaType IsNot Nothing) Then useMediaType = _
            sourceMediaType.SelectedValue

        ' ----- Check for missing data.
        If (useTitle = "") And (useLastName = "") And (useFirstName = "") And _
                (useMediaType = -1) Then
            ' ----- No criteria supplied by the user.
            ResultsGrid.Visible = False
            MatchCount.Text = "No search criteria specified."
            MatchCount.Font.Italic = False
            MatchCount.Font.Bold = True
            Return
        End If

        ' ----- Connect to the database.
        LibraryDB = New SqlClient.SqlConnection( _
            ConfigurationManager.ConnectionStrings( _
            "LibraryConnection").ConnectionString)
        LibraryDB.Open()

        ' ----- Build a data set to hold the results.
        reportData = New Collections.Generic.List(Of BoundSchemaSearchResults)

        ' ------ Prepare to query the database.
        needsAnd = False
        sqlText = "SELECT NI.ID, NI.Title, CMT.FullName AS MediaName, " & _
            "AU.LastName, AU.FirstName FROM NamedItem AS NI INNER JOIN " & _
            "CodeMediaType AS CMT ON NI.MediaType = CMT.ID LEFT JOIN " & _
            "ItemAuthor AS IA ON  NI.ID = IA.ItemID LEFT JOIN Author AS AU " & _
            "ON IA.AuthorID = AU.ID"

        ' ----- Add in the title criteria if supplied.
        If (useTitle <> "") Then
            ' ----- Add the title as text. Always append a wildcard if it's
            '       not yet included.
            If (InStr(useTitle, "*") = 0) Then useTitle &= "*"
            useTitle = Replace(useTitle, "*", "%")
            If (needsAnd = True) Then sqlText &= " AND " Else sqlText &= " WHERE "
            needsAnd = True
            sqlText &= "NI.Title LIKE " & DBText(useTitle)
        End If

        ' ----- Add in the author last name criteria if supplied.
        If (useLastName <> "") Then
            ' ----- Add the last name. Always append a wildcard if it's
            '       not yet included.
            If (InStr(useLastName, "*") = 0) Then useLastName &= "*"
            useLastName = Replace(useLastName, "*", "%")
            If (needsAnd = True) Then sqlText &= " AND " Else sqlText &= " WHERE "
            needsAnd = True
            sqlText &= "AU.LastName LIKE " & DBText(useLastName)
        End If

        ' ----- Add in the author first name criteria if supplied.
        If (useFirstName <> "") Then
            ' ----- Add the first name. Always append a wildcard if it's
            '       not yet included.
            If (InStr(useFirstName, "*") = 0) Then useFirstName &= "*"
            useFirstName = Replace(useFirstName, "*", "%")
            If (needsAnd = True) Then sqlText &= " AND " Else sqlText &= " WHERE "
            needsAnd = True
            sqlText &= "AU.FirstName LIKE " & DBText(useFirstName)
        End If

        ' ----- Add in the media type criteria if supplied.
        If (useMediaType <> -1) Then
            If (needsAnd = True) Then sqlText &= " AND " Else sqlText &= " WHERE "
            needsAnd = True
            sqlText &= "NI.MediaType LIKE " & useMediaType
        End If

        ' ----- Retrieve the results.
        dbInfo = CreateReader(sqlText, LibraryDB)
        Do While dbInfo.Read
            ' ----- Add it to the report data.
            oneEntry = New BoundSchemaSearchResults
            oneEntry.ID = CInt(dbInfo!ID)
            oneEntry.ItemName = CStr(dbInfo!Title)
            If (IsDBNull(dbInfo!LastName) = True) Then useLastName = "" Else _
                useLastName = CStr(dbInfo!LastName)
            If (IsDBNull(dbInfo!FirstName) = True) Then useFirstName = "" Else _
                useFirstName = CStr(dbInfo!FirstName)
            If (useFirstName <> "") Then
                If (useLastName <> "") Then useLastName &= ", "
                useLastName &= useFirstName
            End If
            oneEntry.AuthorName = useLastName
            oneEntry.MediaType = CStr(dbInfo!MediaName)

            reportData.Add(oneEntry)
        Loop
        dbInfo.Close()

        ' ----- Finished with the connection.
        LibraryDB.Close()

        ' ----- Show the results.
        If (reportData.Count = 0) Then
            ' ----- No matching items.
            ResultsGrid.Visible = False
            MatchCount.Text = "No matching items."
            MatchCount.Font.Italic = False
            MatchCount.Font.Bold = True
        Else
            ' ----- Has some results.
            ResultsGrid.DataSource = reportData
            ResultsGrid.DataBind()
            If (reportData.Count = 1) Then
                MatchCount.Text = "1 matching item."
            Else
                MatchCount.Text = reportData.Count & " matching items."
            End If
        End If
    End Sub
End Class
